/*	MiscRubberband.m

	Copyright 1996 Slugg Jello.

	This notice may not be removed from this source code.
	The use and distribution of this software is governed by the
	terms of the MiscKit license agreement.  Refer to the license
	document included with the MiscKit distribution for the terms.

*/


#import "MiscRubberband.h"
#import "Miscrubber.h" 
#include <math.h>

#define MISC_RUBBERBAND_WINDOW_WIDTH	1

@interface MiscRubberband (PrivateMethods)

- (void)_internalCorrectRect:(NSRect *)rect;
- (void)_internalSetPosition;

@end

@implementation MiscRubberband

+ rubberbandWithView:(NSView *)aView
{
	return [[[MiscRubberband alloc] initWithView:aView] autorelease];
}

- initWithView:(NSView *)aView
{	
	[super init];
	view = aView;
	boundsRect = [view convertRect:[view visibleRect] toView:nil];
	return self;
}

- (void)doStretch:(NSEvent *)event
{
	NSEvent *nextEvent;
	NSPoint mouseLocation, p;
	NSRect tRect;
	NSWindow *window = [view window];
			 
	currentRect.origin = [event locationInWindow];
	currentRect.size.width = currentRect.size.height = 0.0;	
        if(![view mouse:currentRect.origin inRect:boundsRect])
		return;
	mouseLocation = currentRect.origin;
	MiscPSWInitRubberband();
	[self _internalSetPosition];
	MiscPSWMakeVisible(YES);
	nextEvent = [window nextEventMatchingMask:NSLeftMouseUpMask|NSLeftMouseDownMask|NSLeftMouseDraggedMask];	
	tRect = currentRect;
	while([nextEvent type] != NSLeftMouseUp){
		p = [nextEvent locationInWindow];
		tRect.size.width += p.x - mouseLocation.x;
		tRect.size.height += p.y - mouseLocation.y;
		currentRect = tRect;
		[self _internalCorrectRect:&currentRect];
		currentRect = NSIntersectionRect(boundsRect,currentRect);
		currentRect = NSIntegralRect(currentRect);
		mouseLocation = p;
		[self _internalSetPosition];
		nextEvent = [window nextEventMatchingMask:NSLeftMouseUpMask|NSLeftMouseDownMask|NSLeftMouseDraggedMask];
	}
	MiscPSWMakeVisible(NO);
}

- (NSRect)currentRect
{
	return [view convertRect:currentRect fromView:nil];	
}
 
@end

@implementation MiscRubberband (PrivateMethods)

- (void)_internalCorrectRect:(NSRect *)rect
{
	if (rect->size.width < 0.0){
		rect->origin.x += rect->size.width;
		rect->size.width = -rect->size.width;
	}
	if (rect->size.height < 0.0){
		rect->origin.y += rect->size.height;
		rect->size.height = -rect->size.height;
	}
}

- (void)_internalSetPosition
/* Position and draw rubberband.  Assumes currentRect is set and rubberband
	windows are initialized.  Also assumes currentRect is 'corrected'. */
{
	struct {
		NSRect top,left,bottom,right;
	} points;
	
	NSRect tRect, screenRect = currentRect;
	
	screenRect.origin = [[view window] convertBaseToScreen:screenRect.origin];
		
	/**** set size of each rubberband side ****/
	
	/* left */
	tRect = screenRect;
	tRect.size.width = MISC_RUBBERBAND_WINDOW_WIDTH;
	tRect.size.height = MAX(MISC_RUBBERBAND_WINDOW_WIDTH,screenRect.size.height);
	points.left = tRect;
	
	/* right */
	tRect.origin.x += screenRect.size.width - 1.0;
	points.right = tRect;
	
	/* bottom */
	tRect = screenRect;
	tRect.origin.x += 1.0;
	tRect.size.width = MAX(MISC_RUBBERBAND_WINDOW_WIDTH,screenRect.size.width - 1.0);
	tRect.size.height = MISC_RUBBERBAND_WINDOW_WIDTH;
	points.bottom = tRect;
	
	/* top */
	tRect.origin.y += screenRect.size.height - 1.0;
	points.top = tRect;
	
	/* this'll erase old one and draw new one in new position */
	MiscPSWPositionRect((float *)&points);

}

@end
