/*	MiscNodeStyle.m

	Copyright 1996 Uwe Hoffmann.

	This notice may not be removed from this source code.
	The use and distribution of this software is governed by the
	terms of the MiscKit license agreement.  Refer to the license
	document included with the MiscKit distribution for the terms.

	Version 2 (August 1996)
*/

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>

#import "MiscNodeStyle.h"

@implementation MiscNodeStyle

- init
{
	[super init];
	fillColor = [[NSColor whiteColor] retain];
	outlineColor = [[NSColor blackColor] retain];
	linewidth = 0.15;
	fill = YES;
	outline = YES;
	automaticResize = NO;
        textPlacement = MiscMiddleTextPlacement;
        childEnding = MiscNoEnding;
        parentEnding = MiscNoEnding;
	return self;
}

- (void)dealloc
{
        [fillColor release];
        [outlineColor release];
        return [super dealloc];
}

- (BOOL)isEqual:(id)anObject
{
    	MiscNodeStyle *co;
    
    	if(!anObject)
            	return NO;
    	if(anObject == self)
            	return YES;
        if(![anObject isKindOfClass:[self class]])
            	return NO;
        co = (MiscNodeStyle *)anObject;
        if([fillColor isEqual:co->fillColor] &&
           [outlineColor isEqual:co->outlineColor] &&
           childEnding == co->childEnding &&
           parentEnding == co->parentEnding &&
           textPlacement == co->textPlacement &&
           automaticResize == co->automaticResize &&
           outline == co->outline &&
           fill == co->fill &&
           linewidth == co->linewidth)
            	return YES;
        else
            	return NO;
}

- (NSColor *)fillColor
{
	return fillColor;
}

- (void)setFillColor:(NSColor *)aColor
{
    	if(!aColor || [fillColor isEqual:aColor])
            	return;
    	[fillColor release];
	fillColor = [aColor copyWithZone:[self zone]];
}

- (NSColor *)outlineColor
{
	return outlineColor;
}

- (void)setOutlineColor:(NSColor *)aColor
{
    	if(!aColor || [outlineColor isEqual:aColor])
            	return;
    	[outlineColor release];
     	outlineColor = [aColor copyWithZone:[self zone]];
}

- (BOOL)fill
{
	return fill;
}

- (void)setFill:(BOOL)aBOOL
{
	fill = aBOOL;
}

- (BOOL)outline
{
	return outline;
}

- (void)setOutline:(BOOL)aBOOL
{
	outline = aBOOL;
}

- (float)linewidth
{
	return linewidth;
}

- (void)setLinewidth:(float)aLinewidth
{
	linewidth = aLinewidth;
}

- (BOOL)automaticResize
{
	return automaticResize;
}

- (void)setAutomaticResize:(BOOL)aBOOL
{
	automaticResize = aBOOL;
}

- (MiscTextPlacement)textPlacement
{
    	return textPlacement;
}

- (void)setTextPlacement:(MiscTextPlacement)aTextPlacement
{
    	textPlacement = aTextPlacement;
}

- (MiscEndingType)childEnding
{
    	return childEnding;
}

- (void)setChildEnding:(MiscEndingType)anEnding
{
    	childEnding = anEnding;
}

- (MiscEndingType)parentEnding
{
    	return parentEnding;
}

- (void)setParentEnding:(MiscEndingType)anEnding
{
    	parentEnding = anEnding;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
	[coder encodeObject:fillColor];
        [coder encodeObject:outlineColor];
	[coder encodeValuesOfObjCTypes:"cccf", &automaticResize,&fill,&outline,&linewidth];
	[coder encodeValueOfObjCType:@encode(MiscTextPlacement) at:&textPlacement];
	[coder encodeValueOfObjCType:@encode(MiscEndingType) at:&childEnding];
	[coder encodeValueOfObjCType:@encode(MiscEndingType) at:&parentEnding];
}

- initWithCoder:(NSCoder *)coder
{
	fillColor = [[coder decodeObject] copyWithZone:[self zone]];
     	outlineColor = [[coder decodeObject] copyWithZone:[self zone]];
	[coder decodeValuesOfObjCTypes:"cccf", &automaticResize,&fill,&outline,&linewidth];
	[coder decodeValueOfObjCType:@encode(MiscTextPlacement) at:&textPlacement];
	[coder decodeValueOfObjCType:@encode(MiscEndingType) at:&childEnding];
	[coder decodeValueOfObjCType:@encode(MiscEndingType) at:&parentEnding];
	return self;
}

- copyWithZone:(NSZone *)zone
{
   	MiscNodeStyle *theCopy;

   	theCopy = (MiscNodeStyle *)NSCopyObject(self, 0, zone);
        theCopy->fillColor = [fillColor copyWithZone:zone];
        theCopy->outlineColor = [outlineColor copyWithZone:zone];
	return theCopy;
}

- copy
{
	return [self copyWithZone:NSDefaultMallocZone()];
}

@end

