/*	MiscTreeDiagramView.h

	Copyright 1996 Uwe Hoffmann.

	This notice may not be removed from this source code.
	The use and distribution of this software is governed by the
	terms of the MiscKit license agreement.  Refer to the license
	document included with the MiscKit distribution for the terms.

	Version 2 (August 1996)
*/

#import <AppKit/AppKit.h>
#import <Foundation/Foundation.h>

#import "MiscDiagramTree.h"	
#import "MiscTreeDiagramViewAPI.h"

#define MISCTDV_REPAINT_NORMAL	1
#define MISCTDV_REPAINT_EVENT	2

@class MiscTreeDiagram, MiscUserPath, MiscHitPath, MiscDiagramTree;

@interface MiscTreeDiagramView:NSView <MiscTreeDiagramViewAPI>
{
@private
	MiscTreeDiagram *treeDiagram;
	MiscUserPath *linePath, *canvasPath;
	MiscHitPath *hitPath;
	BOOL isOwnerOfDiagram;
	NSRect canvasRect;
	int canvasXTimes, canvasYTimes;
	NSMutableArray *nodes;
	MiscDrawState drawState;
	BOOL showAttachments;
	int repaintMode;
	NSImage *buffer;
        float zoomFactor;
	struct {
		MiscDiagramTree *node;
		NSRect textFrame;
	} labelEditingTempData;	
}

- initWithFrame:(NSRect)frameRect treeDiagram:(MiscTreeDiagram *)aDiagram;

- (MiscTreeDiagram *)diagram;
- (void)setDiagram:(MiscTreeDiagram *)diagram;

- (BOOL)writeDiagramToFile:(NSString *)path;
- (BOOL)readDiagramFromFile:(NSString *)path;

- (void)zoomToFactor:(float)aZoomFactor;
- (void)zoomToFit:(id)sender;
- (void)zoomToFactor:(float)aZoomFactor andUpdate:(id)sender;
- (void)zoomToRect:(NSRect)aZoomRect;
- (float)zoomFactor;

@end

@interface MiscTreeDiagramView(Dragging)

+ (void)pasteboard:(NSPasteboard *)pb provideDataForType:(NSString *)type;
- (unsigned int)draggingEntered:sender;
- (unsigned int)draggingUpdated:sender;
- (BOOL)performDragOperation:sender;

@end

@interface MiscTreeDiagramView(TargetAction)

- (void)cut:(id)sender;
- (void)paste:(id)sender;
- (void)delete:(id)sender;
- (void)copy:(id)sender;
- (void)undo:(id)sender;
- (void)redo:(id)sender;
- (void)copyStyle:(id)sender;
- (void)pasteStyle:(id)sender;
- (void)pageLayout:(id)sender;
- (void)collapseExpand:(id)sender;
- (void)toggleRuler:(id)sender;

- (BOOL)validateMenuItem:(NSMenuItem *)anItem;

@end
	