/*	MiscTreeDiagramView.h

	Copyright 1996 Uwe Hoffmann.

	This notice may not be removed from this source code.
	The use and distribution of this software is governed by the
	terms of the MiscKit license agreement.  Refer to the license
	document included with the MiscKit distribution for the terms.

	Version 2 (August 1996)
*/

#import "MiscTreeDiagramView.h"
#import "MiscTreeDiagram.h"
#import "MiscUserPath.h"
#import "MiscHitPath.h"
#import "MiscTDVPrivate.h"
#import "MiscTDUtils.h"
 
@implementation MiscTreeDiagramView

- (float)length
{
    	return 1;
}

- initWithFrame:(NSRect)frameRect treeDiagram:(MiscTreeDiagram *)aDiagram
{
	[super initWithFrame:frameRect];
	treeDiagram = aDiagram;
	isOwnerOfDiagram = NO;
	[self _internalInit];
	return self;
}

- initWithFrame:(NSRect)frameRect
{
	[super initWithFrame:frameRect];
	treeDiagram = [[MiscTreeDiagram allocWithZone:[self zone]] init];
	[treeDiagram setView:self];
	isOwnerOfDiagram = YES;
	[self _internalInit];
	return self;
}

- (void)dealloc
{
	if(isOwnerOfDiagram)
		[treeDiagram release];
	[linePath release];
	[hitPath release];
	[canvasPath release];
	[nodes release];
	if(buffer)
		[buffer release];
	return [super dealloc];
}

- (BOOL)isFlipped
{
    return YES;
}

- (void)lockFocus
{
   	float rot,gruen,blau;

        [super lockFocus];
	if([[NSDPSContext currentContext] isDrawingToScreen]){
		PScurrentlinewidth(&drawState.linewidth);
		PScurrentrgbcolor(&rot,&gruen,&blau);
                if(drawState.color)
                    	[drawState.color release];
		drawState.color = [[NSColor colorWithCalibratedRed:rot green:gruen blue:blau alpha:1.0] retain];
		drawState.font = nil;
   	}
}

- (void)unlockFocus
{
	[super unlockFocus];
        if(drawState.color)
                [drawState.color release];
        drawState.color = nil;
        if(drawState.font)
                 [drawState.font release];
        drawState.font = nil;
}

- (void)repaintVisible
{
	NSRect rx;
	NSWindow *window = [self window];
	
	rx = [self visibleRect];
	if(repaintMode == MISCTDV_REPAINT_NORMAL){
		[window disableFlushWindow];
		[self displayRect:rx];
		[window enableFlushWindow];
		[window flushWindow];
	} else
		[self drawRect:rx];	
}

- (void)repaintAll
{
	NSWindow *window = [self window];

	if(repaintMode == MISCTDV_REPAINT_NORMAL){
		[window disableFlushWindow];
		[self displayRect:[self bounds]];
		[window enableFlushWindow];
		[window flushWindow];
	} else
		[self drawRect:[self bounds]];	
}

- (void)repaintRect:(NSRect)aRect
{
	NSWindow *window = [self window];

	if(repaintMode == MISCTDV_REPAINT_NORMAL){
		[window disableFlushWindow];
		[self displayRect:aRect];
		[window enableFlushWindow];
		[window flushWindow];
	} else
		[self drawRect:aRect];	
}

- (void)repaintRects:(NSRect)rect1 :(NSRect)rect2
{
	NSWindow *window = [self window];

	if(repaintMode == MISCTDV_REPAINT_NORMAL){
		[window disableFlushWindow];
		[self lockFocus];
		[self drawRect:rect1];
		[self drawRect:rect2];
		[self unlockFocus];
		[window enableFlushWindow];
		[window flushWindow];
	} else {
		[self drawRect:rect1];
		[self drawRect:rect2];
	}
}

- (MiscTreeDiagram *)diagram
{
	return treeDiagram;
}

- (void)addToPageSetup
{
    	[super addToPageSetup];
     	if(zoomFactor != 1.0){
             PSscale(1 / zoomFactor, 1 / zoomFactor);
     	}
}

- (void)zoomToFactor:(float)aZoomFactor
{
    	MiscDiagramTree *selectedNode = [treeDiagram selectedNode];
        NSScrollView *sv = [self enclosingScrollView];
        NSRect newFrameRect;
        
        if(!sv || aZoomFactor == zoomFactor)
            return;        
        [self scaleUnitSquareToSize:NSMakeSize(aZoomFactor / zoomFactor,aZoomFactor / zoomFactor)];
        newFrameRect = [self frame];
        newFrameRect.size.width *= aZoomFactor / zoomFactor;
        newFrameRect.size.height *= aZoomFactor / zoomFactor;
        [self setFrame:newFrameRect];
        zoomFactor = aZoomFactor;
        if(selectedNode)
        	[self scrollRectToVisible:[selectedNode nodeDrawBounds]];
        [[self window] display];
}

- (float)zoomFactor
{
    return zoomFactor;
}

- (void)zoomToFit:(id)sender
{
    NSRect docFrame, contentFrame;
    float scaleX, scaleY;
    NSScrollView *sv = [self enclosingScrollView];
    NSString *title;
    int zoom;
    NSArray *itemTitles;
    int index, numberOfTitles;
    NSWindow *window = [self window];

    if(!sv)
        return;
    [window disableFlushWindow];
    docFrame = [self frame];
    contentFrame = [[sv contentView] frame];
    scaleX = contentFrame.size.width / (docFrame.size.width + 1);
    scaleY = contentFrame.size.height /(docFrame.size.height + 1);
    if(scaleX > scaleY)
        scaleX = scaleY;
    else
        scaleY = scaleX;
    [self zoomToFactor:scaleX];
    if([sender isKindOfClass:[NSPopUpButton class]]){
        zoom = (int)(zoomFactor * 100);
        title = [NSString stringWithFormat:@"%d \%", zoom];
        itemTitles = [sender itemTitles];
        index = 0;
        numberOfTitles = [itemTitles count] - 2;
        while(index < numberOfTitles && zoom > [[itemTitles objectAtIndex:index] intValue])
            index++;
        if(index == numberOfTitles)
            [sender insertItemWithTitle:title atIndex:index];
        else if(zoom < [[itemTitles objectAtIndex:index] intValue])
            [sender insertItemWithTitle:title atIndex:index];
        [sender selectItemAtIndex:index];
    }
    [window enableFlushWindow];
    [window flushWindow];
    return;
}

- (void)zoomToFactor:(float)aZoomFactor andUpdate:(id)sender
{
    NSScrollView *sv = [self enclosingScrollView];
    NSString *title;
    int zoom;
    NSArray *itemTitles;
    int index, numberOfTitles;
    NSWindow *window = [self window];

    if(!sv)
        return;
    [window disableFlushWindow];
    [self zoomToFactor:aZoomFactor];
    if([sender isKindOfClass:[NSPopUpButton class]]){
        zoom = (int)(zoomFactor * 100);
        title = [NSString stringWithFormat:@"%d \%", zoom];
        itemTitles = [sender itemTitles];
        index = 0;
        numberOfTitles = [itemTitles count] - 2;
        while(index < numberOfTitles && zoom > [[itemTitles objectAtIndex:index] intValue])
            index++;
        if(index == numberOfTitles)
            [sender insertItemWithTitle:title atIndex:index];
        else if(zoom < [[itemTitles objectAtIndex:index] intValue])
            [sender insertItemWithTitle:title atIndex:index];
        [sender selectItemAtIndex:index];
    }
    [window enableFlushWindow];
    [window flushWindow];
    return;
}

- (void)zoomToRect:(NSRect)aZoomRect
{
    return;
}

- (void)setDiagram:(MiscTreeDiagram *)diagram
{
	if(!diagram || treeDiagram == diagram)
		return;
	if(isOwnerOfDiagram){
		[treeDiagram release];
		treeDiagram = [diagram retain];
		[treeDiagram setView:self];
	} else 
		treeDiagram = diagram;
	[self repaintAll];
}

- (BOOL)writeDiagramToFile:(NSString *)path
{
	BOOL result;
	
	result = [NSArchiver archiveRootObject:treeDiagram toFile:path];
	return result;
}

- (BOOL)readDiagramFromFile:(NSString *)path
{
	BOOL result = NO;
	MiscTreeDiagram *diagram;
	
	diagram = (MiscTreeDiagram *)[NSUnarchiver unarchiveObjectWithFile:path];
	if(diagram){
		[self setDiagram:diagram];
		result = YES;
	}
	return result;
}	

- (void)drawRect:(NSRect)r
{  
	MiscDiagramTree *tree;
	int i,j;
	BOOL resized = NO;
	NSRect theRect;

	tree = [treeDiagram root];
	if(tree && [tree needsUpdateLayout]){		
		[tree layout];
		resized = [self _internalResizeAndPosTree:tree force:NO];
	}
	if(resized)
		theRect = [self bounds];
	else
		theRect = r;
   	PSsetgray(NSWhite);
	NSRectFill(theRect);
        if(drawState.color)
            	[drawState.color release];
	drawState.color = [[NSColor whiteColor] retain];
	if([[NSDPSContext currentContext] isDrawingToScreen]){
		PSrectviewclip(theRect.origin.x,theRect.origin.y,theRect.size.width,theRect.size.height);       
		PSsetgray(NSLightGray);
		PSsetlinewidth(0.15);
                if(drawState.color)
                                [drawState.color release];
		drawState.color = [[NSColor lightGrayColor] retain];
		drawState.linewidth = 0.15;
		for(i = 0;i <= canvasXTimes;i++)
			for(j = 0;j <= canvasYTimes;j++){
				[self _internalUpdateCanvasPos:i * NSWidth(canvasRect) :j * NSHeight(canvasRect)];
				[canvasPath send:dps_ustroke cached:YES];
			}
	}
	if(tree)
		[self _internalDrawTree:tree inRect:theRect];
	if([[NSDPSContext currentContext] isDrawingToScreen])
          	PSinitviewclip();	         	         	     			
}

- (void)mouseDown:(NSEvent *)event 
{
    	NSPoint p;
	MiscDiagramTree *selectedNode;
	int controlPointNumber;

   	p = [event locationInWindow]; 
    	p = [self convertPoint:p fromView:nil];
	if([treeDiagram root]){
		[hitPath moveTo:p];
		selectedNode = [treeDiagram selectNodeWithHitPath:hitPath controlPoint:&controlPointNumber];
		if(selectedNode){
			if([event clickCount] >= 2 && controlPointNumber == -1)
				[self _internalStartLabelEditingFor:selectedNode event:event :YES];
			else if(controlPointNumber > -1)
				[self _internalResize:selectedNode event:event control:controlPointNumber];
			else if(([event modifierFlags] & NSControlKeyMask) == NSControlKeyMask && controlPointNumber == -1)
				[self _internalRearange:selectedNode event:event];
		}
	}
}

- (void)keyDown:(NSEvent *)event 
{
	MiscDiagramTree *selectedNode;
	unichar theChar;
	int i;
    	NSString *evChars = [event characters];
    	int length = [evChars length];
	
	if(!(selectedNode = [treeDiagram selectedNode]))
		return;

	for (i = 0; i < length; i++) {
		theChar = [evChars characterAtIndex:i];
		switch (theChar) {
	    		case NSDeleteFunctionKey:
	    		case 127:
				[self delete:self];
				return; // At this pt, all remaining keys will be useless.
				break;
	    
	   		case NSLeftArrowFunctionKey:
	    		case NSRightArrowFunctionKey:
	    		case NSUpArrowFunctionKey:
	    		case NSDownArrowFunctionKey:
				[self _internalKeyNavigate:selectedNode direction:theChar];
				break;
			case 0xd:
				[self _internalStartLabelEditingFor:selectedNode event:event :NO];
				return;
				break;
		}	
   	 }
}

- (BOOL)acceptsFirstResponder
{
    return YES;
}

- (int)canvasX
{
	return canvasXTimes;
}

- (int)canvasY
{
	return canvasYTimes;
} 

- (BOOL)knowsPagesFirst:(int *)firstPageNum last:(int *)lastPageNum
{
	*lastPageNum = canvasXTimes * canvasYTimes;
	return YES;
}

- (NSRect)rectForPage:(int)page
{
	
    	NSRect theRect;
    	int row, col;
	
	if(page < 1 || page > canvasXTimes * canvasYTimes)
		return NSZeroRect;
	row = 1;
	while(row * canvasXTimes < page)
		row++;	
	col = page - (row - 1) * canvasXTimes;
	theRect.origin.x = (col - 1) * NSWidth(canvasRect);
	theRect.origin.y = (row - 1) * NSHeight(canvasRect);
	theRect.size.width = NSWidth(canvasRect);
	theRect.size.height = NSHeight(canvasRect);
	return theRect;
}

@end

