/*	MiscTreeStyle.m

	Copyright 1996 Uwe Hoffmann.

	This notice may not be removed from this source code.
	The use and distribution of this software is governed by the
	terms of the MiscKit license agreement.  Refer to the license
	document included with the MiscKit distribution for the terms.

	Version 2 (August 1996)
*/

#import <Foundation/Foundation.h>

#import "MiscTreeStyle.h"

@implementation MiscTreeStyle

+ defaultStyle
{
    	return [[[MiscTreeStyle alloc] init] autorelease];
}

- init
{
	[super init];
	lineType = MiscBendLineType;
	bendingFactor = 0.5;
        shadow = YES;
        shadowColor = [[NSColor darkGrayColor] retain];
	return self;
}

- (void)dealloc
{
        [shadowColor release];
        return [super dealloc];
}

- (BOOL)isEqual:(id)anObject
{
        MiscTreeStyle *co;

        if(!anObject)
                return NO;
        if(anObject == self)
                return YES;
        if(![anObject isKindOfClass:[self class]])
                return NO;
        co = (MiscTreeStyle *)anObject;
        if([shadowColor isEqual:co->shadowColor] &&
           lineType == co->lineType &&
           bendingFactor == co->bendingFactor &&
           shadow == co->shadow)
                return YES;
        else
                return NO;
}

- (MiscLineType)lineType
{
	return lineType;
}

- (void)setLineType:(MiscLineType)aLineType
{
	lineType = aLineType;
}

- (float)bendingFactor
{
	return bendingFactor;
}

- (void)setBendingFactor:(float)aBendingFactor
{
	bendingFactor = aBendingFactor;
}

- (BOOL)shadow
{
	return shadow;
}

- (void)setShadow:(BOOL)aBool
{
    	shadow = aBool;
}

- (NSColor *)shadowColor
{
    	return shadowColor;
}

- (void)setShadowColor:(NSColor *)aColor
{
    	if(!aColor || [shadowColor isEqual:aColor])
            	return;
        [shadowColor release];
        shadowColor = [aColor copyWithZone:[self zone]];
}


- (void)encodeWithCoder:(NSCoder *)coder
{
    	[coder encodeObject:shadowColor];
	[coder encodeValuesOfObjCTypes:"cf", &shadow, &bendingFactor];
	[coder encodeValueOfObjCType:@encode(MiscLineType) at:&lineType];
}

- initWithCoder:(NSCoder *)coder
{
    	shadowColor = [[coder decodeObject] copyWithZone:[self zone]];
	[coder decodeValuesOfObjCTypes:"cf", &shadow, &bendingFactor];
	[coder decodeValueOfObjCType:@encode(MiscLineType) at:&lineType];
	return self;
}

- copyWithZone:(NSZone *)zone
{
   	MiscTreeStyle *theCopy;

   	theCopy = (MiscTreeStyle *)NSCopyObject(self, 0, zone);
	return theCopy;
}

- copy
{
	return [self copyWithZone:NSDefaultMallocZone()];
}

@end

