/*	MiscSortedCollection.m

	Copyright 1996 Uwe Hoffmann.

	This notice may not be removed from this source code.
	The use and distribution of this software is governed by the
	terms of the MiscKit license agreement.  Refer to the license
	document included with the MiscKit distribution for the terms.

	Version 2 (August 1996)
*/

#import <Foundation/NSUtilities.h>

#import "MiscSortedCollection.h"
#import "MiscNumberSkipList.h"
#import "MiscStringSkipList.h"
#import "MiscObjectSkipList.h"

//-----------------------------------------------------------------------------------------------------------------
@implementation MiscSortedCollection

- (unsigned)count
{
	return 0;
}
	
- (NSEnumerator *)objectEnumerator
{
	return nil;
}
		
- objectBefore:anObject
{
	return nil;
}

- smallest
{
	return nil;
}

- (NSArray *)allObjectsOrderedSameAs:anObject
{
	return nil;
}

- copyWithZone:(NSZone *)aZone
{
	return nil;
}

- copy
{
	return [self copyWithZone:NSDefaultMallocZone()];
}

- mutableCopyWithZone:(NSZone *)aZone
{
	return nil;
}

- mutableCopy
{
	return [self mutableCopyWithZone:NSDefaultMallocZone()];
}

- (unsigned)hash
{
	return [self count];
}


@end
//-----------------------------------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------------------------------
@implementation MiscSortedCollection (MiscExtendedSortedCollection)

- (NSString *)description
{
	NSMutableString *result;
	unsigned i;
	NSEnumerator *enumerator = [self objectEnumerator];
	id object;
	
	object = [enumerator nextObject];
	if(object){
		result = [NSMutableString stringWithFormat:@"(%@", [object description]];
		i = 1;
		while((object = [enumerator nextObject]) && (i < 15)){
			[result appendFormat:@", %@",[object description]]; 
			i++;
		}
		if([self count] > 15)
			[result appendFormat:@",....)"];
		else
			[result appendFormat:@")"];
		return result;
	} else
		return @"()";

} 

- (BOOL)isEqualToSortedCollection:(MiscSortedCollection *)otherSortedCollection
{
	BOOL result;
	NSEnumerator *otherEnumerator = [otherSortedCollection objectEnumerator];
	NSEnumerator *enumerator = [self objectEnumerator];
	id object, otherObject;
	
	if([self count] == [otherSortedCollection count]){
		result = YES;
		while(result && (object = [enumerator nextObject]) && (otherObject = [otherEnumerator nextObject]))
			result = result && [object isEqual:otherObject];
		return result;
	} else
		return NO;
}

- (BOOL)isEqual:anObject
{
	if([anObject isKindOfClass:[MiscSortedCollection class]])
		return [self isEqualToSortedCollection:(MiscSortedCollection *)anObject];
	else
		return NO;
}

- (NSArray *)allObjects
{
	NSMutableArray *valueArray = [[[NSMutableArray allocWithZone:[self zone]] initWithCapacity:[self count]] autorelease];
	NSEnumerator *enumerator = [self objectEnumerator];
	id object;
	
	while(object = [enumerator nextObject])
		[valueArray addObject:object];
	return valueArray;
} 

@end
//-----------------------------------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------------------------------------------
@implementation MiscSortedCollection (MiscSortedCollectionCreation)

+ allocWithZone:(NSZone *)zone
{
	if((self == [MiscSortedCollection class]) || (self == [MiscMutableSortedCollection class]))
		return [MiscObjectSkipList allocWithZone:zone];
	else
		return [super allocWithZone:zone];
}

+ collection
{
	id collection;
	
	collection = [[[MiscObjectSkipList alloc] init] autorelease];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ collectionWithCompareSelector:(SEL)comparator
{
	id collection;
	
	collection = [[[MiscObjectSkipList alloc] initCompareSelector:comparator] autorelease];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ collectionWithCompareFunction:(int(*)(id, id, void *))comparator context:(void *)context
{
	id collection;
	
	collection = [[[MiscObjectSkipList alloc] initCompareFunction:comparator context:context] autorelease];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}
		
+ stringCollectionOptions:(unsigned)mask range:(NSRange)aRange
{
	id collection;
	
	collection = [[[MiscStringSkipList alloc] initOptions:mask range:aRange] autorelease];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ stringCollectionOptions:(unsigned)mask;
{
	id collection;
	
	collection = [[[MiscStringSkipList alloc] initOptions:mask] autorelease];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}
	
+ stringCollection
{
	id collection;
	
	collection = [[[MiscStringSkipList alloc] init] autorelease];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ numberCollection
{
	id collection;
	
	collection = [[[MiscNumberSkipList alloc] init] autorelease];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}
			
+ collectionWithObjects:(id *)objects count:(unsigned)count
{
	id collection;
	unsigned i;
	
	collection = [[[MiscObjectSkipList alloc] init] autorelease];
	for(i = 0;i < count;i++)
		[collection insertObject:objects[i]];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ collectionWithObjects:(id *)objects count:(unsigned)count compareSelector:(SEL)comparator
{
	id collection;
	unsigned i;
	
	collection = [[[MiscObjectSkipList alloc] initCompareSelector:comparator] autorelease];
	for(i = 0;i < count;i++)
		[collection insertObject:objects[i]];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ collectionWithObjects:(id *)objects count:(unsigned)count compareFunction:(int(*)(id, id, void *))comparator context:(void *)context
{
	id collection;
	unsigned i;
	
	collection = [[[MiscObjectSkipList alloc] initCompareFunction:comparator context:context] autorelease];
	for(i = 0;i < count;i++)
		[collection insertObject:objects[i]];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}
	
+ collectionWithStrings:(NSString **)strings count:(unsigned)count options:(unsigned)mask range:(NSRange)aRange
{
	id collection;
	unsigned i;
	
	collection = [[[MiscStringSkipList alloc] initOptions:mask range:aRange] autorelease];
	for(i = 0;i < count;i++)
		[collection insertObject:strings[i]];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ collectionWithStrings:(NSString **)strings count:(unsigned)count options:(unsigned)mask
{
	id collection;
	unsigned i;
	
	collection = [[[MiscStringSkipList alloc] initOptions:mask] autorelease];
	for(i = 0;i < count;i++)
		[collection insertObject:strings[i]];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ collectionWithStrings:(NSString **)strings count:(unsigned)count
{
	id collection;
	unsigned i;
	
	collection = [[[MiscStringSkipList alloc] init] autorelease];
	for(i = 0;i < count;i++)
		[collection insertObject:strings[i]];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}
	
+ collectionWithNumbers:(NSNumber **)numbers count:(unsigned)count
{
	id collection;
	unsigned i;
	
	collection = [[[MiscNumberSkipList alloc] init] autorelease];
	for(i = 0;i < count;i++)
		[collection insertObject:numbers[i]];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}
	
+ collectionWithObjects:(NSArray *)objects
{
	id collection, object;
	NSEnumerator *enumerator;
	
	collection = [[[MiscStringSkipList alloc] init] autorelease];
	enumerator = [objects objectEnumerator];
	while(object = [enumerator nextObject])
		[collection insertObject:object];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ collectionWithObjects:(NSArray *)objects compareSelector:(SEL)comparator
{
	id collection, object;
	NSEnumerator *enumerator;
	
	collection = [[[MiscObjectSkipList alloc] initCompareSelector:comparator] autorelease];
	enumerator = [objects objectEnumerator];
	while(object = [enumerator nextObject])
		[collection insertObject:object];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ collectionWithObjects:(NSArray *)objects compareFunction:(int(*)(id, id, void *))comparator context:(void *)context
{
	id collection, object;
	NSEnumerator *enumerator;
	
	collection = [[[MiscObjectSkipList alloc] initCompareFunction:comparator context:context] autorelease];
	enumerator = [objects objectEnumerator];
	while(object = [enumerator nextObject])
		[collection insertObject:object];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}
	
+ collectionWithStrings:(NSArray *)strings options:(unsigned)mask range:(NSRange)aRange
{
	id collection, object;
	NSEnumerator *enumerator;
	
	collection = [[[MiscStringSkipList alloc] initOptions:mask range:aRange] autorelease];
	enumerator = [strings objectEnumerator];
	while(object = [enumerator nextObject])
		[collection insertObject:object];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ collectionWithStrings:(NSArray *)strings options:(unsigned)mask
{
	id collection, object;
	NSEnumerator *enumerator;
	
	collection = [[[MiscStringSkipList alloc] initOptions:mask] autorelease];
	enumerator = [strings objectEnumerator];
	while(object = [enumerator nextObject])
		[collection insertObject:object];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}

+ collectionWithStrings:(NSArray *)strings
{
	id collection, object;
	NSEnumerator *enumerator;
	
	collection = [[[MiscStringSkipList alloc] init] autorelease];
	enumerator = [strings objectEnumerator];
	while(object = [enumerator nextObject])
		[collection insertObject:object];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}


+ collectionWithNumbers:(NSArray *)numbers
{
	id collection, object;
	NSEnumerator *enumerator;
	
	collection = [[[MiscNumberSkipList alloc] init] autorelease];
	enumerator = [numbers objectEnumerator];
	while(object = [enumerator nextObject])
		[collection insertObject:object];
	if(self == [MiscMutableSortedCollection class])
		[collection setMutable:YES];
	return collection;
}
	

@end
//-----------------------------------------------------------------------------------------------------------------
