//=============================================================================
//
//		Copyright (C) 1995,1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//
//=============================================================================
//-----------------------------------------------------------------------------
// NameCache.m
//
//		Data structure and routines for caching user names and group names.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id$
// $Log$
//-----------------------------------------------------------------------------

#import "NameCache.h"
#import <Foundation/NSDictionary.h>
#import <Foundation/NSException.h>
#import <Foundation/NSString.h>
#import <Foundation/NSValue.h>
#import <grp.h>
#import <pwd.h>


//=============================================================================
// NAME CACHE
//=============================================================================
@implementation NameCache
//-----------------------------------------------------------------------------
// - init
//-----------------------------------------------------------------------------
- (id) init
	{
	[super init];
	table = [[NSMutableDictionary dictionary] retain];
	return self;
	}


//-----------------------------------------------------------------------------
// - dealloc
//-----------------------------------------------------------------------------
- (void) dealloc
	{
	[table release];
	[super dealloc];
	}


//-----------------------------------------------------------------------------
// - resolve:
//-----------------------------------------------------------------------------
- (NSString*) resolve:(int)x
	{
	[NSException raise:NSInvalidArgumentException
		format:@"*** Subclass responsibility: %@", NSStringFromSelector(_cmd)];
	return 0;
	}


//-----------------------------------------------------------------------------
// - store:
//-----------------------------------------------------------------------------
- (NSString*) store:(int)ident
	{
	NSString* name = [self resolve:ident];
	[table setObject:name forKey:[NSNumber numberWithInt:ident]];
	return name;
	}


//-----------------------------------------------------------------------------
// - lookup:
//-----------------------------------------------------------------------------
- (NSString*) lookup:(int)ident
	{
	NSString* name = [table objectForKey:[NSNumber numberWithInt:ident]];
	if (name == 0)
		name = [self store:ident];
	return name;
	}

@end


//=============================================================================
// OWNER CACHE
//=============================================================================
@implementation OwnerCache
//-----------------------------------------------------------------------------
// - resolve:
//-----------------------------------------------------------------------------
- (NSString*) resolve:(int)uid
	{
	NSString* s;
	struct passwd const* const pw = getpwuid( uid );
	if (pw != 0)
		s = [NSString stringWithCString:pw->pw_name];
	else
		s = [[NSNumber numberWithInt:uid] stringValue];
	return s;
	}


//-----------------------------------------------------------------------------
// + commonInstance
//-----------------------------------------------------------------------------
+ (id) commonInstance
	{
	static NameCache* obj = 0;
	if (obj == 0)
		obj = [[self alloc] init];
	return obj;
	}
@end


//=============================================================================
// GROUP CACHE
//=============================================================================
@implementation GroupCache
//-----------------------------------------------------------------------------
// - resolve:
//-----------------------------------------------------------------------------
- (NSString*) resolve:(int)gid
	{
	NSString* s;
	struct group const* const gr = getgrgid( gid );
	if (gr != 0)
		s = [NSString stringWithCString:gr->gr_name];
	else
		s = [[NSNumber numberWithInt:gid] stringValue];
	return s;
	}


//-----------------------------------------------------------------------------
// + commonInstance
//-----------------------------------------------------------------------------
+ (id) commonInstance
	{
	static NameCache* obj = 0;
	if (obj == 0)
		obj = [[self alloc] init];
	return obj;
	}
@end
