//=============================================================================
//
//		Copyright (C) 1995, 1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscHighlightTracker.M
//
//		Highlight-mode selection tracking.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscHighlightTracker.M,v 1.3 96/05/07 02:12:49 sunshine Exp $
// $Log:		MiscHighlightTracker.M,v $
//	Revision 1.3  96/05/07	02:12:49  sunshine
//	For OpenStep conformance, keyboard events are now treated the same as
//	mouse events (i.e. one must use the same modifiers with keyboard events
//	as one does with mouse events rather than the behavior being different
//	for keyboard events).  Ditched -keyDown:atPos: method.
//	
//	Revision 1.2  96/04/30	05:38:25  sunshine
//	Ported to OpenStep 4.0 for Mach PR2.
//-----------------------------------------------------------------------------
#import "MiscHighlightTracker.h"
#import "MiscSparseSet.h"
#import "MiscTableBorder.h"


@implementation MiscHighlightTracker

//-----------------------------------------------------------------------------
// setLastPos:
//-----------------------------------------------------------------------------
- (void) setLastPos: (MiscCoord_V) pos
	{
	lastPos = pos;
	set->setCursor( pos );
	}


//-----------------------------------------------------------------------------
// mouseDown:atPos:
//-----------------------------------------------------------------------------
- (void) mouseDown:(NSEvent*) event atPos: (MiscCoord_V) pos
	{
	extending = (([event modifierFlags] & NSAlternateKeyMask) != 0);
	if (extending)
		{
		[super mouseDown: event atPos: pos];
		}
	else
		{
		set->toggle( pos );
		[self setLastPos: pos];
		}
	}


//-----------------------------------------------------------------------------
// mouseDragged:atPos:
//-----------------------------------------------------------------------------
- (void) mouseDragged:(NSEvent*) event atPos: (MiscCoord_V) pos
	{
	if (extending)
		{
		[super mouseDragged: event atPos: pos];
		}
	else
		{
		if (lastPos >= 0 && lastPos < border->count())
			set->toggle( lastPos );
		if (pos >= 0 && pos < border->count())
			set->toggle( pos );
		[self setLastPos: pos];
		}
	}


//-----------------------------------------------------------------------------
// mouseUp:atPos:
//-----------------------------------------------------------------------------
- (void) mouseUp:(NSEvent*) event atPos: (MiscCoord_V) pos
	{
	if (extending)
		[super mouseUp: event atPos: pos];
	}

@end
