//=============================================================================
//
//		Copyright (C) 1995, 1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscListTracker.M
//
//		List-mode selection tracking.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscListTracker.M,v 1.3 96/05/07 02:13:05 sunshine Exp $
// $Log:		MiscListTracker.M,v $
//	Revision 1.3  96/05/07	02:13:05  sunshine
//	Fixed bug: Was unable to shift-drag to deselect range of slots.
//	For OpenStep conformance, keyboard events are now treated the same as
//	mouse events (i.e. one must use the same modifiers with keyboard events
//	as one does with mouse events rather than the behavior being different
//	for keyboard events).  Ditched -keyDown:atPos: method.
//	
//	Revision 1.2  96/04/30	05:38:29  sunshine
//	Ported to OpenStep 4.0 for Mach PR2.
//-----------------------------------------------------------------------------
#import "MiscListTracker.h"
#import "MiscSparseSet.h"
#import "MiscTableBorder.h"


@implementation MiscListTracker

//-----------------------------------------------------------------------------
// setAnchor:
//-----------------------------------------------------------------------------
- (void) setAnchor: (MiscCoord_V) pos
	{
	anchor = pos;
	set->setCursor( pos );
	}


//-----------------------------------------------------------------------------
// mouseDown:atPos:
//-----------------------------------------------------------------------------
- (void) mouseDown:(NSEvent*) event atPos: (MiscCoord_V) pos
	{
	deselecting = NO;
	if ([event modifierFlags] & NSShiftKeyMask)
		{
		deselecting = set->contains( pos );
		set->toggle( pos );
		[self setAnchor: pos];
		}
	else if ([event modifierFlags] & NSAlternateKeyMask)
		{
		int cursor = set->getCursor();
		if (cursor < 0 || cursor >= border->count())
			cursor = pos;
		set->add( cursor, pos );
		anchor = cursor;
		set->setCursor( pos );
		}
	else
		{
		set->empty();
		set->add( pos );
		[self setAnchor: pos];
		}
	}


//-----------------------------------------------------------------------------
// mouseDragged:atPos:
//-----------------------------------------------------------------------------
- (void) mouseDragged:(NSEvent*) event atPos: (MiscCoord_V) pos
	{
	if (pos < 0) pos = 0;
	if (pos >= border->count()) pos = border->count() - 1;

	if (deselecting)
		set->remove( anchor, pos );
	else
		{
		int cursor = set->getCursor();
		if (pos != cursor)
			{
			set->remove( anchor, cursor );
			set->add( anchor, pos );
			}
		}
	set->setCursor( pos );
	}


//-----------------------------------------------------------------------------
// mouseUp:atPos:
//-----------------------------------------------------------------------------
- (void) mouseUp:(NSEvent*) event atPos: (MiscCoord_V) pos
	{
	}

@end
