//=============================================================================
//
//		Copyright (C) 1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscTableConnector.M
//
//		A custom sublcass of the internal Interface Builder
//		IBControlConnector class that works for doubleTarget / doubleAction.
//
// NOTE *OPENSTEP-4.0-PR2*
//		The OpenStep Foundation framework does not publish the 
//		object_setInstanceVariable() function.	Therefore, do the next best 
//		thing (try a -set...: style method) until we think of something 
//		better.	 If that doesn't work, call object_setInstanceVariable() which
//		would have otherwise come from the old Object.h but which we declare 
//		locally to avoid including that old header.	 
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableConnector.M,v 1.4 96/05/07 09:38:47 sunshine Exp $
// $Log:		MiscTableConnector.M,v $
//	Revision 1.4  96/05/07	09:38:47  sunshine
//	Bumped OpenStep class version numbers way up (to 1000) to give NEXTSTEP 3.3
//	TableScroll room to grow.
//	
//	Revision 1.3  96/04/30	05:38:55  sunshine
//	Ported to OpenStep 4.0 for Mach PR2.
//	
//-----------------------------------------------------------------------------
#import "MiscTableConnector.h"
extern "Objective-C" {
#import <objc/objc-class.h>						// NOTE *OPENSTEP-4.0-PR2*
Ivar object_setInstanceVariable( id, char const* name, void const* );
}

#define MISC_CONN_VERSION_0		0		// First NEXTSTEP 3.3 version.
#define MISC_CONN_VERSION_1000	1000	// First OpenStep version (4.0 PR2)
#define MISC_CONN_VERSION		MISC_CONN_VERSION_1000


@interface NSIBConnector(MiscTableConnector)
- (NSString*) label;
@end
@implementation NSIBConnector(MiscTableConnector)
- (NSString*) label { return label; }
@end


@implementation MiscTableConnector

//-----------------------------------------------------------------------------
// +initialize
//-----------------------------------------------------------------------------
+ (void) initialize
	{
	if (self == [MiscTableConnector class])
		{
		[self setVersion: MISC_CONN_VERSION];
		}
	}


//-----------------------------------------------------------------------------
// -dealloc
//-----------------------------------------------------------------------------
- (void) dealloc
	{
	[outletName release];
	[actionName release];
	[super dealloc];
	}


//-----------------------------------------------------------------------------
// -readCStringWithCoder:
//-----------------------------------------------------------------------------
- (NSString*) readCStringWithCoder:(NSCoder*)decoder
	{
	NSString* ret = @"";
	char* s = 0;
	[decoder decodeValueOfObjCType:@encode(char*) at:&s];
	if (s != 0)
		{
		ret = [NSString stringWithCString:s];
		NSZoneFree( [decoder objectZone], s );
		}
	return ret;
	}


//-----------------------------------------------------------------------------
// -initWithCoder_v0:
//-----------------------------------------------------------------------------
- (void) initWithCoder_v0:(NSCoder*)decoder
	{
	outletName = [[self readCStringWithCoder:decoder] retain];
	actionName = [[self readCStringWithCoder:decoder] retain];
	[self readCStringWithCoder:decoder]; // Duplicate "label" no longer needed
	}


//-----------------------------------------------------------------------------
// -initWithCoder_v1000:
//-----------------------------------------------------------------------------
- (void) initWithCoder_v1000:(NSCoder*)decoder
	{
	outletName = [[decoder decodeObject] retain];
	actionName = [[decoder decodeObject] retain];
	}


//-----------------------------------------------------------------------------
// -initWithCoder:
//-----------------------------------------------------------------------------
- (id) initWithCoder:(NSCoder*)decoder
	{
	[super initWithCoder:decoder];
	unsigned int const ver = [decoder versionForClassName:
								[[MiscTableConnector class] description]];
	switch (ver)
		{
		case MISC_CONN_VERSION_0:	 [self initWithCoder_v0:   decoder]; break;
		case MISC_CONN_VERSION_1000: [self initWithCoder_v1000:decoder]; break;
		default:
			[NSException raise:NSGenericException
						format:@"Cannot read: unknown version %d", ver];
			break;
		}
	return self;
	}


//-----------------------------------------------------------------------------
// -setInstanceVariable:of:to:	--	NOTE *OPENSTEP-4.0-PR2*
//-----------------------------------------------------------------------------
- (void) setInstanceVariable:(NSString*)var of:(id)obj to:(id)val
	{
	int const varLen = [var length];
	if (var != 0 && varLen > 0)
		{
		NSString* s;
		if (varLen == 1)
			s = [var uppercaseString];
		else
			s = [[[var substringToIndex:1] uppercaseString]
				stringByAppendingString:[var substringFromIndex:1]];
		s = [@"set" stringByAppendingString:[s stringByAppendingString:@":"]];

		SEL cmd = NSSelectorFromString( s );
		if (cmd != 0 && [obj respondsToSelector:cmd])
			[obj performSelector:cmd withObject:val];
		else
			object_setInstanceVariable( obj, [var cString], val );
		}
	}


//-----------------------------------------------------------------------------
// -establishConnection
//-----------------------------------------------------------------------------
- (void)establishConnection
	{
	if (source != 0 && destination != 0 && [outletName length] > 0)
		{
		[self setInstanceVariable:outletName of:source to:destination];
		if ([actionName length] > 0 && [[self label] length] > 0)
			{
			SEL aSel = NSSelectorFromString([self label]);
			if (aSel != 0)
				[self setInstanceVariable:actionName of:source to:(id)aSel];
			}
		} 
	}

@end
