//=============================================================================
//
//		Copyright (C) 1995, 1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscTableFocus.M
//
//		Blinking "cursor" for showing keyboard focus in TableView.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableFocus.M,v 1.3 96/05/07 02:15:25 sunshine Exp $
// $Log:		MiscTableFocus.M,v $
//	Revision 1.3  96/05/07	02:15:25  sunshine
//	-initOwner: renamed to -initWithOwner:.
//	
//	Revision 1.2  96/04/30	05:38:58  sunshine
//	Ported to OpenStep 4.0 for Mach PR2.
//-----------------------------------------------------------------------------

#import "MiscTableFocus.h"

extern "Objective-C" {
#import <AppKit/NSView.h>
}
extern "C" {
#import "MiscTableFocusPS.h"
}

static NSTimeInterval const BLINK = 0.4;		// seconds
enum { FOCUS_BLACK = NO, FOCUS_WHITE = YES };


@implementation MiscTableFocus

//-----------------------------------------------------------------------------
// initOwner:
//-----------------------------------------------------------------------------
- (id) initWithOwner:(NSView*)o
	{
	[super init];
	owner = o;
	frame = [owner bounds];
	state = FOCUS_BLACK;
	timer = 0;
	return self;
	}


//-----------------------------------------------------------------------------
// dealloc
//-----------------------------------------------------------------------------
- (void) dealloc
	{
	if (timer != 0)
		{
		[timer invalidate];
		[timer release];
		}
	[super dealloc];
	}


//-----------------------------------------------------------------------------
// canDraw
//-----------------------------------------------------------------------------
- (BOOL) canDraw
	{
	return ([self isShowingFocus] && NSWidth(frame) > 0 && NSHeight(frame) > 0
				&& [owner canDraw]);
	}


//-----------------------------------------------------------------------------
// drawFocus
//-----------------------------------------------------------------------------
- (void) drawFocus
	{
	if ([self canDraw])
		{
		float gray;
		NSColor* c = (state ? [NSColor whiteColor] : [NSColor blackColor]);
		[c getWhite:&gray alpha:0];

		BOOL needFocus = ([NSView focusView] != owner);
		if (needFocus) [owner lockFocus];
		MISC_TF_drawBox( NSMinX(frame), NSMinY(frame),
						 NSWidth(frame), NSHeight(frame), gray );
		if (needFocus) [owner unlockFocus];
		}
	}


//-----------------------------------------------------------------------------
// eraseFocus
//-----------------------------------------------------------------------------
- (void) eraseFocus
	{
	if ([self canDraw])
		{
		BOOL needFocus = ([NSView focusView] != owner);
		if (needFocus) [owner lockFocus];
		MISC_TF_eraseBox( NSMinX(frame), NSMinY(frame),
						  NSWidth(frame), NSHeight(frame) );
		if (needFocus) [owner unlockFocus];
		}
	}


//-----------------------------------------------------------------------------
// setFrame:
//-----------------------------------------------------------------------------
- (void) setFrame:(NSRect)r
	{
	[self eraseFocus];
	frame = r;
	[self drawFocus];
	}


//-----------------------------------------------------------------------------
// blinker:
//-----------------------------------------------------------------------------
- (void) blinker:(NSTimer*)t
	{
	state = !state;
	[self drawFocus];
	}


//-----------------------------------------------------------------------------
// startFocus
//-----------------------------------------------------------------------------
- (void) startFocus
	{
	if (![self isShowingFocus])
		{
		timer = [[NSTimer scheduledTimerWithTimeInterval:BLINK
				target:self selector:@selector(blinker:) userInfo:nil
				repeats:YES] retain];
		state = FOCUS_BLACK;
		[self drawFocus];
		}
	}


//-----------------------------------------------------------------------------
// stopFocus
//-----------------------------------------------------------------------------
- (void) stopFocus
	{
	if ([self isShowingFocus])
		{
		[self eraseFocus];
		[timer invalidate];
		[timer release];
		timer = 0;
		}
	}


//-----------------------------------------------------------------------------
// isShowingFocus
//-----------------------------------------------------------------------------
- (BOOL) isShowingFocus
	{
	return (timer != 0);
	}

@end
