#ifndef __MiscTableScroll_h
#define __MiscTableScroll_h
//=============================================================================
//
//		Copyright (C) 1995, 1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// <misckit/MiscTableScroll.h>
//
//		ScrollView class that displays a 2-D table of cells.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableScroll.h,v 1.17 96/05/13 07:40:39 sunshine Exp $
// $Log:		MiscTableScroll.h,v $
// Revision 1.17  96/05/13	07:40:39  sunshine
// For consistency with other renamed methods, -border:insertSlotAt: and
// -border:removeSlotAt: were renamed to -border:insertSlot: and
// -border:removeSlot: respectively.
// 
// Revision 1.16  96/05/07	09:45:03  sunshine
// Ditched -reflectSelection; replaced with -selectionChanged (since they
// are identical under OpenStep).
// Bumped OpenStep class version numbers way up (to 1000) to give NEXTSTEP 3.3
// TableScroll room to grow.
// Added some missing explicit (id) return types and argument types.
// Slot-sort-vector changed from int array (int*) to NSArray.
// Renamed the following methods for consistency with other method names:
//		border:selectTags:		->>		border:selectSlotTags:
//		border:setCursor:		->>		border:setCursorSlot:
//		border:setTitlesOn:		->>		border:setSlotTitlesOn:
//		border:setTitleMode:	->>		border:setSlotTitleMode:
// 
// Revision 1.15  96/05/06	04:34:12  sunshine
// Moved keyDown: to here from MiscTableView.
// Combined (Delegate) and (DataDelegate) categories, since all messages
// are now sent to both.
//-----------------------------------------------------------------------------
#import <misckit/MiscTableTypes.h>

MISC_TS_EXTERN_BEGIN( "Objective-C" )
#import <AppKit/NSScrollView.h>
MISC_TS_EXTERN_END

MISC_TS_CLASS_DEF( MiscTableBorder );
@class MiscTableScroll, MiscTableView, MiscBorderView, MiscNullView;
@class NSClipView, NSFont;

#define MISC_TS_VERSION_0		0		// Initial version
#define MISC_TS_VERSION_1		1		// MiscSortType was 0..8, now 0..12
#define MISC_TS_VERSION_1000	1000	// First OpenStep version (4.0 PR2)
#define MISC_TS_VERSION_1001	1001	// Sort vector: (int*) ->> (NSArray)
#define MISC_TS_VERSION			MISC_TS_VERSION_1001


typedef struct MiscBorderInfo
		{
		MiscTableBorder*		border;
		MiscBorderView*			view;
		NSClipView*				clip;
		NSArray*				sort_vector;
		BOOL					isOn;
		BOOL					autoSort;
		BOOL					constrain_min;
		BOOL					constrain_max;
		} MiscBorderInfo;


@interface MiscTableScroll : NSScrollView <NSCoding>
	{
@private
	MiscTableView*		tableView;
	MiscBorderInfo*		info[2];		// { &colInfo, &rowInfo }
	MiscBorderInfo		colInfo;
	MiscBorderInfo		rowInfo;
	MiscNullView*		nullView;
	NSFont*				font;
	NSColor*			textColor;
	NSColor*			backgroundColor;
	NSColor*			highlightTextColor;
	NSColor*			highlightBackgroundColor;
	id					delegate;
	id					dataDelegate;
	id					target;
	id					doubleTarget;
	SEL					action;
	SEL					doubleAction;
	int					tag;
	MiscCompareEntryFunc sort_entry_func;
	MiscCompareSlotFunc sort_slot_func;
	BOOL				enabled;
	BOOL				lazy;
	MiscSelectionMode	mode;
	int					num_cols;		// Currently active number of
	int					num_rows;		// columns and rows.
	int					max_rows;		// Highwater mark for Cell allocations.
	int					max_cells;
	id*					cells;
	}

- (id) initWithFrame:(NSRect)frameRect;
- (void) dealloc;

- (id) initWithCoder:(NSCoder*)decoder;
- (void) encodeWithCoder:(NSCoder*)encoder;


// DELEGATE -------------------------------------------------------------------
- (id) delegate;
- (void) setDelegate:(id)obj;
- (id) dataDelegate;
- (void) setDataDelegate:(id)obj;

// Private: border -> scroll
- (NSString*) border:(MiscBorderType)b getDelegateSlotTitle:(int)slot;
- (id) border:(MiscBorderType)b getDelegateSlotPrototype:(int)s;


// TARGET / ACTION ------------------------------------------------------------
- (id) target;
- (void) setTarget:(id)obj;
- (id) doubleTarget;
- (void) setDoubleTarget:(id)obj;
- (SEL) action;
- (void) setAction:(SEL)new_sel;
- (SEL) doubleAction;
- (void) setDoubleAction:(SEL)new_sel;

- (BOOL) isEnabled;
- (void) setEnabled:(BOOL)flag;

- (BOOL) sendAction:(SEL)theAction to:(id)theTarget;
- (BOOL) sendAction;
- (BOOL) sendActionIfEnabled;
- (BOOL) sendDoubleAction;
- (BOOL) sendDoubleActionIfEnabled;

- (int) tag;
- (void) setTag:(int)x;


// FONT -----------------------------------------------------------------------
- (NSFont*) font;
- (void) setFont:(NSFont*)newFont;
- (void) changeFont:(id)sender;
+ (NSFont*) defaultFont;


// COLOR ----------------------------------------------------------------------
- (NSColor*) backgroundColor;
- (void) setBackgroundColor:(NSColor*)value;			// Sent to all cells.
- (NSColor*) textColor;
- (void) setTextColor:(NSColor*)value;					// Sent to all cells.
- (NSColor*) highlightBackgroundColor;
- (void) setHighlightBackgroundColor:(NSColor*)value;	// Sent to all cells.
- (NSColor*) highlightTextColor;
- (void) setHighlightTextColor:(NSColor*)value;			// Sent to all cells.

- (void) setColor:(NSColor*)value;		// Alias for -setBackgroundColor:
- (NSColor*) color;						// Alias for -backgroundColor

+ (NSColor*) defaultBackgroundColor;
+ (NSColor*) defaultTextColor;
+ (NSColor*) defaultHighlightBackgroundColor;
+ (NSColor*) defaultHighlightTextColor;


// SLOT ORDER -----------------------------------------------------------------
- (NSArray*) slotOrder:(MiscBorderType)b;
- (NSArray*) colOrder;
- (NSArray*) rowOrder;

- (BOOL) border:(MiscBorderType)b setSlotOrder:(NSArray*)list;
- (BOOL) setColOrder:(NSArray*)list;
- (BOOL) setRowOrder:(NSArray*)list;

- (NSString*) slotOrderAsString:(MiscBorderType)b;
- (NSString*) colOrderAsString;
- (NSString*) rowOrderAsString;

- (BOOL) border:(MiscBorderType)b setSlotOrderFromString:(NSString*)s;
- (BOOL) setColOrderFromString:(NSString*)s;
- (BOOL) setRowOrderFromString:(NSString*)s;


// SLOT SIZES -----------------------------------------------------------------
- (NSArray*) slotSizes:(MiscBorderType)b;
- (NSArray*) colSizes;
- (NSArray*) rowSizes;

- (BOOL) border:(MiscBorderType)b setSlotSizes:(NSArray*)list;
- (BOOL) setColSizes:(NSArray*)list;
- (BOOL) setRowSizes:(NSArray*)list;

- (NSString*) slotSizesAsString:(MiscBorderType)b;
- (NSString*) colSizesAsString;
- (NSString*) rowSizesAsString;

- (BOOL) border:(MiscBorderType)b setSlotSizesFromString:(NSString*)s;
- (BOOL) setColSizesFromString:(NSString*)s;
- (BOOL) setRowSizesFromString:(NSString*)s;


// MULTICAST MESSAGES ---------------------------------------------------------
- (void) sendAction:(SEL)aSelector to:(id)anObject forAllCells:(BOOL)flag;

- (int) makeCellsPerform:(SEL)aSel selectedOnly:(BOOL)flag;
- (int) makeCellsPerform:(SEL)aSel with:(id)arg1 selectedOnly:(BOOL)flag;
- (int) makeCellsPerform:(SEL)aSel with:(id)arg1 with:(id)arg2
				selectedOnly:(BOOL)flag;

- (int) makeCellsPerform:(SEL)aSel;
- (int) makeCellsPerform:(SEL)aSel with:(id)arg1;
- (int) makeCellsPerform:(SEL)aSel with:(id)arg1 with:(id)arg2;


// FINDING CELLS --------------------------------------------------------------
- (int) border:(MiscBorderType)b findSlotWithTag:(int)x;
- (int) findColWithTag:(int)x;
- (int) findRowWithTag:(int)x;
- (id) cellWithTag:(int)x;
- (id) cellWithTag:(int)x row:(int*)row col:(int*)col;
- (BOOL) findCell:(id)cell row:(int*)row col:(int*)col;


// SELECTION ------------------------------------------------------------------
- (MiscSelectionMode) selectionMode;
- (void) setSelectionMode: (MiscSelectionMode)x;
- (BOOL) hasSlotSelection:(MiscBorderType)b;
- (BOOL) hasRowSelection;
- (BOOL) hasColSelection;
- (BOOL) hasMultipleSlotSelection:(MiscBorderType)b;
- (BOOL) hasMultipleRowSelection;
- (BOOL) hasMultipleColSelection;
- (unsigned int) numSelectedSlots:(MiscBorderType)b;
- (unsigned int) numSelectedRows;
- (unsigned int) numSelectedCols;
- (BOOL) cellIsSelected: (MiscCoord_P)row : (MiscCoord_P)col;
- (BOOL) border: (MiscBorderType)b slotIsSelected: (MiscCoord_P)slot;
- (BOOL) rowIsSelected: (MiscCoord_P)row;
- (BOOL) colIsSelected: (MiscCoord_P)col;
- (MiscCoord_P) selectedSlot:(MiscBorderType)b;
- (MiscCoord_P) selectedRow;
- (MiscCoord_P) selectedColumn;
- (NSArray*) selectedSlotTags:(MiscBorderType)b;
- (NSArray*) selectedRowTags;
- (NSArray*) selectedColTags;
- (NSArray*) selectedSlots:(MiscBorderType)b;
- (NSArray*) selectedRows;
- (NSArray*) selectedCols;
- (void) border: (MiscBorderType)b selectSlot: (MiscCoord_P)slot;
- (void) selectRow: (MiscCoord_P)row;
- (void) selectCol: (MiscCoord_P)col;
- (void) border:(MiscBorderType)b selectSlotTags:(NSArray*)tags;
- (void) selectRowTags: (NSArray*)tags;
- (void) selectColTags: (NSArray*)tags;
- (void) border: (MiscBorderType)b selectSlots: (NSArray*)slots;
- (void) selectRows: (NSArray*)rows;
- (void) selectCols: (NSArray*)cols;
- (void) selectAllSlots:(MiscBorderType)b;
- (void) selectAllRows;
- (void) selectAllCols;
- (void) selectAll:(id)sender;	// -selectAllRows and sends action to target.
- (void) clearSlotSelection:(MiscBorderType)b;
- (void) clearRowSelection;
- (void) clearColSelection;
- (void) clearSelection;


// MOUSE & KEYBOARD TRACKING --------------------------------------------------
- (void) trackBy: (MiscBorderType)b;
- (MiscBorderType) trackingBy;


// KEYBOARD CURSOR ------------------------------------------------------------
- (MiscCoord_P) cursorSlot:(MiscBorderType)b;
- (MiscCoord_P) cursorRow;
- (MiscCoord_P) cursorCol;
- (void) border:(MiscBorderType)b setCursorSlot:(MiscCoord_P)slot;
- (void) setCursorRow: (MiscCoord_P) row;
- (void) setCursorCol: (MiscCoord_P) col;
- (void) clearCursorSlot:(MiscBorderType)b;
- (void) clearCursorRow;
- (void) clearCursorCol;
- (void) clearCursor;
- (BOOL) hasValidCursorSlot:(MiscBorderType)b;
- (BOOL) hasValidCursorRow;
- (BOOL) hasValidCursorCol;


// GLOBAL SIZE LIMITS ---------------------------------------------------------
- (float) totalSize:(MiscBorderType)b;
- (float) totalWidth;
- (float) totalHeight;

- (float) minTotalSize:(MiscBorderType)b;
- (float) maxTotalSize:(MiscBorderType)b;
- (float) minTotalWidth;
- (float) maxTotalWidth;
- (float) minTotalHeight;
- (float) maxTotalHeight;

- (void) border:(MiscBorderType)b setMinTotalSize:(float)size;
- (void) border:(MiscBorderType)b setMaxTotalSize:(float)size;
- (void) setMinTotalWidth:(float)size;
- (void) setMaxTotalWidth:(float)size;
- (void) setMinTotalHeight:(float)size;
- (void) setMaxTotalHeight:(float)size;

- (void) constrainSize;			// Applies constraints, updates views.

- (BOOL) minTotalSizeIsConstrained:(MiscBorderType)b;
- (BOOL) maxTotalSizeIsConstrained:(MiscBorderType)b;
- (BOOL) minTotalWidthIsConstrained;
- (BOOL) maxTotalWidthIsConstrained;
- (BOOL) minTotalHeightIsConstrained;
- (BOOL) maxTotalHeightIsConstrained;

- (void) border:(MiscBorderType)b constrainMinTotalSize:(BOOL)flag;
- (void) border:(MiscBorderType)b constrainMaxTotalSize:(BOOL)flag;
- (void) constrainMinTotalWidth:(BOOL)flag;
- (void) constrainMaxTotalWidth:(BOOL)flag;
- (void) constrainMinTotalHeight:(BOOL)flag;
- (void) constrainMaxTotalHeight:(BOOL)flag;


// SLOTS ----------------------------------------------------------------------
- (MiscTableBorder*) border:(MiscBorderType)b;
- (BOOL) slotTitlesOn:(MiscBorderType)b;
- (BOOL)border:(MiscBorderType)b setSlotTitlesOn:(BOOL)on_off;
		// -border:setTitlesOn: Returns YES if changed.
- (MiscTableTitleMode) slotTitleMode:(MiscBorderType)b;
- (void) border:(MiscBorderType)b setSlotTitleMode:(MiscTableTitleMode)x;

- (void) border:(MiscBorderType)b moveSlotFrom:(int)pos to:(int)pos;
- (int) border:(MiscBorderType)b slotPosition:(int)slot;
- (int) border:(MiscBorderType)b slotAtPosition:(int)pos;
- (NSArray*) border:(MiscBorderType)b physicalToVisual:(NSArray*)list;
- (NSArray*) border:(MiscBorderType)b visualToPhysical:(NSArray*)list;

- (BOOL) sizeableSlots:(MiscBorderType)b;
- (BOOL) draggableSlots:(MiscBorderType)b;
- (BOOL) modifierDragSlots:(MiscBorderType)b;
- (float) uniformSizeSlots:(MiscBorderType)b;

- (float) border:(MiscBorderType)b slotAdjustedSize:(int)slot;
- (float) border:(MiscBorderType)b slotSize:(int)slot;
- (float) border:(MiscBorderType)b slotMinSize:(int)slot;
- (float) border:(MiscBorderType)b slotMaxSize:(int)slot;
- (float) border:(MiscBorderType)b slotDataSize:(int)slot;
- (BOOL) border:(MiscBorderType)b slotIsSizeable:(int)slot;
- (BOOL) border:(MiscBorderType)b slotExpandsToData:(int)slot;
- (BOOL) border:(MiscBorderType)b slotIsAutosize:(int)slot;
- (NSString*) border:(MiscBorderType)b slotTitle:(int)slot;
- (int) border:(MiscBorderType)b slotTag:(int)slot;
- (MiscTableCellStyle) border:(MiscBorderType)b slotCellType:(int)slot;
- (id) border:(MiscBorderType)b slotCellPrototype:(int)slot;

- (void) border:(MiscBorderType)b setSizeableSlots:(BOOL)flag;
- (void) border:(MiscBorderType)b setDraggableSlots:(BOOL)flag;
- (void) border:(MiscBorderType)b setModifierDragSlots:(BOOL)flag;
- (void) border:(MiscBorderType)b setUniformSizeSlots:(float)uniform_size;

- (void) border:(MiscBorderType)b setSlot:(int)n size:(float)size;
- (void) border:(MiscBorderType)b setSlot:(int)n minSize:(float)size;
- (void) border:(MiscBorderType)b setSlot:(int)n maxSize:(float)size;
- (void) border:(MiscBorderType)b setSlot:(int)n dataSize:(float)size;
- (void) border:(MiscBorderType)b setSlot:(int)n sizeable:(BOOL)flag;
- (void) border:(MiscBorderType)b setSlot:(int)n expandsToData:(BOOL)flag;
- (void) border:(MiscBorderType)b setSlot:(int)n autosize:(BOOL)flag;
- (void) border:(MiscBorderType)b setSlot:(int)n title:(NSString*)title;
- (void) border:(MiscBorderType)b setSlot:(int)n tag:(int)tag;
- (void) border:(MiscBorderType)b setSlot:(int)n
				cellType:(MiscTableCellStyle)t;
- (void) border:(MiscBorderType)b setSlot:(int)n cellPrototype:(id)cell;

- (int) numVisibleSlots:(MiscBorderType)b;
- (int) firstVisibleSlot:(MiscBorderType)b;
- (int) lastVisibleSlot:(MiscBorderType)b;
- (BOOL) border:(MiscBorderType)b slotIsVisible:(int)n;
- (void) border:(MiscBorderType)b setFirstVisibleSlot:(int)n;
- (void) border:(MiscBorderType)b setLastVisibleSlot:(int)n;


// COLS -----------------------------------------------------------------------
- (MiscTableBorder*) colBorder;
- (BOOL) colTitlesOn;
- (BOOL) setColTitlesOn: (BOOL) on_off; // Returns YES if changed.
- (MiscTableTitleMode) colTitleMode;
- (void) setColTitleMode:(MiscTableTitleMode)x;

- (void) moveColFrom:(int)pos to:(int)pos;
- (int) colPosition:(int)col;
- (int) colAtPosition:(int)pos;

- (float) uniformSizeCols;
- (BOOL) sizeableCols;
- (BOOL) draggableCols;
- (BOOL) modifierDragCols;
- (float) colAdjustedSize:(int)col;
- (float) colSize:(int)col;
- (float) colMinSize:(int)col;
- (float) colMaxSize:(int)col;
- (float) colDataSize:(int)col;
- (BOOL) colIsSizeable:(int)col;
- (BOOL) colExpandsToData:(int)col;
- (BOOL) colIsAutosize:(int)col;
- (NSString*) colTitle:(int)col;
- (int) colTag:(int)col;
- (MiscTableCellStyle) colCellType:(int)col;
- (id) colCellPrototype:(int)col;

- (void) setSizeableCols:(BOOL)flag;
- (void) setDraggableCols:(BOOL)flag;
- (void) setModifierDragCols:(BOOL)flag;
- (void) setUniformSizeCols:(float)uniform_size;

- (void) setCol:(int)col size:(float)size;
- (void) setCol:(int)col minSize:(float)size;
- (void) setCol:(int)col maxSize:(float)size;
- (void) setCol:(int)col dataSize:(float)size;
- (void) setCol:(int)col sizeable:(BOOL)flag;
- (void) setCol:(int)col expandsToData:(BOOL)flag;
- (void) setCol:(int)col autosize:(BOOL)flag;
- (void) setCol:(int)col title:(NSString*)title;
- (void) setCol:(int)col tag:(int)tag;
- (void) setCol:(int)col cellType:(MiscTableCellStyle)type;
- (void) setCol:(int)col cellPrototype:(id)cell;

- (int) numVisibleCols;
- (int) firstVisibleCol;
- (int) lastVisibleCol;
- (BOOL) colIsVisible:(int)n;
- (void) setFirstVisibleCol:(int)n;
- (void) setLastVisibleCol:(int)n;


// ROWS -----------------------------------------------------------------------
- (MiscTableBorder*) rowBorder;
- (BOOL) rowTitlesOn;
- (BOOL) setRowTitlesOn: (BOOL) on_off;
- (MiscTableTitleMode) rowTitleMode;
- (void) setRowTitleMode:(MiscTableTitleMode)x;

- (void) moveRowFrom:(int)fromPos to:(int)toPos;
- (int) rowPosition:(int)row;
- (int) rowAtPosition:(int)pos;

- (BOOL) sizeableRows;
- (BOOL) draggableRows;
- (BOOL) modifierDragRows;
- (float) uniformSizeRows;
- (float) rowAdjustedSize:(int)row;
- (float) rowSize:(int)row;
- (float) rowMinSize:(int)row;
- (float) rowMaxSize:(int)row;
- (float) rowDataSize:(int)row;
- (BOOL) rowIsSizeable:(int)row;
- (BOOL) rowExpandsToData:(int)row;
- (BOOL) rowIsAutosize:(int)row;
- (NSString*) rowTitle:(int)row;
- (int) rowTag:(int)row;
- (MiscTableCellStyle) rowCellType:(int)row;
- (id) rowCellPrototype:(int)row;

- (void) setSizeableRows:(BOOL)flag;
- (void) setDraggableRows:(BOOL)flag;
- (void) setModifierDragRows:(BOOL)flag;
- (void) setUniformSizeRows:(float)uniform_size;

- (void) setRow:(int)row size:(float)size;
- (void) setRow:(int)row minSize:(float)size;
- (void) setRow:(int)row maxSize:(float)size;
- (void) setRow:(int)row dataSize:(float)size;
- (void) setRow:(int)row sizeable:(BOOL)flag;
- (void) setRow:(int)row expandsToData:(BOOL)flag;
- (void) setRow:(int)row autosize:(BOOL)flag;
- (void) setRow:(int)row title:(NSString*)title;
- (void) setRow:(int)row tag:(int)tag;
- (void) setRow:(int)row cellType:(MiscTableCellStyle)type;
- (void) setRow:(int)row cellPrototype:(id)cell;

- (int) numVisibleRows;
- (int) firstVisibleRow;
- (int) lastVisibleRow;
- (BOOL) rowIsVisible:(int)n;
- (void) setFirstVisibleRow:(int)n;
- (void) setLastVisibleRow:(int)n;


// DRAWING --------------------------------------------------------------------
- (NSRect) getDocClipFrame;

- (void) drawCellAtRow:(int)row column:(int)col;		// Physical coords

- (void) drawRow:(int)row;								// Physical coords
- (void) drawCol:(int)col;								// Physical coords
- (void) border:(MiscBorderType)b drawSlot:(int)n;

- (void) scrollCellToVisibleAtRow:(int)row column:(int)col; // Physical coords
- (void) scrollRowToVisible: (int)row;					// Physical coord
- (void) scrollColToVisible: (int)col;					// Physical coord
- (void) scrollSelToVisible;

- (void) border:(MiscBorderType)b drawSlotTitle:(int)n;
- (void) drawRowTitle:(int)n;
- (void) drawColTitle:(int)n;

- (void) selectionChanged;

- (void) sizeToCells;

// Private: BorderView -> TableScroll
- (void) border:(MiscBorderType)b slotDraggedFrom:(int)fromPos to:(int)toPos;
- (void) border:(MiscBorderType)b slotResized:(int)n;

@end


// SORTING --------------------------------------------------------------------
@interface MiscTableScroll(Sort)

- (MiscCompareSlotFunc) compareSlotFunc;
- (void) setCompareSlotFunc: (MiscCompareSlotFunc) f;

- (void) sortInfoInit:(MiscSlotSortInfo*)ip border:(MiscBorderType)b;
- (void) sortInfoDone:(MiscSlotSortInfo*)ip;

// Slots
- (BOOL) autoSortSlots:(MiscBorderType)b;
- (void) border:(MiscBorderType)b setAutoSortSlots:(BOOL)flag;
- (NSArray*) slotSortVector:(MiscBorderType)b;
- (void) border:(MiscBorderType)b setSlotSortVector:(NSArray*)v;

- (void) sortSlots:(MiscBorderType) b;
- (BOOL) slotsAreSorted:(MiscBorderType)b;
- (BOOL) border:(MiscBorderType) b sortSlot:(int)n;
- (BOOL) border:(MiscBorderType)b slotIsSorted:(int)n;

- (int) border:(MiscBorderType)b compareSlots:(int)slot1 :(int)slot2
		info:(MiscSlotSortInfo*)ip;
- (int) border:(MiscBorderType)b compareSlots:(int)slot1 :(int)slot2;


- (MiscCompareEntryFunc) border:(MiscBorderType)b slotSortFunc:(int)n;
- (MiscSortDirection) border:(MiscBorderType)b slotSortDirection:(int)n;
- (MiscSortType) border:(MiscBorderType)b slotSortType:(int)n;
- (void) border:(MiscBorderType)b setSlot:(int)n
				sortFunc:(MiscCompareEntryFunc)x;
- (void) border:(MiscBorderType)b setSlot:(int)n
				sortDirection:(MiscSortDirection)x;
- (void) border:(MiscBorderType)b setSlot:(int)n
				sortType:(MiscSortType)x;

// Cols
- (BOOL) autoSortCols;
- (void) setAutoSortCols:(BOOL)flag;
- (NSArray*) colSortVector;
- (void) setColSortVector:(NSArray*)v;

- (void) sortCols;
- (BOOL) colsAreSorted;
- (BOOL) sortCol:(int)n;
- (BOOL) colIsSorted:(int)n;
- (int) compareCols:(int)c1 :(int)c2 info:(MiscSlotSortInfo*)ip;
- (int) compareCols:(int)c1 :(int)c2;

- (MiscCompareEntryFunc) colSortFunc:(int)n;
- (MiscSortDirection) colSortDirection:(int)n;
- (MiscSortType) colSortType:(int)n;
- (void) setCol:(int)n sortFunc:(MiscCompareEntryFunc)x;
- (void) setCol:(int)n sortDirection:(MiscSortDirection)x;
- (void) setCol:(int)n sortType:(MiscSortType)x;

// Rows
- (BOOL) autoSortRows;
- (void) setAutoSortRows:(BOOL)flag;
- (NSArray*) rowSortVector;
- (void) setRowSortVector:(NSArray*)v;

- (void) sortRows;
- (BOOL) rowsAreSorted;
- (BOOL) sortRow:(int)n;
- (BOOL) rowIsSorted:(int)n;
- (int) compareRows:(int)r1 :(int)r2 info:(MiscSlotSortInfo*)ip;
- (int) compareRows:(int)r1 :(int)r2;

- (MiscCompareEntryFunc) rowSortFunc:(int)n;
- (MiscSortDirection) rowSortDirection:(int)n;
- (MiscSortType) rowSortType:(int)n;
- (void) setRow:(int)n sortFunc:(MiscCompareEntryFunc)x;
- (void) setRow:(int)n sortDirection:(MiscSortDirection)x;
- (void) setRow:(int)n sortType:(MiscSortType)x;
@end


// DATA MANIPULATION ----------------------------------------------------------
@interface MiscTableScroll(Data)
- (BOOL) isLazy;
- (void) setLazy:(BOOL)flag;

- (id) cellAtRow:(int)row column:(int)col;

// Lazy mode tables that perform multiple-buffering:
- (int) buffCount;

// Lazy mode asks delegate/dataDelegate first then uses cellAtRow:column:
// Eager mode goes straight to cellAtRow:column:
- (int) tagAtRow:(int)row column:(int)col;
- (int) intValueAtRow:(int)row column:(int)col;
- (float) floatValueAtRow:(int)row column:(int)col;
- (double) doubleValueAtRow:(int)row column:(int)col;
- (NSString*) stringValueAtRow:(int)row column:(int)col;
// For ButtonCells.
- (NSString*) titleAtRow:(int)row column:(int)col;
- (int) stateAtRow:(int)row column:(int)col;

// Invoke delegate methods if possible.
- (id) reviveCell:(id)cell atRow:(int)row column:(int)col;
- (id) retireCell:(id)cell atRow:(int)row column:(int)col;

// Builtin default implementation.
- (id) doReviveCell:(id)cell atRow:(int)row column:(int)col;
- (id) doRetireCell:(id)cell atRow:(int)row column:(int)col;

- (void) addSlot:(MiscBorderType)b;
- (void) border:(MiscBorderType)b insertSlot:(int)pos;
- (void) border:(MiscBorderType)b removeSlot:(int)pos;
- (int) numberOfSlots:(MiscBorderType)b;

- (void) addColumn;
- (void) insertColumn:(int)pos;
- (void) removeColumn:(int)pos;
- (int) numberOfColumns;

- (void) addRow;
- (void) insertRow:(int)pos;
- (void) removeRow:(int)pos;
- (int) numberOfRows;

- (void) renewRows:(int)count;
- (void) empty;									// [self renewRows:0];
- (void) emptyAndReleaseCells;
@end


// PASTEBOARD SUPPORT ---------------------------------------------------------
@interface MiscTableScroll(Pasteboard)
- (void) copy:(id)sender;
- (void) cut:(id)sender;
- (id) validRequestorForSendType:(NSString*)t_send
		returnType:(NSString*)t_return;
- (id) builtinValidRequestorForSendType:(NSString*)t_send
		returnType:(NSString*)t_return;
- (BOOL) canWritePboardType:(NSString*)type;
- (BOOL) builtinCanWritePboardType:(NSString*)type;
- (BOOL) writeSelectionToPasteboard:(NSPasteboard*)pboard types:(NSArray*)types;
- (BOOL) builtinWriteSelectionToPasteboard:(NSPasteboard*)pboard
		types:(NSArray*)types;
- (NSString*) stringForPboardType:(NSString*)type;
- (NSString*) builtinStringForPboardType:(NSString*)type;
- (NSString*) builtinStringForPboardType:(NSString*)type;
- (NSString*) stringForNSStringPboardType;
- (NSString*) stringForNSTabularTextPBoardType;
- (BOOL) readSelectionFromPasteboard:(NSPasteboard*)pboard;
- (BOOL) builtinReadSelectionFromPasteboard:(NSPasteboard*)pboard;
- (void) builtinRegisterServicesTypes;
- (void) registerServicesTypes;
@end


// KEYBOARD EVENTS ------------------------------------------------------------
@interface MiscTableScroll(Keyboard)
- (void) keyDown:(NSEvent*)event;
@end


// INCREMENTAL SEARCH ---------------------------------------------------------
@interface MiscTableScroll(IncrementalSearch)
- (BOOL) incrementalSearch:(NSEvent*)p;
- (BOOL) doIncrementalSearch:(NSEvent*)p col:(int)col;
- (BOOL) getISearchCol:(int*)col;
- (BOOL) doGetISearchCol:(int*)col;
@end


// DELEGATE PROTOCOL ----------------------------------------------------------
@interface MiscTableScroll(Delegate)
- (void) tableScroll:(MiscTableScroll*)scroll border:(MiscBorderType)b
		slotDraggedFrom:(int)fromPos to:(int)toPos;
- (void) tableScroll:(MiscTableScroll*)scroll border:(MiscBorderType)b
		slotResized:(int)n;

- (void) tableScroll:(MiscTableScroll*)scroll
		changeFont:(NSFont*)oldFont to:(NSFont*)newFont;
- (void) tableScroll:(MiscTableScroll*)scroll
		fontChangedFrom:(NSFont*)oldFont to:(NSFont*)newFont;

- (void) tableScroll:(MiscTableScroll*)scroll
		backgroundColorChangedTo:(NSColor*)x;
- (void) tableScroll:(MiscTableScroll*)scroll
		highlightBackgroundColorChangedTo:(NSColor*)x;
- (void) tableScroll:(MiscTableScroll*)scroll
		highlightTextColorChangedTo:(NSColor*)x;
- (void) tableScroll:(MiscTableScroll*)scroll
		textColorChangedTo:(NSColor*)x;

- (BOOL) tableScroll:(MiscTableScroll*)scroll getISearchCol:(int*)col;
- (int) tableScrollBuffCount:(MiscTableScroll*)scroll;

- (id) tableScroll:(MiscTableScroll*)scroll
		border:(MiscBorderType)b slotPrototype:(int)slot;
- (NSString*) tableScroll:(MiscTableScroll*)scroll
		border:(MiscBorderType)b slotTitle:(int)slot;

- (id) tableScroll:(MiscTableScroll*)scroll cellAtRow:(int)row column:(int)col;
- (id) tableScroll:(MiscTableScroll*)scroll reviveCell:(id)cell
		atRow:(int)row column:(int)col;
- (id) tableScroll:(MiscTableScroll*)scroll retireCell:(id)cell
		atRow:(int)row column:(int)col;

- (int) tableScroll:(MiscTableScroll*)scroll
		tagAtRow:(int)row column:(int)col;
- (int) tableScroll:(MiscTableScroll*)scroll
		intValueAtRow:(int)row column:(int)col;
- (float) tableScroll:(MiscTableScroll*)scroll
		floatValueAtRow:(int)row column:(int)col;
- (double) tableScroll:(MiscTableScroll*)scroll
		doubleValueAtRow:(int)row column:(int)col;
- (NSString*) tableScroll:(MiscTableScroll*)scroll
		stringValueAtRow:(int)row column:(int)col;
- (NSString*) tableScroll:(MiscTableScroll*)scroll
		titleAtRow:(int)row column:(int)col;
- (int) tableScroll:(MiscTableScroll*)scroll
		stateAtRow:(int)row column:(int)col;

- (void) tableScrollRegisterServicesTypes: (MiscTableScroll*) scroll;
- (id) tableScroll:(MiscTableScroll*)scroll
		validRequestorForSendType:(NSString*)t_write
		returnType:(NSString*)t_read;
- (BOOL) tableScroll:(MiscTableScroll*)scroll 
		canWritePboardType:(NSString*)type;
- (NSString*) tableScroll:(MiscTableScroll*)scroll
		stringForPboardType:(NSString*)type;
- (BOOL) tableScroll:(MiscTableScroll*)scroll
		writeSelectionToPasteboard:(NSPasteboard*)pboard types:(NSArray*)types;
- (BOOL) tableScroll:(MiscTableScroll*)scroll
		readSelectionFromPasteboard:(NSPasteboard*)pboard;
@end


// DATA CELL PROTOCOL ---------------------------------------------------------
@interface MiscTableScroll(DataCell)
- (id) tableScroll:(MiscTableScroll*)scroll
		reviveAtRow:(int)row column:(int)col;
- (id) tableScroll:(MiscTableScroll*)scroll
		retireAtRow:(int)row column:(int)col;
@end

#endif // __MiscTableScroll_h
