//=============================================================================
//
//		Copyright (C) 1995,1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscTableScrollData.M
//
//		The (DATA) category of MiscTableScroll.	 Implements the management
//		of the cells of the table.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableScrollData.M,v 1.14 96/05/13 07:41:32 sunshine Exp $
// $Log:		MiscTableScrollData.M,v $
//	Revision 1.14  96/05/13	 07:41:32  sunshine
//	For consistency with other renamed methods, -border:insertSlotAt: and
//	-border:removeSlotAt: were renamed to -border:insertSlot: and
//	-border:removeSlot: respectively.
//	
//	Revision 1.13  96/05/07	 09:45:31  sunshine
//	Added some missing explicit (id) return types and argument types.
//	
//	Revision 1.12  96/04/30	 05:39:04  sunshine
//	Ported to OpenStep 4.0 for Mach PR2.
//-----------------------------------------------------------------------------
#import <misckit/MiscTableScroll.h>
#import <misckit/MiscTableCell.h>
#import "MiscTableBorder.h"
extern "Objective-C" {
#import <AppKit/NSCell.h>
#import <AppKit/NSImage.h>
}
extern "C" {
#import <assert.h>
#import <limits.h>
#import <stdlib.h>
#import <string.h>
}

@implementation MiscTableScroll(DATA)

//-----------------------------------------------------------------------------
// - lazyCellAtRow:column:
//-----------------------------------------------------------------------------
- (id) lazyCellAtRow:(int)row column:(int)col
	{
	if (delegate != 0 &&
		[delegate respondsToSelector:@selector(tableScroll:cellAtRow:column:)])
			return [delegate tableScroll:self cellAtRow:row column:col];
	if (dataDelegate != 0 &&
		[dataDelegate respondsToSelector:
		@selector(tableScroll:cellAtRow:column:)])
			return [dataDelegate tableScroll:self cellAtRow:row column:col];
	return 0;
	}


//-----------------------------------------------------------------------------
// - eagerCellAtRow:column:
//-----------------------------------------------------------------------------
- (id) eagerCellAtRow:(int)row column:(int)col
	{
	return cells[ row * num_cols + col ];
	}


//-----------------------------------------------------------------------------
// - cellAtRow:column:
//-----------------------------------------------------------------------------
- (id) cellAtRow:(int)row column:(int)col
	{
	id cell = 0;
	if ((unsigned int) row < (unsigned int) num_rows &&
		(unsigned int) col < (unsigned int) num_cols)
		{
		if (!lazy)
			cell = [self eagerCellAtRow:row column:col];
		else
			cell = [self lazyCellAtRow:row column:col];

		MiscCoord_V vRow = rowInfo.border->physicalToVisual(row);
		MiscCoord_V vCol = colInfo.border->physicalToVisual(col);
		if ([cell respondsToSelector:@selector(setState:)])
			[cell setState:colInfo.border->selectionSet().contains(vCol) ||
							rowInfo.border->selectionSet().contains(vRow)];
		}
	return cell;
	}


//-----------------------------------------------------------------------------
// tagAtRow:column:, intValueAtRow:column:, floatValueAtRow:column:,
// doubleValueAtRow:column:, stringValueAtRow:column:, titleAtRow:column:,
// stateAtRow:column:
//-----------------------------------------------------------------------------
#define MISC_CELL_VAL( DATA_TYPE, NAME )\
- (DATA_TYPE) NAME##AtRow:(int)r column:(int)c\
	{\
	if (delegate != 0 &&\
		[delegate respondsToSelector:\
		@selector(tableScroll:NAME##AtRow:column:)])\
		return [delegate tableScroll:self NAME##AtRow:r column:c];\
	if (dataDelegate != 0 &&\
		[dataDelegate respondsToSelector:\
		@selector(tableScroll:NAME##AtRow:column:)])\
		return [dataDelegate tableScroll:self NAME##AtRow:r column:c];\
	id cell = (!lazy ?	[self eagerCellAtRow:r column:c] :\
						[self lazyCellAtRow:r column:c]);\
	if (cell != 0 && [cell respondsToSelector:@selector(NAME)])\
		return [cell NAME];\
	return 0;\
	}

MISC_CELL_VAL( int, tag )						// tagAtRow:column:
MISC_CELL_VAL( int, intValue )					// intValueAtRow:column:
MISC_CELL_VAL( float, floatValue )				// floatValueAtRow:column:
MISC_CELL_VAL( double, doubleValue )			// doubleValueAtRow:column:
MISC_CELL_VAL( NSString*, stringValue )			// stringValueAtRow:column:
MISC_CELL_VAL( int, state )						// stateAtRow:column:
MISC_CELL_VAL( NSString*, title )				// titleAtRow:column:
#undef MISC_CELL_VAL


//-----------------------------------------------------------------------------
// - buffCount
//		For lazy-mode tables that perform multiple-buffering.  This can
//		avoid copying when accessing more than one cell at a time (such
//		as during sorting.)
//-----------------------------------------------------------------------------
- (int) buffCount
	{
	if (delegate != 0 &&
		[delegate respondsToSelector:@selector(tableScrollBuffCount:)])
		return [delegate tableScrollBuffCount:self];
	if (dataDelegate != 0 &&
		[dataDelegate respondsToSelector:@selector(tableScrollBuffCount:)])
		return [dataDelegate tableScrollBuffCount:self];
	return 1;
	}


//-----------------------------------------------------------------------------
// - expandIfNeeded
//-----------------------------------------------------------------------------
- (void) expandIfNeeded
	{
	int const num_cells = max_rows * num_cols;
	if (num_cells > max_cells)
		{
		NSZone* const z = [self zone];
		int const nbytes = num_cells * sizeof(*cells);
		if (max_cells == 0)
			cells = (id*) NSZoneMalloc( z, nbytes );
		else
			cells = (id*) NSZoneRealloc( z, cells, nbytes );
		max_cells = num_cells;
		}
	}


//-----------------------------------------------------------------------------
// - lazyInsertColumn:
//-----------------------------------------------------------------------------
- (void) lazyInsertColumn:(int)n
	{
	colInfo.border->insertAt(n,n);		// FIXME: Check this.
	num_cols++;
	}


//-----------------------------------------------------------------------------
// - eagerInsertColumn:
//-----------------------------------------------------------------------------
- (void) eagerInsertColumn:(int)n
	{
	assert( 0 <= n );  assert( n <= num_cols );
	assert( 0 < num_cols );				// lazyInsert must increment num_cols.

	if (max_rows > 0 || num_rows > 0)
		{
		if (max_rows == 0)
			max_rows = num_rows;
		[self expandIfNeeded];
		int const num_new = num_cols;
		int const num_old = num_cols - 1;
		if (num_old > 0)				// Shift old, existing cells.
			{
			int const frag = num_old - n; // #cols right of insert col.
			id* src = cells + (max_rows * num_old) - frag;
			id* dst = cells + (max_rows * num_new) - frag;
			if (frag > 0)				// Last partial row.
				memmove( dst, src, frag * sizeof(*dst) );

			int const nbytes = num_old * sizeof(*dst);
			src -= num_old;
			dst -= num_new;
			while (src > cells)			// All full rows.
				{
				memmove( dst, src, nbytes );
				src -= num_old;
				dst -= num_new;
				}
			}
		int row = 0;
		id const proto = colInfo.border->getPrototype_P(n);
		id* const plim = cells + (max_rows * num_new);
		id* p = cells + n;
		while (p < plim)				// Initialize cells in new column.
			{
			*p = [self reviveCell:[proto copy] atRow:row++ column:n];
			p += num_new;
			}
		}
	}


//-----------------------------------------------------------------------------
// - insertColumn:
//-----------------------------------------------------------------------------
- (void) insertColumn:(int)n
	{
	if (0 <= n && n <= num_cols)
		{
		[self lazyInsertColumn:n];
		if (!lazy)
			[self eagerInsertColumn:n];
		[self constrainSize];
		[self setNeedsDisplay:YES];
		}
	}


//-----------------------------------------------------------------------------
// - addColumn
//-----------------------------------------------------------------------------
- (void) addColumn
	{
	[self insertColumn:num_cols];
	}


//-----------------------------------------------------------------------------
// - lazyRemoveColumn:
//-----------------------------------------------------------------------------
- (void) lazyRemoveColumn:(int)n
	{
	colInfo.border->deleteAt_P(n);
	num_cols--;
	}


//-----------------------------------------------------------------------------
// - eagerRemoveColumn:
//-----------------------------------------------------------------------------
- (void) eagerRemoveColumn:(int)n
	{
	assert( 0 < num_cols );
	assert( 0 <= n );  assert( n < num_cols );
	if (max_rows > 0)
		{
		int const num_old = num_cols;
		int const num_new = num_cols - 1;
		id* dst = cells + n;
		id* src = dst + 1;
		id* const dst_lim = cells + (num_old * max_rows);
		int row = 0;
		while (dst < dst_lim)			// Destroy deleted cells.
			{
			if (row < num_rows)
				[[self retireCell:*dst atRow:row++ column:n] release];
			else
				[*dst release];
			dst += num_old;
			}
		if (num_new > 0)				// Shift remaining columns.
			{
			dst = cells + n;
			int const nbytes = num_new * sizeof(*dst);
			id* const src_lim = cells + (max_rows - 1) * num_old + n;
			while (src < src_lim)
				{
				memmove( dst, src, nbytes );
				dst += num_new;
				src += num_old;
				}
			if (n < num_new)			// Shift last partial row.
				memmove( dst, src, (num_new - n) * sizeof(*dst) );
			}
		else // (num_new <= 0)
			{
			max_rows = 0;
			}
		}
	}


//-----------------------------------------------------------------------------
// - removeColumn:
//-----------------------------------------------------------------------------
- (void) removeColumn:(int)n
	{
	if (0 <= n && n < num_cols)
		{
		if (!lazy)
			[self eagerRemoveColumn:n];
		[self lazyRemoveColumn:n];
		[self constrainSize];
		[self setNeedsDisplay:YES];
		}
	}


//-----------------------------------------------------------------------------
// - numberOfColumns
//-----------------------------------------------------------------------------
- (int) numberOfColumns
	{
	return num_cols;
	}


//-----------------------------------------------------------------------------
// - lazyInsertRow:
//-----------------------------------------------------------------------------
- (void) lazyInsertRow:(int)n
	{
	rowInfo.border->insertAt(n,n);		// FIXME.  Check this.
	num_rows++;
	}


//-----------------------------------------------------------------------------
// - eagerInsertRow:
//-----------------------------------------------------------------------------
- (void) eagerInsertRow:(int)n
	{
	assert( 0 < num_rows );				// lazy-insert must increment num_rows
	assert( 0 <= n );  assert( n < num_rows );

	if (num_cols > 0)
		{
		max_rows++;								// Incremental growth.
		assert( num_rows <= max_rows );
		[self expandIfNeeded];

		int const old_max = max_rows - 1;
		id* p = cells + n * num_cols;
		if (n < old_max)						// Shift other rows.
			{
			id* src = p;
			id* dst = src + num_cols;
			memmove( dst, src, (old_max - n) * num_cols * sizeof(*dst) );
			}
		MiscTableBorder* const bp = colInfo.border;
		for (int c = 0;	 c < num_cols;	c++)	// Initialize new cells.
			*p++ = [self reviveCell:[bp->getPrototype_P(c) copy]
						atRow:n column:c];
		}
	}


//-----------------------------------------------------------------------------
// - insertRow:
//-----------------------------------------------------------------------------
- (void) insertRow:(int)n
	{
	if (0 <= n && n <= num_rows)
		{
		[self lazyInsertRow:n];
		if (!lazy)
			[self eagerInsertRow:n];
		[self constrainSize];
		[self setNeedsDisplay:YES];
		}
	}


//-----------------------------------------------------------------------------
// - eagerAddRow
//
// NOTE *1*
//		We always initialize new rows when they are created, so if num_rows
//		is less than max_rows, the new row has already been initialized.
//
// NOTE *2*
//		The initialization loop is organized in column-major order because
//		we want to maximize locality of reference for both code and data
//		during the -copy calls.
//-----------------------------------------------------------------------------
- (void) eagerAddRow
	{
	assert( 0 < num_rows );		// lazy insertRow: must increment num_rows.
	if (num_cols > 0)
		{
		if (num_rows >= max_rows)						// NOTE *1*
			{
			int const MISC_TS_MAX_ROWS = ((INT_MAX >> 1) + 1);
			assert( num_rows <= MISC_TS_MAX_ROWS );

			int const old_max = max_rows;
			int n = ( max_rows ? max_rows : 16 );
			while (n < num_rows)
				n <<= 1;
			int const new_max = max_rows = n;

			[self expandIfNeeded];
			id* p0 = cells + old_max * num_cols;
			for (int c = 0;	 c < num_cols;	c++)		// NOTE *2*
				{
				id const proto = colInfo.border->getPrototype_P(c);
				id* p = p0++;
				id* const plim = p + (new_max - old_max) * num_cols;
				while (p < plim)
					{
					*p = [proto copy];					// Initialize new cells.
					p += num_cols;
					}
				}
			}
		int const row = num_rows - 1;
		id* p = cells + (num_cols * row);
		for (int col = 0;  col < num_cols;	col++)
			*p++ = [self reviveCell:*p atRow:row column:col];
		}
	}


//-----------------------------------------------------------------------------
// - addRow
//
// NOTE: We explicitly do NOT -constrainSize here, because we expect that the
// user will call this routine many times successively.	 
//-----------------------------------------------------------------------------
- (void) addRow
	{
	[self lazyInsertRow:num_rows];
	if (!lazy)
		[self eagerAddRow];
	}


//-----------------------------------------------------------------------------
// - lazyRemoveRow:
//-----------------------------------------------------------------------------
- (void) lazyRemoveRow:(int)n
	{
	rowInfo.border->deleteAt_P(n);
	num_rows--;
	}


//-----------------------------------------------------------------------------
// - eagerRemoveRow:
//-----------------------------------------------------------------------------
- (void) eagerRemoveRow:(int)n
	{
	assert( 0 < num_rows );
	assert( 0 <= n );  assert( n < num_rows );
	if (num_cols > 0)
		{
		assert( num_rows <= max_rows );
		max_rows--;
		id* const dst = cells + (n * num_cols );
		id* p = dst;
		id* const plim = p + num_cols;
		int col = 0;
		while (p < plim)		// Destroy the cells.
			{
			[[self retireCell:*p atRow:n column:col++] release];
			p++;
			}
		if (n < max_rows)
			memmove( dst, plim, (max_rows - n) * num_cols * sizeof(*dst) );
		}
	}


//-----------------------------------------------------------------------------
// - removeRow:
//-----------------------------------------------------------------------------
- (void) removeRow:(int)n
	{
	if (0 <= n && n < num_rows)
		{
		if (!lazy)
			[self eagerRemoveRow:n];
		[self lazyRemoveRow:n];
		[self constrainSize];
		[self setNeedsDisplay:YES];
		}
	}


//-----------------------------------------------------------------------------
// - numberOfRows
//-----------------------------------------------------------------------------
- (int) numberOfRows
	{
	return num_rows;
	}


//-----------------------------------------------------------------------------
// - lazyRenewRows:
//-----------------------------------------------------------------------------
- (void) lazyRenewRows:(int)n
	{
	rowInfo.border->setCount(n);
	num_rows = n;
	}


//-----------------------------------------------------------------------------
// - eagerRenewRows:
//
// NOTE *1*
//		Since we initialize all cells when they are allocated, we only have
//		to do extra work if we need to allocate more rows.
//
// NOTE *2*
//		The initialization loop is organized in column-major order to
//		increase locality of reference for both code and data during the
//		call to -copy.
//-----------------------------------------------------------------------------
- (void) eagerRenewRows:(int)n
	{
	assert( 0 <= n );
	if (num_cols > 0)
		{
		int const old_num_rows = num_rows;
		int const new_num_rows = n;
		if (n > max_rows)								// NOTE *1*
			{
			int const max_old = max_rows;
			int const max_new = max_rows = n;
			[self expandIfNeeded];
			id* p0 = cells + max_old * num_cols;
			id* const plim = cells + max_new * num_cols;
			for (int c = 0; c < num_cols; c++)			// NOTE *2*
				{
				id* p = p0++;
				id const proto = colInfo.border->getPrototype_P(c);
				while (p < plim)
					{
					*p = [proto copy];
					p += num_cols;
					}
				}
			}
		if (old_num_rows < new_num_rows)				// Growing
			{
			id* p0 = cells + old_num_rows * num_cols;
			id* const plim = cells + new_num_rows * num_cols;
			for (int c = 0; c < num_cols; c++)			// NOTE *2*
				{
				int r = old_num_rows;
				id* p = p0++;
				while (p < plim)
					{
					*p = [self reviveCell:*p atRow:r++ column:c];
					p += num_cols;
					}
				}
			}
		else if (old_num_rows > new_num_rows)			// Shrinking
			{
			id* p0 = cells + new_num_rows * num_cols;
			id* const plim = cells + old_num_rows * num_cols;
			for (int c = 0; c < num_cols; c++)			// NOTE *2*
				{
				int r = new_num_rows;
				id* p = p0++;
				while (p < plim)
					{
					*p = [self retireCell:*p atRow:r++ column:c];
					p += num_cols;
					}
				}
			}
		}
	}


//-----------------------------------------------------------------------------
// - renewRows:
//-----------------------------------------------------------------------------
- (void) renewRows:(int)n
	{
	[self clearSelection];
	if (0 <= n && n != num_rows)
		{
		if (!lazy)
			[self eagerRenewRows:n];
		[self lazyRenewRows:n];
		}
	[self constrainSize];
	[self setNeedsDisplay:YES];
	}


//-----------------------------------------------------------------------------
// - empty
//-----------------------------------------------------------------------------
- (void) empty
	{
	return [self renewRows:0];
	}


//-----------------------------------------------------------------------------
// - releaseCells
//-----------------------------------------------------------------------------
- (void) releaseCells
	{
	if (max_rows > 0 && num_cols > 0)
		{
		id* p0 = cells;
		for (int c = 0;	 c < num_cols;	c++)
			{
			id* p = p0++;
			for (int r = 0;	 r < max_rows;	r++)
				{
				[*p release];
				p += num_cols;
				}
			}
		}
	if (cells != 0)
		NSZoneFree( [self zone], cells );
	cells = 0;
	max_cells = 0;
	max_rows = 0;
	}


//-----------------------------------------------------------------------------
// - emptyAndReleaseCells
//-----------------------------------------------------------------------------
- (void) emptyAndReleaseCells
	{
	[self empty];
	[self releaseCells]; 
	}


//-----------------------------------------------------------------------------
// - setLazy:
//-----------------------------------------------------------------------------
- (void) setLazy:(BOOL)flag
	{
	if (lazy != flag)
		{
		lazy = flag;
		if (lazy)
			[self emptyAndReleaseCells];
		else
			[self eagerRenewRows:num_rows];
		}
	}


//-----------------------------------------------------------------------------
// - isLazy
//-----------------------------------------------------------------------------
- (BOOL) isLazy
	{
	return lazy;
	}


//-----------------------------------------------------------------------------
// Generic Slot methods
//-----------------------------------------------------------------------------
- (void) addSlot:(MiscBorderType)b
		{ return (b == MISC_COL_BORDER) ? [self addColumn] : [self addRow]; }

- (void) border:(MiscBorderType)b insertSlot:(int)n
		{ return (b == MISC_COL_BORDER) ?
				[self insertColumn:n] : [self insertRow:n]; }

- (void) border:(MiscBorderType)b removeSlot:(int)n
		{ return (b == MISC_COL_BORDER) ?
				[self removeColumn:n] : [self removeRow:n]; }

- (int) numberOfSlots:(MiscBorderType)b
		{ return (b == MISC_COL_BORDER) ?
				[self numberOfColumns] : [self numberOfRows]; }


//-----------------------------------------------------------------------------
// REVIVE
//-----------------------------------------------------------------------------
- (id) doReviveCell:(id)cell atRow:(int)row column:(int)col
	{
	if (cell != 0)
		{
		if ([cell respondsToSelector:@selector(setOwner:)])
			[cell setOwner:self];

		if ([cell respondsToSelector:@selector(setUseOwnerFont:)])
			[cell setUseOwnerFont:YES];
		if ([cell respondsToSelector:@selector(setUseOwnerTextColor:)])
			[cell setUseOwnerTextColor:YES];
		if ([cell respondsToSelector:@selector(setUseOwnerBackgroundColor:)])
			[cell setUseOwnerBackgroundColor:YES];
		if ([cell respondsToSelector:
			@selector(setUseOwnerHighlightTextColor:)])
			[cell setUseOwnerHighlightTextColor:YES];
		if ([cell respondsToSelector:
			@selector(setUseOwnerHighlightBackgroundColor:)])
			[cell setUseOwnerHighlightBackgroundColor:YES];

		if ([cell respondsToSelector:@selector(setOwnerFont:)])
			[cell setOwnerFont:[self font]];
		else if ([cell respondsToSelector:@selector(setFont:)])
			[cell setFont:[self font]];

		if ([cell respondsToSelector:@selector(setOwnerTextColor:)])
			[cell setOwnerTextColor:[self textColor]];
		else if ([cell respondsToSelector:@selector(setTextColor:)])
			[cell setTextColor:[self textColor]];

		if ([cell respondsToSelector:@selector(setOwnerBackgroundColor:)])
			[cell setOwnerBackgroundColor:[self backgroundColor]];
		else if ([cell respondsToSelector:@selector(setBackgroundColor:)])
			[cell setBackgroundColor:[self backgroundColor]];

		if ([cell respondsToSelector:@selector(setOwnerHighlightTextColor:)])
			[cell setOwnerHighlightTextColor:[self highlightTextColor]];
		else if ([cell respondsToSelector:@selector(setHighlightTextColor:)])
			[cell setHighlightTextColor:[self highlightTextColor]];

		if ([cell respondsToSelector:
			@selector(setOwnerHighlightBackgroundColor:)])
			[cell setOwnerHighlightBackgroundColor:
						[self highlightBackgroundColor]];
		else if ([cell respondsToSelector:
			@selector(setHighlightBackgroundColor:)])
			[cell setHighlightBackgroundColor:[self highlightBackgroundColor]];
		}
	return cell;
	}


- (id) reviveCell:(id)cell atRow:(int)row column:(int)col
	{
	if (delegate != 0 &&
		[delegate respondsToSelector:
		@selector(tableScroll:reviveCell:atRow:column:)])
		return [delegate tableScroll:self reviveCell:cell
						atRow:row column:col];

	if (dataDelegate != 0 &&
		[dataDelegate respondsToSelector:
		@selector(tableScroll:reviveCell:atRow:column:)])
		return [dataDelegate tableScroll:self reviveCell:cell
						atRow:row column:col];

	if (cell != 0 && [cell respondsToSelector:
		@selector(tableScroll:reviveAtRow:column:)])
		return [cell tableScroll:self reviveAtRow:row column:col];

	return [self doReviveCell:cell atRow:row column:col];
	}


//-----------------------------------------------------------------------------
// RETIRE
//-----------------------------------------------------------------------------
- (id) doRetireCell:(id)cell atRow:(int)row column:(int)col
	{
	if ([cell respondsToSelector:@selector(type)] &&
		[(NSCell*)cell type] == NSImageCellType)
		[cell setImage:[[[NSImage alloc] initWithSize:NSZeroSize] autorelease]];
	else if ([cell respondsToSelector:@selector(setTitle:)])
		[cell setTitle:@""];
	else if ([cell respondsToSelector:@selector(setStringValue:)])
		[cell setStringValue:@""];
	return cell;
	}


- (id) retireCell:(id)cell atRow:(int)row column:(int)col
	{
	if (delegate != 0 && [delegate respondsToSelector:
		@selector(tableScroll:retireCell:atRow:column:)])
		return [delegate tableScroll:self retireCell:cell
						atRow:row column:col];

	if (dataDelegate != 0 && [dataDelegate respondsToSelector:
		@selector(tableScroll:retireCell:atRow:column:)])
		return [dataDelegate tableScroll:self retireCell:cell
						atRow:row column:col];

	if (cell != 0 && [cell respondsToSelector:
		@selector(tableScroll:retireAtRow:column:)])
		return [cell tableScroll:self retireAtRow:row column:col];

	return [self doRetireCell:cell atRow:row column:col];
	}

@end
