//=============================================================================
//
//		Copyright (C) 1995, 1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscTableScrollInspector.M
//
//		Interface Builder inspector for MiscTableScroll.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableScrollInspector.M,v 1.12 96/05/13 07:41:15 sunshine Exp $
// $Log:		MiscTableScrollInspector.M,v $
//	Revision 1.12  96/05/13	 07:41:15  sunshine
//	For consistency with other renamed methods, -border:insertSlotAt: and
//	-border:removeSlotAt: were renamed to -border:insertSlot: and
//	-border:removeSlot: respectively.
//	
//	Revision 1.11  96/05/07	 09:47:23  sunshine
//	-setTitlesOn: ->> -setSlotTitlesOn:, -setTitleMode: ->> -setSlotTitleMode:
//	
//	Revision 1.10  96/05/06	 04:35:13  sunshine
//	Now performs a revert after toggling lazy-mode to recognize the fact
//	that the rows get deleted when toggling from non-lazy to lazy.
//-----------------------------------------------------------------------------
#import "MiscTableScrollInspector.h"
#import "MiscTableBorder.h"
#import <misckit/MiscTableScroll.h>

//=============================================================================
// IB-ONLY METHODS FOR MiscTableScroll
//=============================================================================
@interface MiscTableScroll(IB)
- (NSString*) inspectorClassName;
- (void) editSelf:(id)sender in:(id)owner;
@end

@implementation MiscTableScroll(IB)
- (NSString*) inspectorClassName
	{
	NSString* s = @"MiscTableScrollInspector";
	return s;
	}
// Prevent double-click from editing NSScrollView in IB
- (void) editSelf:(id)sender in:(id)owner {}
@end


//=============================================================================
// CONVENIENCE CATAGORIES
//=============================================================================
@interface NSPopUpButton(MiscTableScroll)
- (int) popUpTag;
- (void) setPopUpTag: (int) new_tag;
@end

@implementation NSPopUpButton(MiscTableScroll)

- (int) popUpTag
	{
	return [[self selectedItem] tag];
	}

- (void) setPopUpTag: (int) new_tag
	{
	NSMatrix* matrix = [self itemMatrix];
	NSMenuItem* m_cell = [matrix cellWithTag:new_tag];
	[self selectItemWithTitle:[m_cell title]];
	}

@end


//=============================================================================
// IMPLEMENTATION
//=============================================================================
@interface MiscTableScrollInspector(Forward)
- (void) doSlot:sender;
- (void) doTitleMode:sender;
@end

@implementation MiscTableScrollInspector

//-----------------------------------------------------------------------------
// init
//-----------------------------------------------------------------------------
- init
	{
	[super init];
	[NSBundle loadNibNamed:[[self class] description] owner:self];
	[[upButton cell] setImageDimsWhenDisabled:YES];
	[[downButton cell] setImageDimsWhenDisabled:YES];
	border_type = MISC_COL_BORDER;
	dirty = NO;
	return self;
	}


//-----------------------------------------------------------------------------
// wantsButtons
//-----------------------------------------------------------------------------
- (BOOL) wantsButtons
	{
	return NO;
	}


//-----------------------------------------------------------------------------
// setTitleControls
//-----------------------------------------------------------------------------
- (void) setTitleControls
	{
	id o = [self object];
	if ([o slotTitlesOn:border_type])
		{
		BOOL sizable = border->isSizeable();
		[titlesSwitch setState:1];
		[sizableSwitch setEnabled:YES];
		[sizableSwitch setState:sizable];
		[draggableSwitch setEnabled:YES];
		[draggableSwitch setState:border->isDraggable()];
		[titleModePopUp setEnabled:YES];
		[titleModePopUp setPopUpTag:(int)[o slotTitleMode:border_type]];
		[uniformSizeField setEnabled:!sizable];
		if (sizable)
			[uniformSizeField setStringValue:@""];
		else
			[uniformSizeField setIntValue:
						int([o uniformSizeSlots:border_type])];
		}
	else
		{
		[titlesSwitch setState:0];
		[sizableSwitch setEnabled:NO];
		[sizableSwitch setState:0];
		[draggableSwitch setEnabled:NO];
		[draggableSwitch setState:0];
		[titleModePopUp setEnabled:NO];
		[titleModePopUp setPopUpTag:(int)[o slotTitleMode:border_type]];
		[uniformSizeField setEnabled:YES];
		[uniformSizeField setIntValue:(int)[o uniformSizeSlots:border_type]];
		}
	}


//-----------------------------------------------------------------------------
// fillScroll
//-----------------------------------------------------------------------------
- (void) fillScroll
	{
	id o = [self object];
	num_slots = [o numberOfSlots:border_type];
	if (num_slots == 0)
		{
		[slotScroll deselectAllCells];
		[slotScroll renewRows:0 columns:0]; // Gets rid of horiz scroller knob
		}
	else
		{
		[slotScroll renewRows:num_slots columns:1];
		for (int i = 0;	 i < num_slots;	 i++)
			{
			int const pslot = [o border:border_type slotAtPosition:i];
			[[slotScroll cellAtRow:i column:0]
						setTitle:[o border:border_type slotTitle:pslot]];
			}
		}
	[slotScroll sizeToCells];

	if (num_slots > 0)
		{
		if (slot < 0) slot = 0;
		[slotScroll selectCellAtRow:slot column:0];
		[slotScroll scrollCellToVisibleAtRow:slot column:0];
		}
	else
		{
		slot = -1;
		}
	[self doSlot:self];
	}


//-----------------------------------------------------------------------------
// setConstrainSwitches
//-----------------------------------------------------------------------------
- (void) setConstrainSwitches
	{
	id o = [self object];
	[constrainMaxSwitch setState: [o maxTotalSizeIsConstrained:border_type]];
	[constrainMinSwitch setState: [o minTotalSizeIsConstrained:border_type]];
	}



//-----------------------------------------------------------------------------
// doConstrainMax:
//-----------------------------------------------------------------------------
- (void) doConstrainMax:sender
	{
	[[self object] border:border_type
				constrainMaxTotalSize:[constrainMaxSwitch state]];
	[self ok:sender];
	}



//-----------------------------------------------------------------------------
// doConstrainMin:
//-----------------------------------------------------------------------------
- (void) doConstrainMin:sender
	{
	[[self object] border:border_type
				constrainMinTotalSize:[constrainMinSwitch state]];
	[self ok:sender];
	}



//-----------------------------------------------------------------------------
// doEnabled:
//-----------------------------------------------------------------------------
- (void) doEnabled:sender
	{
	[[self object] setEnabled: [enabledSwitch state]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doLazy:
//-----------------------------------------------------------------------------
- (void) doLazy:sender
	{
	[[self object] setLazy: [lazySwitch state]];
	[self ok:sender];
	[self revert:sender];
	}


//-----------------------------------------------------------------------------
// doMode:
//-----------------------------------------------------------------------------
- (void) doMode:sender
	{
	[[self object] setSelectionMode: MiscSelectionMode([modePopUp popUpTag])];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doBorder:
//-----------------------------------------------------------------------------
- (void) doBorder:sender
	{
	MiscBorderType bt = MiscBorderType([borderPopUp popUpTag]);
	if (bt != border_type)
		{
		border_type = bt;
		[self revert:sender];
		}
	}


//-----------------------------------------------------------------------------
// doTitles:
//-----------------------------------------------------------------------------
- (void) doTitles:sender
	{
	BOOL const isOn = [titlesSwitch state];
	[[self object] border:border_type setSlotTitlesOn:isOn];
	[self setTitleControls];
	[self doSlot:self];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doSizable:
//-----------------------------------------------------------------------------
- (void) doSizable:sender
	{
	BOOL const val = [sizableSwitch state];
	id o = [self object];
	[o border:border_type setSizeableSlots:val];
	if (val)
		{
		[uniformSizeField setStringValue:@""];
		[uniformSizeField setEnabled:NO];
		[o border:border_type setUniformSizeSlots:0];
		}
	else
		{
		[uniformSizeField setEnabled:YES];
		[uniformSizeField setIntValue:(int)[o uniformSizeSlots:border_type]];
		}
	[self doSlot:0];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doDraggable:
//-----------------------------------------------------------------------------
- (void) doDraggable:sender
	{
	[[self object] border:border_type
				setDraggableSlots:[draggableSwitch state]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doAutoSort:
//-----------------------------------------------------------------------------
- (void) doAutoSort:sender
	{
	[[self object] border:border_type setAutoSortSlots:[autoSortSwitch state]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doTitleMode:
//-----------------------------------------------------------------------------
- (void) doTitleMode:sender
	{
	[[self object] border:border_type
		setSlotTitleMode:(MiscTableTitleMode)[titleModePopUp popUpTag]];
	[self fillScroll];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doDelete:
//-----------------------------------------------------------------------------
- (void) doDelete:sender
	{
	id o = [self object];
	[o border:border_type
				removeSlot:[o border:border_type slotAtPosition:slot]];
	[o tile];
	[o display];
	slot--;
	num_slots--;
	if (slot < 0 && num_slots > 0)
		slot = 0;
	[self fillScroll];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doNew:
//-----------------------------------------------------------------------------
- (void) doNew:sender
	{
	id o = [self object];
	[o addSlot:border_type];
	[o tile];
	[o display];
	slot = num_slots;
	[self fillScroll];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doUserSizeable:
//-----------------------------------------------------------------------------
- (void) doUserSizeable:sender
	{
	id o = [self object];
	[o border:border_type setSlot:[o border:border_type slotAtPosition:slot]
				sizeable:[userSizeableSwitch state]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doExpandToData:
//-----------------------------------------------------------------------------
- (void) doExpandToData:sender
	{
	id o = [self object];
	[o border:border_type setSlot:[o border:border_type slotAtPosition:slot]
				expandsToData:[expandToDataSwitch state]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doAutosize:
//-----------------------------------------------------------------------------
- (void) doAutosize:sender
	{
	id o = [self object];
	[o border:border_type setSlot:[o border:border_type slotAtPosition:slot]
				autosize:[autosizeSwitch state]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doCellClass:
//-----------------------------------------------------------------------------
- (void) doCellClass:sender
	{
	id o = [self object];
	[o border:border_type setSlot:[o border:border_type slotAtPosition:slot]
				cellType:(MiscTableCellStyle)[cellClassPopUp popUpTag]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doSortType:
//-----------------------------------------------------------------------------
- (void) doSortType:sender
	{
	id o = [self object];
	[o border:border_type setSlot:[o border:border_type slotAtPosition:slot]
				sortType:(MiscSortType)[sortTypePopUp popUpTag]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doSortDirection:
//-----------------------------------------------------------------------------
- (void) doSortDirection:sender
	{
	id o = [self object];
	[o border:border_type setSlot:[o border:border_type slotAtPosition:slot]
		sortDirection:(MiscSortDirection)[sortDirectionPopUp popUpTag]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doSlot:
//-----------------------------------------------------------------------------
- (void) doSlot:sender
	{
	slot = [slotScroll selectedRow];
	if (slot >= 0)
		{
		id o = [self object];
		BOOL const titlesOn = [o slotTitlesOn:border_type];
		BOOL const notUniform = ([o uniformSizeSlots:border_type] == 0);
		int pslot = [o border:border_type slotAtPosition:slot];
		[deleteButton setEnabled:YES];
		[upButton setEnabled:(slot > 0)];
		[downButton setEnabled:(slot < num_slots - 1)];
		[cellClassPopUp setEnabled:YES];
		[cellClassPopUp setPopUpTag:(int)
				[o border:border_type slotCellType:pslot]];
		[sortTypePopUp setEnabled:YES];
		[sortTypePopUp setPopUpTag:(int)
				[o border:border_type slotSortType:pslot]];
		[sortDirectionPopUp setEnabled:YES];
		[sortDirectionPopUp setPopUpTag:(int)
				[o border:border_type slotSortDirection:pslot]];
		[autosizeSwitch setEnabled:notUniform];
		[autosizeSwitch setState:[o border:border_type slotIsAutosize:pslot]];
		[expandToDataSwitch setEnabled:notUniform];
		[expandToDataSwitch setState:
				[o border:border_type slotExpandsToData:pslot]];
		BOOL const enabled = titlesOn && notUniform &&
				[[self object] sizeableSlots:border_type];
		[userSizeableSwitch setEnabled:enabled];
		[userSizeableSwitch setState:
				enabled && [o border:border_type slotIsSizeable:pslot]];
		[sizeField setEnabled:notUniform];
		[sizeField setIntValue:(int)[o border:border_type slotSize:pslot]];
		[sizeMinField setEnabled:notUniform];
		[sizeMinField setIntValue:(int)
				[o border:border_type slotMinSize:pslot]];
		[sizeMaxField setEnabled:notUniform];
		[sizeMaxField setIntValue:(int)
				[o border:border_type slotMaxSize:pslot]];
		[titleField setEnabled:titlesOn &&
				([o slotTitleMode:border_type] == MISC_CUSTOM_TITLE)];
		[titleField setStringValue:[o border:border_type slotTitle:pslot]];
		}
	else
		{
		[deleteButton setEnabled:NO];
		[upButton setEnabled:NO];
		[downButton setEnabled:NO];
		[cellClassPopUp setEnabled:NO];
		[cellClassPopUp setPopUpTag:0];
		[sortTypePopUp setEnabled:NO];
		[sortTypePopUp setPopUpTag:0];
		[sortDirectionPopUp setEnabled:NO];
		[sortDirectionPopUp setPopUpTag:0];
		[autosizeSwitch setEnabled:NO];
		[autosizeSwitch setState:0];
		[expandToDataSwitch setEnabled:NO];
		[expandToDataSwitch setState:0];
		[userSizeableSwitch setEnabled:NO];
		[userSizeableSwitch setState:0];
		[sizeField setEnabled:NO];
		[sizeField setStringValue:@""];
		[sizeMinField setEnabled:NO];
		[sizeMinField setStringValue:@""];
		[sizeMaxField setEnabled:NO];
		[sizeMaxField setStringValue:@""];
		[titleField setEnabled:NO];
		[titleField setStringValue:@""];
		} 
	}


//-----------------------------------------------------------------------------
// doUp:
//-----------------------------------------------------------------------------
- (void) doUp:sender
	{
	if (slot > 0)
		{
		[[self object] border:border_type moveSlotFrom:slot to:slot-1];
		border->clearVMap();
		slot--;
		[self fillScroll];
		[self ok:sender];
		}
	}


//-----------------------------------------------------------------------------
// doDown:
//-----------------------------------------------------------------------------
- (void) doDown:sender
	{
	if (slot < num_slots - 1)
		{
		[[self object] border:border_type moveSlotFrom:slot to:slot+1];
		border->clearVMap();
		slot++;
		[self fillScroll];
		[self ok:sender];
		}
	}


//-----------------------------------------------------------------------------
// get_pslot
//		Get the physical slot-id of the current slot.
//-----------------------------------------------------------------------------
- (int) get_pslot
	{
	return [[self object] border:border_type slotAtPosition:slot];
	}


//-----------------------------------------------------------------------------
// controlTextDidBeginEditing:
//-----------------------------------------------------------------------------
- (void) controlTextDidBeginEditing:(NSNotification*)n
	{
	dirty = YES;
	}


//-----------------------------------------------------------------------------
// controlTextDidEndEditing:
//-----------------------------------------------------------------------------
- (void) controlTextDidEndEditing:(NSNotification*)n
	{
	if (dirty)
		{
		dirty = NO;
		id o = [self object];
		NSTextField* field = [n object];
		if (field == titleField)
			{
			int const pslot = [self get_pslot];
			[o border:border_type setSlot:pslot
						title:[titleField stringValue]];
			[[slotScroll cellAtRow:slot column:0]
						setTitle:[titleField stringValue]];
			[slotScroll drawCellAtRow:slot column:0];
			}
		else if (field == uniformSizeField)
			{
			int old_size = (int)[o uniformSizeSlots:border_type];
			int new_size = [uniformSizeField intValue];
			if (new_size != 0)
				{
				if (new_size < MISC_MIN_PIXELS_SIZE)
					new_size = MISC_MIN_PIXELS_SIZE;
				if (new_size > MISC_MAX_PIXELS_SIZE)
					new_size = MISC_MAX_PIXELS_SIZE;
				[uniformSizeField setIntValue:new_size];
				}
			if (old_size != new_size)
				{
				[o border:border_type setUniformSizeSlots:(float)new_size];
				[self ok:self];
				[self revert:self];
				}
			}
		else if (field == sizeField || field == sizeMinField ||
				 field == sizeMaxField)
			{
			int const pslot = [self get_pslot];
			int i = [sizeField intValue];
			int imax = [sizeMaxField intValue];
			int imin = [sizeMinField intValue];
			if (field == (id)sizeMaxField)
				{
				if (imax > MISC_MAX_PIXELS_SIZE) imax = MISC_MAX_PIXELS_SIZE;
				else if (imax < imin) imax = imin;
				[sizeMaxField setIntValue: imax];
				[o border:border_type setSlot:pslot maxSize:(float)imax];
				}
			else if (field == sizeMinField)
				{
				if (imin < MISC_MIN_PIXELS_SIZE) imin = MISC_MIN_PIXELS_SIZE;
				else if (imin > imax) imin = imax;
				[sizeMinField setIntValue: imin];
				[o border:border_type setSlot:pslot minSize:(float)imin];
				}
	
			if (i < imin || i > imax) // May need to do every time
				{
				if (i < imin) i = imin;
				if (i > imax) i = imax;
				[sizeField setIntValue: i];
				}
			[o border:border_type setSlot:pslot size:(float)i];
			}
		[self ok:self];
		}
	}


//-----------------------------------------------------------------------------
// doColorText:
//-----------------------------------------------------------------------------
- (void) doColorText:sender
	{
	[[self object] setTextColor: [sender color]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doColorBack:
//-----------------------------------------------------------------------------
- (void) doColorBack:sender
	{
	[[self object] setBackgroundColor: [sender color]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doColorTextH:
//-----------------------------------------------------------------------------
- (void) doColorTextH:sender
	{
	[[self object] setHighlightTextColor: [sender color]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// doColorBackH:
//-----------------------------------------------------------------------------
- (void) doColorBackH:sender
	{
	[[self object] setHighlightBackgroundColor: [sender color]];
	[self ok:sender];
	}


//-----------------------------------------------------------------------------
// ok:
//-----------------------------------------------------------------------------
- (void) ok:sender
	{
	[super ok:sender];
	}


//-----------------------------------------------------------------------------
// revert:
//-----------------------------------------------------------------------------
- (void) revert:sender
	{
	dirty = NO;
	id o = [self object];
	[super revert: sender];

	border = [o border:border_type];
	[autoSortSwitch setState:[o autoSortSlots:border_type]];
	[self setTitleControls];

	[self setConstrainSwitches];

	[enabledSwitch setState: [o isEnabled]];
	[lazySwitch setState: [o isLazy]];
	[modePopUp setPopUpTag:(int) [o selectionMode]];

	[colorText	setColor: [o textColor]];
	[colorTextH setColor: [o highlightTextColor]];
	[colorBack	setColor: [o backgroundColor]];
	[colorBackH setColor: [o highlightBackgroundColor]];

	slot = -1;
	[self fillScroll];
	}

@end
