//=============================================================================
//
//		Copyright (C) 1995, 1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscTableScrollPB.M
//
//		Pasteboard and services support for MiscTableScroll.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableScrollPB.M,v 1.5 96/04/30 05:39:44 sunshine Exp $
// $Log:		MiscTableScrollPB.M,v $
//	Revision 1.5  96/04/30	05:39:44  sunshine
//	Ported to OpenStep 4.0 for Mach PR2.
//	
//-----------------------------------------------------------------------------
#import <misckit/MiscTableScroll.h>

extern "Objective-C" {
#import <AppKit/NSApplication.h>
#import <AppKit/NSCell.h>
#import <AppKit/NSPasteboard.h>
}

#define MISC_PB_FIELD_SEPARATOR			@"\t"
#define MISC_PB_RECORD_TERMINATOR		@"\n"

@implementation MiscTableScroll(Pasteboard)

//-----------------------------------------------------------------------------
// - sortSel:border:
//-----------------------------------------------------------------------------
- (NSArray*) sortSel:(NSArray*)sel_list border:(MiscBorderType)b
	{
	return [self border:b visualToPhysical:
				[[self border:b physicalToVisual:sel_list]
				sortedArrayUsingSelector:@selector(compare:)]];
	}



//-----------------------------------------------------------------------------
// - builtinRegisterServicesTypes
//
// FIXME: Deal with these also:
//		writeTypes[*] = NXRTFPboardType
//		writeTypes[*] = NXFontPboardType;
//		writeTypes[*] = NXColorPboardType
//		writeTypes[*] = NXSelectionPboardType
//		writeTypes[*] = NXDataLinkPboardType
//-----------------------------------------------------------------------------
- (void) builtinRegisterServicesTypes
	{
	NSArray* sendTypes = [NSArray arrayWithObjects:
				NSTabularTextPboardType, NSStringPboardType, 0];
	NSArray* returnTypes = [NSArray array];
	[NSApp registerServicesMenuSendTypes:sendTypes returnTypes:returnTypes];
	}



//-----------------------------------------------------------------------------
// - registerServicesTypes
//-----------------------------------------------------------------------------
- (void) registerServicesTypes
	{
	if (delegate != 0 &&
		[delegate respondsToSelector:
				@selector(tableScrollRegisterServicesTypes:)])
		[delegate tableScrollRegisterServicesTypes:self];

	else if (dataDelegate != 0 &&
		[dataDelegate respondsToSelector:
				@selector(tableScrollRegisterServicesTypes:)])
		[dataDelegate tableScrollRegisterServicesTypes:self];

	else
		[self builtinRegisterServicesTypes];
	}



//-----------------------------------------------------------------------------
// - builtinValidRequestorForSendType:returnType:
//-----------------------------------------------------------------------------
- (id) builtinValidRequestorForSendType:(NSString*)t_send
	returnType:(NSString*)t_return
	{
	if (t_return == 0 &&		// We only send stuff, we never take stuff.
		([t_send isEqualToString:NSTabularTextPboardType] ||
		 [t_send isEqualToString:NSStringPboardType]) &&
		([self hasRowSelection] || [self hasColSelection]))
		return self;

	return [super validRequestorForSendType:t_send returnType:t_return];
	}


//-----------------------------------------------------------------------------
// - validRequestorForSendType:returnType:
//-----------------------------------------------------------------------------
- (id) validRequestorForSendType:(NSString*)t_send
	returnType:(NSString*)t_return
	{
	if (delegate != 0 && [delegate respondsToSelector:
		@selector(tableScroll:validRequestorForSendType:returnType:)])
		return [delegate tableScroll:self validRequestorForSendType:t_send
				returnType:t_return];

	if (dataDelegate != 0 && [dataDelegate respondsToSelector:
		@selector(tableScroll:validRequestorForSendType:returnType:)])
		return [dataDelegate tableScroll:self validRequestorForSendType:t_send
				returnType:t_return];

	return [self builtinValidRequestorForSendType:t_send returnType:t_return];
	}


//-----------------------------------------------------------------------------
// - builtinReadSelectionFromPasteboard:
//-----------------------------------------------------------------------------
- (BOOL) builtinReadSelectionFromPasteboard:(NSPasteboard*)pboard
	{
	return NO;
	}


//-----------------------------------------------------------------------------
// - readSelectionFromPasteboard:
//-----------------------------------------------------------------------------
- (BOOL) readSelectionFromPasteboard:(NSPasteboard*)pb
	{
	if (delegate != 0 && [delegate respondsToSelector:
		@selector(tableScroll:readSelectionFromPasteboard:)])
		return [delegate tableScroll:self readSelectionFromPasteboard:pb];

	if (dataDelegate != 0 && [dataDelegate respondsToSelector:
		@selector(tableScroll:readSelectionFromPasteboard:)])
		return [dataDelegate tableScroll:self readSelectionFromPasteboard:pb];

	return [self builtinReadSelectionFromPasteboard:pb];
	}


//-----------------------------------------------------------------------------
// - builtinCanWritePboardType:
//-----------------------------------------------------------------------------
- (BOOL) builtinCanWritePboardType:(NSString*)type
	{
	return ([type isEqualToString:NSStringPboardType] ||
			[type isEqualToString:NSTabularTextPboardType]);
	}


//-----------------------------------------------------------------------------
// - canWritePboardType:
//-----------------------------------------------------------------------------
- (BOOL) canWritePboardType:(NSString*)type
	{
	if (delegate != 0 &&
		[delegate respondsToSelector:
		@selector(tableScroll:canWritePboardType:)])
		return [delegate tableScroll:self canWritePboardType:type];

	if (dataDelegate != 0 &&
		[dataDelegate respondsToSelector:
		@selector(tableScroll:canWritePboardType:)])
		return [dataDelegate tableScroll:self canWritePboardType:type];

	return [self builtinCanWritePboardType:type];
	}


//-----------------------------------------------------------------------------
// - stringForNSStringPboardTypeAtRow:column:
//-----------------------------------------------------------------------------
- (NSString*) stringForNSStringPboardTypeAtRow:(int)row column:(int)col
	{
	NSString* s = 0;
	id cell = [self cellAtRow:row column:col];

	if (cell != 0)
		{
		if ([cell respondsToSelector:@selector(title)])
			s = [cell title];
		else if ([cell respondsToSelector:@selector(stringValue)])
			s = [cell stringValue];
		}

	if (s == 0)
		s = @"";
	else
		{
		NSRange r = { 0, 0 };
		NSMutableString* ms = [[s mutableCopy] autorelease];
		for (;;)
			{
			r = [ms rangeOfString:MISC_PB_FIELD_SEPARATOR options:0
				range:(NSRange){ r.location, [s length] - r.location }];
			if (r.length == 0)
				break;
			else
				[ms replaceCharactersInRange:r withString:@" "];
			}
		s = ms;
		}

	return s;
	}


//-----------------------------------------------------------------------------
// - stringForNSStringPboardType
//-----------------------------------------------------------------------------
- (NSString*) stringForNSStringPboardType
	{
	NSMutableString* s =
		[[[NSMutableString allocWithZone:[self zone]] init] autorelease];
	unsigned int i, i_lim;
	MiscCoord_V j, j_lim;
	MiscCoord_P row, col;

	if ([self numSelectedRows] > 0)
		{
		NSArray* sel_list =
				[self sortSel:[self selectedRows] border:MISC_ROW_BORDER];
		i_lim = [sel_list count];
		j_lim = (MiscCoord_V) [self numberOfColumns];
		for (i = 0;	 i < i_lim;	 i++)
			{
			row = (MiscCoord_P) [[sel_list objectAtIndex:i] intValue];
			for (j = 0;	 j < j_lim;	 j++)
				{
				if (j > 0) [s appendString:MISC_PB_FIELD_SEPARATOR];
				col = [self colAtPosition:j];
				[s appendString:
					[self stringForNSStringPboardTypeAtRow:row column:col]];
				}
			[s appendString:MISC_PB_RECORD_TERMINATOR];
			}
		}
	else if ([self numSelectedCols] > 0)
		{
		NSArray* sel_list =
				[self sortSel:[self selectedCols] border:MISC_COL_BORDER];
		i_lim = [sel_list count];
		j_lim = (MiscCoord_V) [self numberOfRows];
		for (j = 0;	 j < j_lim;	 j++)
			{
			row = [self rowAtPosition:j];
			for (i = 0;	 i < i_lim;	 i++)
				{
				if (i > 0) [s appendString:MISC_PB_FIELD_SEPARATOR];
				col = (MiscCoord_P) [[sel_list objectAtIndex:i] intValue];
				[s appendString:
					[self stringForNSStringPboardTypeAtRow:row column:col]];
				}
			[s appendString:MISC_PB_RECORD_TERMINATOR];
			}
		}
	return s;
	}


//-----------------------------------------------------------------------------
// - stringForNSTabularTextPBoardType
//-----------------------------------------------------------------------------
- (NSString*) stringForNSTabularTextPBoardType
	{
	return [self stringForNSStringPboardType];
	}


//-----------------------------------------------------------------------------
// - builtinStringForPboardType:
//-----------------------------------------------------------------------------
- (NSString*) builtinStringForPboardType:(NSString*)type
	{
	NSString* s = @"";
	if ([type isEqualToString:NSStringPboardType])
		s = [self stringForNSStringPboardType];
	else if ([type isEqualToString:NSTabularTextPboardType])
		s = [self stringForNSTabularTextPBoardType];
	return s;
	}


//-----------------------------------------------------------------------------
// - stringForPboardType:
//-----------------------------------------------------------------------------
- (NSString*) stringForPboardType:(NSString*)s
	{
	if (delegate != 0 && [delegate respondsToSelector:
		@selector(tableScroll:stringForPboardType:)])
		return [delegate tableScroll:self stringForPboardType:s];

	else if (dataDelegate != 0 && [dataDelegate respondsToSelector:
		@selector(tableScroll:stringForPboardType:)])
		return [dataDelegate tableScroll:self stringForPboardType:s];

	else
		return [self builtinStringForPboardType:s];
	}


//-----------------------------------------------------------------------------
// - builtinWriteSelectionToPasteboard:types:
//-----------------------------------------------------------------------------
- (BOOL) builtinWriteSelectionToPasteboard:(NSPasteboard*)pboard
	types:(NSArray*)original_types
	{
	BOOL result = NO;
	NSMutableArray* types = [NSMutableArray array];

	if (original_types != 0)
		{
		for (unsigned int i = 0, lim = [original_types count]; i < lim; i++)
			{
			id t = [original_types objectAtIndex:i];
			if ([self canWritePboardType:t])
				[types addObject:t];
			}
		}

	unsigned int const nTypes = [types count];
	if (nTypes > 0 && ([self hasRowSelection] || [self hasColSelection]))
		{
		[pboard declareTypes:types owner:0];

		for (unsigned int i = 0;  i < nTypes;  i++)
			{
			NSString* s = [types objectAtIndex:i];
			[pboard setString:[self stringForPboardType:s] forType:s];
			}

		result = YES;
		}

	return result;
	}


//-----------------------------------------------------------------------------
// - writeSelectionToPasteboard:types:
//-----------------------------------------------------------------------------
- (BOOL) writeSelectionToPasteboard:(NSPasteboard*)pboard types:(NSArray*)types
	{
	if (delegate != 0 && [delegate respondsToSelector:
		@selector(tableScroll:writeSelectionToPasteboard:types:)])
		return [delegate tableScroll:self
				writeSelectionToPasteboard:pboard types:types];

	if (dataDelegate != 0 && [dataDelegate respondsToSelector:
		@selector(tableScroll:writeSelectionToPasteboard:types:)])
		return [dataDelegate tableScroll:self
				writeSelectionToPasteboard:pboard types:types];

	return [self builtinWriteSelectionToPasteboard:pboard types:types];
	}


//-----------------------------------------------------------------------------
// - copy:
//-----------------------------------------------------------------------------
- (void) copy:(id)sender
	{
	NSArray* types = [NSArray arrayWithObjects:
						NSTabularTextPboardType, NSStringPboardType, 0];
	[self writeSelectionToPasteboard:[NSPasteboard generalPasteboard]
						types:types];
	}



//-----------------------------------------------------------------------------
// - cut:
//-----------------------------------------------------------------------------
- (void) cut:(id)sender
	{
	[self copy:sender];
	}

@end
