#ifndef __MiscTableView_h
#define __MiscTableView_h
//=============================================================================
//
//		Copyright (C) 1995, 1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscTableView.h
//
//		General-purpose 2-D display object that works with the
//		MiscTableScroll to provide row/column sizing and dragging.
//
//		This object is responsible for drawing, mouse and keyboard
//		events in the content portion of the display.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableView.h,v 1.5 96/05/07 09:49:45 sunshine Exp $
// $Log:		MiscTableView.h,v $
//	Revision 1.5  96/05/07	09:49:45  sunshine
//	Ditched -reflectSelection in favor of -selectionChanged:.
//	
//	Revision 1.4  96/05/06	04:36:00  sunshine
//	Moved keyDown: from MiscTableView to MiscTableScroll.
//	
//	Revision 1.3  96/04/30	05:40:07  sunshine
//	Ported to OpenStep 4.0 for Mach PR2.
//-----------------------------------------------------------------------------
extern "Objective-C" {
#import <AppKit/NSView.h>
}
#import <misckit/MiscTableTypes.h>

@class MiscMouseTracker, MiscTableFocus, MiscTableScroll, NSCell;
class MiscSparseSet;
class MiscTableBorder;

@interface MiscTableView : NSView
	{
	MiscTableScroll*	scroll;
	MiscTableBorder*	col_border;
	MiscTableBorder*	row_border;
	MiscBorderType		tracker_border;
	MiscMouseTracker*	tracker;
	MiscSparseSet*		oldColSel;
	MiscSparseSet*		oldRowSel;
	MiscTableFocus*		focuser;
	}

- (id) initWithFrame:(NSRect)r scroll:(MiscTableScroll*)s
	colInfo:(MiscTableBorder*)col_border rowInfo:(MiscTableBorder*)row_border;
- (void) adjustSize;

- (void) drawCellAtRow:(int)row column:(int)col;		// Physical coords
- (void) drawRow:(int)row;								// Physical coord
- (void) drawCol:(int)col;								// Physical coord

- (void) scrollCellToVisibleAtRow:(int)row column:(int)col; // Physical coords
- (void) scrollRowToVisible:(int)row;					// Physical coord
- (void) scrollColToVisible:(int)col;					// Physical coord

- (int) numVisibleSlots:(MiscBorderType)b;				// all physical coords.
- (int) firstVisibleSlot:(MiscBorderType)b;
- (int) lastVisibleSlot:(MiscBorderType)b;
- (BOOL) border:(MiscBorderType)b slotIsVisible:(int)n;
- (void) border:(MiscBorderType)b setFirstVisibleSlot:(int)n;
- (void) border:(MiscBorderType)b setLastVisibleSlot:(int)n;

- (void) setSelectionMode:(MiscSelectionMode)mode;
- (void) selectionChanged;
- (void) reflectCursor;
- (void) trackBy:(MiscBorderType)b;
- (MiscBorderType) trackingBy;

- (void) mouseDown:(NSEvent*)event;
- (void) keyboardSelect:(NSEvent*)p;
- (void) moveFocusBy:(int)delta;

@end

#endif // __MiscTableView_h
