
/*
   MiscController.h
   Authors: Todd Thomas and Dave Slotnick
  	
   $Id:$
*/

// Headers
#import <Foundation/NSObject.h>

// Forward class declarations
@class NSDictionary;
@class NSString;
@class NSNotification;
@class MiscControllerManager;

// Global constants
extern NSString* MiscControllerDidEditNotification;


@interface MiscController : NSObject
{
#if HACK_FOR_IB_OUTLETS
    id controllerManager;
#endif
    
@private
    MiscControllerManager* _controllerManager;	
    NSString* _name;	/*" For named managers "*/
    BOOL _dirty;		/*" Are we currently edited? "*/
}

/*" Initialization/deallocation "*/
- (id) init;
- (void) dealloc;

/*" Our manager "*/
- (MiscControllerManager*) controllerManager;
- (void) setControllerManager:(MiscControllerManager*)aManager;

/*" Our state "*/
- (BOOL) isDirty;
- (void) setDirty:(BOOL)nowDirty;
- (BOOL) isEditable;
- (void) setEditable:(BOOL)nowEditable;

/*" Notification sender "*/
- (void) didEdit;
- (void) didEditWithUserInfo:(NSDictionary*)userInfo;

/*" Edit notification "*/
- (void) controllerDidEdit:(NSNotification*)notification;

/*" Subclass responsibility "*/
- (BOOL) save;
- (BOOL) revert;

/*" Named controllers "*/
- (NSString*) name;
- (void) setName:(NSString*)newName;

@end


/*
   Any headers from here on are protected and should only be used by
   subclasses. They should be considered more volitile than the
   public API.
*/


@interface MiscController (Protected)

/*" UI initialization hook "*/
- (void) _initializeUI;

/*" Notification dispatch "*/
- (void) _postControllerDidEditNotificationWithUserInfo:(NSDictionary*)userInfo;

/*" Notification registration "*/
- (void) _registerForControllerDidEditNotification:(MiscController*)aController;
- (void) _unregisterForControllerDidEditNotification:(MiscController*)aController;

@end


@interface MiscController (NibInitialization)

- (void) awakeFromNib;

@end

