
/*
   MiscControllerManager.h
   Authors: Todd Thomas and Dave Slotnick

  $Id:$
*/

#import "MiscWindowManager.h"

// Forward class declarations
@class NSMutableArray;
@class NSArray;
@class NSEnumerator;
@class MiscController;


@interface MiscControllerManager : MiscWindowManager
{
@private
    NSMutableArray* _controllers; 	 /*" Our controllers "*/
}

/*" Initialization / deallocation "*/
- (id) init;
- (void) dealloc;

/*" Accessing the managed controllers "*/
- (void) addController:(MiscController*)newController;
- (void) removeController:(MiscController*)oldController;
- (NSArray*) controllers;
- (NSEnumerator*) controllerEnumerator;

/*" Controllers by class "*/
- (NSArray*) controllersOfClass:(Class)aClass;
- (MiscController*) controllerOfClass:(Class)aClass;

/*" Controller by name "*/
- (NSArray*) controllersWithName:(NSString*)name;
- (MiscController*) controllerWithName:(NSString*)name;

/*" Updating the document status "*/
- (void) updateDocumentEdited;

/*" Save/revert "*/
- (BOOL) save;
- (BOOL) revert;

/*" Our editability "*/
- (BOOL) isEditable;
- (void) setEditable:(BOOL)nowEditable;

@end


/*
   Any headers from here on are protected and should only be used by
   subclasses. They should be considered more volitile than the
   public API.
*/


@interface MiscControllerManager (Protected)

/*" Protected accessors "*/
- (NSMutableArray*) mutableControllers;

/*" Extended methods "*/
- (void) _initializeUI;

@end

