
/*
   MiscNibManager.h
   Authors: Todd Thomas and Dave Slotnick
  
   $Id:$
*/


// Headers
#import <Foundation/NSObject.h>

// Forward class declarations
@class NSWindow;
@class NSMutableArray;
@class NSArray;
@class NSString;
@class NSEnumerator;
@class MiscWindowManager;

// Global constants
extern NSString* MiscNibManagerWillReleaseNotification;


@interface MiscNibManager : NSObject
{
@private
    NSMutableArray* _windowManagers;    	/*" Our window managers "*/
    BOOL _releasedWhenAllWindowManagersReleased;
}

/*" Initialization / deallocation "*/
- (id) init;
- (id) initWithNibFilename:(NSString*)nibFilename;
- (id) initWithBundle:(NSBundle*)aBundle;
- (void) dealloc;

/*" Our nib to load "*/
- (NSString*) nibFilename;

/*" Window manager control "*/
- (void) addWindowManager:(MiscWindowManager*)newManager;
- (void) removeWindowManager:(MiscWindowManager*)oldManager;
- (NSArray*) windowManagers;
- (NSEnumerator*) windowManagerEnumerator;

/*" Our window manager access "*/
- (NSArray*) windowManagersWithName:(NSString*)name;
- (MiscWindowManager*) windowManagerWithName:(NSString*)name;
- (NSArray*) windowManagersOfClass:(Class)aClass;
- (MiscWindowManager*) windowManagerOfClass:(Class)aClass;

/*" Releasing "*/ 
- (BOOL) isReleasedWhenAllWindowManagersReleased;
- (void) setReleasedWhenAllWindowManagersReleased:(BOOL)freeUs;
 
@end


@interface MiscNibManager (MiscWindowManagerNotifications)

- (void) windowManagerWillClose:(MiscWindowManager*)closingWindowManager;

@end


/*
   Any headers from here on are protected and should only be used by
   subclasses. They should be considered more volitile than the
   public API.
*/


@interface MiscNibManager (Protected)

/*" Protected accessors "*/
- (NSMutableArray*) mutableWindowManagers;

/*" UI hook "*/
- (void) _initializeUI;

@end


@interface MiscNibManager (NibNotification)

- (void) awakeFromNib;

@end

