/**************************************************************************
 * CLASS:		MiscImageDragView
 * Copyright (C) 1995 Robert Todd Thomas
 * Use is governed by the MiscKit license
 *
 *	See the header file for more information on this class.
 **************************************************************************/

#import <AppKit/NSPasteboard.h> 
#import "MiscImageDragView.h"

#define MISC_IMAGEDRAGVIEW_VERSION 1


@implementation MiscImageDragView

+ (void)initialize
{
	// Set the versioning for archiving.
	if (self == [MiscImageDragView class])
		[self setVersion: MISC_IMAGEDRAGVIEW_VERSION];
	
	return;
}



// Register to accept dragging, and setup some defaults.

- initWithFrame:(NSRect)frameRect
{	
	[super initWithFrame:frameRect];
	[self setDragIconRepresentation: YES];

	return self;
}



- initDragTypes
{
  const char *const types[] = { [NSPostScriptPboardType cString], [NSTIFFPboardType cString], 	
#error StreamConversion: NSFilenamesPboardType used to be NXFilenamePboardType. Pasteboard data of type NSFilenamesPboardType will be an NSArray of NSString. Use 'setPropertyList:forType:' and 'propertyListForType:'
#error StreamConversion: NSFilenamesPboardType used to be NXFilenamePboardType. Pasteboard data of type NSFilenamesPboardType will be an NSArray of NSString. Use 'setPropertyList:forType:' and 'propertyListForType:'
  								[NSFilenamesPboardType cString] };

#error StringConversion: Dragged types are now stored in an NSArray of NSStrings (used to use char **).  Change your variable declaration.
	[self registerForDraggedTypes:(const char *const *)&types];
	return self;
}



//  Sets whether the TIFF itself or its TIFF icon representation is dragged.
 
- setDragIconRepresentation: (BOOL)aBool
{
	dragIconRepresentation = aBool;
	return self;
}



- (BOOL)dragIconRepresentation
{
	return dragIconRepresentation;
}



// Put the image data on the NXTIFFPboardType. Also set the image that is
// going to be used for the drag, depending upon whether 
// dragIconRepresentation is true.

- (BOOL)setupForSourceDrag
{
  id  dragPB = [self draggingPasteboard];
  NXStream  *imageStream = NXOpenMemory (NULL, 0, NX_WRITEONLY);
  const char *const types[] = { [NSTIFFPboardType cString] };
	
	// write the NXImage in TIFF form (it's original size)
	
	[dragPB declareTypes:[NSArray arrayWithObject:[NSString stringWithCString:*types]] owner:self];
	
	// put the image back to regular size and write it
	
	[[self image] setSize:*imageSize];
	imageStream = [[self image] TIFFRepresentation];
#error StreamConversion: 'setData:forType:' (used to be 'writeType:fromStream:') takes an NSData instance as its first argument (used to take NXStream).
	[dragPB setData:imageStream forType:NSTIFFPboardType];

	NXCloseMemory (imageStream, NX_FREEBUFFER);		

	// if use icon rep. grab the image from somewhere that everyone has
	
	if ([self dragIconRepresentation])
		[self setDragImage:  [[NSImage alloc] initByReferencingFile:@"/NextApps/Preview.app/tiff.tiff"] freeWhenDone: YES];	
	else
		[self setDragImage: [self image] ];	

	return YES;
}



// Adjust the dragPoint so that you grab the image in the center.

- calculateDragPoint: (NSPoint *)dragPoint andOffset: (NSPoint *)offset
{
	if ([self dragIconRepresentation])
	{
		dragPoint->x -= 24.0;
		dragPoint->y -= 24.0;
	 }
	else
	{
	  NSSize  imSize;
	  
	  	imSize = [[self image] size];
		dragPoint->x -= imSize.width / 2.0;
		dragPoint->y -= imSize.height / 2.0;
	 }
	 
	return self;
}



// This sure is a complicated method. Actually NXImage takes care
// of everything. It checks for filenames, correct extensions, data
// on the correct pasteboards, and even accepts non native images if 
// you have the filters.
		
- (BOOL)prepareForDragOperation: sender
{
	if ([NSImage canInitWithPasteboard:[sender draggingPasteboard]])
	    return YES;

	[self cleanupAfterDestinationDrag];
	[self display];
	
	return NO;
}



// Take the pasteboard data and create the image.

- (BOOL)performDragOperation: sender
{
	[self setImage:[[NSImage alloc] initWithPasteboard:[sender draggingPasteboard]]];

	return YES;
}



// Archiving methods

- (id)initWithCoder:(NSCoder *)aDecoder
{
	int  version;
	
	[super initWithCoder:aDecoder];
	
	version = [aDecoder versionForClassName:@"MiscImageDragView"];
	switch (version)
	{
		case 0:
		break;
		
		case MISC_IMAGEDRAGVIEW_VERSION:
			[aDecoder decodeValueOfObjCType:"c" at:&dragIconRepresentation];
		break;
	 }
	 	
	return self;
}



- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];
	[aCoder encodeValueOfObjCType:"c" at:&dragIconRepresentation];
}



- (NSString *)stringValue
{	// we don't currently remember the filename, so we don't
	// know where the image came from...
    return [super stringValue];
}



- (void)setStringValue:(NSString *)aValue
{
    return [self setImageByFilename:[aValue cString]];
}

@end


/*****************************************************************************
  CHANGES:
  	Version 0.4, November 30, 1994 (Todd)
		1) Just made it compatible with the changes made in MiscDragView and
			added archive versioning.

 *****************************************************************************/
 