# lib.make
# by Mike Ferris  (original by Art Isbell - art@cubisol.com)
# Part of MOKit
# Copyright 1993, all rights reserved

# About lib.make
#
# This file defines several make targets that extend the Tool project 
# type in ProjectBuilder to support the building of libraries.  This 
# makefile leverages heavily off the Next makefile architecture.
#
# This file is my modifications to the original file by Art Isbell.

include $(LOCALMAKEDIR)/version.make

version:: $(SYM_DIR) $(OFILE_DIR)
	@(echo "$(NAME) Library Version is $(LIB_VERSION)." ; \
	$(LOCALMAKEDIR)/buildversion $(SYM_DIR)/version.c "$(MAJOR_PROJECT_NAME)" "$(VERSION)" "$(RELEASE)" "$(NAME)" "$(LIB_VERSION)" "$(ALL_CFLAGS)" ; \
	$(CC) $(ALL_CFLAGS) -c $(SYM_DIR)/version.c -o $(OFILE_DIR)/version.o)

LIBTOOL = /bin/libtool -o
# LIBTOOL = /bin/ar r

lib::
	@($(MKDIRS) $(SYMROOT) ; \
 	$(process_target_archs)_obj ; \
	if [ "$(ENGLISH)" = "YES" -o "$(JAPANESE)" != "YES" ] ; then \
		language_cflags="" ; \
	else \
		language_cflags="-DKANJI" ; \
	fi ; \
	$(MAKE) libproject \
		"PRODUCT_ROOT = $(SYMROOT)" \
		"LIBPRODUCT = $(LOCALLIBDIR)/lib$(NAME).a" \
		"OFILE_DIR = $(OBJROOT)/$$obj_dir" \
		"PROJ_CFLAGS = $(NORMAL_CFLAGS) $$language_cflags" \
		"APP_MAKEFILE_DIR = $(APP_MAKEFILE_DIR)" \
		"RC_CFLAGS = $$arch_cflags" \
		"RC_ARCHS = $$archs" \
		"SRCROOT = $(SRCROOT)" \
		"OBJROOT = $(OBJROOT)" \
		"SYMROOT = $(SYMROOT)/sym" \
		"SYM_DIR = $(SYMROOT)/sym" )

debuglib::
	@($(MKDIRS) $(SYMROOT) ; \
 	$(process_target_archs)_debug_obj ; \
	if [ "$(ENGLISH)" = "YES" -o "$(JAPANESE)" != "YES" ] ; then \
		language_cflags="" ; \
	else \
		language_cflags="-DKANJI" ; \
	fi ; \
	$(MAKE) libproject \
		"PRODUCT_ROOT = $(SYMROOT)" \
		"LIBPRODUCT = $(LOCALLIBDIR)/lib$(NAME)_g.a" \
		"OFILE_DIR = $(OBJROOT)/$$obj_dir" \
		"PROJ_CFLAGS = $(DEBUG_CFLAGS) $$language_cflags" \
		"APP_MAKEFILE_DIR = $(APP_MAKEFILE_DIR)" \
		"RC_CFLAGS = $$arch_cflags" \
		"RC_ARCHS = $$archs" \
		"SRCROOT = $(SRCROOT)" \
		"OBJROOT = $(OBJROOT)" \
		"SYMROOT = $(SYMROOT)/sym" \
		"SYM_DIR = $(SYMROOT)/sym" )

profilelib::
	@($(MKDIRS) $(SYMROOT) ; \
 	$(process_target_archs)_profile_obj ; \
	if [ "$(ENGLISH)" = "YES" -o "$(JAPANESE)" != "YES" ] ; then \
		language_cflags="" ; \
	else \
		language_cflags="-DKANJI" ; \
	fi ; \
	$(MAKE) libproject \
		"PRODUCT_ROOT = $(SYMROOT)" \
		"LIBPRODUCT = $(LOCALLIBDIR)/lib$(NAME)_p.a" \
		"OFILE_DIR = $(OBJROOT)/$$obj_dir" \
		"PROJ_CFLAGS = $(PROFILE_CFLAGS) $$language_cflags" \
		"APP_MAKEFILE_DIR = $(APP_MAKEFILE_DIR)" \
		"RC_CFLAGS = $$arch_cflags" \
		"RC_ARCHS = $$archs" \
		"SRCROOT = $(SRCROOT)" \
		"OBJROOT = $(OBJROOT)" \
		"SYMROOT = $(SYMROOT)/sym" \
		"SYM_DIR = $(SYMROOT)/sym" )

libproject:: $(INITIAL_TARGETS) toplevel_subcomponents resources $(LIBPRODUCT)

$(LOCALLIBDIR)/lib$(NAME).a: $(PRODUCT_DEPENDS)
	@(if [ "`$(ECHO) $(OFILES) $(OTHER_OFILES) | wc -w`" != "       0" ] ; \
	then \
		$(STRIP) -x $(OFILES) $(OTHER_OFILES); \
		cmd="$(LIBTOOL) $(LIBPRODUCT) $(OFILES) $(OTHER_OFILES)"; \
		echo $$cmd; \
		$$cmd; \
	fi	)

$(LOCALLIBDIR)/lib$(NAME)_g.a : $(PRODUCT_DEPENDS)
	@(if [ "`$(ECHO) $(OFILES) $(OTHER_OFILES) | wc -w`" != "       0" ] ; \
	then \
		cmd="$(LIBTOOL) $(LIBPRODUCT) $(OFILES) $(OTHER_OFILES)"; \
		echo $$cmd; \
		$$cmd; \
	fi	)

$(LOCALLIBDIR)/lib$(NAME)_p.a: $(PRODUCT_DEPENDS)
	@(if [ "`$(ECHO) $(OFILES) $(OTHER_OFILES) | wc -w`" != "       0" ] ; \
	then \
		cmd="$(LIBTOOL) $(LIBPRODUCT) $(OFILES) $(OTHER_OFILES)"; \
		echo $$cmd; \
		$$cmd; \
	fi	)
