/*************************************************************************
*	Copyright (c) 1994 Stone Design Corp.  All rights reserved. 
*	programmer:	Andrew C. Stone
*	File name:	MiscExplodingMenu.h
*	Date:		May 2 1994 
*	Purpose:	To show and hide all the menus in your application 
* 	This code may be used freely provided you leave this copyright in place
*   This code is licensed under the terms of the MiscKit license.
***************************************************************************/

#import <misckit/MiscExplodingMenu.h>
#import <Foundation/NSBundle.h>
#import <AppKit/NSApplication.h>
#import <AppKit/NSMatrix.h>
#import <AppKit/NSMenuItem.h>

#define EXPLODE_TITLE [NSLocalizedStringFromTable(@"Show Menus", @"MiscKit", "Show Menus menu title to show all of the menus in an app.") cString]

#define IMPLODE_TITLE [NSLocalizedStringFromTable(@"Hide Menus", @"MiscKit", "Hide Menus menu title") cString]

#define SERVICES_TITLE [NSLocalizedStringFromTable(@"Services", @"MiscKit", "Services menu title") cString]

// Menu needs to be alerted when you rip off a menu so that it gets the close
// box

@interface NSMenu(MyMenu)
- setAttached;
@end

@implementation NSMenu(MyMenu)
- setAttached;
{
    [self isAttached] = 1;
    return self;
}
@end


@implementation MiscExplodingMenu

static float largest;

+ sharedInstance
{
    static id sharedInstance = nil;

    if (sharedInstance) {
        return sharedInstance;
    } else {
        NSZone *zone = NSApp ? [NSApp zone] : NSDefaultMallocZone();
	sharedInstance=[[super allocWithZone:zone] init];
    }
    
    return sharedInstance;
}

+ new
{
    return [self sharedInstance];
}

- (void)dealloc
{
    
}

- init
{
    [super init];
    return self;
}

- recursiveExplode:(NSMenu *)menu where:(float *)left :(float *)y not:orig
{
    id list,sub;
    int i,count;
    NSRect frame;
    float wid;
    NSEvent *bogusEvent;
    NSSize sz;
    
    sz = [[NSScreen mainScreen] frame].size;
    
    /* if we get to services menu, stop, else leaves poop */
    if ([[menu title] isEqualToString:[NSString stringWithCString:SERVICES_TITLE]]) return self;
    /* this line is required because NEXT does not translate the services menu.. */
    if ([[menu title] isEqualToString:@"Services"]) return self;
    
    /* first, emulate a move to get the closebox since there is no UI to do so */	
    
    frame = [menu frame];
    wid = frame.size.width;
    
    // this caused more problems than it was worth! so let's add a closebox
    if (menu != [NSApp mainMenu] /* && menu != orig */) {
	[menu setAttached];
	
	[bogusEvent locationInWindow].x = frame.origin.x + 3.;	// must be different
	[bogusEvent locationInWindow].y = frame.origin.y + 3.;	// ditto
#error WindowConversion: windowMoved:bogusEvent is obsolete.  Register for a notification from the window instead
	[menu windowMoved:bogusEvent];
	
	[menu setFrameTopLeftPoint:NSMakePoint(*left, *y)];
	[menu orderFront:self];
    }
    if (wid+*left>sz.width) {
	*left = 0.;
	*y -= (largest);
	[menu setFrameTopLeftPoint:NSMakePoint(*left, *y)];
	*left += wid;
    } else *left += wid;
	   
#warning PopUpConversion: This message should be sent to an NSPopUpButton, but is probably being sent to an NSPopUpList
#warning PopUpConversion: Consider NSPopUpButton methods instead of using itemMatrix to access items in a pop-up list.
    list = [menu itemMatrix];
    count = [[list cells] count];
    
    for (i = 0;i<count;i++) {
	if ([(sub =[list cellAtRow:i column:0]) hasSubmenu]) {
	[self recursiveExplode:[sub  target] where:left :y not:orig]; 
	}
    }
    return self;
}


- recursiveImplode:menu not:parent
{
    id list,sub;
    int i,count;
    float top;
    BOOL isTornOff;
    NSRect r;
    NSSize sz;
    
    r = [menu frame];
    sz = [[NSScreen mainScreen] frame].size;
    
    top = r.origin.y+r.size.height;
    isTornOff =!(((sz.height - top) <1.) || top == sz.height -largest ||
				top == sz.height - 2*largest);
    
#warning PopUpConversion: This message should be sent to an NSPopUpButton, but is probably being sent to an NSPopUpList
#warning PopUpConversion: Consider NSPopUpButton methods instead of using itemMatrix to access items in a pop-up list.
    list = [menu itemMatrix];
    count = [[list cells] count];
    for (i = 0;i<count;i++) {
	if ([(sub =[list cellAtRow:i column:0]) hasSubmenu]) {
	[self recursiveImplode:[sub target] not:parent]; 
	}
    }
    
    /* If it has not been moved or is not NXApp or parent, close it */
    
    if (!isTornOff && menu != [NSApp mainMenu] && menu!=parent) [menu close];
    
    return self;
}


- _explodeMenus:sender
{
    const char *which = [[[sender selectedCell] title] cString];
    if (which) {
	if (!strcmp(which,EXPLODE_TITLE)) {
	    NSSize sz;
	    float x = -1.,y;
	    NSRect r;
	    
	    r = [[NSApp mainMenu] frame];
	    sz = [[NSScreen mainScreen] frame].size;
	    y = sz.height;
	    
	    largest = r.size.height +2.;
	    [sender setTitle:[NSString stringWithCString:IMPLODE_TITLE] atRow:[sender selectedRow] column:0];
	    [self recursiveExplode:(NSMenu *)[NSApp mainMenu] where:&x :&y
		    not:[sender window]];    
	    [[sender window] display];
	}
	else {
	    [self recursiveImplode:[NSApp mainMenu] not:[sender window]];
	    [sender setTitle:[NSString stringWithCString:EXPLODE_TITLE] atRow:[sender selectedRow] column:0];
	    [[sender window] display];
	}
    }
    return self;
}



/* FINALLY: actual called routine */
/* this delay stuff sort of solves the problem..... */
- explodeMenus:sender 
{
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(_explodeMenus:) object:sender], [self performSelector:@selector(_explodeMenus:) object:sender afterDelay:(.01) / 1000.0];
    return self;
}


@end
