// MiscDocManager.h
//
// by Mike Ferris
// Part of MOKit
// Copyright 1993, all rights reserved.

// ABOUT MOKit
// 
// MOKit is a collection of useful and general objects.  Permission is 
// granted by the author to use MOKit in your own programs in any way 
// you see fit.  All other rights to the kit are reserved by the author 
// including the right to sell these objects as part of a LIBRARY or as 
// SOURCE CODE.  In plain English, I wish to retain rights to these 
// objects as objects, but allow the use of the objects as pieces in a 
// fully functional program.  NO WARRANTY is expressed or implied.  The author 
// will under no circumstances be held responsible for ANY consequences to 
// you from the use of these objects.  Since you don't have to pay for 
// them, and full source is provided, I think this is perfectly fair.

// ABOUT MODocManager
//
// MODocManager manages a bunch of MODocControllers.

#import <AppKit/AppKit.h>

// Forward declarations of class names we'll use for static typing.
@class MiscDocument;

@interface MiscDocManager:NSObject
{
	List *docList;
	MiscDocument *currentDoc;
	List *docClassList;
	
	int untitledCount;
	int frameCycle;
	NSPoint windowStartingLocation;
	
	// quit panel outlets
	id	quitPanel;
	id	buttonMatrix;
	
	// menu outlets
	id	docMenu;
	id	openCell;
	id	newCell;
	id	saveCell;
	id	saveAsCell;
	id	saveToCell;
	id	saveAllCell;
	id	revertCell;
	id	closeCell;
	
	NSMenu *newMenu;
	NSMenuItem *newSubmenuCell;
	NSMenu *openMenu;
	NSMenuItem *openSubmenuCell;
}

+ (void)initialize;

- init;
- initDocumentClass:(Class)docControllerClass;
- (void)dealloc;
- (void)awakeFromNib;

- new:sender;
- newFromDocumentClass:sender;
- open:sender;
- openFromDocumentClass:sender;
- openDocument:(const char *)path withClass:(Class)docClass;

- save:sender;
- saveAs:sender;
#warning ColorConversion: 'writeToFile:' was 'saveTo:' ; conversion should only apply to NSColorList ; 'writeToFile:' returns BOOL rather than raising
- (BOOL)writeToFile:(id)sender;
- saveAll:sender;
- revert:sender;
- close:sender;

#warning PrintingConversion:  printPSCode: has been renamed to print:.  Rename this method?
- (void)print:(id)sender;

- getNextWindowLocation:(NSPoint *)pt;
- (int)nextUntitledNum;
- setWindowStartingLocation:(const NSPoint *)pt;

- addDocument:(MiscDocument *)aDocument;
- removeDocument:(MiscDocument *)aDocument;
- (MiscDocument *)findDocumentForWindow:aWindow;
- (List *)documentList;
- makeDocumentsPerform:(SEL)aMethod with:anArg;

- (BOOL)areDocumentsDirty;

- setCurrentDocument:(MiscDocument *)aDocument;
- currentDocument;

- addDocumentClass:(Class)docControllerClass;
- removeDocumentClass:(Class)docControllerClass;
- (List *)documentClassList;
- (int)docClassCount;

- (BOOL)applicationShouldTerminate:(id)sender;
- quitPanelStopModalAction:sender;

#warning NotificationConversion: windowDidBecomeKey:(NSNotification *)notification is an NSWindow notification method (used to be a delegate method); delegates of NSWindow are automatically set to observe this notification; subclasses of NSWindow do not automatically receive this notification
- (void)windowDidBecomeKey:(NSNotification *)notification;
#warning NotificationConversion: windowDidResignKey:(NSNotification *)notification is an NSWindow notification method (used to be a delegate method); delegates of NSWindow are automatically set to observe this notification; subclasses of NSWindow do not automatically receive this notification
- (void)windowDidResignKey:(NSNotification *)notification;

- (BOOL)menuUpdate:menuCell;

#error ArchiverConversion: put the contents of your 'awake' method at the end of your 'initWithCoder:' method instead
- awake;
- (id)initWithCoder:(NSCoder *)aDecoder;
- (void)encodeWithCoder:(NSCoder *)aCoder;

@end

