//
//	MiscDocType.h -- encapsulating class for document info
//		Written by Mike Ferris Copyright (c) 1993, 1995 by Mike Ferris.
//		Modified for use in the MiscKit by Don Yacktman from MODocType.
//				Version 1.0.  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

// ABOUT MiscDocType
//
// MiscDocType is small data-class used by MiscDocController.  It stores
// a document tag, a name, a file extension, and two selectors pointing
// to the revert(open) and save methods to be called in the controller
// for the given type.

#import <AppKit/AppKit.h>

// Forward declarations of class names we'll use for static typing.
@class MiscString;

@interface MiscDocType:NSObject
{
	Class controllerClass;
	int typeTag;
	MiscString *typeName;
	MiscString *typeExtension;
	SEL openSelector;
	SEL saveSelector;
}

+ (void)initialize;

- init;
- initForControllerClass:(Class)controller tag:(int)tag 
			name:(const char *)name extension:(const char *)ext 
			openSelector:(SEL)openSel saveSelector:(SEL)saveSel;
- (void)dealloc;

- (Class)controllerClass;
- (int)typeTag;
- (const char *)typeName;
- (const char *)typeExtension;
- (BOOL)canOpenType;
- (SEL)openSelector;
- (BOOL)canSaveType;
- (SEL)saveSelector;

- (id)initWithCoder:(NSCoder *)aDecoder;
- (void)encodeWithCoder:(NSCoder *)aCoder;

@end

