/************************************************************************
  CATEGORY:			MiscFile (Info)
  PROGRAMMER:		Todd Thomas, Copyright 1994, 1995.
  BEGAN:			December 22, 1994
  LAST CHANGED:		January 25, 1995
  CHANGES:			-
  VERSION:			0.3
  DECLARED IN:		<misckit/MiscFile.h>
  
  Adds the ability to get some useful information about the MiscFile.
  If you are more familiar with standard UNIX calls, check out the
  MiscFile+Unix category. Most all the methods use either -stat, -lstat
  or -access for their information.

  This object is included in the MiscKit by permission from the author
  and its use is governed by the MiscKit license, found in the file
  "LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
  for a list of all applicable permissions and restrictions.
       
 *************************************************************************/

@class MiscUser;
@class MiscUserGroup;
@class MiscTime;
@class NSImage;


@interface MiscFile (Info)

// Asking a MiscFile about it's access rights.
- (BOOL)hasPermissions: (int)perms for: (int)who;

// Some more inuitive method names that use stat(2) or lstat(2), depending
// upon the value of +(BOOL)useLinkInfo.
- (BOOL)isSymbolicLink;
- (BOOL)isDirectory;
- (dev_t)device;
- (ino_t)inode;
- (u_short)permissions;
- (short)hardLinks;
- (MiscUser *)owner;
- (MiscUserGroup *)group;
- (dev_t)deviceType;
- (off_t)size;
- (long)blocksAllocated;
- (long)optimalBlockSize;

- (MiscTime *)lastAccessTime;
- (MiscTime *)lastModifyTime;
- (MiscTime *)lastStatusChangeTime;

// Some methods to extract info about the MiscFile (uses access(2))
- (BOOL)readAccess;
- (BOOL)writeAccess;
- (BOOL)executeAccess;
- (BOOL)doesExist;

// This will likely be moved to some AppKit dependent category someday.
- (NSImage *)iconImage;

@end

