//
//	MiscListExtensions.h -- various extensions to List
//		Originally written by Drew Davidson and Don Yacktman
//		Copyright (c) 1994 by Drew Davidson.
//		Modified and extended by Don Yacktman for inclusion into the MiscKit.
//				Version 1.0.  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

@interface List(Extensions)

- deepCopy;
- deepCopyFromZone:(NSZone *)zone;
- makeObjectsPerform:(SEL)aSelector with:object1 with:object2;
- makeObjectsPerformInOrder:(SEL)aSelector;
- makeObjectsPerformInOrder:(SEL)aSelector with:anObject;
- makeObjectsPerformInOrder:(SEL)aSelector with:object1 with:object2;
- addLists:firstList, ...;
- mergeLists:firstList, ...;

@end

@interface NSObject(MiscListForeach)

// Make self perform aSelector once for each object in a List.  The
// object from the List is sent as the first argument to aSelector,
// so aSelector should take a single argument of type id.
- foreach:aList perform:(SEL)aSelector;

// As above, but aSelector should take two arguments of type id.
- foreach:aList perform:(SEL)aSelector with:anObject;

@end

@interface List (MiscStringSort)

/* Assumes everything in the list is a MiscString -- no check is made.
 * This category is written by Bill Bumgarner and (C) Bill Bumgarner.
 */
- miscStringSort;
- miscStringSortCaseSensitive:(BOOL)aFlag; // added by DAY

@end



// This next category is from R. Todd Thomas and was implemented to
// help support the MiscUser* classes.

@interface List (StringFinding)

- (BOOL)matchString: (const char *)str usingSel: (SEL)comparator;

@end

