//
//	MiscMergeDriver.h -- a simple loop for driving bulk merges
//		Written by Don Yacktman Copyright (c) 1995 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <AppKit/AppKit.h>
#import <misckit/MiscMergeProtocols.h>

@class MiscMergeCommand;
@class MiscMergeEngine;
@class MiscMergeTemplate;
@class HashTable;

@interface MiscMergeDriver : NSObject <MiscMergeDriver>
{
	MiscMergeTemplate *template; /*" MiscMergeTemplate for merging "*/
	List *dictionaries; /*" List of MiscDictionaries used for merges "*/
	List *output; /*" The output list that will be returned by -#{doMerge:} "*/
	MiscMergeEngine *engine; /*" The merge engine to be used for merges "*/
	BOOL merging; /*" YES if merging, NO if not "*/

	int _mergeLoopIndex; /*" Index to #{dictionaries} when merge is in progress "*/
}

// The three basic steps to a successful merge:
/*" Accessing the template "*/
- (MiscMergeTemplate *)template;
- setTemplate:(MiscMergeTemplate *)aTemplate;

/*" Accessing the data "*/
- (List *)mergeData;
- setMergeData:(List *)aList;

/*" Performing a merge "*/
- (List *)doMerge:sender;

/*" Accessing the engine "*/
- (MiscMergeEngine *)engine;
- setEngine:(MiscMergeEngine *)anEngine;

@end