/* MiscSwapViewByPopUp.h				 
 *
 * This is a SwapView that can handle popUps either by tag or by title.
 * If a tag is not 0 it will try to choose the viewController by title.
 *
 * Usage: Like the normal swapView but we have to set the popUpCover
 *		  to ensure that there will be a swapAction anywhere.
 *
 * Notes: The popUps could be handled as a matrix object too but there are some
 *		  problems with that when the popUp appears for the first time.
 *        
 *		  We compare tags by default first. So if you work with multiple NIBs
 *		  you must use a tag for every cell (to make it work at all) 
 *		  because there is no way in IB to make inter NIB connections. This 
 *		  will never be possible anyway. Not NeXTs fault. It's a limitation
 *		  of the NIB concept. (OneNIB to MultiNIB problem)
 *
 * Written by: 		Thomas Engel
 * Created:    		24.01.1994 (Copyright 1994 Thomas Engel)
 * Last modified: 	24.02.1994
 */

//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.


#import <misckit/MiscSwapView.h>

@interface MiscSwapViewByPopUp:MiscSwapView
{
	id		popUpCover;
}

+ (void)initialize;
- setPopUpCover:anObject;
- popUpCover;

// To handle the popUps we need a new method that can find the right popUpItem
// by its title and do the comparison...by tag or item.
  
- swapContentView:sender;
- realTrigger;

- (id)initWithCoder:(NSCoder *)aDecoder;
- (void)encodeWithCoder:(NSCoder *)aCoder;

@end

/*
 * History: 24.02.94 Made it a direkt subclass of SwapView.
 *
 *			24.01.94 Made it MiscSwap conform.
 *
 *			08.01.94 Conplete redesign and adding of whatPanelByObject/Tag etc.
 *
 *			21.12.93 First subclass of the swapController. whatPanel was im-
 *					 proved to handle tags!
 *
 *
 * Bugs: - does only handle real popUps. PullDowns might not realy work 
 *		   because I always set the title inside the trigger method. Hmm
 *		   Maybe its better to write a seperate pullDown Controller!
 */