// Copyright (C) 1995 Jon Kutemeier
// Use is governed by the MiscKit license

/******************************************************************************
 *      $Log: DateSelectionCell.m,v $
# Revision 1.1  1993/10/08  22:04:14  jkute
# Checkpoint
#
# Revision 1.1  1993/04/13  18:02:34  gpopick
# done
#
# Revision 1.1  1993/01/29  19:45:21  jkute
# *** empty log message ***
#
# Revision 1.1  1992/11/17  20:48:22  jkute
# Initial revision
#
******************************************************************************/
static char *ident = "$Id: DateSelectionCell.m,v 1.1 1993/10/08 22:04:14 jkute Exp $";
static int __hack(int x){if(x)return x;else return __hack((int)*ident);}
/*****************************************************************************/

#import <strings.h>

#import <AppKit/psops.h>
#import <AppKit/NSGraphics.h>
#import <AppKit/NSColor.h>
#import <AppKit/NSFont.h>
#import <AppKit/NSText.h>

#import "DateSelectionCell.h"

#define COLOR_DKBLUE	[NSColor colorWithCalibratedRed:34.0/255.0 green:68.0/255.0 blue:119.0/255.0 alpha:1.0]

@implementation DateSelectionCell

- init
{
    id retCode;

    retCode = [super init];

    newCellFont = [NSFont fontWithName:@"Helvetica" size:10];
    [NSFont setUserFont:newCellFont];

    [self setTextColor:[NSColor whiteColor]];
    [self setHighlightTextColor:[NSColor blackColor]];
    [self setBackgroundColor:COLOR_DKBLUE];
    [self setHighlightColor:[NSColor whiteColor]];

    [self setDrawRightEdge:NO];
    [self setDrawEntireBottomEdge:YES];

    return retCode;
}

- (void)setTextColor:(NSColor *)aColor
{
    textColor = aColor;
}

- (NSColor *)textColor
{
    return textColor;
}

- setHighlightTextColor:(NSColor *)aColor
{
    highlightTextColor = aColor;

    return self;
}

- (NSColor *)highlightTextColor
{
    return highlightTextColor;
}

- (void)setBackgroundColor:(NSColor *)aColor
{
    backgroundColor = aColor;
}

- (NSColor *)backgroundColor
{
    return backgroundColor;
}

- setHighlightColor:(NSColor *)aColor
{
    highlightColor = aColor;

    return self;
}

- (NSColor *)highlightColor
{
    return highlightColor;
}

- setDrawRightEdge:(BOOL)yn
{
    drawRightEdge = yn;

    return self;
}

- (BOOL)drawRightEdge
{
    return drawRightEdge;
}

- setDrawEntireBottomEdge:(BOOL)yn
{
    drawEntireBottomEdge = yn;

    return self;
}

- (BOOL)drawEntireBottomEdge
{
    return drawEntireBottomEdge;
}

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
    float	yPos, 
		xPos,
		width;
    const char	*stringValue;
    NSRect 	rectArray[2];
    int		numOfRects;

    stringValue = [[self stringValue] cString];

    /* erase the cell */

    [(cFlags1.state || cFlags1.highlighted) ? highlightColor: [self backgroundColor] set];
    NSRectFill(cellFrame);
    
    if (stringValue && *stringValue)
    {
	/* set the font according to our drawing status */

	if (![[NSDPSContext currentContext] isDrawingToScreen])
	  [newCellFont set];
	else
	  [[newCellFont screenFont] set];
    
	/* draw the text */

	[[self textColor] set];

	if ( (cFlags1.state || cFlags1.highlighted) )
	  [[self highlightTextColor] set];

	yPos = NSMinY(cellFrame) + 10.0;// + lineHeight - descender;
	xPos = NSMinX(cellFrame) + ((NSWidth(cellFrame) - 
				   [newCellFont widthOfString:[NSString stringWithCString:stringValue]])/2.0);

	PSmoveto( xPos, yPos );

	PSshow( stringValue );
    }

    [[NSColor blackColor] set];

    numOfRects = 1;

    if (![self drawEntireBottomEdge])
    {
	xPos = NSMinX(cellFrame) + 1.0;
	width = NSWidth(cellFrame) - 1.0;
    }
    else
    {
	xPos = NSMinX(cellFrame);
	width = NSWidth(cellFrame);
    }

    (rectArray[0]) = NSMakeRect(xPos, NSMaxY(cellFrame), width, 1.0);

    if ([self drawRightEdge])
    {
	(rectArray[1]) = NSMakeRect(NSMaxX(cellFrame), NSMinY(cellFrame) + 1.0, 1.0, NSHeight(cellFrame));
	numOfRects++;
    }

    NSRectFillList(rectArray, numOfRects);
}

- (void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
    if (cFlags1.highlighted != flag) 
    {
	cFlags1.highlighted = flag;

	[self drawInteriorWithFrame:cellFrame inView:controlView];
    }
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    float	red,
		green,
		blue;

    [super initWithCoder:aDecoder];

    [aDecoder decodeValuesOfObjCTypes:"fff", &red, &green, &blue];
    textColor = [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0];

    [aDecoder decodeValuesOfObjCTypes:"fff", &red, &green, &blue];
    highlightTextColor = [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0];

    [aDecoder decodeValuesOfObjCTypes:"fff", &red, &green, &blue];
    backgroundColor = [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0];

    [aDecoder decodeValuesOfObjCTypes:"fff", &red, &green, &blue];
    highlightColor = [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0];

    [aDecoder decodeValuesOfObjCTypes:"@cc", &newCellFont, &drawEntireBottomEdge, 
		&drawRightEdge];

    return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
    float	red,
		green,
		blue;

    [super encodeWithCoder:aCoder];

    [[textColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:&red green:&green blue:&blue alpha:0];
    [aCoder encodeValuesOfObjCTypes:"fff", &red, &green, &blue];

    [[highlightTextColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:&red green:&green blue:&blue alpha:0];
    [aCoder encodeValuesOfObjCTypes:"fff", &red, &green, &blue];

    [[backgroundColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:&red green:&green blue:&blue alpha:0];
    [aCoder encodeValuesOfObjCTypes:"fff", &red, &green, &blue];

    [[highlightColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:&red green:&green blue:&blue alpha:0];
    [aCoder encodeValuesOfObjCTypes:"fff", &red, &green, &blue];

    [aCoder encodeValuesOfObjCTypes:"@cc", &newCellFont, &drawEntireBottomEdge,
		 &drawRightEdge];
}

@end

