// Copyright (C) 1995 Jon Kutemeier
// Use is governed by the MiscKit license

#import <AppKit/NSApplication.h>
#import <AppKit/NSFont.h>

#import "misckit/MiscCalendarViewConstants.h"
#import "misckit/DateDelegateProtocol.h"
#import "MiscCalendarMatrix.h"
#import "DateSelectionCell.h"
#import "SimpleDate.h"

@implementation MiscCalendarMatrix

+ (void)initialize
{
    [MiscCalendarMatrix setVersion:1];

    return;
}

- initWithFrame:(NSRect)frameRect
{
    NSSize 	spacing,
		newCellSize;
    id		aCell;

    [super initWithFrame:frameRect mode:NSRadioModeMatrix prototype:[[DateSelectionCell alloc] init] numberOfRows:NUM_MATRIX_ROWS numberOfColumns:NUM_MATRIX_COLS];
    [self setAllowsEmptySelection:YES];

    [self deselectAllCells];

    [NSFont setUserFont:[NSFont fontWithName:@"Helvetica" size:11]];

    [self setFont:[NSFont fontWithName:@"Helvetica" size:11]];

    spacing.height = MATRIX_SPACING_HEIGHT;
    spacing.width = MATRIX_SPACING_WIDTH;

    [self setIntercellSpacing:spacing];

    newCellSize.height = MATRIX_CELL_HEIGHT;
    newCellSize.width = MATRIX_CELL_WIDTH;

    [self setCellSize:newCellSize];

    [self sizeToCells];

    [self sizeBy:1 :1];

    simpleDate = [[SimpleDate alloc] init];

    [self setDateDelegate:simpleDate];

    aCell = [[self cells] objectAt:0];

    cellTextColor = [aCell textColor];
    cellHighlightTextColor = [aCell highlightTextColor];
    cellHighlightColor = [aCell highlightColor];
    cellBackgroundColor = [aCell backgroundColor];

    overwriteCellColors = NO;

    [self displayDate:self];

    return self;
}

- initWithFrame:(NSRect)frameRect mode:(int)aMode cellClass:cellId numberOfRows:(int)numRows numberOfColumns:(int)numCols
{
    return [self initWithFrame:frameRect];
}

- initWithFrame:(NSRect)frameRect mode:(int)aMode prototype:aCell numberOfRows:(int)numRows numberOfColumns:(int)numCols
{
    return [self initWithFrame:frameRect];
}

- (void)dealloc
{
    [simpleDate release];

    { [super dealloc]; return; };
}

- setDateDelegate:aDateObject
{
    if ([aDateObject conformsToProtocol:@protocol(DateDelegate)])
      dateDelegate = aDateObject;

    return self;
}

- dateDelegate
{
    return dateDelegate;
}

- overwriteCellColors:(BOOL)yn
{
    overwriteCellColors = yn;

    return self;
}

- (BOOL)cellColorsOverwritten
{
    return overwriteCellColors;
}

- (void)setCellBackgroundColor:(NSColor *)aColor
{
    NSArray *myCellList;
    id		aCell;
    int		i;

    myCellList = [self cells];

    for (i = 0; i < [myCellList count]; i++)
    {
	aCell = [myCellList objectAtIndex:i];

	if (overwriteCellColors || 
	    [cellBackgroundColor isEqual:[aCell backgroundColor]])
	  [aCell setBackgroundColor:aColor];
    }

    cellBackgroundColor = aColor;
}

- (NSColor *)cellBackgroundColor
{
    return cellBackgroundColor;
}

- setBackgroundColor:(NSColor *)aColor forCellAt:(int)xPos :(int)yPos
{
    [[self cellAtRow:xPos column:yPos] setBackgroundColor:aColor];

    return self;
}

- (NSColor *)backgroundColorForCellAt:(int)xPos :(int)yPos
{
    return [[self cellAtRow:xPos column:yPos] backgroundColor];
}

- setCellHighlightColor:(NSColor *)aColor
{
    NSArray *myCellList;
    id		aCell;
    int		i;

    myCellList = [self cells];

    for (i = 0; i < [myCellList count]; i++)
    {
	aCell = [myCellList objectAtIndex:i];

	if (overwriteCellColors || 
	    [cellHighlightColor isEqual:[aCell highlightColor]])
	  [aCell setHighlightColor:aColor];
    }

    cellHighlightColor = aColor;

    return self;
}

- (NSColor *)cellHighlightColor
{
    return cellHighlightColor;
}

- setHighlightColor:(NSColor *)aColor forCellAt:(int)xPos :(int)yPos
{
    [[self cellAtRow:xPos column:yPos] setHighlightColor:aColor];

    return self;
}

- (NSColor *)highlightColorForCellAt:(int)xPos :(int)yPos
{
    return [[self cellAtRow:xPos column:yPos] highlightColor];
}

- setCellTextColor:(NSColor *)aColor
{
    NSArray *myCellList;
    id		aCell;
    int		i;

    myCellList = [self cells];

    for (i = 0; i < [myCellList count]; i++)
    {
	aCell = [myCellList objectAtIndex:i];
	if (overwriteCellColors || 
	    [cellTextColor isEqual:[aCell textColor]])
	[aCell setTextColor:aColor];
    }

    cellTextColor = aColor;

    return self;
}

- (NSColor *)cellTextColor
{
    return cellTextColor;
}

- setTextColor:(NSColor *)aColor forCellAt:(int)xPos :(int)yPos
{
    [[self cellAtRow:xPos column:yPos] setTextColor:aColor];

    return self;
}

- (NSColor *)textColorForCellAt:(int)xPos :(int)yPos
{
    return [[self cellAtRow:xPos column:yPos] textColor];
}

- setCellHighlightTextColor:(NSColor *)aColor
{
    NSArray *myCellList;
    id		aCell;
    int		i;

    myCellList = [self cells];

    for (i = 0; i < [myCellList count]; i++)
    {
	aCell = [myCellList objectAtIndex:i];

	if (overwriteCellColors || 
	    [cellHighlightTextColor isEqual:[aCell highlightTextColor]])
	  [aCell setHighlightTextColor:aColor];
    }

    cellHighlightTextColor = aColor;

    return self;
}

- (NSColor *)cellHighlightTextColor
{
    return cellHighlightTextColor;
}

- setHighlightTextColor:(NSColor *)aColor forCellAt:(int)xPos :(int)yPos
{
    [[self cellAtRow:xPos column:yPos] setHighlightTextColor:aColor];

    return self;
}

- (NSColor *)highlightTextColorForCellAt:(int)xPos :(int)yPos
{
    return [[self cellAtRow:xPos column:yPos] highlightTextColor];
}

- (void)mouseDown:(NSEvent *)theEvent 
{
    id		theCell = nil,
		previousCell = nil;
    NSEvent *nextEvent;
    NSPoint	mouseLocation;
    BOOL	looping = YES;
    int		row = -1,
		col = -1,
    		previousRow = -1,
    		previousCol = -1,
		oldMask;

    nextEvent = theEvent;

    if ([self mode] == NSIMHighlightModeMatrix)
    {
	/* Save our normal window event mask, and add the mouse dragged */
	/* event mask */

#error EventConversion: addToEventMask:NX_MOUSEDRAGGEDMASK: is obsolete; you no longer need to use the eventMask methods; for mouse moved events, see 'setAcceptsMouseMovedEvents:'
	oldMask = [[self window] addToEventMask:NSLeftMouseDraggedMask];

	while (looping)
	{
	    /* Find the current coordinates of the mouse... */

	    mouseLocation = [nextEvent locationInWindow];

	    /* ... and convert them into our view coordinate system */

	    mouseLocation = [self convertPoint:mouseLocation fromView:nil];

	    /* Find the row and column of the mouse */

	    [self getRow:&row column:&col forPoint:mouseLocation];

	    if ( (row >= 0) && (col >= 0) && ((row != previousRow) ||
					      (col != previousCol)) )
	    {
		theCell = [self cellAtRow:row column:col];

		if ([theCell isEnabled])
		{
		    if (previousCell && (previousCell != theCell))
#warning MatrixAndCellConversion: 'setState:' used to be 'incrementState' ; this conversion is not valid for multistate cells
		      [previousCell setState:![previousCell  state]];

#warning MatrixAndCellConversion: 'setState:' used to be 'incrementState' ; this conversion is not valid for multistate cells
		    [theCell setState:![theCell  state]];

		    [self display];

		    previousCell = theCell;
		}

		previousRow = row;
		previousCol = col;
	    }
	    else if (((row < 0) || (col < 0)) && previousCell && 
		     [previousCell isEnabled])
	    {
#warning MatrixAndCellConversion: 'setState:' used to be 'incrementState' ; this conversion is not valid for multistate cells
		[previousCell setState:![previousCell  state]];
		[self display];
		previousCell = nil;
	    }
		
	    nextEvent = [[self window] nextEventMatchingMask:NSLeftMouseUpMask | 
			 NSLeftMouseDraggedMask];

	    looping = ([nextEvent type] == NSLeftMouseDragged);
	}

#error EventConversion: setEventMask:oldMask: is obsolete; you no longer need to use the eventMask methods; for mouse moved events, see 'setAcceptsMouseMovedEvents:'
	[[self window] setEventMask:oldMask];
    }
    else
      [super mouseDown:theEvent];
}

- displayDate:sender
{
    int	numberOfDays,
	startCell,
    	currentDay = 0,
	x,
	y,
    	day;
    id	dayCell,
	aDate = nil;
    
    [[self window] disableFlushWindow];


    if ((aDate = [self dateDelegate]))
    {
	currentDay = [aDate day];

	/* Get the number of days for the month that the date object */
	/* is set to */

	numberOfDays = [aDate numberOfDaysInMonth];

	/* Get the starting day of the month */

	startCell = [aDate startDayOfMonth];

    }
    else
    {
	currentDay = 1;
	numberOfDays = 31;
	startCell = 4;
    }

    // Clear out the titles of the first row of dayCells
    // from the start of the matrix to the x coordinate
    // that corresponds to the day of the week that the
    // month start on
    
    for (x=0, y=0; x < startCell; x++)
    {
	dayCell = [self cellAtRow:y column:x];

	if (x == 0)
	  [dayCell setDrawEntireBottomEdge:NO];

	[dayCell setStringValue:@""];
	[dayCell setEnabled:NO];
    }
    
    // Fill in the titles of all the dayCells. When x hits
    // the seventh column of the matrix, we wrap around
    // to the first column and continue filling in values
    // on the next row
    
    for (day = 1; day <= numberOfDays; day++, x++)
    {
	if (x == 7)
	{
	    x = 0;
	    y++;
	}

	dayCell = [self cellAtRow:y column:x];

	[dayCell setIntValue:day];
	[dayCell setEnabled:YES];

	if (day == currentDay)
	  [self selectCellAtRow:y column:x];

	if (x == 0)
	  [dayCell setDrawEntireBottomEdge:NO];

	if (x == 6)
	  [dayCell setDrawRightEdge:YES];
    }
    
    // Clear out the titles of the rest of the dayCells
    // from the last day of the month to the end of
    // the matrix. When x hits the seventh column of 
    // the matrix, we wrap around to the first column
    // and continue blanking dayCells on the next row until
    // y hits the seventh row. We are then done with 
    // this matrix
    
    for (; y < 7; x++)
    {
	if (x == 7)
	{
	    x = 0;
	    y++;
	}
	
	dayCell = [self cellAtRow:y column:x];

	if (x == 0)
	  [dayCell setDrawEntireBottomEdge:NO];

	if (x == 6)
	  [dayCell setDrawRightEdge:YES];

	[dayCell setStringValue:@""];
	[dayCell setEnabled:NO];

    }
    
    [self display];

    [[self window] enableFlushWindow];
    [[self window] flushWindowIfNeeded];

    return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    float	red,
		green,
		blue;
    int 	version;

    [super initWithCoder:aDecoder];

    version = [aDecoder versionForClassName:@"MiscCalendarMatrix"];

    [aDecoder decodeValuesOfObjCTypes:"@@", &dateDelegate, &simpleDate];

    [aDecoder decodeValuesOfObjCTypes:"fff", &red, &green, &blue];
    cellBackgroundColor = [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0];

    [aDecoder decodeValuesOfObjCTypes:"fff", &red, &green, &blue];
    cellHighlightColor = [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0];

    [aDecoder decodeValuesOfObjCTypes:"fff", &red, &green, &blue];
    cellTextColor = [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0];

    [aDecoder decodeValuesOfObjCTypes:"fff", &red, &green, &blue];
    cellHighlightTextColor = [NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0];

    if (version == 1)
      [aDecoder decodeValuesOfObjCTypes:"c", &overwriteCellColors];

    return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
    float	red,
		green,
		blue;

    [super encodeWithCoder:aCoder];

    [aCoder encodeValuesOfObjCTypes:"@@", &dateDelegate, &simpleDate];

    [[cellBackgroundColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:&red green:&green blue:&blue alpha:0];
    [aCoder encodeValuesOfObjCTypes:"fff", &red, &green, &blue];

    [[cellHighlightColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:&red green:&green blue:&blue alpha:0];
    [aCoder encodeValuesOfObjCTypes:"fff", &red, &green, &blue];

    [[cellTextColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:&red green:&green blue:&blue alpha:0];
    [aCoder encodeValuesOfObjCTypes:"fff", &red, &green, &blue];

    [[cellHighlightTextColor colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:&red green:&green blue:&blue alpha:0];
    [aCoder encodeValuesOfObjCTypes:"fff", &red, &green, &blue];

    [aCoder encodeValuesOfObjCTypes:"c", &overwriteCellColors];
}

@end
