// Copyright (C) 1995 Jon Kutemeier
// Use is governed by the MiscKit license

#import <AppKit/NSView.h>

#import "misckit/DateDelegateProtocol.h"
#import "misckit/MiscCalendarViewConstants.h"

@class MiscCalendarMatrix;
@class DateSelectionCell;
@class NSTextField;
@class List;
@class NSMatrix;
@class NSBox;
@class NSFont;

@interface MiscCalendarView : NSView
{
    id			monthField;
    id			yearField;
    id			dateField;
    id			dateDelegate;
    id			textDelegate;

    NSRect		defaultFrame;
    MiscCalendarMatrix	*calendarMatrix;
    DateSelectionCell	*previousDayCell;
    NSTextField		*monthSubview,
			*yearSubview,
    			*inlineMonthSubview,
			*inlineYearSubview;
    NSFont		*helvetica_11,
			*system_11_bold,
			*helvetica_12,
     			*system_12_bold;
    List		*dowSubviewList;
    int			titleDisplayMode;
    BOOL		displayDOWHeader,
			displayMonthAndYearHeader,
			displayMonthHeader,
			displayYearHeader,
			colorDragAndDrop;

}

- awakeAfterUsingCoder:(NSCoder *)aDecoder;
- (void)awakeFromNib;

- initWithFrame:(NSRect)frameRect;

- getMinSize:(NSSize *)minSize maxSize:(NSSize *)maxSize from:(int)where;

- calendarMatrix;

- preserveCellColors:(BOOL)yn;
- (BOOL)cellColorsPreserved;

- setColorOf:(int)element to:(NSColor *)aColor forCellAt:(int)xPos :(int)yPos;
- (NSColor *)colorOf:(int)element forCellAt:(int)xPos :(int)yPos;

- setColorOf:(int)element to:(NSColor *)aColor;
- (NSColor *)colorOf:(int)element;

- cellAt:(int)xPos :(int)yPos lock:(BOOL)yn;
- forRow:(int)row lock:(BOOL)yn;
- forColumn:(int)col lock:(BOOL)yn;

- setHighlightMode:(int)mode;
- (int)highlightMode;

- allowColorDragAndDrop:(BOOL)yn;
- (BOOL)canDragAndDropColor;

- forHeader:(int)aHeader display:(BOOL)doDisplay;
- (BOOL)isHeaderDisplayed:(int)aHeader;

- setColorTo:(NSColor *)aColor forDay:(MiscCVDays)aDay;
- (NSColor *)colorForDay:(MiscCVDays)aDay;

- incrementMonth:sender;
- decrementMonth:sender;

- incrementYear:sender;
- decrementYear:sender;

- updateDate:sender;
- displayDate:sender;

- setDateDelegate:(id <DateDelegate>)anObject;
- (id <DateDelegate>)dateDelegate;

- (const char *)getInspectorClassName;
- (const char *)getEditorClassName;

- (id)initWithCoder:(NSCoder *)aDecoder;
- (void)encodeWithCoder:(NSCoder *)aCoder;

@end
