//
//	MiscClipTextField.m -- a TextField subclass for displaying long string 
//						   values
//		Written and Copyright (c) 1995 by Balazs Pataki. 
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <AppKit/AppKit.h>
#import	<misckit/MiscString.h>
#import <objc/objc-runtime.h>

#import "MiscClipTextField.h"
#import "MiscClipTextFieldCell.h"


#define	CLASS_NAME		"MiscClipTextField"
#define	CLASS_VERSION	1
#define CELL_CLASS		[MiscClipTextFieldCell class]


#define DELIMITERS		[[delimiters stringValue] cString]


static id	CellClass;	// Class variable of MiscClipTextField



/*
						********************************
						*                              *
						*      MiscClipTextField       *
						*                              *
						********************************
*/

@implementation MiscClipTextField

+ (void)initialize
// Set class version
{
#warning NSNameConversion:  Class names have changed. Check that the class name passed in is real.
	if (self == objc_lookUpClass(CLASS_NAME))  {
		[self setVersion:CLASS_VERSION];
		CellClass = CELL_CLASS;
	}
	return;
}

+ (void) setCellClass:(Class)classId
{
	CellClass=classId;
	return;
}


- initWithFrame:(NSRect)rect
// Designated initializer
{
	id	oldCell;
	
	[super initWithFrame:rect];
										/* Plug in our own cell				*/
	oldCell = [self cell];
										/* Plug in our own cell				*/
	[self setCell:[[CellClass alloc] initTextCell:@""]];
	[oldCell release];
	
	return self;
}

- (void)setStringValue:(NSString *)stringValue
{
	id theCell = [self cell];
	// Change the alignemt to origAlignment because if previously there was
	// a triple-click the cells alignemnt changed to left, which may not have
	// been our original alignment
	[theCell setSelectable:NO];		/* Disable selecting and scrolling		*/
	[theCell setScrollable:NO];
	[theCell setAlignment:origAlignment];
	[super setStringValue:stringValue];
}

- (void)setAlignment:(int)align
{
	// Save alignment to change it back after a triple-click (triple-click 
	// event changes the cell to display the cell left aligned and we have to
	// take care to undo it) 
	origAlignment = align;
}


- resetStringValue:sender
{
	id theCell = [self cell];
	
	[theCell setSelectable:NO];		/* Disable selecting and scrolling		*/
	[theCell setScrollable:NO];
	[theCell setAlignment:origAlignment];
	[[self cell] resetStringValue:sender];
	return self;
}


- setClipEnabled:(BOOL)flag
// Sets whether the next `setStringValue:' message should clip the text or not 
{
	[[self cell] setClipEnabled:flag];
	return self;
}


- setClipOnRight:(BOOL)flag
// If flag is YES clipping happens on the right, otherwise on the left of the 
// string in the cell. 
{
	[[self cell] setClipOnRight:flag];
	return self;
}


- setClipperString:(const char*)aString
// Sets  `aString' as the string that is displayed in place of the clipped part
// of the original string. 
{
	[[self cell] setClipperString:aString];
	return self;
}


- setClipDelimiters:(const char*)delimChars
// Sets  `delimChars' as the delimiters by which the clipping has to happen. 
// If `delimChars' is NULL or an empty string clipping will occur to any 
// character in the string.
{
	[[self cell] setClipDelimiters:delimChars];
	return self;
}

- (void)setFrameSize:(NSSize)_newSize
// Adjusts the current text to the new size and redisplays it
{
	[super setFrameSize:_newSize];
	[self resetStringValue:self];
}


- (void)mouseDown:(NSEvent *)event 
// In case of triple-click event displays the full text instead of the clipped
// one and allows selecting and scrolling the string
// NOTE: This trick alters the alignment of the cell and changes it to 
//		 align its text to left and to truncate the text - don't know any
//		 workaround (maybe we should stay with left aligned text only :-(
{ 
	// Handling only triple-click event
	if ([event clickCount] == 3) {
		id	theCell  = [self cell];		
#error WindowConversion: 'disableDisplay' is obsolete.  You can probably remove this call.  Typically drawing should happen as part of the update mechanism after every event.  Display is now optimized using the View setNeedsDisplay: method.  See the conversion doc for more info.
		[[self window] disableDisplay];
		[theCell setSelectable:YES];
		[theCell setScrollable:YES];
		[theCell setClipEnabled:NO]; 	/* Turn off clipping for a moment	*/
		[theCell resetStringValue:self];
		[theCell setClipEnabled:YES];	/* ... and turn it back				*/
#error WindowConversion: 'reenableDisplay' is obsolete.  You can probably remove this call.  Typically drawing should happen as part of the update mechanism after every event.  Display is now optimized using the View setNeedsDisplay: method.  See the conversion doc for more info.
		[[self window] reenableDisplay];
		[self display];
	}
	[super mouseDown:event];
}

- (const char*)fullStringValue;	{ return [[self cell] fullStringValue]; }
- clipper					{ return [[self cell] clipper];			}
- delimiters				{ return [[self cell] delimiters];		}
- (BOOL) doesClipOnRight	{ return [[self cell] doesClipOnRight];	}
- (BOOL) isClipEnabled		{ return [[self cell] isClipEnabled];	}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
    [super encodeWithCoder:aCoder];
    
    [aCoder encodeValueOfObjCType:@encode(int) at:&origAlignment];
}


- (id)initWithCoder:(NSCoder *)aDecoder
{
    [super initWithCoder:aDecoder];
   
	[aDecoder decodeValueOfObjCType:@encode(int) at:&origAlignment];

	return self;
}

@end


/*
							********************************
							*                              *
							*  MiscClipTextField(IBStuff)  *
							*                              *
							* -- InterfaceBuilder Stuff--  *
							*                              *
							********************************
*/

@implementation MiscClipTextField(IBStuff)

- (const char *)getInspectorClassName
// Return the class name of our inspector.
{
	return "MiscClipTextFieldInspector";
}

@end


/*
							********************************
							*                              *
							*   MiscClipTextField(Test)    *
							*                              *
							* -- Debug and Test Methods -- *
							*                              *
							********************************
*/

@implementation MiscClipTextField(Test)

- (void)changeFont:(id)sender
{
	[[self cell] setFont:[sender selectedFont]];
	[[self cell] resetStringValue:self];
}

@end

