//
//	MiscClipTextFieldInspector.m -- IB inspector of MiscClipTextField(Cell)
//		Written and Copyright (c) 1995 by Balazs Pataki. 
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

// Most of the code in this inspector is borrowed form  Mike Ferris's 
// MORegexTextCellInspector of MOKit
// Mike Ferris, Copyright 1993, all rights reserved.

#import "MiscClipTextFieldInspector.h"
#import <misckit/MiscClipTextFieldCell.h>
#import <misckit/MiscClipTextField.h>
#import <misckit/MiscPathField.h>
#import <objc/objc-runtime.h>

#define CLASS_VERSION	0
#define CLASS_NAME		"MiscClipTextFieldInspector"


#define BG_NONE_TAG		0
#define BG_BLACK_TAG	1
#define BG_DKGRAY_TAG	2
#define BG_LTGRAY_TAG	3
#define BG_WHITE_TAG	4

#define TG_BLACK_TAG	0
#define TG_DKGRAY_TAG	1
#define TG_LTGRAY_TAG	2
#define TG_WHITE_TAG	3

#define ALIGN_LEFT_TAG		0
#define ALIGN_CENTER_TAG	1
#define ALIGN_RIGHT_TAG		2

#define BORDER_NONE_TAG		0
#define BORDER_LINE_TAG		1
#define BORDER_BEZEL_TAG	2

#define LEFT_TAG		0
#define RIGHT_TAG		1


#define NIB_TYPE							"nib"
#define NIB_NAME							"MiscClipTextFieldInspector"


@implementation MiscClipTextFieldInspector

+ (void)initialize
// Set the version.
{
#warning NSNameConversion:  Class names have changed. Check that the class name passed in is real.
	if (self == objc_lookUpClass(CLASS_NAME))  {
		[self setVersion:CLASS_VERSION];
	}
	return;
}

- init
// Load our nib file.
{
	char buf[MAXPATHLEN+1];
	id bundle;
	
	[super init];
	
	// load our nib file.
	bundle = [NSBundle bundleForClass:[self class]];
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
	buf = [bundle pathForResource:[NSString stringWithCString:NIB_NAME] ofType:[NSString stringWithCString:NIB_TYPE]];
	[NSBundle loadNibFile:[NSString stringWithCString:buf] externalNameTable:[NSDictionary dictionaryWithObjectsAndKeys:self, @"NSOwner", nil] withZone:[self zone]];
	
	return self;
}

- ok:sender
// set the text field-ish stuff and the allowEmptyString attribute.  
// The patternButtonAction takes care of the regex stuff.  The 
// optionCheckboxAction takes care of editable, selectable, scrollable.
{
	int bgTag = [backgroundGrayMatrix selectedTag];
	int tgTag = [textGrayMatrix selectedTag];
	int alTag = [alignmentMatrix selectedTag];
	int boTag = [borderMatrix selectedTag];
	int clTag = [clipOnMatrix selectedTag];
	
	const char *stringVal=NULL;
	
	if (bgTag == BG_NONE_TAG)  {
#warning ColorConversion: [object setDrawsBackground:NO] was [object setBackgroundGray:-1.0]
		[object setDrawsBackground:NO];
	}  else if (bgTag == BG_BLACK_TAG)  {
		[object setBackgroundColor:[NSColor blackColor]];
	}  else if (bgTag == BG_DKGRAY_TAG)  {
		[object setBackgroundColor:[NSColor darkGrayColor]];
	}  else if (bgTag == BG_LTGRAY_TAG)  {
		[object setBackgroundColor:[NSColor lightGrayColor]];
	}  else  {
		[object setBackgroundColor:[NSColor whiteColor]];
	}
	if (tgTag == TG_BLACK_TAG)  {
		[object setTextColor:[NSColor blackColor]];
	}  else if (tgTag == TG_DKGRAY_TAG)  {
		[object setTextColor:[NSColor darkGrayColor]];
	}  else if (tgTag == TG_LTGRAY_TAG)  {
		[object setTextColor:[NSColor lightGrayColor]];
	}  else  {
		[object setTextColor:[NSColor whiteColor]];
	}
	if (alTag == ALIGN_LEFT_TAG)  {
		[object setAlignment:NSLeftTextAlignment];
	}  else if (alTag == ALIGN_CENTER_TAG)  {
		[object setAlignment:NSCenterTextAlignment];
	}  else  {
		[object setAlignment:NSRightTextAlignment];
	}
	if (boTag == BORDER_NONE_TAG)  {
		[object setBordered:NO];
		[object setBezeled:NO];
	}  else if (boTag == BORDER_LINE_TAG)  {
		[object setBordered:YES];
	}  else  {
		[object setBezeled:YES];
	}

	[object setTag:[[tagForm cellAtIndex:0] intValue]];

	// My settings
	if (clTag == LEFT_TAG)
		[object setClipOnRight:NO];
	else
		[object setClipOnRight:YES];

	if (stringVal=[[clipperStringField stringValue] cString])
		[object setClipperString:stringVal];
	//else
	//	[object setClipperString:"..."];
	[object setClipDelimiters:[[clipDelimitersField stringValue] cString]];

	return [super ok:sender];
}

- revert:sender
// fill in the inspector with the attributes of "object"
{	
#warning ColorConversion: Do you really want gray?
#warning ColorConversion: Do you really want gray?
	float bg = [[[object backgroundColor] colorUsingColorSpaceName:NSCalibratedWhiteColorSpace] whiteComponent], tg = [[[object textColor] colorUsingColorSpaceName:NSCalibratedWhiteColorSpace] whiteComponent];
	int alignment = [object alignment];
	BOOL isBordered = [object isBordered], isBezeled = [object isBezeled];
	
	if (bg < 0)  {
		[backgroundGrayMatrix selectCellAtRow:0 column:BG_NONE_TAG];
	}  else  if ((bg >= 0) && (bg < .2))  {
		[backgroundGrayMatrix selectCellAtRow:0 column:BG_BLACK_TAG];
	}  else  if ((bg >= .2) && (bg < .45))  {
		[backgroundGrayMatrix selectCellAtRow:0 column:BG_DKGRAY_TAG];
	}  else  if ((bg >= .45) && (bg < .8))  {
		[backgroundGrayMatrix selectCellAtRow:0 column:BG_LTGRAY_TAG];
	}  else  if (bg >= .8)  {
		[backgroundGrayMatrix selectCellAtRow:0 column:BG_WHITE_TAG];
	}
	[backgroundGrayMatrix display];
	if (tg < .2)  {
		[textGrayMatrix selectCellAtRow:0 column:TG_BLACK_TAG];
	}  else  if ((tg >= .2) && (tg < .45))  {
		[textGrayMatrix selectCellAtRow:0 column:TG_DKGRAY_TAG];
	}  else  if ((tg >= .45) && (tg < .8))  {
		[textGrayMatrix selectCellAtRow:0 column:TG_LTGRAY_TAG];
	}  else  if (tg >= .8)  {
		[textGrayMatrix selectCellAtRow:0 column:TG_WHITE_TAG];
	}
	[textGrayMatrix display];
	if (alignment == NSLeftTextAlignment)  {
		[alignmentMatrix selectCellAtRow:0 column:ALIGN_LEFT_TAG];
	}  else if (alignment == NSCenterTextAlignment)  {
		[alignmentMatrix selectCellAtRow:0 column:ALIGN_CENTER_TAG];
	}  else  {
		[alignmentMatrix selectCellAtRow:0 column:ALIGN_RIGHT_TAG];
	}
	[alignmentMatrix display];
	if (isBezeled)  {
		[borderMatrix selectCellAtRow:0 column:BORDER_BEZEL_TAG];
	}  else if (isBordered)  {
		[borderMatrix selectCellAtRow:0 column:BORDER_LINE_TAG];
	}  else  {
		[borderMatrix selectCellAtRow:0 column:BORDER_NONE_TAG];
	}

	[[tagForm cellAtIndex:0] setIntValue:[object tag]];
	
	// My settings
	if ([object doesClipOnRight])
		[clipOnMatrix selectCellWithTag:RIGHT_TAG];
	else
		[clipOnMatrix selectCellWithTag:LEFT_TAG];
		
	[clipDelimitersField setStringValue:[[object delimiters] stringValue]];
	[clipperStringField setStringValue:[[object clipper] stringValue]];

	return [super revert:sender];
}

- (BOOL)wantsButtons
// Our inspector does not have OK or Revert buttons.
{
	return NO;
}


@end