//
//	MiscReadOnlyColorWell.m -- an NXColorWell that can bemade read-only
//		Written by Morgan D. Wise Copyright (c) 1994 by Morgan D. Wise.
//		Adapted for MiscKit inclusion by Don Yacktman.
//				Version 0.1.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "MiscReadOnlyColorWell.h"

@implementation MiscReadOnlyColorWell

- setReadOnly:(BOOL)flag
{
	_isReadOnly = flag;
	[self setBordered:!flag];		
	return self;
}

- (BOOL)isReadOnly
{
	return _isReadOnly;
}

- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
{
	if (_isReadOnly) return NSDragOperationNone;
	else return [super draggingEntered:sender];
}

- (const char *)getInspectorClassName
{
    return "ReadOnlyColorWellInspector";
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	[super initWithCoder:aDecoder];
	[aDecoder decodeValueOfObjCType:"c" at:&_isReadOnly];
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];
	[aCoder encodeValueOfObjCType:"c" at:&_isReadOnly];
}

// This drawSelf:: allows a ColorWell to be placed in a ScrollView;
// Perhaps it should be placed in a category used to override the
// one in NXColorWell instead...
// donated by Michael T. H. Scott mths@longshot.demon.co.uk
#warning RectConversion: drawRect:(NSRect)rects (used to be drawSelf:(const NXRect *)rects :(int)rectCount) no longer takes an array of rects
- (void)drawRect:(NSRect)rects
{
	NSRect rect = [self bounds];
	int index = ( rectCount == 1 ) ? 0 : 1;
	
	PSgsave();
	
	NSRectClipList(&&rects[index] , index + 1);
		
	if ( [self isBordered] )
	{
		NSDrawButton(rect , &rects[0]);
	
		if ( [self isActive] &&
			 NSWidth(rect) > 3.0 && NSHeight(rect) > 3.0 )
		{
			NSRect highlightRect;
			
			highlightRect = NSMakeRect(NSMinX(rect) + 1.0, NSMinY(rect) + 2.0, NSWidth(rect) - 3.0, NSHeight(rect) - 3.0);
						
			PSsetgray(NSWhite);
			NSRectFill(highlightRect);
		}
		
		rect = NSInsetRect(rect , 6.0 , 6.0);
		NSDrawGrayBezel(rect , &rects[0]);
		rect = NSInsetRect(rect , 2.0 , 2.0);
	}
	else
	{
		NSDrawGrayBezel(rect , &rects[0]);
		rect = NSInsetRect(rect , 2.0 , 2.0);
	}

	if ( NSWidth(rect) > 0.0 && NSHeight(rect) > 0.0 )
	{
		[self drawWellInside:rect];
	}
	
	PSgrestore();
}

@end
