//
//	MiscDateView.m -- a view for dealing with input and display of dates
//		Written by Hugh Ashton, Copyright (c) 1994 by Hugh Ashton.
//				Version 1.0  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#include <time.h>
#include <libc.h>
#include <strings.h>
#import "MiscDateView.h"

#define CURRENT_VERSION 1

id	BundleForStrings = nil;

#define MONDAY [NSLocalizedStringFromTableInBundle(@"Monday", nil, BundleForStrings, zeroth day of week) cString]
#define TUESDAY [NSLocalizedStringFromTableInBundle(@"Tuesday", nil, BundleForStrings, first day of week) cString]
#define WEDNESDAY [NSLocalizedStringFromTableInBundle(@"Wednesday", nil, BundleForStrings, second day of week) cString]
#define THURSDAY [NSLocalizedStringFromTableInBundle(@"Thursday", nil, BundleForStrings, third day of week) cString]
#define FRIDAY [NSLocalizedStringFromTableInBundle(@"Fri", nil, BundleForStrings, fourth day of week) cString]
#define SATURDAY [NSLocalizedStringFromTableInBundle(@"Sat", nil, BundleForStrings, fifth day of week) cString]
#define SUNDAY [NSLocalizedStringFromTableInBundle(@"Sun", nil, BundleForStrings, sixth day of week) cString]
#define DATE_ERROR [NSLocalizedStringFromTableInBundle(@"Date entry error", nil, BundleForStrings, error in date entry) cString]

@implementation MiscDateView

+ (void)initialize
{
	if ( self == [MiscDateView class] )
		[MiscDateView setVersion:CURRENT_VERSION];

	return;
}

- decDay:sender
{
	int mm;
	
	if([[day stringValue] length]>0)
	{
		[day setIntValue:[day intValue]-1];
	}
	if([day intValue]<1)
	{
		[month setIntValue:[month intValue]-1];
		if([month intValue]<1)
		{
			[month setIntValue:12];
			[year setIntValue:[year intValue]-1];
		}
		mm=[month intValue];
		if(mm==4 || mm==6 || mm==9 || mm==11)
/*
	30 days hath September...
*/		{
			[day setIntValue:30];
			[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
			return self;
		}
		if(mm==2)
		{
			if([self isItaLeapYear:[year intValue]])
/*
leap year
*/
			{
				[day setIntValue:29];
			}
			else
			{
				[day setIntValue:28];
			}
			[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
			return self;
		}
		[day setIntValue:31];
	}
	[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
	[self isDateValid:self];
				
	return self;
}

- decMonth:sender
{
	if([[month stringValue] length]>0)
	{
		[month setIntValue:[month intValue]-1];
	}
	if([month intValue]<1)
	{
		[month setIntValue:12];
		[year setIntValue:[year intValue]-1];
	}
	[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
	[self isDateValid:self];
    return self;
}

- outputDOW:(unsigned int)number
{
	number=number%7;
	
	if(number==0)
	{
		[dow setStringValue:[NSString stringWithCString:MONDAY]];
		return self;
	}
	if(number==1)
	{
		[dow setStringValue:[NSString stringWithCString:TUESDAY]];
		return self;
	}
	if(number==2)
	{
		[dow setStringValue:[NSString stringWithCString:WEDNESDAY]];
		return self;
	}
	if(number==3)
	{
		[dow setStringValue:[NSString stringWithCString:THURSDAY]];
		return self;
	}
	if(number==4)
	{
		[dow setStringValue:[NSString stringWithCString:FRIDAY]];
		return self;
	}
	if(number==5)
	{
		[dow setStringValue:[NSString stringWithCString:SATURDAY]];
		return self;
	}
	if(number==6)
	{
		[dow setStringValue:[NSString stringWithCString:SUNDAY]];
		return self;
	}
	return self;
}
	
- decYear:sender
{
	if([[year stringValue] length]>0)
	{
		[year setIntValue:[year intValue]-1];
	}
	[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
	[self isDateValid:self];
	return self;
}

- checkDay:sender
{
	if(([[day stringValue] isEqualToString:@""]) || ([[month stringValue] isEqualToString:@""]) || ([[year stringValue] isEqualToString:@""]))
	{
		[dow setStringValue:@"XXX"];
		return self;
	}
	[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
	[self isDateValid:self];
	return self;
}
- (unsigned int)whatDayIs:(int)theDay :(int)theMonth :(int)theYear
/*
	we take 1 January 1901 as our base
	Mon=0, Tue=1, etc
*/
{
	unsigned int i;
	unsigned int current_count;
	current_count=1;
/*
	as it happens, 1/1/1901 was a Tuesday
*/
	if(theYear<1901)
	{
		return 00;
	}
	for(i=1901;i<theYear;i++)
	{
		if([self isItaLeapYear:i])
		{
			current_count=(current_count+=366);
		}
		else
		{
			current_count=(current_count+=365);
		}
	}
	for(i=1;i<theMonth;i++)
	{
		if((i==4) || (i==6) || (i==9) || (i==11))
		{
			current_count+=30;
		}
		if((i==2) && ([self isItaLeapYear:theYear]))
		{
			current_count+=29;
		}
		if((i==2) && (![self isItaLeapYear:theYear]))
		{
			current_count+=28;
		}
		if((i==1) || (i==3) || (i==5) || (i==7) || (i==8) || (i==10) || (i==12))
		{
			current_count+=31;
		}
	}
	for(i=1;i<theDay;i++)
	{
		current_count+=1;
	}
	
/*
	fencepost error, hence this kludge
	current_count=current_count-1;
*/
	return current_count;
}
- (BOOL)isItaLeapYear:(int)theYear
{
	// Fix from Rod Ragner:
    return ((!(theYear % 4) && (theYear % 200)) ? YES : NO);
// old way is incorrect:
//	if((theYear/4.0)==(theYear/4))
//	{
//		return YES;
//	}
//	else
//	{
//		return NO;
//	}
}
- getTodaysDate:sender
{
	long currenttime;
	char dateString[60]="";
	int dd;
	int mm=0;
	char mmStr[4]="";
	int yyyy;
	char dowStr[4]="";
	char timeString[20]="";

	currenttime = time(0L);
	strcpy(dateString,ctime(&currenttime));
	
	sscanf(dateString,"%s %s %d %s %d",dowStr,mmStr,&dd,timeString,&yyyy);
	[dow setStringValue:[NSString stringWithCString:dowStr]];
	[year setIntValue:yyyy];
	[day setIntValue:dd];
	if(strcmp(mmStr,"Jan")==0)
	{
		mm=1;
	}
	if((strcmp(mmStr,"Feb"))==0)
	{
		mm=2;
	}
	if((strcmp(mmStr,"Mar"))==0)
	{
		mm=3;
	}
	if((strcmp(mmStr,"Apr"))==0)
	{
		mm=4;
	}
	if((strcmp(mmStr,"May"))==0)
	{
		mm=5;
	}
	if((strcmp(mmStr,"Jun"))==0)
	{
		mm=6;
	}
	if((strcmp(mmStr,"Jul"))==0)
	{
		mm=7;
	}
	if((strcmp(mmStr,"Aug"))==0)
	{
		mm=8;
	}
	if((strcmp(mmStr,"Sep"))==0)
	{
		mm=9;
	}
	if((strcmp(mmStr,"Oct"))==0)
	{
		mm=10;
	}
	if((strcmp(mmStr,"Nov"))==0)
	{
		mm=11;
	}
	if((strcmp(mmStr,"Dec"))==0)
	{
		mm=12;
	}
	[month setIntValue:mm];
	return self;
}

- incDay:sender
{
	int mm=0;
	
	mm=[month intValue];
	
	if([[day stringValue] length]>0)
	{
		[day setIntValue:[day intValue]+1];
	}
	if((mm==4 || mm==6 || mm==9 || mm==11) && [day intValue]>30)
	{
		[month setIntValue:[month intValue]+1];
		[day setIntValue:1];
		[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
		return self;
	}
	if((mm==2) && ([day intValue]>28) && (![self isItaLeapYear:[year intValue]]))
	{
		[month setIntValue:3];
		[day setIntValue:1];
		[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
		return self;
	}
	if((mm==2) && ([day intValue]>29) && ([self isItaLeapYear:[year intValue]]))
	{
		[month setIntValue:3];
		[day setIntValue:1];
		[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
		return self;
	}
	
	if([day intValue]>31)
	{
		[month setIntValue:[month intValue]+1];
		[day setIntValue:1];
		if([month intValue]>12)
		{
			[month setIntValue:1];
			[year setIntValue:[year intValue]+1];
		}
	}
	[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
	[self isDateValid:self];
	return self;
}

- incMonth:sender
{
	if([[month stringValue] length]>0)
	{
		[month setIntValue:[month intValue]+1];
	}
	if([month intValue]>12)
	{
		[month setIntValue:1];
		[year setIntValue:[year intValue]+1];
	}
	[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
	[self isDateValid:self];
	return self;
}

- incYear:sender
{
	if([[year stringValue] length]>0)
	{
		[year setIntValue:[year intValue]+1];
	}
	[self outputDOW:[self whatDayIs:[day intValue] :[month intValue] : [year intValue]]];
	[self isDateValid:self];
	return self;
}

- (BOOL)isDateValid:sender
{
	if(([year intValue]>2050) || ([year intValue]<1901))
	{
		NSRunAlertPanel([NSString stringWithCString:DATE_ERROR], @"Year is outside range 1901 to 2050", @"OK", nil, nil);
		[year selectText:self];
		[dow setStringValue:@"XXX"];
		return NO;
	}
	if(([month intValue]>12) || ([month intValue]<1))
	{
		NSRunAlertPanel([NSString stringWithCString:DATE_ERROR], @"Month is outside range 1 to 12", @"OK", nil, nil);
		[month selectText:self];
		[dow setStringValue:@"XXX"];
		return NO;
	}
	if(([day intValue]<1) || ([day intValue]>31))
	{
		NSRunAlertPanel([NSString stringWithCString:DATE_ERROR], @"Day is outside range 1 to 31", @"OK", nil, nil);
		[day selectText:self];
		[dow setStringValue:@"XXX"];
		return NO;
	}
	if(([month intValue]==4 || [month intValue]==6 || [month intValue]==9 || [month intValue]==11) && [day intValue]>30)
	{
		NSRunAlertPanel([NSString stringWithCString:DATE_ERROR], @"Day is outside range for this month (1 to 30)", @"OK", nil, nil);
		[day selectText:self];
		[dow setStringValue:@"XXX"];
		return NO;
	}
	if([month intValue]==2 && ![self isItaLeapYear:[year intValue]] && [day intValue]>28)
	{
		NSRunAlertPanel([NSString stringWithCString:DATE_ERROR], @"Day is outside range for this month (1 to 28). %d is not a leap year", @"OK", nil, nil, [year intValue]);
		[day selectText:self];
		[dow setStringValue:@"XXX"];
		return NO;
	}
	if([month intValue]==2 && [self isItaLeapYear:[year intValue]] && [day intValue]>29)
	{
		NSRunAlertPanel([NSString stringWithCString:DATE_ERROR], @"Day is outside range for this month (1 to 29). %d is a leap year", @"OK", nil, nil, [year intValue]);
		[day selectText:self];
		[dow setStringValue:@"XXX"];
		return NO;
	}
	
	
	return YES;
}

- initWithFrame:(NSRect)frameRect
{
	[super initWithFrame:frameRect];

	return self;
}

#warning RectConversion: drawRect:(NSRect)rects (used to be drawSelf:(const NXRect *)rects :(int)rectCount) no longer takes an array of rects
- (void)drawRect:(NSRect)rects
{
	PSsetgray(NSLightGray);
	NSRectFill([self bounds]);
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	int version;
	[super initWithCoder:aDecoder];
	version = [aDecoder versionForClassName:@"MiscDateView"];
	if ( version == CURRENT_VERSION ) {
		day			= [[aDecoder decodeObject] retain];
		dayDown		= [[aDecoder decodeObject] retain];
		dayUp		= [[aDecoder decodeObject] retain];
		dow			= [[aDecoder decodeObject] retain];
		month		= [[aDecoder decodeObject] retain];
		monthDown	= [[aDecoder decodeObject] retain];
		monthUp		= [[aDecoder decodeObject] retain];
		year		= [[aDecoder decodeObject] retain];
		yearDown	= [[aDecoder decodeObject] retain];
		yearUp		= [[aDecoder decodeObject] retain];
	} else {
		[self error:"Unknown version!"];
	}
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];
	[aCoder encodeObject:day];
	[aCoder encodeObject:dayDown];
	[aCoder encodeObject:dayUp];
	[aCoder encodeObject:dow];
	[aCoder encodeObject:month];
	[aCoder encodeObject:monthDown];
	[aCoder encodeObject:monthUp];
	[aCoder encodeObject:year];
	[aCoder encodeObject:yearDown];
	[aCoder encodeObject:yearUp];
}


- (int)getDay:sender
{
	return [day intValue];
}

- (int)getMonth:sender
{
	return [month intValue];
}

- (int)getYear:sender
{
	return [year intValue];
}

- day:sender
{
	return day;
}

- month:sender
{
	return month;
}

- year:sender
{
	return year;
}
@end
