/***************************************************************************
 * CLASS:		MiscDVInspector (MiscDragViewInspector)
 * Copyright (C) 1995 Robert Todd Thomas
 * Use is governed by the MiscKit license
 * 
 *	See the header file for more info about this class.
 ***************************************************************************/
 
#import "MiscViews.subproj/MiscIconWell.h"
#import "MiscDVInspector.h"


@implementation MiscDVInspector

// Load the Inspector.

- init
{
    char buf[MAXPATHLEN + 1];
    id bundle;
    
    [super init];
    
    bundle = [NSBundle bundleForClass:[MiscDragView class]];

#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
    buf = [bundle pathForResource:@"MiscDVInspector" ofType:@"nib"];

    [NSBundle loadNibFile:[NSString stringWithCString:buf] externalNameTable:[NSDictionary dictionaryWithObjectsAndKeys:self, @"NSOwner", nil] withZone:[self zone]];
		   
    return self;
}



// This is called to set the options located on the inspector to
// be the same as the current selected object.

- revert: sender
{	
	// set all the radio buttons for dragging options
	
	[[optionMatrix cellAtRow:0 column:0] setState:[object allowSourceDragging]];	
   	[[optionMatrix cellAtRow:1 column:0] setState:[object allowDestinationDragging]];	
	[optionMatrix display];	// get matrix to redisplay values

	// set icon stuff

	[[iconMatrix cellAtRow:1 column:0] setEnabled:[object allowDestinationDragging]];

	[[iconMatrix cellAtRow:0 column:0] setStringValue:[NSString stringWithCString:[object imageName]]];
	[[iconMatrix cellAtRow:1 column:0] setStringValue:[NSString stringWithCString:[object acceptingImageName]]];

	// set the border type
	
	[borderMatrix selectCellWithTag:[object borderType]];
	[borderMatrix display];
		
    return [super revert:sender];
}



// This method is called when one of the radio buttons in the inspector
// is clicked. The object is then notified of the change so that
// it can change it's state.

- optionsChanged: sender
{
  BOOL  state = [ [sender selectedCell] state];
  
  	switch ([ [sender selectedCell] tag])
    {
	  case 0:					
	  	[object setAllowSourceDragging: state];
		break;
	  case 1: 					
	  	[object setAllowDestinationDragging: state];
		break;
	 }
	[window disableFlushWindow];
	[self revert: sender];
	[window enableFlushWindow];
	return [super ok: sender];
}

- iconsChanged: sender
{	const char *str;
	if(str=[[[iconMatrix cellAtRow:0 column:0] stringValue] cString])
		[object setImageByName:str];
	if(str=[[[iconMatrix cellAtRow:1 column:0] stringValue] cString])
		[object setAcceptingImageByName:str];
	return [super ok: sender];
}

- borderChanged: sender
{
	[object setBorderType:[ [borderMatrix selectedCell] tag]];
	[object display];
	
	return [super ok: sender];
}



// Determines whether the inspector panel will have an "OK" and
// "Revert" button. 

- (BOOL)wantsButtons
{
    return NO;
}


@end


@implementation MiscDragView (IBInspector)

- (const char *)getInspectorClassName
{
	return "MiscDVInspector";
}

@end


