//		Written by Thomas Engel Copyright (c) 1995 by Thomas Engel.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/MiscColorView.h>

#define MISC_COLORVIEW_VERSION 0
#define MISC_COLORVIEW_CLASSNAME "MiscColorView"


@implementation MiscColorView

+ (void)initialize
{
	// Initialize the current version number which is used when archiving 
	// objects. This way we will be able to read all versions if we are 
	// careful.

	if( self == [MiscColorView class] )
		[self setVersion:MISC_COLORVIEW_VERSION];
		
	return;
}

- initWithFrame:(NSRect)frameRect
{
	// Designated initilizer. We will set allow resizing by default and
	// we will draw in Lightgray if we have to swap 'nil' in.

	self = [super initWithFrame:frameRect];
	if( !self ) return self;

	// By default we have a white background and don't focus on the
	// desktops background.

	backgroundColor = [NSColor colorWithCalibratedRed:1.0 green:1.0 blue:1.0 alpha:1.0];

	return self;
}

- (void)setBackgroundColor:(NSColor *)color
{
	backgroundColor = color;
}

- (NSColor *)backgroundColor
{
	return backgroundColor;
}

- setUseSameColorAsDesktop:(BOOL)flag
{
	sameColorAsDesktop = flag;
	return self;
}

- (BOOL)hasSameColorAsDesktop
{
	return sameColorAsDesktop;
}

#warning RectConversion: drawRect:(NSRect)rects (used to be drawSelf:(const NXRect *)rects :(int)rectCount) no longer takes an array of rects
- (void)drawRect:(NSRect)rects
{
	const char *	defaultVal;
	float	red;
	float	green;
	float	blue;
	
	if( sameColorAsDesktop )
	{
		// Read the desktops color form the defaults database.

#warning DefaultsConversion: If you were using NXReadDefault() to avoid searching the GLOBAL domain or to search a different domain than your app domain, you must set the NSUserDefault searchlist to the appropriate domains and use NSUserDefaults 'objectForKey:'
		defaultVal = ([[NSUserDefaults standardUserDefaults] synchronize], [[[NSUserDefaults standardUserDefaults] objectForKey:@"BackgroundColor"] cString]);

		if( defaultVal != NULL )
			sscanf( defaultVal, "%f %f %f",&red, &green, &blue );
		
		// If there is no default we will take a shade of blue.
		// This seems to be the color NeXT uses as a default.

		else
		{	
			red = 0.333333;
			green = 0.333333;
			blue = 0.466666;
		}
		[[NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0] set];	}
	else	
		[backgroundColor set];

	NSRectFill([self bounds]);
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	int  version;
  
	[super initWithCoder:aDecoder];
	
	version = [aDecoder versionForClassName:[NSString stringWithCString:MISC_COLORVIEW_CLASSNAME]];
	
	switch( version )
	{
		case 0:
#warning ColorConversion: [[aDecoder decodeNXColor] retain] used to be NXReadColor(aDecoder).  Use 'decodeNXColor' to read old style colors, but use 'decodeObject' to read OpenStep colors.
			backgroundColor = [[aDecoder decodeNXColor] retain];
			[aDecoder decodeValueOfObjCType:"c" at:&sameColorAsDesktop];
			break;
		
		default:
			break;
	}

	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];

	[aCoder encodeObject:backgroundColor];
	[aCoder encodeValueOfObjCType:"c" at:&sameColorAsDesktop];
}

@end
