//		Written by Thomas Engel Copyright (c) 1995 by Thomas Engel.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/MiscPaperView.h>

#define MISC_PAPERVIEW_VERSION 0
#define MISC_PAPERVIEW_CLASSNAME "MiscPaperView"


@implementation MiscPaperView

+ (void)initialize
{
	// Initialize the current version number which is used when archiving 
	// objects. This way we will be able to read all versions if we are 
	// careful.

	if( self == [MiscPaperView class] )
		[self setVersion:MISC_PAPERVIEW_VERSION];
		
	return;
}

- initWithFrame:(NSRect)frameRect
{
	// Designated initilizer. We will set allow resizing by default and
	// we will draw in Lightgray if we have to swap 'nil' in.

	self = [super initWithFrame:frameRect];
	if( !self ) return self;

	// By default will draw a simple horizontal light gray grid with a dark
	// gray right side border.

	gridColor = [NSColor lightGrayColor];
	gridType = Misc_PaperGridHorizontal;
	gridOrigin = Misc_PaperGridStartsUpperLeft;
	gridVertOffset = gridHorOffset = 20;

	sidelineColor = [NSColor darkGrayColor];
	sidelineType = Misc_PaperSidelineRight;
	sidelineOffset = 4;

	return self;
}

- setGridColor:(NSColor *)color
{
	gridColor = color;
	return self;
}

- (NSColor *)gridColor
{
	return gridColor;
}

- setGridType:(int)aType withOrigin:(int)theOrigin
{
	gridType = aType;
	gridOrigin = theOrigin;
	return self;
}

- (int)gridType
{
	return gridType;
}

- (int)gridOrigin
{
	return gridOrigin;
}

- setGridSizeVertical:(int)vert horizontal:(int)hor
{
	// Well there has to be some progress so forbit useless offsets

	if( vert < 1 ) vert = 1;
	if( hor < 1 ) hor = 1;

	gridVertOffset = vert;
	gridHorOffset = hor;

	return self;
}

- (int)verticalGridSize
{
	return gridVertOffset;
}

- (int)horizontalGridSize
{
	return gridHorOffset;
}

- setSidelineColor:(NSColor *)color
{
	sidelineColor = color;
	return self;
}

- (NSColor *)sidelineColor
{
	return sidelineColor;
}

- setSidelineType:(int)aType
{
	sidelineType = aType;
	return self;
}

- (int)sidelineType
{
	return sidelineType;
}

- setSidelineOffset:(int)offset
{
	sidelineOffset = offset;
	return self;
}

- (int)sidelineOffset
{
	return sidelineOffset;
}

#warning RectConversion: drawRect:(NSRect)rects (used to be drawSelf:(const NXRect *)rects :(int)rectCount) no longer takes an array of rects
- (void)drawRect:(NSRect)rects
{	
	float	xMin;
	float	xMax;
	float	yMin;
	float	yMax;

	float	offset;
	float	coord;
	int		i;
	int		count;

#warning RectConversion: drawRect:*rects (used to be drawSelf:rects :rectCount) no longer takes an array of rects
	[super drawRect:rects];

	xMin = [self bounds].origin.x;
	xMax = [self bounds].origin.x + [self bounds].size.width - 1;
	yMin = [self bounds].origin.y;
	yMax = [self bounds].origin.y + [self bounds].size.height - 1;

	// BUG: Please tell me whats wrong ?
	// Well everybody would think that this should be the right calculation ?
	// At least I do...but somehow you have to adjust the Y values of the bounds
	// because otherwise you'll start 1 pixel too low (out side the view) and
	// you will end 1 pixel too early ????. 
	// Perhaps I missed some kind of stroke adjust ?

	yMin += 1;
	yMax += 1;

	// Draw the grid first...but lets see where we have to start from
	// and how many lines we need.

	PSsetlinewidth( 1.0 );
	[gridColor set];

	if( gridType & Misc_PaperGridHorizontal )
	{
		count = [self bounds].size.height / gridVertOffset;

		// If we start from the top then the values are different...

		if( gridOrigin & Misc_PaperGridStartsLow )
		{
			coord = yMin - 1;
			offset = gridVertOffset;
		}
		else
		{
			coord = yMax + 1;
			offset = -gridVertOffset;
		}
		for( i=0; i<count; i++ )
		{
			coord += offset;
			PSmoveto( xMin, coord );
			PSlineto( xMax, coord );
			PSstroke();
		}
	}

	// Now lets do the same for Vertical lines.

	if( gridType & Misc_PaperGridVertical )
	{
		count = [self bounds].size.width / gridHorOffset;

		// If we start from the left side then the values are different...

		if( gridOrigin & Misc_PaperGridStartsRight )
		{
			coord = xMax + 1;
			offset = -gridHorOffset;
		}
		else
		{
			coord = xMin - 1;
			offset = gridHorOffset;
		}
		for( i=0; i<count; i++ )
		{
			coord += offset;
			PSmoveto( coord, yMin );
			PSlineto( coord, yMax );
			PSstroke();
		}
	}	

	// Now draw the borders..

	[sidelineColor set];
	
	if( sidelineType & Misc_PaperSidelineTop )
	{
		PSmoveto( xMin, yMax - sidelineOffset );
		PSlineto( xMax, yMax - sidelineOffset );
		PSstroke();
	}

	if( sidelineType & Misc_PaperSidelineBottom )
	{
		PSmoveto( xMin, yMin + sidelineOffset );
		PSlineto( xMax, yMin + sidelineOffset );
		PSstroke();
	}

	if( sidelineType & Misc_PaperSidelineLeft )
	{
		PSmoveto( xMin + sidelineOffset, yMin );
		PSlineto( xMin + sidelineOffset, yMax );
		PSstroke();
	}

	if( sidelineType & Misc_PaperSidelineRight )
	{
		PSmoveto( xMax - sidelineOffset, yMin );
		PSlineto( xMax - sidelineOffset, yMax );
		PSstroke();
	}
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	int  version;
  
	[super initWithCoder:aDecoder];
	
	version = [aDecoder versionForClassName:[NSString stringWithCString:MISC_PAPERVIEW_CLASSNAME]];
	
	switch( version )
	{
		case 0:
#warning ColorConversion: [[aDecoder decodeNXColor] retain] used to be NXReadColor(aDecoder).  Use 'decodeNXColor' to read old style colors, but use 'decodeObject' to read OpenStep colors.
			gridColor = [[aDecoder decodeNXColor] retain];
			[aDecoder decodeValueOfObjCType:"i" at:&gridType];
			[aDecoder decodeValueOfObjCType:"i" at:&gridOrigin];
			[aDecoder decodeValueOfObjCType:"i" at:&gridVertOffset];
			[aDecoder decodeValueOfObjCType:"i" at:&gridHorOffset];

#warning ColorConversion: [[aDecoder decodeNXColor] retain] used to be NXReadColor(aDecoder).  Use 'decodeNXColor' to read old style colors, but use 'decodeObject' to read OpenStep colors.
			sidelineColor = [[aDecoder decodeNXColor] retain];
			[aDecoder decodeValueOfObjCType:"i" at:&sidelineType];
			[aDecoder decodeValueOfObjCType:"i" at:&sidelineOffset];
			break;
		
		default:
			break;
	}

	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];

	[aCoder encodeObject:gridColor];
	[aCoder encodeValueOfObjCType:"i" at:&gridType];
	[aCoder encodeValueOfObjCType:"i" at:&gridOrigin];
	[aCoder encodeValueOfObjCType:"i" at:&gridVertOffset];
	[aCoder encodeValueOfObjCType:"i" at:&gridHorOffset];

	[aCoder encodeObject:sidelineColor];
	[aCoder encodeValueOfObjCType:"i" at:&sidelineType];
	[aCoder encodeValueOfObjCType:"i" at:&sidelineOffset];
}

@end
