// Copyright (C) 1995 Steve Hayman
// Use is governed by the MiscKit license

#import "MiscAwakeAction.h"
#import <apps/InterfaceBuilder.h>
#define AWAKE_ACTION_VERSION 0

@implementation MiscAwakeAction

/*
 * The meat of the class.  Send a message somewhere.
 */
 
- fire
{
    if ( _target && action && [_target respondsToSelector:action] ) {
	[_target perform:action withObject:self];
    }    
    return self;
}

/*
 * We get "awakeFromNib" when our nib file is loaded by a running
 * application, after all the outlets are set up.
 * Good time to fire the target/action.
 */
- (void)awakeFromNib
{
    return [self fire];
}

/*
 * Bug in 3.1 and later: [NXApp isTestingInterface] always yields
 * NO during read/write/awake.  Use this workaround.  See NeXTanswers
 * #1525.  This idea is from FilenameList.m in the StringList miniexample
 */
#error ArchiverConversion: put the contents of your 'awake' method at the end of your 'initWithCoder:' method instead
- awake
{
    // test to see if mainMenu is visible, which it isn't while test
    // interface mode is being set up. 
    
    if([NSApp conformsToProtocol:@protocol(IB)]) {
	if(! [[NSApp mainMenu] isVisible]) {
	    // main menu not visible, must be entering test mode
	    // (although this also happens when we are loading a .nib
	    //  at IB startup time, i.e. if you doubleclick on a .nib
	    //  and ib is opening it while starting up. no big deal.

	    return [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(fire) object:nil], [self performSelector:@selector(fire) object:nil afterDelay:(1) / 1000.0];
	}
    }
    return nil;
}

- target
{
    return _target;
}
- (void)setTarget:t
{
    _target = t;
}
- (SEL)action
{
    return action;
}
- (void)setAction:(SEL)a
{
    action = a;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    int version;

    [super initWithCoder:aDecoder];
    version = [aDecoder versionForClassName:@"MiscAwakeAction"];
    switch (version) {
    case AWAKE_ACTION_VERSION:
	[aDecoder decodeValuesOfObjCTypes:"@:", &_target, &action];
	
	if ( [NSApp respondsToSelector:@selector(isTestingInterface)]
	    && [NSApp isTestingInterface] ) {
	

	    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(fire) object:nil], [self performSelector:@selector(fire) object:nil afterDelay:(1) / 1000.0];
	}

	break;
    default:
#warning StringConversion: NSStringFromSelector return value is an NSString; use [ cString] if needed, otherwise change the target type to an NSString.
	NSLog(@"[%s %s] - unknown version of %s in typed stream", [[[self class] name] cString], NSStringFromSelector(_cmd), 
		[[[self class] name] cString]);
	break;
    }
    return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
    [super encodeWithCoder:aCoder];
    
	[aCoder encodeValuesOfObjCTypes:"@:", &_target, &action];
}


@end

