// Copyright (C) 1995 Steve Hayman
// Use is governed by the MiscKit license

#import "MiscShell.subproj/MiscShell.h"
#import "MiscShellInspector.h"
#import "EmacsText.h"

@implementation MiscShellInspector
// Simple view swapping methods in response to the popup
//
- swapToBox:aBox
{
    [swapBox setContentView:[aBox contentView]];
    [swapBox display];
    return self;
}
// the popup buttons send this
- swapView:sender
{
    switch( [[sender selectedCell] tag] ) {
    case 0:	
	[self swapToBox:scriptBox ]; 
	break;
    case 1:	
	[self swapToBox:optionsBox]; 
	break;
    }
    
    return self;
}

// Load the Inspector.
// TODO - replace this with an EmacsText object

- init
{
    char buf[MAXPATHLEN + 1];
    id bundle;
    NSRect textFrame;
    NSFont *originalFont;
    
    [super init];
    
    bundle = [NSBundle bundleForClass:[MiscShell class]];
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
    buf = [bundle pathForResource:@"MiscShellInspector" ofType:@"nib"];
    [NSBundle loadNibFile:[NSString stringWithCString:buf] externalNameTable:[NSDictionary dictionaryWithObjectsAndKeys:self, @"NSOwner", nil] withZone:[self zone]];
    
    [self swapToBox:scriptBox];
    /*
     * at this point, "text" is pointing to the old scrolling text.
     */
    originalFont = [text font];

    /*
     * Now we want to create an EmacsText object and replace the text
     * inside the "document" scroll view with it.
     * Some of this code is borrowed from Scott Anguish's enhanced Yap.
     * Thanks, Scott.
     */
    textFrame = [[scrollView documentView] frame];
    text = [[EmacsText alloc] initWithFrame:textFrame text:@"" alignment:NSLeftTextAlignment];
    /*
     * Put this new text object in the window and free the IB-created one.
     */
    [[scrollView setDocumentView:text] release];
    
    /*
     * Finish setting up the new text object
     */
    [text setFont:originalFont];		// Whatever was set in IB

    [text setVerticallyResizable:YES];		// Grow down as you type
    [text setHorizontallyResizable:NO];		// But not sideways 
    [text setAutoresizingMask:NSViewWidthSizable];	// Size horizontally when resized
    [text setRichText:NO];
#error ViewConversion: View's 'setOpaque:' is obsolete; you must override 'isOpaque' instead of setting externally (if sent to text, 'setDrawsBackground:' can be used to eliminate background gray; if sent to NSImageRep, it's OK)
    [text setOpaque:YES];
    [text setMinSize:textFrame.size];
    (&textFrame)->size.width = (&textFrame)->size.height = 1.0e38;
    [text setMaxSize:textFrame.size];	// Can grow
    [text setSelectionStart:0 end:0];			// Set the selection
    [text setDelegate:self];
    [text sizeToFit];
		   
    return self;
}


/*
 * revert  - set inspector to reflect object's state
 */
- revert: sender
{
    int tag;
    [text setString:[[object script] stringValue]];
    [runToCompletionCheck setState:[object runToCompletion]];
    switch( [object delimiter] ) {
    case 0:	tag = 0; break;
    case '\t':	tag = 1; break;
    case ' ':	tag = 2; break;
    case ':':	tag = 3; break;
    default:	tag = 0; break;	// ??? TODO - print an error message
    }
    
    [delimiters selectCellWithTag:tag];
    [customDelimiters setStringValue:[[object customDelimiters] stringValue]];
    [tableSort selectCellWithTag:[object sortWhenColumnsMove]];
    return [super revert:sender];
}


/*
 * ok - set object to reflect inspector's contents.  various objects in
 * the inspector panel send us this message when they're clicked.
 *
 */
- ok: sender
{
    MiscString *newScript = [[MiscString alloc] init];
    MiscString *aLine = [[MiscString alloc] init];
    NXStream *textStream = [text stream];
    char newDelim;
    MiscString *newDelimiters = [[MiscString alloc] init];
    
    [self touch:self];			// mark inspector panel edited
    [[NSApp activeDocument] touch];	// mark nib as edited
    /*
     * The textStream isn't necessarily at the start when we do [text stream];
     * it might be where we left it before.  which could cause us to
     * hit EOS immediately.  which did happen.  which drove me nuts for
     * a while as my carefully setup scripts kept vanishing in the inspector.
     */
    NXSeek(textStream, 0L, NX_FROMSTART);
    
    while ( [aLine streamGets:textStream] != EOF) {
	[newScript concatenate:aLine];
    }
    [newScript concatenate:aLine];	// last incomplete line if any

    [object setScript:newScript];	// it makes a copy
   
    [newDelimiters setStringValue:[customDelimiters stringValue]];
    [object setCustomDelimiters:newDelimiters];	// it makes a copy
    [newDelimiters release];
    
    [newScript release];
    [aLine release];
    
    [object setRunToCompletion: [runToCompletionCheck state]];
    
    switch ( [[delimiters selectedCell] tag] ) {
    case 0:	newDelim = 0; break;
    case 1:	newDelim = '\t'; break;
    case 2:	newDelim = ' '; break;
    case 3:	newDelim = ':'; break;
    default:	newDelim = 0; break;	// ??? todo - add an error msg
    }
    
    [object setDelimiter:newDelim];
    [object setSortWhenColumnsMove: [[tableSort selectedCell] tag]];
    return self;
}
- (BOOL)wantsButtons
{
    return NO;
}

// Text delegate methods - we're the delegate of the script text, so we
// want to do "ok" whenever the text changes.
// thanks to the StringList palette for this clever method.
#warning NotificationConversion: 'textDidChange:' used to be 'textDidGetKeys:'.  This conversion assumes this method is implemented or sent to a delegate of NSText.  If this method was implemented by a NSMatrix or NSTextField textDelegate, use the text notifications in NSControl.h.
- (void)textDidChange:(NSNotification *)notification 
{
#warning NotificationConversion: if this notification was not posted by NSText (eg. was forwarded by NSMatrix or NSTextField from the field editor to their textDelegate), then the text object is found by [[notification userInfo] objectForKey:@"NSFieldEditor"] rather than [notification object]
    NSText *theText = [notification object];
    BOOL flag = [[theText text] isEqualToString:@""];
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(ok:) object:self], [self performSelector:@selector(ok:) object:self afterDelay:(500) / 1000.0];
}

// same deal for text fields
#warning NotificationConversion: 'textDidBeginEditing:' used to be 'textDidChange:'.  This conversion assumes this method is implemented or sent to a delegate of NSText.  If this method was implemented by a NSMatrix or NSTextField textDelegate, use the text notifications in NSControl.h.
- (void)textDidBeginEditing:(NSNotification *)notification
{
#warning NotificationConversion: if this notification was not posted by NSText (eg. was forwarded by NSMatrix or NSTextField from the field editor to their textDelegate), then the text object is found by [[notification userInfo] objectForKey:@"NSFieldEditor"] rather than [notification object]
    NSText *theText = [notification object];
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(ok:) object:self], [self performSelector:@selector(ok:) object:self afterDelay:(500) / 1000.0];
}

@end
