//=============================================================================
//
//		Copyright (C) 1995 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscBorderCell.M
//
//		Cell class used by MiscBorderView to manage column headings and row 
//		labels for MiscTableScroll.	 
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscBorderCell.M,v 1.1 95/09/27 12:21:21 zarnuk Exp $
// $Log:		MiscBorderCell.M,v $
//	Revision 1.1  95/09/27	12:21:21  zarnuk
//	Initial revision
//	
//-----------------------------------------------------------------------------
#import "MiscBorderCell.h"

extern "Objective-C" {
#import <AppKit/NSFont.h>
#import <AppKit/NSText.h>
#import <AppKit/psopsOpenStep.h>
}

@implementation MiscBorderCell

- initTextCell:(NSString *)s
	{
	[super initTextCell:s];
	[self setBordered:NO];
	[self setWrap:NO];
	[self setAlignment:NSCenterTextAlignment];
	[self setFont:[NSFont boldSystemFontOfSize:12.0]];
	return self;
	}


- (float) backgroundGray
	{
	return [self state] ? (1.0/6.0) : NSDarkGray;
	}


- setUpFieldEditorAttributes:textObject
	{
	[super setUpFieldEditorAttributes:textObject];
	[textObject setTextColor:[NSColor whiteColor]];
#warning ColorConversion: Do you really want gray?
	[textObject setBackgroundColor:[NSColor colorWithCalibratedWhite:[[[self backgroundColor] colorUsingColorSpaceName:NSCalibratedWhiteColorSpace] whiteComponent] alpha:1.0]];
	return textObject;
	}


- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
	{
	NSRect r;
#warning ColorConversion: Do you really want gray?
	PSsetgray( [[[self backgroundColor] colorUsingColorSpaceName:NSCalibratedWhiteColorSpace] whiteComponent] );
	r = NSMakeRect(NSMinX(cellFrame)+2, NSMinY(cellFrame)+2, NSWidth(cellFrame) - 3, NSHeight(cellFrame) - 3);
	NSRectFill(r);
	[super drawInteriorWithFrame:r inView:controlView];
}


- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)aView
	{
	NSRect r[2];

	PSsetgray( NSLightGray );
	(r[0]) = NSMakeRect(NSMinX(cellFrame), NSMinY(cellFrame), 1, NSHeight(cellFrame));
	(r[1]) = NSMakeRect(NSMinX(cellFrame) + 1, NSMinY(cellFrame), NSWidth(cellFrame) - 1, 1);
	NSRectFillList(r, 2);
	
	PSsetgray( NSWhite );
	(r[0]) = NSMakeRect(NSMinX(cellFrame) + 1, NSMinY(cellFrame) + 1, 1, NSHeight(cellFrame) - 2);
	(r[1]) = NSMakeRect(NSMinX(cellFrame) + 2, NSMinY(cellFrame) + 1, NSWidth(cellFrame) - 3, 1);
	NSRectFillList(r, 2);

	PSsetgray( NSBlack );
	(r[0]) = NSMakeRect(NSMaxX(cellFrame) - 1, NSMinY(cellFrame) + 2, 1, NSHeight(cellFrame) - 2);
	(r[1]) = NSMakeRect(NSMinX(cellFrame) + 2, NSMaxY(cellFrame) - 1, NSWidth(cellFrame) - 3, 1);
	NSRectFillList(r, 2);

	PSsetgray( NSDarkGray );
	(r[0]) = NSMakeRect(NSMinX(cellFrame)+1, NSMaxY(cellFrame) - 1, 1, 1);
	(r[1]) = NSMakeRect(NSMaxX(cellFrame) - 1, NSMinY(cellFrame) + 1, 1, 1);
	NSRectFillList(r, 2);

	[self drawInteriorWithFrame:cellFrame inView:aView];
}


// - highlight:(NXRect const*)cellFrame inView:controlView lit:(BOOL)flag
//	   {
// //	  if (cFlags1.highlighted != flag)
// //	{
// //	cFlags1.highlighted = flag;
// //	[self drawInside:cellFrame inView:controlView];
// //	}
//	   cFlags1.highlighted = flag;
//	   [self drawInside:cellFrame inView:controlView];
//	   return self;
//	   }

@end
