//=============================================================================
//
//		Copyright (C) 1995 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscTableFocus.M
//
//		Blinking "cursor" for showing keyboard focus in TableView.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableFocus.M,v 1.1 95/09/27 12:21:21 zarnuk Exp $
// $Log:		MiscTableFocus.M,v $
//	Revision 1.1  95/09/27	12:21:21  zarnuk
//	Initial revision
//	
//-----------------------------------------------------------------------------

#import "MiscTableFocus.h"

extern "Objective-C" {
#import <AppKit/NSApplication.h>
#import <AppKit/NSView.h>
}
extern "C" {
#import "MiscTableFocusPS.h"
}

double const BLINK = 0.4;		// seconds
enum { FOCUS_BLACK = NO, FOCUS_WHITE = YES };


@implementation MiscTableFocus

//-----------------------------------------------------------------------------
// initOwner:
//-----------------------------------------------------------------------------
- initOwner: (NSView *)o
	{
	[super init];
	owner = o;
	frame = [owner bounds];
	state = FOCUS_BLACK;
	return self;
	}


//-----------------------------------------------------------------------------
// free
//-----------------------------------------------------------------------------
- (void)dealloc
	{
	if (timer != 0)
		[timer invalidate]; [timer release];;
	{ [super dealloc]; return; };
	}


//-----------------------------------------------------------------------------
// canDraw
//-----------------------------------------------------------------------------
- (BOOL) canDraw
	{
	return ([self isShowingFocus] && frame.size.width > 0 &&
				frame.size.height > 0 && [owner canDraw]);
	}


//-----------------------------------------------------------------------------
// drawFocus
//-----------------------------------------------------------------------------
- (void) drawFocus
	{
	if ([self canDraw])
		{
#error ViewConversion: '-focusView' in NSApplication has been replaced by '+focusView' in NSView
		BOOL needFocus = (!([NSView focusView] == owner));
		if (needFocus) [owner lockFocus];
		MISC_TF_drawBox( frame.origin.x, frame.origin.y,
						 frame.size.width, frame.size.height,
						 state ? NSWhite : NSBlack );
		if (needFocus) [owner unlockFocus];
		}
	}


//-----------------------------------------------------------------------------
// eraseFocus
//-----------------------------------------------------------------------------
- (void) eraseFocus
	{
	if ([self canDraw])
		{
#error ViewConversion: '-focusView' in NSApplication has been replaced by '+focusView' in NSView
		BOOL needFocus = (!([NSView focusView] == owner));
		if (needFocus) [owner lockFocus];
		MISC_TF_eraseBox( frame.origin.x, frame.origin.y,
						  frame.size.width, frame.size.height );
		if (needFocus) [owner unlockFocus];
		}
	}


//-----------------------------------------------------------------------------
// setFrame:
//-----------------------------------------------------------------------------
- (void)setFrame:(NSRect)r
	{
	[self eraseFocus];
	frame = r;
	[self drawFocus];
}


//-----------------------------------------------------------------------------
// blinker
//-----------------------------------------------------------------------------
- (void) blinker
	{
	state = !state;
	[self drawFocus];
	}

static void blinker( NSTimer *te, double now, void* userData )
	{
	[id(userData) blinker];
	}


//-----------------------------------------------------------------------------
// startFocus
//-----------------------------------------------------------------------------
- (void) startFocus
	{
	if (![self isShowingFocus])
		{
#error DPSConversion: timer should be converted from a DPSTimedEntry to an NSTimer object
		
#error DPSConversion: 'timerWithTimeInterval:target:selector:userInfo:repeats:' used to be DPSAddTimedEntry(BLINK, blinker, self, NSRunModalThreshold).  blinker should be converted to a method with one argument (the timer), and <target> should be converted to the implementor of that method ([aTarget aSelector:timer]).  The argument to userInfo: needs to be converted to an object and can be retrieved with [timer userInfo].
	{
	timer = [[NSTimer timerWithTimeInterval:BLINK target:<target> selector:blinker userInfo:self repeats:YES] retain];
	[[NSRunLoop currentRunLoop] addTimer:timer forMode:NSDefaultRunLoopMode];
	[[NSRunLoop currentRunLoop] addTimer:timer forMode:NSModalPanelRunLoopMode];
	}
		state = FOCUS_BLACK;
		[self drawFocus];
		}
	}


//-----------------------------------------------------------------------------
// stopFocus
//-----------------------------------------------------------------------------
- (void) stopFocus
	{
	if ([self isShowingFocus])
		{
		[self eraseFocus];
		[timer invalidate]; [timer release];;
		timer = 0;
		}
	}


//-----------------------------------------------------------------------------
// isShowingFocus
//-----------------------------------------------------------------------------
- (BOOL) isShowingFocus
	{
	return (timer != 0);
	}

@end
