//=============================================================================
//
//		Copyright (C) 1995 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscTableScrollInspector.M
//
//		Interface Builder inspector for MiscTableScroll.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableScrollInspector.M,v 1.6 95/10/20 00:13:20 sunshine Exp $
// $Log:		MiscTableScrollInspector.M,v $
//	Revision 1.6  95/10/20	00:13:20  sunshine
//	Was including MiscTableScroll.h with "" instead of <>.
//	
//	Revision 1.5  95/10/19	05:40:38  sunshine
//	Fixed bugs and inter-locks.	 Now does more sensible enabling of controls.
//	For instance "sizable" is enabled even when there is a uniform size.  Now
//	if the user turns on "sizable" it sets uniform size to 0 and disables the
//	field (this is more convenient since in the past the user had to manually
//	set it to 0 before being able to turn on the switch).
//	
//	Revision 1.4  95/10/03	23:20:46  zarnuk
//	comments.
//-----------------------------------------------------------------------------
#import "MiscTableScrollInspector.h"
#import "MiscTableBorder.h"
#import <misckit/MiscTableScroll.h>

extern "Objective-C" {
#import <AppKit/NSApplication.h>
#import <AppKit/NSButton.h>
#import <AppKit/NSButtonCell.h>
#import <AppKit/NSMatrix.h>
#import <AppKit/NSColorWell.h>
#import <AppKit/NSPopUpButton.h>
#import <AppKit/NSScrollView.h>
#import <AppKit/NSTextField.h>
}

//=============================================================================
// IB-ONLY METHODS FOR MiscTableScroll
//=============================================================================
@interface MiscTableScroll(IB)
- (char const*)getInspectorClassName;
- editSelf: sender in: owner;
- (BOOL) isInIB;
- (void) initIB;
@end
@implementation MiscTableScroll(IB)
- (char const*)getInspectorClassName
	{
	return "MiscTableScrollInspector";
	}

- editSelf: sender in: owner // keep IB from allowing double-click to edit SV
	{
	return nil;
	}

- (BOOL) isInIB { return YES; }

- (void) initIB
	{
	[self addColumn];
	[self addColumn];
	[self setCol: 1 autosize: YES];
	[self tile];
	}
@end

//=============================================================================
// CONVENIENCE CATAGORIES
//=============================================================================
@interface NSButton(MiscPopUpList)
- (int) popUpTag;
- (void) setPopUpTag: (int) new_tag;
@end

@interface NSMatrix(MiscPopUpList)
- (int) popUpTag;
@end

@implementation NSButton(MiscPopUpList)

- (int) popUpTag
	{
	id popUp = [self target];
#warning PopUpConversion: This message should be sent to an NSPopUpButton, but is probably being sent to an NSPopUpList
#warning PopUpConversion: Consider NSPopUpButton methods instead of using itemMatrix to access items in a pop-up list.
	return [[[popUp itemMatrix] selectedCell] tag];
	}

- (void) setPopUpTag: (int) new_tag
	{
	id popUp = [self target];
#warning PopUpConversion: This message should be sent to an NSPopUpButton, but is probably being sent to an NSPopUpList
#warning PopUpConversion: Consider NSPopUpButton methods instead of using itemMatrix to access items in a pop-up list.
	NSMatrix* matrix = [popUp itemMatrix];
	NSMenuItem* m_cell = [matrix cellWithTag:new_tag];
	[matrix selectCell: m_cell];
	[self setTitle:[m_cell title]];
	}

@end

@implementation NSMatrix(MiscPopUpList)

- (int) popUpTag
	{
	return [[self selectedCell] tag];
	}

@end


//=============================================================================
// IMPLEMENTATION
//=============================================================================
@interface MiscTableScrollInspector(Forward)
- doSlot:sender;
- doTitleMode:sender;
@end

@implementation MiscTableScrollInspector

//-----------------------------------------------------------------------------
// init
//-----------------------------------------------------------------------------
- init
	{
	[super init];
	char buf[ MAXPATHLEN + 1 ];
	id bundle = [NSBundle bundleForClass: [MiscTableScrollInspector class]];
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
	buf = [bundle pathForResource:[[self class] name] ofType:@"nib"];
	[NSBundle loadNibFile:[NSString stringWithCString:buf] externalNameTable:[NSDictionary dictionaryWithObjectsAndKeys:self, @"NSOwner", nil] withZone:[self zone]];
	border_type = MISC_COL_BORDER;
	dirty = NO;
	return self;
	}


//-----------------------------------------------------------------------------
// wantsButtons
//-----------------------------------------------------------------------------
- (BOOL) wantsButtons
	{
	return NO;
	}


//-----------------------------------------------------------------------------
// setTitleControls
//-----------------------------------------------------------------------------
- (void) setTitleControls
	{
	if ([object borderTitlesOn:border_type])
		{
		BOOL sizable = border->isSizeable();
		[titlesSwitch setState:1];
		[sizableSwitch setEnabled:YES];
		[sizableSwitch setState:sizable];
		[draggableSwitch setEnabled:YES];
		[draggableSwitch setState:border->isDraggable()];
		[titleModePopUp setEnabled:YES];
		[titleModePopUp setPopUpTag:(int)[object borderTitleMode:border_type]];
		[uniformSizeField setEnabled:!sizable];
		if (sizable)
			[uniformSizeField setStringValue:@""];
		else
			[uniformSizeField setIntValue:int([object uniformSizeSlots:border_type])];
		}
	else
		{
		[titlesSwitch setState:0];
		[sizableSwitch setEnabled:NO];
		[sizableSwitch setState:0];
		[draggableSwitch setEnabled:NO];
		[draggableSwitch setState:0];
		[titleModePopUp setEnabled:NO];
		[titleModePopUp setPopUpTag:(int)[object borderTitleMode:border_type]];
		[uniformSizeField setEnabled:YES];
		[uniformSizeField setIntValue:int([object uniformSizeSlots:border_type])];
		}
	}


//-----------------------------------------------------------------------------
// fillScroll
//-----------------------------------------------------------------------------
- (void) fillScroll
	{
	[window disableFlushWindow];
	num_slots = [object numSlots:border_type];
	if (num_slots == 0)
		{
		[slotScroll deselectAllCells];
		[slotScroll renewRows:0 columns:0]; // Gets rid of horiz scroller knob
		}
	else
		{
		[slotScroll renewRows:num_slots columns:1];
		for (int i = 0;	 i < num_slots;	 i++)
			{
			int const pslot = [object border:border_type slotAtPosition:i];
			[[slotScroll cellAtRow:i column:0] setTitle:[NSString stringWithCString:[object border:border_type
						slotTitle:pslot]]];
			}
		}
	[slotScroll sizeToCells];
	[slotScroll display];

	if (num_slots > 0)
		{
		if (slot < 0) slot = 0;
		[slotScroll selectCellAtRow:slot column:0];
		[slotScroll scrollCellToVisibleAtRow:slot column:0];
		}
	else
		{
		slot = -1;
		}
	[self doSlot:self];
	[window enableFlushWindow];
	[window flushWindow];
	}


//-----------------------------------------------------------------------------
// setConstrainSwitches
//-----------------------------------------------------------------------------
- (void) setConstrainSwitches
	{
	[constrainMaxSwitch setState:[object maxTotalSizeIsConstrained:border_type]];
	[constrainMinSwitch setState:[object minTotalSizeIsConstrained:border_type]];
	}



//-----------------------------------------------------------------------------
// doConstrainMax:
//-----------------------------------------------------------------------------
- doConstrainMax:sender
	{
	[object border:border_type
				constrainMaxTotalSize:[constrainMaxSwitch state]];
	return [self ok:sender];
	}



//-----------------------------------------------------------------------------
// doConstrainMin:
//-----------------------------------------------------------------------------
- doConstrainMin:sender
	{
	[object border:border_type
				constrainMinTotalSize:[constrainMinSwitch state]];
	return [self ok:sender];
	}



//-----------------------------------------------------------------------------
// doEnabled:
//-----------------------------------------------------------------------------
- doEnabled: sender
	{
	[object setEnabled:[enabledSwitch state]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doLazy:
//-----------------------------------------------------------------------------
- doLazy: sender
	{
	[object setLazy: [lazySwitch state]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doMode:
//-----------------------------------------------------------------------------
- doMode: sender
	{
	[object setSelectionMode: MiscSelectionMode([modePopUp popUpTag])];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doBorder:
//-----------------------------------------------------------------------------
- doBorder:sender
	{
	MiscBorderType bt = MiscBorderType([borderPopUp popUpTag]);
	if (bt != border_type)
		{
		border_type = bt;
		[self revert:sender];
		}
	return self;
	}


//-----------------------------------------------------------------------------
// doTitles:
//-----------------------------------------------------------------------------
- doTitles: sender
	{
	BOOL const isOn = [titlesSwitch state];
	[object border:border_type setTitlesOn: isOn];
	[self setTitleControls];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doSizable:
//-----------------------------------------------------------------------------
- doSizable: sender
	{
	BOOL const val = [sizableSwitch state];
	[object border:border_type setSizeableSlots:val];
	if (val)
		{
		[uniformSizeField setStringValue:@""];
		[uniformSizeField setEnabled:NO];
		[object border:border_type setUniformSizeSlots:0];
		}
	else
		{
		[uniformSizeField setEnabled:YES];
		[uniformSizeField setIntValue:int([object uniformSizeSlots:border_type])];
		}
	[self doSlot:0];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doDraggable:
//-----------------------------------------------------------------------------
- doDraggable: sender
	{
	[object border:border_type setDraggableSlots:[draggableSwitch state]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doAutoSort:
//-----------------------------------------------------------------------------
- doAutoSort: sender
	{
	[object border:border_type setAutoSortSlots:[autoSortSwitch state]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doTitleMode:
//-----------------------------------------------------------------------------
- doTitleMode:sender
	{
	[object border:border_type
		setTitleMode: (MiscTableTitleMode) [titleModePopUp popUpTag]];
	[self fillScroll];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doDelete:
//-----------------------------------------------------------------------------
- doDelete: sender
	{
	[object border:border_type
				deleteSlotAt:[object border:border_type slotAtPosition:slot]];
	[object tile];
	[object display];
	slot--;
	num_slots--;
	if (slot < 0 && num_slots > 0)
		slot = 0;
	[self fillScroll];
	[self ok:sender];
	return self;
	}


//-----------------------------------------------------------------------------
// doNew:
//-----------------------------------------------------------------------------
- doNew: sender
	{
	[object addSlot:border_type];
	[object tile];
	[object display];
	slot = num_slots;
	[self fillScroll];
	[self ok:sender];
	return self;
	}


//-----------------------------------------------------------------------------
// doUserSizeable:
//-----------------------------------------------------------------------------
- doUserSizeable: sender
	{
	[object border:border_type
				setSlot:[object border:border_type slotAtPosition:slot]
				sizeable:[userSizeableSwitch state]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doExpandToData:
//-----------------------------------------------------------------------------
- doExpandToData: sender
	{
	[object border:border_type
				setSlot:[object border:border_type slotAtPosition:slot]
				expandsToData:[expandToDataSwitch state]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doAutosize:
//-----------------------------------------------------------------------------
- doAutosize: sender
	{
	[object border:border_type
				setSlot:[object border:border_type slotAtPosition:slot]
				autosize:[autosizeSwitch state]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doCellClass:
//-----------------------------------------------------------------------------
- doCellClass: sender
	{
	[object border:border_type
		setSlot:[object border:border_type slotAtPosition:slot]
		cellType: (MiscTableCellStyle) [cellClassPopUp popUpTag]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doSortType:
//-----------------------------------------------------------------------------
- doSortType: sender
	{
	[object border:border_type
		setSlot:[object border:border_type slotAtPosition:slot]
		sortType: (MiscSortType) [sortTypePopUp popUpTag]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doSortDirection:
//-----------------------------------------------------------------------------
- doSortDirection: sender
	{
	[object border:border_type
		setSlot:[object border:border_type slotAtPosition:slot]
		sortDirection: (MiscSortDirection) [sortDirectionPopUp popUpTag]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doSlot:
//-----------------------------------------------------------------------------
- doSlot: sender
	{
	slot = [slotScroll selectedRow];
	if (slot >= 0)
		{
		BOOL const titlesOn = [object borderTitlesOn:border_type];
		BOOL const notUniform = ([object uniformSizeSlots:border_type] == 0);
		int pslot = [object border:border_type slotAtPosition:slot];
		[deleteButton setEnabled:YES];
		[upButton setEnabled:(slot > 0)];
		[downButton setEnabled:(slot < num_slots - 1)];
		[cellClassPopUp setEnabled:YES];
		[cellClassPopUp setPopUpTag:(int)
						[object border:border_type slotCellType:pslot]];
		[sortTypePopUp setEnabled:YES];
		[sortTypePopUp setPopUpTag:(int)
						[object border:border_type slotSortType:pslot]];
		[sortDirectionPopUp setEnabled:YES];
		[sortDirectionPopUp setPopUpTag:(int)
						[object border:border_type slotSortDirection:pslot]];
		[autosizeSwitch setEnabled:notUniform];
		[autosizeSwitch setState:[object border:border_type slotIsAutosize:pslot]];
		[expandToDataSwitch setEnabled:notUniform];
		[expandToDataSwitch setState:[object border:border_type slotExpandsToData:pslot]];
		[userSizeableSwitch setEnabled:titlesOn && notUniform && [object sizeableSlots:border_type]];
		[userSizeableSwitch setState:[object border:border_type slotIsSizeable:pslot]];
		[sizeField setEnabled:notUniform];
		[sizeField setIntValue:(int)
						[object border:border_type slotSize:pslot]];
		[sizeMinField setEnabled:notUniform];
		[sizeMinField setIntValue:(int)
						[object border:border_type slotMinSize:pslot]];
		[sizeMaxField setEnabled:notUniform];
		[sizeMaxField setIntValue:(int)
						[object border:border_type slotMaxSize:pslot]];
		[titleField setEnabled:[object borderTitlesOn:border_type] &&
				([object borderTitleMode:border_type] == MISC_CUSTOM_TITLE)];
		[titleField setStringValue:[NSString stringWithCString:[object border:border_type slotTitle:pslot]]];
		[titleField selectText:self];
		}
	else
		{
		[deleteButton setEnabled:NO];
		[upButton setEnabled:NO];
		[downButton setEnabled:NO];
		[cellClassPopUp setEnabled:YES];
		[cellClassPopUp setPopUpTag:0];
		[sortTypePopUp setEnabled:YES];
		[sortTypePopUp setPopUpTag:0];
		[sortDirectionPopUp setEnabled:YES];
		[sortDirectionPopUp setPopUpTag:0];
		[autosizeSwitch setEnabled:NO];
		[autosizeSwitch setState:0];
		[expandToDataSwitch setEnabled:NO];
		[expandToDataSwitch setState:0];
		[userSizeableSwitch setEnabled:NO];
		[userSizeableSwitch setState:0];
		[sizeField setEnabled:NO];
		[sizeField setStringValue:@""];
		[sizeMinField setEnabled:NO];
		[sizeMinField setStringValue:@""];
		[sizeMaxField setEnabled:NO];
		[sizeMaxField setStringValue:@""];
		[titleField setEnabled:NO];
		[titleField setStringValue:@""];
		}
	return self;
	}


//-----------------------------------------------------------------------------
// doUp:
//-----------------------------------------------------------------------------
- doUp: sender
	{
	if (slot > 0)
		{
		[object border:border_type moveSlotFrom:slot to:slot-1];
		border->clearVMap();
		slot--;
		[self fillScroll];
		[self ok:sender];
		}
	return self;
	}


//-----------------------------------------------------------------------------
// doDown:
//-----------------------------------------------------------------------------
- doDown: sender
	{
	if (slot < num_slots - 1)
		{
		[object border:border_type moveSlotFrom:slot to:slot+1];
		border->clearVMap();
		slot++;
		[self fillScroll];
		[self ok:sender];
		}
	return self;
	}


//-----------------------------------------------------------------------------
// get_pslot
//		Get the physical slot-id of the current slot.
//-----------------------------------------------------------------------------
- (int) get_pslot
	{
	return [object border:border_type slotAtPosition:slot];
	}


//-----------------------------------------------------------------------------
// textDidChange:
//-----------------------------------------------------------------------------
#warning NotificationConversion: 'textDidBeginEditing:' used to be 'textDidChange:'.  This conversion assumes this method is implemented or sent to a delegate of NSText.  If this method was implemented by a NSMatrix or NSTextField textDelegate, use the text notifications in NSControl.h.
- (void)textDidBeginEditing:(NSNotification *)notification
	{
#warning NotificationConversion: if this notification was not posted by NSText (eg. was forwarded by NSMatrix or NSTextField from the field editor to their textDelegate), then the text object is found by [[notification userInfo] objectForKey:@"NSFieldEditor"] rather than [notification object]
	NSText *theText = [notification object];
    dirty = YES;
#warning NotificationConversion: 'textDidBeginEditing:' used to be 'textDidChange:'.  This conversion assumes this method is implemented or sent to a delegate of NSText.  If this method was implemented by a NSMatrix or NSTextField textDelegate, use the text notifications in NSControl.h.
	return [[NSNotificationCenter defaultCenter] postNotificationName:NSTextDidBeginEditingNotification object:theText];
	}


//-----------------------------------------------------------------------------
// textDidEnd:endChar:
//-----------------------------------------------------------------------------
#warning NotificationConversion: 'textDidEndEditing:' used to be 'textDidEnd:'.  This conversion assumes this method is implemented or sent to a delegate of NSText.  If this method was implemented by a NSMatrix or NSTextField textDelegate, use the text notifications in NSControl.h.
- (void)textDidEndEditing:(NSNotification *)notification
	{
#warning NotificationConversion: if this notification was not posted by NSText (eg. was forwarded by NSMatrix or NSTextField from the field editor to their textDelegate), then the text object is found by [[notification userInfo] objectForKey:@"NSFieldEditor"] rather than [notification object]
	NSText *theText = [notification object];
    int whyEnd = [[[notification userInfo] objectForKey:@"NSTextMovement"] intValue];
    if (dirty)
		{
		dirty = NO;
		id field = 0;
		if ([theText respondsToSelector:@selector(delegate)])
			field = [theText delegate];
		if (field == (id)titleField)
			{
			int const pslot = [self get_pslot];
			[object border:border_type setSlot:pslot
											title:[[titleField stringValue] cString]];
			[[slotScroll cellAtRow:slot column:0] setTitle:[titleField stringValue]];
			[slotScroll drawCellAtRow:slot column:0];
			}
		else if (field == (id)uniformSizeField)
			{
			int old_size = (int) [object uniformSizeSlots:border_type];
			int new_size = [uniformSizeField intValue];
			if (new_size != 0)
				{
				if (new_size < MISC_MIN_PIXELS_SIZE)
					new_size = MISC_MIN_PIXELS_SIZE;
				if (new_size > MISC_MAX_PIXELS_SIZE)
					new_size = MISC_MAX_PIXELS_SIZE;
				[uniformSizeField setIntValue:new_size];
				}
			if (old_size != new_size)
				{
				[object border:border_type
						setUniformSizeSlots:(float)new_size];
				[self ok:theText];
				[self revert:0];
				}
			}
		else if (field == (id)sizeField || field == (id)sizeMinField ||
				 field == (id)sizeMaxField)
			{
			int const pslot = [self get_pslot];
			int i = [sizeField intValue];
			int imax = [sizeMaxField intValue];
			int imin = [sizeMinField intValue];
			if (field == (id)sizeMaxField)
				{
				if (imax > MISC_MAX_PIXELS_SIZE) imax = MISC_MAX_PIXELS_SIZE;
				else if (imax < imin) imax = imin;
				[sizeMaxField setIntValue:imax];
				[object border:border_type
						setSlot:pslot maxSize:(float)imax];
				}
			else if (field == (id)sizeMinField)
				{
				if (imin < MISC_MIN_PIXELS_SIZE) imin = MISC_MIN_PIXELS_SIZE;
				else if (imin > imax) imin = imax;
				[sizeMinField setIntValue:imin];
				[object border:border_type
						setSlot:pslot minSize:(float)imin];
				}
	
			if (i < imin || i > imax) // May need to do every time
				{
				if (i < imin) i = imin;
				if (i > imax) i = imax;
				[sizeField setIntValue:i];
				}
			[object border:border_type setSlot:pslot size:(float)i];
			}
		[self ok: theText];
		}
}


//-----------------------------------------------------------------------------
// doColorText
//-----------------------------------------------------------------------------
- doColorText: sender
	{
	[object setTextColor:[sender color]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doColorBack
//-----------------------------------------------------------------------------
- doColorBack: sender
	{
	[object setBackgroundColor:[sender color]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doColorTextH
//-----------------------------------------------------------------------------
- doColorTextH: sender
	{
	[object setHighlightTextColor: [sender color]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// doColorBackH
//-----------------------------------------------------------------------------
- doColorBackH: sender
	{
	[object setHighlightBackgroundColor: [sender color]];
	return [self ok:sender];
	}


//-----------------------------------------------------------------------------
// ok:
//-----------------------------------------------------------------------------
- ok: sender
	{
	return [super ok: sender];
	}


//-----------------------------------------------------------------------------
// revert:
//-----------------------------------------------------------------------------
- revert: sender
	{
	dirty = NO;
	[super revert: sender];
	[window disableFlushWindow];

	border = [object border:border_type];
	[autoSortSwitch setState:[object autoSortSlots:border_type]];
	[self setTitleControls];

	[self setConstrainSwitches];

	[enabledSwitch setState:[object isEnabled]];
	[lazySwitch setState:[object isLazy]];
	[modePopUp setPopUpTag:(int) [object selectionMode]];

	[colorText setColor:[object textColor]];
	[colorTextH setColor:[object highlightTextColor]];
	[colorBack setColor:[object backgroundColor]];
	[colorBackH setColor:[object highlightBackgroundColor]];

	slot = -1;
	[self fillScroll];

	id fr = [window firstResponder];
	if (fr == 0 || fr == window)
		[titleField selectText:self];
	[window enableFlushWindow];
	[window flushWindow];
	return self;
	}

@end
