#ifndef __MiscTableView_h
#define __MiscTableView_h
//=============================================================================
//
//		Copyright (C) 1995 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscTableView.h
//
//		General-purpose 2-D display object that works with the
//		MiscTableScroll to provide row/column sizing and dragging.
//
//		This object is responsible for drawing, mouse and keyboard
//		events in the content portion of the display.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableView.h,v 1.1 95/09/27 12:21:21 zarnuk Exp $
// $Log:		MiscTableView.h,v $
//	Revision 1.1  95/09/27	12:21:21  zarnuk
//	Initial revision
//	
//-----------------------------------------------------------------------------
extern "Objective-C" {
#import <AppKit/NSView.h>
#import <AppKit/dpsOpenStep.h>
#import <AppKit/NSDPSContext.h>
}
#import <misckit/MiscTableTypes.h>

@class MiscMouseTracker, MiscTableFocus, MiscTableScroll, NSCell;
class MiscSparseSet;
class MiscTableBorder;

@interface MiscTableView : NSView
	{
	MiscTableScroll*	scroll;
	MiscTableBorder*	col_border;
	MiscTableBorder*	row_border;
	MiscBorderType		tracker_border;
	MiscMouseTracker*	tracker;
	MiscSparseSet*		oldColSel;
	MiscSparseSet*		oldRowSel;
	MiscTableFocus*		focuser;
	}

- initFrame:(NSRect const*)frameRect
		scroll:(MiscTableScroll*)i_scroll
		colInfo:(MiscTableBorder*)i_col_border
		rowInfo:(MiscTableBorder*)i_row_border;
- (void) adjustSize;

- (void)drawCellAtRow:(int)row column:(int)col;						// Physical coords
- drawRow: (int)row;									// Physical coord
- drawCol: (int)col;									// Physical coord

- (void)scrollCellToVisibleAtRow:(int)row column:(int)col;		// Physical coords
- (void) scrollRowToVisible: (int)row;					// Physical coord
- (void) scrollColToVisible: (int)col;					// Physical coord

- (void) setSelectionMode: (MiscSelectionMode)mode;
- (void) reflectSelection;
- (void) reflectCursor;
- (void) trackBy: (MiscBorderType)b;
- (MiscBorderType) trackingBy;

- (void)mouseDown:(NSEvent *)event;
- (void)keyDown:(NSEvent *)event;

@end

#endif // __MiscTableView_h
