//
//	MiscDimpleCell.m -- An NXBrowserCell subclass that adds a mark image so
//						a list can have an appearance similar to the IB
//						connection inspector list.
//
//		Written by David Fedchenko.  Copyright 1994 by David Fedchenko.
//				Version 1.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "MiscDimpleCell.h"

@implementation MiscDimpleCell

static id MDMarkImage;
static id MDAltMarkImage;

+ (void)initialize
	{
    id	idBundle;
	char buf[MAXPATHLEN + 1];
	
	[super initialize];
	
	idBundle = [NSBundle bundleForClass:[self class]];
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
    buf = [idBundle pathForResource:@"Dimple" ofType:@"tiff"];
    MDMarkImage = [[NSImage alloc] initByReferencingFile:[NSString stringWithCString:buf]];
	
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
    buf = [idBundle pathForResource:@"DimpleH" ofType:@"tiff"];
    MDAltMarkImage = [[NSImage alloc] initByReferencingFile:[NSString stringWithCString:buf]];
	
	return;
	}

+ setMarkImage:anImage
	{
	id idT = MDMarkImage;
	
	MDMarkImage = anImage;
	
	return idT;
	}

+ markImage
	{
	return MDMarkImage;
	}

+ setAltMarkImage:anImage
	{
	id idT = MDAltMarkImage;
	
	MDAltMarkImage = anImage;
	
	return idT;
	}

+ altMarkImage
	{
	return MDAltMarkImage;
	}

- setMarked:(BOOL)flag
	{
	fMarked = flag;
	
	if (fMarked)
		{
		[self setLeaf:YES];
		}
	
	return self;
	}

-(BOOL) marked
	{
	return fMarked;
	}

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
	{
    [super drawInteriorWithFrame:cellFrame inView:controlView];
    
	[self drawMark:&cellFrame];
}

- (void)highlight:(BOOL)lit withFrame:(NSRect)cellFrame inView:(NSView *)aView
	{
	[super highlight:lit withFrame:cellFrame inView:aView];
	
	[self drawMark:&cellFrame];
}

- drawMark:(const NSRect *)cellFrame
	{
	NSRect	rect;
	id		idImage;
	
     if ([self isLeaf])
	 	{
		idImage = ([self state] || [self isHighlighted])
					? [[self class] altMarkImage]
					: [[self class] markImage];
		rect.size = [idImage size];
		rect.origin.x = NSWidth(*cellFrame) - rect.size.width - 5.0;
		rect.origin.y = NSMinY(*cellFrame) + NSHeight(*cellFrame)
						- ((NSHeight(*cellFrame) - rect.size.height) / 2);
		
		if (fMarked)
			{
			[idImage compositeToPoint:rect.origin operation:NSCompositeSourceOver];
			}
		else
			{
			PSsetgray(([self state] || [self isHighlighted])
						? NSWhite : NSLightGray);
			rect.origin.y -= rect.size.height;
			NSRectFill(rect);
			}
		}
	
	return self;
	}

@end
