//
//	MiscDistInspector.m -- Inspector object for MiscDistributor.
//
//		Written by Don Yacktman.  Copyright 1994 by Don Yacktman.
//				Version 1.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "MiscDistInspector.h"
#import "MiscDistributorPrivate.h"
#import "_MiscDistributorConnection.h"

@implementation MiscDistributor(IBSupport)

- (const char *)getInspectorClassName
{
	return "MiscDistributorInspector";
}

- (NSImage *)getIBImage
{
    id	idBundle;
    id	idImage;
	char buf[MAXPATHLEN + 1];
	
	idBundle = [NSBundle bundleForClass:[MiscDistributorInspector class]];
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
    buf = [idBundle pathForResource:@"MiscDistributor" ofType:@"tiff"];
    idImage = [[NSImage alloc] initByReferencingFile:[NSString stringWithCString:buf]];
	
	return idImage;
}

@end

@implementation MiscDistributorInspector

- init
{	
    char buf[MAXPATHLEN + 1];
    id bundle;
	[super init];
    bundle = [NSBundle bundleForClass:[MiscDistributor class]];
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
    buf = [bundle pathForResource:@"MiscDistributorInspector" ofType:@"nib"];
    [NSBundle loadNibFile:[NSString stringWithCString:buf] externalNameTable:[NSDictionary dictionaryWithObjectsAndKeys:self, @"NSOwner", nil] withZone:[self zone]];
	return self;
}

- (BOOL)wantsButtons { return NO; }

- revert:sender
{
	connections = [object _connectionList];
	[self setUpMatrix];
	[self selectRow:0 notify:YES];
	return [super revert:sender];
}

- ok:sender
{
	return [super ok:sender];
}

- setUpMatrix
{
	int i;
	// populate matrix with connections' names
	[connectionMatrix renewRows:[connections count] columns:1];
	for (i=0; i<[connections count]; i++) {
		[[connectionMatrix cellAtRow:i column:0] setTitle:[NSString stringWithCString:[[connections objectAt:i] connectionName]]];
	}
	[connectionMatrix sizeToCells];
	[connectionMatrix display];
	return self;
}

- editName
{
	[window makeFirstResponder:connectionNameText];
	[connectionNameText selectText:self];
	return self;
}

- selectRow:(int)i notify:(BOOL)flag
{
	[connectionMatrix selectCellAtRow:i column:0];
	[self selectionChanged:connectionMatrix];
	[connectionMatrix display];
	[self editName];
	return self;
}

- selectionChanged:sender
{
	// change values of name/dir widgets to reflect selected connection
	selObject = [connections objectAt:[connectionMatrix selectedRow]];
	[connectionNameText setStringValue:[NSString stringWithCString:[selObject connectionName]]];
	if ([selObject direction] == MiscIn) {
		[connectionDirMatrix selectCellAtRow:0 column:0];
	} else if ([selObject direction] == MiscOut) {
		[connectionDirMatrix selectCellAtRow:0 column:1];
	} else if ([selObject direction] == MiscInout) {
		[connectionDirMatrix selectCellAtRow:0 column:2];
	}
	[self editName];
	return self;
}

- addConnection:sender
{
	id newConnection = [[_MiscDistributorConnection alloc] init];
	// add a connection to the list and matrix, and select it in the matrix
	[connections addObject:newConnection];
	[newConnection setConnectionName:"UNTITLED"];
	[newConnection setDirection:MiscInout];
	[self setUpMatrix];
	[self selectRow:([connections count] - 1) notify:YES];
	return self;
}

- changeConnection:sender
{
	int theRow = [connectionMatrix selectedRow];
	// take values of name/dir widgets and change selected connection
	switch ([connectionDirMatrix selectedColumn]) {
		case 0 : [selObject setDirection:MiscIn]; break;
		case 1 : [selObject setDirection:MiscOut]; break;
		case 2 : [selObject setDirection:MiscInout]; break;
		default: [selObject setDirection:MiscIn]; break;
	}
	[selObject setConnectionName:[[connectionNameText stringValue] cString]];
	[self setUpMatrix];
	[self selectRow:theRow notify:NO];
	return self;
}

@end
