//
//	_MiscDistributorConnection.m -- A helper for MiscDistributor.
//
//		Written by Don Yacktman.  Copyright 1994 by Don Yacktman.
//				Version 1.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#define INSUBPROJ
#import "../_MiscDistributorConnection.h"

#define CURRENT_VERSION 1

@implementation _MiscDistributorConnection

+ (void)initialize
{
	// set up class version number and set up the waiting list and
	// distributors dictionary
	if (self == [_MiscDistributorConnection class]) {
		[_MiscDistributorConnection setVersion:CURRENT_VERSION];
	}
	return;
}

- init
{
	connectionName = NULL;
	direction = MiscInout;
	return self;
}

- (void)dealloc
{
	if (connectionName) free(connectionName);
	{ [super dealloc]; return; };
}

- (const char *)connectionName { return connectionName; }
- setConnectionName:(const char *)aString
{
	if (connectionName) free(connectionName);
	if (aString)
			connectionName = NXCopyStringBufferFromZone(aString, (NXZone *)[self zone]);
	return self;
}

- (MiscDCDirection)direction { return direction; }
- setDirection:(MiscDCDirection)aDirection
{
	direction = aDirection;
	return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	int version, d;

	[super initWithCoder:aDecoder];
	version = [aDecoder versionForClassName:@"MiscDistributor"];
	if (version == CURRENT_VERSION) {
		[aDecoder decodeValuesOfObjCTypes:"*i", &connectionName, &d];
		direction = d;
	} else {
		connectionName = "UNTITLED";
		direction = 0;
	}
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	int d = direction;
	[super encodeWithCoder:aCoder];
	[aCoder encodeValuesOfObjCTypes:"*i", &connectionName, &d];
}


@end
