//
//	MiscTeeConnector.m -- an IBConnector that allows a multitude of connections
//						  to spread out from it.
//
//		Written by David Fedchenko.  Copyright 1994 by David Fedchenko.
//				Version 1.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "MiscTeeConnector.h"
#import "MiscTee.h"

@implementation MiscTeeConnector

- setSource:anObject
    {
    idSource = anObject;

    return self;
    }

- setDestination:anObject
    {
    idDest = anObject;

    return self;
    }

- (void)setAction:(SEL)anAction
	{
	selAction = anAction;
}

- source
    {
    return idSource;
    }

- destination
    {
    return idDest;
    }

-(SEL) action
	{
	return selAction;
	}

- establishConnection
    {
    [idSource addConnection:idDest with:selAction];

    return self;
    }

- nibInstantiate
    {
    idSource = [idSource nibInstantiate];
    idDest = [idDest nibInstantiate];
	
    return self;
    }

- renewObject:old to:new
    {
    if (old == idSource)
        {
		idSource = new;
        }
    else if (old == idDest)
        {
		idDest = new;
        }

    return self;
    }

- (id)initWithCoder:(NSCoder *)aDecoder
    {
    [super initWithCoder:aDecoder];
	
	[aDecoder decodeValueOfObjCType:":" at:&selAction];
    idSource = [[aDecoder decodeObject] retain];
    idDest = [[aDecoder decodeObject] retain];

    return self;
    }

- (void)encodeWithCoder:(NSCoder *)aCoder
    {
    [super encodeWithCoder:aCoder];

	[aCoder encodeValueOfObjCType:":" at:&selAction];
    [aCoder encodeObject:idSource];
    [aCoder encodeObject:idDest];
}

- (void)dealloc
    {
    { [super dealloc]; return; };
    }

@end
