//
//	MiscThreeStateButtonCell.m -- cell class for the three state button
//		Written by Don Yacktman Copyright (c) 1993 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//		This is a free object!  Contact the author for the latest version.
//		Don Yacktman, 4279 N. Ivy Lane, Provo, UT, 84604
//		e-mail:  Don_Yacktman@byu.edu
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "MiscThreeStateButtonCell.h"
#import "MiscThreeStateButton.h"

// This would be a hell of a lot easier with NeXT's source code.  Since
// I don't have it, but I want to make sure that the same drawing mechanisms
// are used to render the third state of the button, I do a nasty hack.
// Whenever I want to go to the third state, I fake it and make the button
// think it's in the alternate state (of a normal button) but whenever it
// attempts to get the title or image, I swap in the third state button/image
// before the render and then switch back the true alt image/title upon
// finishing the render.  I use the same trick when the button is being queried
// about it's sizes for drawing areas, etc.  I swap icons and titles in and get
// the size they'd use, then I get the size of the correct icons, and then
// return the MAX of the two.  Without knowing exactly how NeXT does the
// calculations, I have to do a hack like this.  Since I'm essentially re-using
// their code when I take my measurements and do my rendering, that means that
// if it changes in the future, this class will follow the changes and keep
// doing the right thing, whatever that is.  I still think it's ugly, but it is
// a cute trick.

// As usual, ***** denotes an unfinished method.  This class is not at all
// done.  It won't do what it is supposed to do yet, at all!

// Allowing NX_CONTENTS ("show alternate icon/title") as a highlighting mode
// just creates a ton of problems, so it needs to be disallowed.  Below is the
// set of allowable highlight modes to be ANDed with whatever highlight mode
// is sent in, which should remove NX_CONTENTS as a possibility.   -Carl
#define ALLOWABLE_HIGHLIGHTS (NSNoCellMask|NSPushInCellMask|NSChangeGrayCellMask|NSChangeBackgroundCellMask)

@implementation MiscThreeStateButtonCell

- init
{
	id ret = [super init];
	thirdTitle = NULL;
	_thirdImage = nil;
	isCyclic = YES;
	thirdState = NO;

	altClicked = NO;		// Set these hacks correctly :-)  -Carl
	dontIncrement = NO;

	// Disallow NX_CONTENTS as highlight mode  -Carl
	[super setHighlightsBy:([self highlightsBy] & ALLOWABLE_HIGHLIGHTS)];
	return ret;
}

- copyWithZone:(NSZone *)zone
{
	id newClass = [super copyWithZone:zone];
	// copy these two strings so they won't get freed twice or more...
	thirdTitle =
		NXCopyStringBufferFromZone([newClass thirdTitle], (NXZone *)[self zone]);
	trueAltTitle =
		NXCopyStringBufferFromZone([[newClass alternateTitle] cString], (NXZone *)[self zone]);
	return newClass;
}

- (void)dealloc
{
	if (thirdTitle) free(thirdTitle);
//	[_thirdImage free];  // should I do this?
	{ [super dealloc]; return; };
}

- (NSString *)alternateTitle { return [NSString stringWithCString:trueAltTitle]; }

- (void)setAlternateTitle:(NSString *)aString
{
	if (trueAltTitle) free(trueAltTitle);
	trueAltTitle = NXCopyStringBufferFromZone([aString cString], (NXZone *)[self zone]);
}

- (NSImage *)alternateImage { return _trueAltImage; }

- (void)setAlternateImage:(NSImage *)iconName
{
	[self setAlternateImage:iconName];
}

- alternateImage { return _trueAltImage; }

- (void)setAlternateImage:(NSImage *)image
{
//	[_trueAltImage free];  // should I do this?
	_trueAltImage = image;
}

- (const char *)thirdTitle { return thirdTitle; }

- setThirdTitle:(const char *)aString
{
	if (thirdTitle) free(thirdTitle);
	thirdTitle = NXCopyStringBufferFromZone(aString, (NXZone *)[self zone]);
	return self;
}

- (const char *)thirdIcon { return [[_thirdImage name] cString]; }

- setThirdIcon:(const char *)iconName
{
	return [self setThirdImage:[NSImage imageNamed:[NSString stringWithCString:iconName]]];
}

- thirdImage { return _thirdImage; }

- setThirdImage:image
{
//	[_thirdImage free];  // should I do this?
	_thirdImage = image;
	return self;
}

- setButtonType:(NSButtonType)aType
{
	switch (aType) {
		case MISC_CYCLIC_THREE_STATE: isCyclic = YES; break;
		case MISC_PLAIN_THREE_STATE:  isCyclic = NO;  break;
		default: [super setType:aType]; return self;
	}
	return self;
}

- (BOOL)isCyclic { return isCyclic; }

- (NSString *)stringValue
{
	if (thirdState) return @"alt";
	return [super stringValue];
}

- (void)setStringValue:(NSString *)aString
{
	thirdState = NO;
	if (aString) if ([aString length]) thirdState = YES;
	[super setStringValue:aString];
}

- (void)setStringValue:(NSString *)aString
{
	thirdState = NO;
	if (aString) if ([aString length]) thirdState = YES;
	[super setStringValue:aString];
}

- (int)intValue
{
	if (thirdState) return 2;
	return [super intValue];
}

- (void)setIntValue:(int)anInt
{
	if (anInt == 2) thirdState = YES;
	else thirdState = NO;
	[super setIntValue:anInt];
}

- (float)floatValue
{
	if (thirdState) return 2.0;
	return [super floatValue];
}

- (void)setFloatValue:(float)aFloat
{
	if ((aFloat > 1.0) && (aFloat <= 2.0)) thirdState = YES;
	else thirdState = NO;
	[super setFloatValue:aFloat];
}

- (double)doubleValue
{
	if (thirdState) return 2.0;
	return [super doubleValue];
}

- (void)setDoubleValue:(double)aDouble
{
	if ((aDouble > 1.0) && (aDouble <= 2.0)) thirdState = YES;
	else thirdState = NO;
	[super setDoubleValue:aDouble];
}

// Here's where the fun begins with us intercepting and fooling the button.
- (NSRect)drawingRectForBounds:(NSRect)theRect
{	// perform the super method for both alt and third state, then merge rects
	// Note, this may not really be necessary, since the NeXT implementation
	// may use the two methods below to give the answer to this method.  If
	// that's the case, then this method doesn't need the override, since the
	// two methods below take the third state into account.  If they don't we
	// need this method.  The docs don't say one way or the other, and I don't
	// feel like disassembling the Appkit, so at worst, we'll be inefficient
	// here.  This is a good reason for why NeXT should let us look at source,
	// or make MAJOR improvements in the docs!
	NSRect tempRect  = { { 0.0, 0.0 }, { 0.0, 0.0 } };
	icon.bmap.alternate = _thirdImage;
	[self replaceAltTitle:thirdTitle];
	tempRect = [super drawingRectForBounds:tempRect];
	[self replaceAltTitle:trueAltTitle];
	icon.bmap.alternate = _trueAltImage;
	theRect = [super drawingRectForBounds:theRect];
	if (thirdState) {
		icon.bmap.alternate = _thirdImage;
		[self replaceAltTitle:thirdTitle];
	}
	theRect  = NSUnionRect(tempRect , theRect);
	return theRect;
}

- (NSRect)titleRectForBounds:(NSRect)theRect
{ // perform the super method for both alt and third state, then merge rects
	NSRect tempRect  = { { 0.0, 0.0 }, { 0.0, 0.0 } };
	icon.bmap.alternate = _thirdImage;
	[self replaceAltTitle:thirdTitle];
	tempRect = [super titleRectForBounds:tempRect];
	[self replaceAltTitle:trueAltTitle];
	icon.bmap.alternate = _trueAltImage;
	theRect = [super titleRectForBounds:theRect];
	if (thirdState) {
		icon.bmap.alternate = _thirdImage;
		[self replaceAltTitle:thirdTitle];
	}
	theRect  = NSUnionRect(tempRect , theRect);
	return theRect;
}

- (NSRect)imageRectForBounds:(NSRect)theRect
{ // perform the super method for both alt and third state, then merge rects
	NSRect tempRect  = { { 0.0, 0.0 }, { 0.0, 0.0 } };
	icon.bmap.alternate = _thirdImage;
	[self replaceAltTitle:thirdTitle];
	tempRect = [super imageRectForBounds:tempRect];
	[self replaceAltTitle:trueAltTitle];
	icon.bmap.alternate = _trueAltImage;
	theRect = [super imageRectForBounds:theRect];
	if (thirdState) {
		icon.bmap.alternate = _thirdImage;
		[self replaceAltTitle:thirdTitle];
	}
	theRect  = NSUnionRect(tempRect , theRect);
	return theRect;
}


- (NSSize)cellSizeForBounds:(NSRect)aRect
{
	
    NSSize theSize;
    NSSize tempSize  = { 0.0, 0.0 };
	icon.bmap.alternate = _thirdImage;
	[self replaceAltTitle:thirdTitle];
	tempSize = [super cellSizeForBounds:aRect];
	[self replaceAltTitle:trueAltTitle];
	icon.bmap.alternate = _trueAltImage;
	theSize = [super cellSizeForBounds:aRect];
	if (thirdState) {
		icon.bmap.alternate = _thirdImage;
		[self replaceAltTitle:thirdTitle];
	}
	theSize.width  = MAX(theSize.width,  tempSize.width);
	theSize.height = MAX(theSize.height, tempSize.height);
	return theSize;
}

- (void)drawInteriorWithFrame:(NSRect)aRect inView:(NSView *)controlView
{
	if (thirdState) {
		icon.bmap.alternate = _thirdImage;
		[self replaceAltTitle:thirdTitle];
	} else {
		[self replaceAltTitle:trueAltTitle];
		icon.bmap.alternate = _trueAltImage;
	}
	[super drawInteriorWithFrame:aRect inView:controlView];
}

- (void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
	if (thirdState) {
		icon.bmap.alternate = _thirdImage;
		[self replaceAltTitle:thirdTitle];
	} else {
		[self replaceAltTitle:trueAltTitle];
		icon.bmap.alternate = _trueAltImage;
	}
	[super highlight:flag withFrame:cellFrame inView:controlView];
}

#define ALTKEY (theEvent->flags & NSAlternateKeyMask)

#warning RectConversion: 'trackMouse:inRect:ofView:untilMouseUp:' used to be 'trackMouse:inRect:ofView:'.  untilMouseUp == YES when inRect used to be NULL
- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView untilMouseUp:(BOOL)_untilMouseUp
{
	altClicked = (ALTKEY)? YES:NO;
#warning RectConversion: 'trackMouse:inRect:ofView:untilMouseUp:' used to be 'trackMouse:inRect:ofView:'.  untilMouseUp == YES when inRect used to be NULL
	return [super trackMouse:theEvent inRect:cellFrame ofView:controlView untilMouseUp:<mouseup_arg>];
}

- replaceTitle:(const char *)aString
{
	if ([self stringValue]) free([self stringValue]);
	[self stringValue] = NXCopyStringBufferFromZone(aString, (NXZone *)[self zone]);
	return self;
}

- replaceAltTitle:(const char *)aString
{
	if (altContents) free(altContents);
	altContents = NXCopyStringBufferFromZone(aString, (NXZone *)[self zone]);
	return self;
}

- (void)performClick:(id)sender
{
	[super performClick:sender];
}

- performAltClick:sender
{
	altClicked = YES;

	// This is a total hack...  For some reason, whenever -performClick
	// or -performAltClick was called and the current state was 0,
	// -incrementState was called *twice*.  This did not happen when using
	// the mouse, nor did it happen when state was 1 or 2.
	// This did not seem to affect -performClick -- I have no idea why --
	// but this was a problem with -performAltClick.  Therefore, I ask
	// -incrementState to not execute on the first call to it after this
	// method by way of the special case ivar, 'dontIncrement'.   -Carl
	if ([self intValue] == 0) dontIncrement = YES;

	[super performClick:sender];

	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	id [super encodeWithCoder:aCoder]; ret = self;
	[aCoder encodeValuesOfObjCTypes:"**cc", &thirdTitle, &trueAltTitle,
			&isCyclic, &thirdState];
	[aCoder encodeObject:_thirdImage];
	[aCoder encodeObject:_trueAltImage];
	return ret;
	// altClicked and dontIncrement don't really need to be
	// written... they are always NO at this point.  See -read.
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	id ret = [super initWithCoder:aDecoder];
	[aDecoder decodeValuesOfObjCTypes:"**cc", &thirdTitle, &trueAltTitle,
			&isCyclic, &thirdState];
	_thirdImage = [[aDecoder decodeObject] retain];
	_trueAltImage =  [[aDecoder decodeObject] retain];

	// Disallow NX_CONTENTS as highlight mode  -Carl
	[super setHighlightsBy:([self highlightsBy] & ALLOWABLE_HIGHLIGHTS)];

	// The following two ivars are used to inform -incrementState
	// of a couple things.  They are only YES for very brief periods
	// of time, so they really don't need to be archived in -write.
	// Someone would have to be rather diabolical to manage to have
	// -write called when they were YES... :-)   -Carl
	altClicked = NO;
	dontIncrement = NO;
	return ret;
}

#error MatrixAndCellConversion: 'incrementState' is obsolete ; use 'setState:'
- incrementState
{
	int next = 0;
	int curState = [self intValue];
	
	if (dontIncrement) {  //see note in -performAltClick
	  dontIncrement = NO;
	  return self;
	}
	switch (curState) {
		case 0 : next = 1; break;
		case 1 : if (isCyclic) next = 2;
				else next = 0;
			 break;
		case 2 : next = 0; break;
		default: break;
	}
	if (altClicked && (curState < 2)) next = 2;
	[self setIntValue:next];  //disable flushWindow around this??
	altClicked = NO;
  return self;
}

- (void)setHighlightsBy:(int)aType;
{
	// Disallow NX_CONTENTS as highlight mode  -Carl
    [super setHighlightsBy:(aType & ALLOWABLE_HIGHLIGHTS)];
	// Disallow NX_CONTENTS as highlight mode  -Carl
    return self;
}

@end
