//
//	MiscThreeStateButtonCell.h -- cell class for the three state button
//		Written by Don Yacktman Copyright (c) 1993 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//		This is a free object!  Contact the author for the latest version.
//		Don Yacktman, 4279 N. Ivy Lane, Provo, UT, 84604
//		e-mail:  Don_Yacktman@byu.edu
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <AppKit/AppKit.h>

// The three state button...

// the values of these are arbitrary and chosen to avoid conflict
// with the NeXT #defines in ButtonCell.h...  use them with -setType:
#define MISC_CYCLIC_THREE_STATE	100	// cycles through three states
#define MISC_PLAIN_THREE_STATE	101	// alt click for third state

@interface MiscThreeStateButtonCell:NSButtonCell
{
	char *thirdTitle;	// title for the third state
	id _thirdImage;		// image for the third state
	char *trueAltTitle;	// title for the third state
	id _trueAltImage;	// image for the third state
	BOOL isCyclic;		// if not, ALT click gives third state.
	BOOL thirdState;	// if we're in the third state.
	BOOL altClicked;
	BOOL dontIncrement;
}

// the basics
- init;
- copyWithZone:(NSZone *)zone;
- (void)dealloc;

// override to set the "true" instance variables...
- (NSString *)alternateTitle;
- (void)setAlternateTitle:(NSString *)aString;
- (NSImage *)alternateImage;
- (void)setAlternateImage:(NSImage *)iconName;
- alternateImage;
- (void)setAlternateImage:(NSImage *)image;

// setting and getting the third image/title
- (const char *)thirdTitle;
- setThirdTitle:(const char *)aString;
- (const char *)thirdIcon;
- setThirdIcon:(const char *)iconName;
- thirdImage;
- setThirdImage:image;
- setButtonType:(NSButtonType)aType;
- (BOOL)isCyclic;

// Need to override these to provide for the third state
- (NSString *)stringValue;
- (void)setStringValue:(NSString *)aString;
- (void)setStringValue:(NSString *)aString;
- (int)intValue;
- (void)setIntValue:(int)anInt;
- (float)floatValue;
- (void)setFloatValue:(float)aFloat;
- (double)doubleValue;
- (void)setDoubleValue:(double)aDouble;

// these override the drawing to deal with the extra
// image/icon and title
- (NSRect)drawingRectForBounds:(NSRect)theRect;
- (NSRect)titleRectForBounds:(NSRect)theRect;
- (NSRect)imageRectForBounds:(NSRect)theRect;
- (NSSize)cellSizeForBounds:(NSRect)aRect;
- (void)drawInteriorWithFrame:(NSRect)aRect inView:(NSView *)controlView;
- (void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)controlView;
#warning RectConversion: 'trackMouse:inRect:ofView:untilMouseUp:' used to be 'trackMouse:inRect:ofView:'.  untilMouseUp == YES when inRect used to be NULL
- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView untilMouseUp:(BOOL)_untilMouseUp;
- (void)performClick:(id)sender;
- performAltClick:sender;
- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aDecoder;

- replaceAltTitle:(const char *)aString;
- replaceTitle:(const char *)aString;

#error MatrixAndCellConversion: 'incrementState' is obsolete ; use 'setState:'
- incrementState;
- (void)setHighlightsBy:(int)aType;
@end
