//
//	MiscTriStateInspector.m -- inspector class for the three state button
//		Written by Don Yacktman Copyright (c) 1993 by Don Yacktman.
//				Version 1.1.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//		This is a free object!  Contact the author for the latest version.
//		Don Yacktman, 4279 N. Ivy Lane, Provo, UT, 84604
//		e-mail:  Don_Yacktman@byu.edu
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "MiscTriStateInspector.h"
#import <AppKit/AppKit.h>
#import "MiscThreeStateButton.subproj/MiscThreeStateButtonCell.h"
#import "MiscThreeStateButton.subproj/MiscThreeStateButton.h"

@implementation MiscTriStateInspector

- init
{
    char buf[MAXPATHLEN + 1];
    id bundle;
    
    [super init];
    bundle = [NSBundle bundleForClass:[MiscThreeStateButton class]];
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
    buf = [bundle pathForResource:@"TriStateInspector" ofType:@"nib"];
    [NSBundle loadNibFile:[NSString stringWithCString:buf] externalNameTable:[NSDictionary dictionaryWithObjectsAndKeys:self, @"NSOwner", nil] withZone:[self zone]];
    return self;
}

- (BOOL)wantsButtons { return YES; }

- revert:sender
{
	// Put string in text object
	[thirdTitle setStringValue:[NSString stringWithCString:[object thirdTitle]]];
	[thirdIcon setStringValue:[NSString stringWithCString:[object thirdIcon]]];

	// If we are dealing with a button send message to its cell
	if ([object isKindOfClass:[MiscThreeStateButton class]])
		[cyclic setIntValue:[[object cell] isCyclic]];
	// otherwise send direct
	else [cyclic setIntValue:[object isCyclic]];

	return [super revert:sender];
}

- ok:sender
{
	[object setThirdTitle:[[thirdTitle stringValue] cString]];
	[object setThirdIcon:[[thirdIcon stringValue] cString]];
	[object setType:([cyclic intValue] ?
			MISC_CYCLIC_THREE_STATE : MISC_PLAIN_THREE_STATE)];
	return [super ok:sender];
}

@end

@implementation MiscThreeStateButton(IBSupport)

- (const char *)getInspectorClassName
{	// alt-click to bring up the button class inspector instead...
	NSEvent *ev = [[self window] currentEvent];
	if ([ev modifierFlags] & NSAlternateKeyMask) return [super getInspectorClassName];
	else return "MiscTriStateInspector";
}

@end

@implementation MiscThreeStateButtonCell(IBSupport)

- (const char *)getInspectorClassName
{	// alt-click to bring up the button class inspector instead...
	NSEvent *ev = [NSApp currentEvent];
	if ([ev modifierFlags] & NSAlternateKeyMask) 
		return [[MiscThreeStateButton class] getInspectorClassName];
	else return "MiscTriStateInspector";
}

@end
