//
//	MiscSliderCell.h -- a TextFieldCell subclass that adds a SliderCell.
//	   Written by David Fedchenko. Copyright 1994 by David Fedchenko.
//	       		 Version 1.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <AppKit/AppKit.h>

#define MSC_SPACING      4
#define MSC_WIDTH        16
#define MSC_MINWIDTH     30
// these values are the cell tags from the matrix in the inspector
#define MSC_ABOVELEFT    1
#define MSC_ABOVECENTER  2
#define MSC_ABOVERIGHT   3
#define MSC_LEFT         4
#define MSC_RIGHT        5
#define MSC_BELOWLEFT    6
#define MSC_BELOWCENTER  7
#define MSC_BELOWRIGHT   8

@interface MiscSliderCell:NSTextFieldCell
    {
    id      (idSlider);
    id      stringList;
    
    int     layout;
    float   split;
    double  actualValue;
    double  limitMinValue;
    double  limitMaxValue;
    double  limitMinBound;
    double  limitMaxBound;
    BOOL    fExpandMin;
    BOOL    fExpandMax;
    BOOL    fInt;
    
    BOOL    fLoop;
    BOOL    fFeedback;
    BOOL    fValidRect;
    NSRect  rectFrame;
    }

- initTextCell:(NSString *)sz;
- copyWithZone:(NSZone *)zone;

- (void)drawInteriorWithFrame:(NSRect)aRect inView:(NSView *)controlView;
- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView;
- (void)resetCursorRect:(NSRect)cellFrame inView:(NSView *)aView;

- (NSSize)cellSizeForBounds:(NSRect)aRect;
- (void)editWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject event:(NSEvent *)theEvent;
- (NSRect)drawingRectForBounds:(NSRect)theRect;
- (NSRect)titleRectForBounds:(NSRect)theRect;
- (void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)aView;
- (void)selectWithFrame:(NSRect)aRect inView:(NSView *)aView editor:(NSText *)aTextObject delegate:(id)anObject start:(int)selStart length:(int)selLength;
- (void)setEditable:(BOOL)flag;

- (void)setIntValue:(int)anInt;
- (void)setFloatValue:(float)aFloat;
- (void)setDoubleValue:(double)anDouble;
- (void)setStringValue:(NSString *)aString;
-(int) intValue;
-(float) floatValue;
-(double) doubleValue;

- moved:sender;

- (void)setMinValue:(double)value;
- (void)setMaxValue:(double)value;
- setMinBoundary:(double)value;
- setMaxBoundary:(double)value;
-(double) minValue;
-(double) maxValue;
-(double) minBoundary;
-(double) maxBoundary;
- setExpandMin:(BOOL)flag;
- setExpandMax:(BOOL)flag;
-(BOOL) expandMin;
-(BOOL) expandMax;
- setPosition:(int)where;
- setSplit:(int)percent;
- setIntegerOnly:(BOOL)flag;
-(int) position;
-(int) split;
-(BOOL) integerOnly;

- setStringList:anObject;
- stringList;

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aDecoder;
#error ArchiverConversion: put the contents of your 'awake' method at the end of your 'initWithCoder:' method instead
- awake;
- (void)awakeFromNib;

@end

@interface NSObject(MiscValueDelegate)
-(const char *) stringAt:(int)item;
@end
