//
//	MiscSliderCell.m -- a TextFieldCell subclass that adds a SliderCell
//	   Written by David Fedchenko. Copyright 1994 by David Fedchenko.
//	       		 Version 1.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "MiscSliderCell.h"
#import "MiscSliderField.h"

@implementation MiscSliderCell

- initTextCell:(NSString *)sz
    {
    [super initTextCell:sz];

    layout = MSC_LEFT;
    split = 0.5;
    limitMinValue = 0;
    limitMaxValue = 100;
    limitMinBound = limitMinValue;
    limitMaxBound = limitMaxValue;
    fExpandMin = fExpandMax = NO;
    
    fValidRect = NO;
    
    idSlider = [[NSSliderCell allocWithZone:[self zone]] init];
    
    [idSlider setTarget:self];
    
    [idSlider setAction:@selector(moved:)];
    [idSlider setContinuous:YES];
    [idSlider sendActionOn:NSLeftMouseDownMask | NSLeftMouseDraggedMask];
    [idSlider setMinValue:limitMinValue];
    [idSlider setMaxValue:limitMaxValue];
    
#warning VoidConversion:  creating new variable to deal with methods returning void or bool (previously self)
    {
    id newVar = [self setBezeled:YES];
    [newVar setEditable:YES];
    [newVar setContinuous:NO];
}
    
    return self;
    }

- copyWithZone:(NSZone *)zone
    {
    MiscSliderCell * idNew = [super copyWithZone:zone];
    
    if (idNew)
        {
        if ((idNew->idSlider = [idSlider copyWithZone:zone]))
            {
            [idNew->idSlider setTarget:idNew];
            }
        }
    
    return idNew;
    }

- textRect:(NSRect *)rect
    {
    float width = rect->size.width;
    
    switch (layout)
        {
	case MSC_LEFT:
    	    rect->size.width *= split;
	    rect->size.width = MIN(width - MSC_MINWIDTH, rect->size.width);
            rect->size.width -= (MSC_SPACING / 2);
	    break;
	
	case MSC_RIGHT:
	    rect->size.width *= split;
	    rect->size.width = MAX(MSC_MINWIDTH, rect->size.width);
	    rect->size.width = MIN(width - MSC_MINWIDTH, rect->size.width);
            rect->size.width -= (MSC_SPACING / 2);
	    rect->origin.x = width - rect->size.width;
	    break;
	    
	case MSC_BELOWLEFT:
	case MSC_BELOWCENTER:
	case MSC_BELOWRIGHT:
	    // the view is flipped
	    rect->origin.y += MSC_WIDTH + MSC_SPACING;
	    // fall through
	    
	case MSC_ABOVELEFT:
	case MSC_ABOVECENTER:
	case MSC_ABOVERIGHT:
	    // the view is flipped
	    rect->size.height -= MSC_WIDTH + MSC_SPACING;
	    rect->size.width *= split;
	    rect->size.width = MAX(MSC_MINWIDTH, rect->size.width);
	    rect->size.width = MIN(width, rect->size.width);
	    switch (layout)
	        {
		case MSC_ABOVECENTER:
		case MSC_BELOWCENTER:
		    rect->origin.x += (width - rect->size.width) / 2;
		    break;
		
		case MSC_ABOVERIGHT:
		case MSC_BELOWRIGHT:
		    rect->origin.x += width - rect->size.width;
		    break;
		}
	    break;
        }
    
    return self;
    }

- sliderRect:(NSRect *)rect
    {
    float width = rect->size.width;
    
    switch (layout)
        {
	case MSC_LEFT:
	    rect->size.width *= (1 - split);
	    rect->size.width = MAX(MSC_MINWIDTH, rect->size.width);
	    rect->size.width = MIN(width - MSC_MINWIDTH, rect->size.width);
            rect->size.width -= (MSC_SPACING / 2);
	    rect->origin.x = width - rect->size.width;
	    rect->origin.y += (rect->size.height - MSC_WIDTH) / 2;
	    rect->size.height = MSC_WIDTH;
	    break;
	
	case MSC_RIGHT:
	    rect->size.width *= (1 - split);
	    rect->size.width = MAX(MSC_MINWIDTH, rect->size.width);
	    rect->size.width = MIN(width - MSC_MINWIDTH, rect->size.width);
            rect->size.width -= (MSC_SPACING / 2);
	    rect->origin.y += (rect->size.height - MSC_WIDTH) / 2;
	    rect->size.height = MSC_WIDTH;
	    break;
	    
	case MSC_BELOWLEFT:
	case MSC_BELOWCENTER:
	case MSC_BELOWRIGHT:
	    // the view is flipped
	    rect->size.height = MSC_WIDTH;
	    break;
	    
	case MSC_ABOVELEFT:
	case MSC_ABOVECENTER:
	case MSC_ABOVERIGHT:
	    // the view is flipped
	    rect->origin.y += rect->size.height - MSC_WIDTH;
	    rect->size.height = MSC_WIDTH;
	    break;
        }
    
    return self;
    }

- (void)drawInteriorWithFrame:(NSRect)aRect inView:(NSView *)controlView
    {
    if (!fValidRect)
        {
        rectFrame = aRect;
        [self textRect:&rectFrame];
        }
    
    [super drawInteriorWithFrame:rectFrame inView:controlView];
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
    {
    NSRect rectSlider;
    
    rectFrame = cellFrame;
    rectSlider = cellFrame;
    [self textRect:&rectFrame];
    [self sliderRect:&rectSlider];
    
    fValidRect = YES;
    [super drawWithFrame:rectFrame inView:controlView];
    fValidRect = NO;
    
    [idSlider drawWithFrame:rectSlider inView:controlView];
}

- (void)resetCursorRect:(NSRect)cellFrame inView:(NSView *)aView
    {
    NSRect rect = cellFrame;
    
    [self textRect:&rect];
    [super resetCursorRect:rect inView:aView];
}

- (NSSize)cellSizeForBounds:(NSRect)aRect
    {
    
    NSSize theSize;
    theSize = [super cellSizeForBounds:aRect];
    
    switch (layout)
	{
	case MSC_LEFT:
	case MSC_RIGHT:
	    theSize.width = MSC_MINWIDTH * 2 + MSC_SPACING;
	    break;
	    
	case MSC_ABOVELEFT:
	case MSC_ABOVECENTER:
	case MSC_ABOVERIGHT:
	case MSC_BELOWLEFT:
	case MSC_BELOWCENTER:
	case MSC_BELOWRIGHT:
	    theSize.width = MSC_MINWIDTH;
	    theSize.height += MSC_WIDTH + MSC_SPACING;
	    break;
	}

    return theSize;
    }

- (void)editWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject event:(NSEvent *)theEvent
    {
    NSRect  rectT = aRect;
    NSRect  rectS = aRect;
    NSPoint p = theEvent->location;
    
    [self textRect:&rectT];
    [self sliderRect:&rectS];
    
    p = [controlView convertPoint:p fromView:nil];
    
    if (NSPointInRect(p , rectT))
        {
        [super editWithFrame:rectT inView:controlView editor:textObj delegate:anObject event:theEvent];
        }
    else if (NSPointInRect(p , rectS))
        {
	id sl = stringList;
	
	if (!sl)
            {
	    if ([[self controlView] class] == [MiscSliderField class])
	        {
		sl = [[self controlView] stringList];
		}
	    }
    
	if (sl)
	    {
	    [idSlider setMinValue:0];
	    [idSlider setMaxValue:[sl count]];
	    }
	
        [controlView lockFocus];
#error RectConversion: 'trackMouse:inRect:ofView:untilMouseUp:' (used to be 'trackMouse:inRect:ofViewinRect:') should receive cells bounds for 'inRect:'
        [idSlider trackMouse:theEvent inRect:??? ofView:controlView untilMouseUp:YES];
        [controlView unlockFocus];
        
        [NSApp sendAction:[self action] to:[self target] from:controlView];
        }
}

- (NSRect)drawingRectForBounds:(NSRect)theRect
    {
    [self textRect:&theRect];
    theRect = [super drawingRectForBounds:theRect];
    
    return theRect;
    }

- (NSRect)titleRectForBounds:(NSRect)theRect
    {
    [self textRect:&theRect];
    theRect = [super titleRectForBounds:theRect];
    
    return theRect;
    }

- (void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)aView
    {
    NSRect rect = cellFrame;
    
    [self textRect:&rect];
    [super highlight:flag withFrame:rect inView:aView];
}

- (void)selectWithFrame:(NSRect)aRect inView:(NSView *)aView editor:(NSText *)aTextObject delegate:(id)anObject start:(int)selStart length:(int)selLength
    {
    NSRect rect = aRect;
    
    [self textRect:&rect];
    [super selectWithFrame:rect inView:aView editor:aTextObject delegate:anObject start:selStart length:selLength];
}

- (void)setEditable:(BOOL)flag
    {
    [super setEditable:flag];
    
    if (!flag)
        {
        [self setSelectable:YES];
        }
}


- (void)setIntValue:(int)anInt
    {
    [self setDoubleValue:(double)anInt];
}

- (void)setFloatValue:(float)aFloat
    {
    [self setDoubleValue:(double)aFloat];
}

- (void)setDoubleValue:(double)aDouble
    {
    const char * sz;
    id           sl = stringList;
    
    fLoop = YES; // so setStringValue doesn't get loopy
    
    if (!sl)
        {
	if ([[self controlView] class] == [MiscSliderField class])
	    {
	    sl = [[self controlView] stringList];
	    }
        }
    
    if (sl)
	{
	int indx = (int)aDouble;
	int cnt = (int)limitMaxValue;
        
	cnt = [sl count] - 1;
		
	indx = MAX(indx, 0);
	indx = MIN(indx, cnt);
	actualValue = indx;
        
	sz = (char *)[sl stringAt:indx];
	if (sz)
	    {
	    [super setStringValue:[NSString stringWithCString:sz]];
	    }
	else
	    {
	    [super setDoubleValue:actualValue];
       	    }
	}
    else
        {
        aDouble = MIN(aDouble, limitMaxValue);
        aDouble = MAX(aDouble, limitMinValue);
        
        if (!fFeedback)
            {
      	    if (fExpandMin && (aDouble < limitMinBound))
	        {
	        limitMinBound = aDouble;
	        [idSlider setMinValue:aDouble];
	        }
	    
	    if (fExpandMax && (aDouble > limitMaxBound))
	        {
	        limitMaxBound = aDouble;
	        [idSlider setMaxValue:aDouble];
	        }
	    }
        
        if (fInt)
            {
	    aDouble = rint(aDouble);
	    }
    
        actualValue = aDouble;
        [super setDoubleValue:aDouble];
        if (!fFeedback)
            {
	    [idSlider setDoubleValue:aDouble];
	    [[self controlView] updateCell:self];
            }
        }
    
    fLoop = NO;
}

- (void)setStringValue:(NSString *)aString
    {
    double  value;
    
    [super setStringValue:aString];
    
    if (!fLoop)
        {
        sscanf([aString cString], "%lf", &value);
        [self setDoubleValue:value];
        }
}

- (void)takeDoubleValueFrom:(id)sender
    {
    [self setDoubleValue:[sender doubleValue]];
}

-(int) intValue
    {
	[[self controlView] validateEditing];

    return (int)actualValue;
    }

-(float) floatValue
    {
	[[self controlView] validateEditing];

    return (float)actualValue;
    }

-(double) doubleValue
    {
	[[self controlView] validateEditing];

    return actualValue;
    }


- moved:sender
    {
    fFeedback = YES;
    [self setDoubleValue:[idSlider doubleValue]];
    fFeedback = NO;
    
    if ([self isContinuous])
        {
        [NSApp sendAction:[self action] to:[self target] from:[self controlView]];
        }
    
    return self;
    }


- (void)setMinValue:(double)value
    {
    limitMinValue = value;
    
    if (!fExpandMin && !stringList)
        {
	[idSlider setMinValue:value];
	[self setDoubleValue:MAX(actualValue, value)];
	}
}

- (void)setMaxValue:(double)value
    {
    limitMaxValue = value;

    if (!fExpandMax && !stringList)
        {
	[idSlider setMaxValue:value];
	[self setDoubleValue:MIN(actualValue, value)];
	}
}

- setMinBoundary:(double)value
    {
    limitMinBound = value;

    if (fExpandMin && !stringList)
        {
	[idSlider setMinValue:value];
	[self setDoubleValue:MAX(actualValue, value)];
	}
    
    return self;
    }

- setMaxBoundary:(double)value
    {
    limitMaxBound = value;

    if (fExpandMin && !stringList)
        {
	[idSlider setMinValue:value];
	[self setDoubleValue:MIN(actualValue, value)];
	}
    
    return self;
    }

-(double) minValue
    {
    return limitMinValue;
    }

-(double) maxValue
    {
    return limitMaxValue;
    }

-(double) minBoundary
    {
    return limitMinBound;
    }

-(double) maxBoundary
    {
    return limitMaxBound;
    }

- setExpandMin:(BOOL)flag
    {
    fExpandMin = flag;
    
    if (flag)
        {
        if (!stringList)
            {
	    [idSlider setMinValue:limitMinBound];
	    [self setDoubleValue:MAX(actualValue, limitMinBound)];
            }
	}
    else
        {
        if (!stringList)
            {
            [idSlider setMinValue:limitMinValue];
	    [self setDoubleValue:actualValue];
            }
	}
    
    return self;
    }

- setExpandMax:(BOOL)flag
    {
    fExpandMax = flag;
    
    if (flag)
        {
        if (!stringList)
            {
	    [idSlider setMaxValue:limitMaxBound];
	    [self setDoubleValue:MIN(actualValue, limitMaxBound)];
            }
	}
    else
        {
        if (!stringList)
            {
            [idSlider setMaxValue:limitMaxValue];
	    [self setDoubleValue:actualValue];
            }
	}
    
    return self;
    }

-(BOOL) expandMin
    {
    return fExpandMin;
    }

-(BOOL) expandMax
    {
    return fExpandMax;
    }

- setPosition:(int)where
    {
    layout = where;
    
    return self;
    }

- setSplit:(int)percent
    {
    split = percent / 100.0;
    
    return self;
    }

- setIntegerOnly:(BOOL)flag
    {
    fInt = flag;
    
    return self;
    }

-(int) position
    {
    return layout;
    }

-(int) split
    {
    return (int)(split * 100);
    }

-(BOOL) integerOnly
    {
    return fInt;
    }


- setStringList:anObject
    {
    if (anObject && [anObject respondsToSelector:@selector(stringAt:)] &&
        [anObject respondsToSelector:@selector(count)])
        {
        stringList = anObject;
	[self setEditable:NO];
        [self setDoubleValue:actualValue];
        }
    else
        {
        [self setMinBoundary:limitMinBound];
        [self setMaxBoundary:limitMaxBound];
        [self setMinValue:limitMinValue];
        [self setMaxValue:limitMaxValue];
        }
    
    return self;
    }

- stringList
    {
    return stringList;
    }


- (void)encodeWithCoder:(NSCoder *)aCoder
    {
    [super encodeWithCoder:aCoder];
    
    [aCoder encodeValuesOfObjCTypes:"ifd", &layout, &split, &actualValue];
    [aCoder encodeValuesOfObjCTypes:"dd", &limitMinValue, &limitMaxValue];
    [aCoder encodeValuesOfObjCTypes:"dd", &limitMinBound, &limitMaxBound];
    [aCoder encodeValueOfObjCType:@encode(BOOL) at:&fExpandMin];
    [aCoder encodeValueOfObjCType:@encode(BOOL) at:&fExpandMax];
    [aCoder encodeValueOfObjCType:@encode(BOOL) at:&fInt];
    [aCoder encodeObject:idSlider];
}

- (id)initWithCoder:(NSCoder *)aDecoder
    {
    [super initWithCoder:aDecoder];
    
    [aDecoder decodeValuesOfObjCTypes:"ifd", &layout, &split, &actualValue];
    [aDecoder decodeValuesOfObjCTypes:"dd", &limitMinValue, &limitMaxValue];
    [aDecoder decodeValuesOfObjCTypes:"dd", &limitMinBound, &limitMaxBound];
    [aDecoder decodeValueOfObjCType:@encode(BOOL) at:&fExpandMin];
    [aDecoder decodeValueOfObjCType:@encode(BOOL) at:&fExpandMax];
    [aDecoder decodeValueOfObjCType:@encode(BOOL) at:&fInt];
    idSlider = [[aDecoder decodeObject] retain];
    
    return self;
    }

#error ArchiverConversion: put the contents of your 'awake' method at the end of your 'initWithCoder:' method instead
- awake
    {
    [self setDoubleValue:actualValue];
    
    return self;
    }

- (void)awakeFromNib
    {
#error ArchiverConversion: put the contents of your 'awake' method at the end of your 'initWithCoder:' method instead
    [self awake];
}

@end
