//
//	MiscSliderField.m -- a TextField subclass that specializes in using
//                           the MiscSliderCell.
//	   Written by David Fedchenko. Copyright 1994 by David Fedchenko.
//	       		 Version 1.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "MiscSliderField.h"
#import "MiscSliderCell.h"

@implementation MiscSliderField

static id _idMiscSliderCellClass;

+ (void)initialize
    {
    /* Class initialization code. */
    if (self == [MiscSliderField class])
        {
        _idMiscSliderCellClass = [MiscSliderCell class];  // Default class
        }
    
    return;
    }

+ (void) setCellClass:(Class)classId
    {
    _idMiscSliderCellClass = classId;
    
    return;
    }

- initWithFrame:(NSRect)frameRect
    {
    id oldCell;

    [super initWithFrame:frameRect];
    oldCell = [self cell];
    [self setCell:[[_idMiscSliderCellClass alloc] initTextCell:@"0"]];
    [oldCell release];

    return self;
    }

// the methods to communicate with the cell
- (void)setMinValue:(double)value
    {
    [[self cell] setMinValue:value];
}

- (void)setMaxValue:(double)value
    {
    [[self cell] setMaxValue:value];
}

- setMinBoundary:(double)value
    {
    [[self cell] setMinBoundary:value];
    
    return self;
    }

- setMaxBoundary:(double)value
    {
    [[self cell] setMaxBoundary:value];
    
    return self;
    }

-(double) minValue
    {
    return [(MiscSliderCell *)[self cell] minValue];
    }

-(double) maxValue
    {
    return [(MiscSliderCell *)[self cell] maxValue];
    }

-(double) minBoundary
    {
    return [[self cell] minBoundary];
    }

-(double) maxBoundary
    {
    return [[self cell] maxBoundary];
    }

- setExpandMin:(BOOL)flag
    {
    [[self cell] setExpandMin:flag];
    
    return self;
    }

- setExpandMax:(BOOL)flag
    {
    [[self cell] setExpandMax:flag];
    
    return self;
    }

-(BOOL) expandMin
    {
    return [[self cell] expandMin];
    }

-(BOOL) expandMax
    {
    return [[self cell] expandMax];
    }

- setPosition:(int)where
    {
    [[self cell] setPosition:where];
    
    return self;
    }

- setSplit:(int)percent
    {
    [[self cell] setSplit:percent];
    
    return self;
    }

- setIntegerOnly:(BOOL)flag
    {
    [[self cell] setIntegerOnly:flag];
    
    return self;
    }

-(int) position
    {
    return [[self cell] position];
    }

-(int) split
    {
    return [[self cell] split];
    }

-(BOOL) integerOnly
    {
    return [[self cell] integerOnly];
    }


- setStringList:anObject
    {
    stringList = anObject;
    
    [[self cell] setStringList:anObject];
    
    return self;
    }

- stringList
    {
    if ([[self cell] stringList])
        {
	return [[self cell] stringList];
	}
    else
        {
	return stringList;
	}
    }

@end
