//
//	MiscValueCellInspector.m -- an IBInspector that edits a MiscValueCell
//                                  or its MiscValueField.
//	   Written by David Fedchenko. Copyright 1994 by David Fedchenko.
//	       		 Version 1.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "MiscValueCellInspector.h"

@implementation MiscValueCellInspector

- init
    {
    char    buf[MAXPATHLEN + 1];
    id      bundle;
    
    [super init];
    
    bundle = [NSBundle bundleForClass:[MiscValueCell class]];
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
    buf = [bundle pathForResource:@"MiscValueCellInspector" ofType:@"nib"];
    [NSBundle loadNibFile:[NSString stringWithCString:buf] externalNameTable:[NSDictionary dictionaryWithObjectsAndKeys:self, @"NSOwner", nil] withZone:[self zone]];
    
    return self;
    }

- updateControls
    {
    [idSoftLower setEnabled:[idExpandLower state]];
    [idSoftLowerLabel setEnabled:[idExpandLower state]];
    [idSoftUpper setEnabled:[idExpandUpper state]];
    [idSoftUpperLabel setEnabled:[idExpandUpper state]];

    return self;
    }

- ok:sender
    {
    [object setMinValue:[idHardLower doubleValue]];
    [object setMaxValue:[idHardUpper doubleValue]];
    [object setMinBoundary:[idSoftLower doubleValue]];
    [object setMaxBoundary:[idSoftUpper doubleValue]];
    [object setStepSize:[idNormal doubleValue]];
    [object setAltStepSize:[idAlternate doubleValue]];
    [object setExpandMin:[idExpandLower state]];
    [object setExpandMax:[idExpandUpper state]];
    [object setContinuous:[idContinuous state]];
    
    [self updateControls];

    return [super ok:sender];
    }

- revert:sender
    {
    // there is casting here because the NeXT headers have a type conflict
    // for these two methods.
    [idHardLower setDoubleValue:[(MiscValueField *)object minValue]];
    [idHardUpper setDoubleValue:[(MiscValueField *)object maxValue]];
    [idSoftLower setDoubleValue:[object minBoundary]];
    [idSoftUpper setDoubleValue:[object maxBoundary]];
    [idNormal setDoubleValue:[object stepSize]];
    [idAlternate setDoubleValue:[object altStepSize]];
    [idExpandLower setState:[object expandMin]];
    [idExpandUpper setState:[object expandMax]];
    [idContinuous setState:[object isContinuous]];
    
    [self updateControls];

    return [super revert:sender];
    }

- (BOOL)wantsButtons
    {
    return NO;
    }

// the text delegate functions to get the info more better
#warning NotificationConversion: 'textDidEndEditing:' used to be 'textDidEnd:'.  This conversion assumes this method is implemented or sent to a delegate of NSText.  If this method was implemented by a NSMatrix or NSTextField textDelegate, use the text notifications in NSControl.h.
- (void)textDidEndEditing:(NSNotification *)notification 
    {
#warning NotificationConversion: if this notification was not posted by NSText (eg. was forwarded by NSMatrix or NSTextField from the field editor to their textDelegate), then the text object is found by [[notification userInfo] objectForKey:@"NSFieldEditor"] rather than [notification object]
    NSText *theText = [notification object];
    int whyEnd = [[[notification userInfo] objectForKey:@"NSTextMovement"] intValue];
    [self ok:theText];
}

@end

@implementation MiscValueField (AttributesInspector)

- (const char *)getInspectorClassName
    {
    NSEvent *ev;

    ev = [[self window] currentEvent];
    if ([ev modifierFlags]&NSAlternateKeyMask) // get the superclass inspector
        return [super getInspectorClassName];
    else // the real inspector
        return "MiscValueCellInspector";
    }

@end

@implementation MiscValueCell (AttributesInspector)

- (const char *)getInspectorClassName
    {
    NSEvent *ev;

    ev = [NSApp currentEvent];
    if ([ev modifierFlags]&NSAlternateKeyMask) // get the superclass inspector
        return [super getInspectorClassName];
    else // the real inspector
        return "MiscValueCellInspector";
    }

@end
