//
//	MiscValueCell.h -- a TextFieldCell subclass that adds arrow buttons
//	   Written by David Fedchenko. Copyright 1994 by David Fedchenko.
//	       		 Version 2.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <AppKit/AppKit.h>

@interface MiscValueCell:NSTextFieldCell
    {
    id      (idUp);
    id      (idDown);
    
    id      stringList;
    
    double  actualValue;
    double  limitMinValue;
    double  limitMaxValue;
    double  limitMinBound;
    double  limitMaxBound;
    double  sizeStep;
    double  sizeAltStep;
    BOOL    fExpandLow;
    BOOL    fExpandHigh;
    
    BOOL    fLoop;
    BOOL    fValidRect;
    int     fAltKeyDown;
    NSRect  rectFrame;
    }

- initTextCell:(NSString *)sz;
- copyWithZone:(NSZone *)zone;

- (void)drawInteriorWithFrame:(NSRect)aRect inView:(NSView *)controlView;
- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView;
- (void)resetCursorRect:(NSRect)cellFrame inView:(NSView *)aView;

- (NSSize)cellSizeForBounds:(NSRect)aRect;
- (void)editWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject event:(NSEvent *)theEvent;
- (NSRect)drawingRectForBounds:(NSRect)theRect;
- (NSRect)titleRectForBounds:(NSRect)theRect;
- (void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)aView;
- (void)selectWithFrame:(NSRect)aRect inView:(NSView *)aView editor:(NSText *)aTextObject delegate:(id)anObject start:(int)selStart length:(int)selLength;
- (void)setBezeled:(BOOL)flag;
- (void)setBordered:(BOOL)flag;
- (void)setEditable:(BOOL)flag;

- (void)setDoubleValue:(double)aDouble;
- (void)setFloatValue:(float)aFloat;
- (void)setIntValue:(int)anInt;
- (void)setStringValue:(NSString *)aString;
-(int) intValue;
-(float) floatValue;
-(double) doubleValue;

- increment:sender;
- decrement:sender;

- (void)setMinValue:(double)value;
- (void)setMaxValue:(double)value;
- setMinBoundary:(double)value;
- setMaxBoundary:(double)value;
-(double) minValue;
-(double) maxValue;
-(double) minBoundary;
-(double) maxBoundary;
- setStepSize:(double)size;
- setAltStepSize:(double)size;
-(double) stepSize;
-(double) altStepSize;
- setExpandMin:(BOOL)flag;
- setExpandMax:(BOOL)flag;
-(BOOL) expandMin;
-(BOOL) expandMax;

- setStringList:anObject;
- stringList;

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aDecoder;
#error ArchiverConversion: put the contents of your 'awake' method at the end of your 'initWithCoder:' method instead
- awake;
- (void)awakeFromNib;

@end

@interface NSObject(MiscValueDelegate)
-(const char *) stringAt:(int)item;
@end
