//
//	MiscValueCell.m -- a TextFieldCell subclass that adds arrow buttons
//	   Written by David Fedchenko. Copyright 1994 by David Fedchenko.
//	       		 Version 2.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "MiscValueCell.h"
#import "MiscValueField.h"

@implementation MiscValueCell

- initTextCell:(NSString *)sz
    {
    [super initTextCell:sz];
    
    actualValue = 0;
    limitMinValue = 0;
    limitMaxValue = 100;
    limitMinBound = 0;
    limitMaxBound = 100;
    fExpandLow = NO;
    fExpandHigh = NO;
    sizeStep = 1;
    sizeAltStep = 10;
    fLoop = fValidRect = NO;
    
    // Set up the NXImages so they can draw without the files.
    // This only works it's magic for cells set up in IB since
    // a regular initialization will look for the images.
    if (![NSImage imageNamed:@"MVCUpImage"])
        {
        char buf[MAXPATHLEN + 1];
        NSImage * idImage;
        
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
        buf = [[NSBundle bundleForClass:[self class]] pathForResource:@"MVCUp" ofType:@"tiff"];
        idImage = [[NSImage alloc] init];
        [idImage setDataRetained:YES];
        [idImage addRepresentations:[NSImageRep imageRepsWithContentsOfFile:[NSString stringWithCString:buf]]];
        [idImage setName:@"MVCUpImage"];
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called buf will be changed into an (NSString *).  You must change the type of the variable called buf by hand.
        buf = [[NSBundle bundleForClass:[self class]] pathForResource:@"MVCDown" ofType:@"tiff"];
        idImage = [[NSImage alloc] init];
        [idImage setDataRetained:YES];
        [idImage addRepresentations:[NSImageRep imageRepsWithContentsOfFile:[NSString stringWithCString:buf]]];
        [idImage setName:@"MVCDownImage"];
        }
    
    idUp = [[NSButtonCell allocWithZone:[self zone]] initImageCell:[NSImage imageNamed:@"MVCUpImage"]];
    
    [idUp setTarget:self];
    
    [idUp setAction:@selector(increment:)];
    [idUp setContinuous:YES];
    [idUp sendActionOn:NSLeftMouseDownMask | NSPeriodicMask];
    
    idDown = [[NSButtonCell allocWithZone:[self zone]] initImageCell:[NSImage imageNamed:@"MVCDownImage"]];
    
    [idDown setTarget:self];
    
    [idDown setAction:@selector(decrement:)];
    [idDown setContinuous:YES];
    [idDown sendActionOn:NSLeftMouseDownMask | NSPeriodicMask];
    
#warning VoidConversion:  creating new variable to deal with methods returning void or bool (previously self)
    {
    id newVar = [self setBezeled:YES];
    [newVar setEditable:YES];
    [newVar setContinuous:NO];
}
    
    return self;
    }

- copyWithZone:(NSZone *)zone
    {
    MiscValueCell * idNew = [super copyWithZone:zone];
    
    if (idNew)
        {
        if ((idNew->idUp = [idUp copyWithZone:zone]))
            {
            [idNew->idUp setTarget:idNew];
            }
        
        if ((idNew->idDown = [idDown copyWithZone:zone]))
            {
            [idNew->idDown setTarget:idNew];
            }
        }
    
    return idNew;
    }

- (void)drawInteriorWithFrame:(NSRect)aRect inView:(NSView *)controlView
    {
    if (!fValidRect)
        {
        rectFrame = aRect;
        rectFrame.size.width -= 17;
        }
    
    [super drawInteriorWithFrame:rectFrame inView:controlView];
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
    {
    rectFrame = cellFrame;
    rectFrame.size.width -= 17;
    
    fValidRect = YES;
    [super drawWithFrame:rectFrame inView:controlView];
    fValidRect = NO;
    
    rectFrame.origin.x += rectFrame.size.width + 1;
    rectFrame.size.width = 16;
    rectFrame.size.height /= 2;
    [idUp drawWithFrame:rectFrame inView:controlView];
    
    rectFrame.origin.y += rectFrame.size.height;
    [idDown drawWithFrame:rectFrame inView:controlView];
}

- (void)resetCursorRect:(NSRect)cellFrame inView:(NSView *)aView
    {
    NSRect rect = cellFrame;
    
    rect.size.width -= 17;
    [super resetCursorRect:rect inView:aView];
}

- (NSSize)cellSizeForBounds:(NSRect)aRect
    {
    
    NSSize theSize;
    NSRect rect = aRect;
    
    rect.size.width -= 17;
    theSize = [super cellSizeForBounds:rect];
    
    theSize.height = MAX(21, theSize.height);
    theSize.width += 17;
    
    return theSize;
    }

- (void)editWithFrame:(NSRect)aRect inView:(NSView *)controlView editor:(NSText *)textObj delegate:(id)anObject event:(NSEvent *)theEvent
    {
    NSRect  rect = aRect;
    NSPoint p = theEvent->location;
    id      idWhich;
    BOOL    fR;
    BOOL    fSpin;
    int     oldMask;
    float   delay;
    float   interv;
    
    rect.size.width -= 17;
    
    p = [controlView convertPoint:p fromView:nil];
    
    if (NSPointInRect(p , rect))
        {
        [super editWithFrame:rect inView:controlView editor:textObj delegate:anObject event:theEvent];
        }
    else
        {
        rect = aRect;
        rect.origin.x += rect.size.width - 16;
        rect.size.width = 16;
        rect.size.height /= 2;
        fAltKeyDown = theEvent->flags & NSAlternateKeyMask;
        
        if (p.y < aRect.origin.y + (aRect.size.height / 2))
            {
            idWhich = idUp;
            }
        else
            {
            rect.origin.y += rect.size.height;
            idWhich = idDown;
            }
        
#error EventConversion: addToEventMask:NX_MOUSEDRAGGEDMASK: is obsolete; you no longer need to use the eventMask methods; for mouse moved events, see 'setAcceptsMouseMovedEvents:'
        oldMask = [[controlView window] addToEventMask:NSLeftMouseDraggedMask];
        [controlView lockFocus];
        [idWhich getPeriodicDelay:&delay interval:&interv];
        fSpin = YES;
        do
            {
            [idWhich highlight:YES withFrame:rect inView:controlView];
#warning RectConversion: 'trackMouse:inRect:ofView:untilMouseUp:' used to be 'trackMouse:inRect:ofView:'.  untilMouseUp == YES when inRect used to be NULL
            fR = [idWhich trackMouse:theEvent inRect:rect ofView:controlView untilMouseUp:<mouseup_arg>];
            [idWhich highlight:NO withFrame:rect inView:controlView];
            [controlView displayRect:rect];
            [idWhich setPeriodicDelay:0.0 interval:interv];
            if (fR)
                {
                fSpin = NO;
                }
            else
                {
                theEvent = [NSApp nextEventMatchingMask:NSLeftMouseUpMask | NSLeftMouseDraggedMask untilDate:[NSDate distantFuture] inMode:NSEventTrackingRunLoopMode dequeue:YES];
                if (theEvent->type == NSLeftMouseUp)
                    {
                    fSpin = NO;
                    }
                }
            } while (fSpin);
        [idWhich setPeriodicDelay:delay interval:interv];
        [controlView unlockFocus];
#error EventConversion: setEventMask:oldMask: is obsolete; you no longer need to use the eventMask methods; for mouse moved events, see 'setAcceptsMouseMovedEvents:'
        [[controlView window] setEventMask:oldMask];
        
        [NSApp sendAction:[self action] to:[self target] from:controlView];
        }
}

- (NSRect)drawingRectForBounds:(NSRect)theRect
    {
    theRect.size.width -= 17;
    theRect = [super drawingRectForBounds:theRect];
    
    return theRect;
    }

- (NSRect)titleRectForBounds:(NSRect)theRect
    {
    theRect.size.width -= 17;
    theRect = [super titleRectForBounds:theRect];
    
    return theRect;
    }

- (void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)aView
    {
    NSRect rect = cellFrame;
    
    rect.size.width -= 17;
    [super highlight:flag withFrame:rect inView:aView];
}

- (void)selectWithFrame:(NSRect)aRect inView:(NSView *)aView editor:(NSText *)aTextObject delegate:(id)anObject start:(int)selStart length:(int)selLength
    {
    NSRect rect = aRect;
    
    rect.size.width -= 17;
    [super selectWithFrame:rect inView:aView editor:aTextObject delegate:anObject start:selStart length:selLength];
}

- (void)setBezeled:(BOOL)flag
    {
    [super setBezeled:flag];
    
    [idUp setBordered:flag];
    [idDown setBordered:flag];
}

- (void)setBordered:(BOOL)flag
    {
    [super setBordered:flag];
    
    [idUp setBordered:NO];
    [idDown setBordered:NO];
}

- (void)setEditable:(BOOL)flag
    {
    [super setEditable:flag];
    
    if (!flag)
        {
        [self setSelectable:YES];
        }
}


- (void)setIntValue:(int)anInt
    {
    [self setDoubleValue:(double)anInt];
}

- (void)setFloatValue:(float)aFloat
    {
    [self setDoubleValue:(double)aFloat];
}

- (void)setDoubleValue:(double)aDouble
    {
    const char *    sz;
    id      sl = stringList;
    
    fLoop = YES; // so setStringValue doesn't get loopy
    
    if (!sl)
        {
	if ([[self controlView] class] == [MiscValueField class])
	    {
	    sl = [[self controlView] stringList];
	    }
        }
    
    if (sl)
	{
	int indx = (int)aDouble;
	int cnt = (int)limitMaxValue;
        
	cnt = [sl count] - 1;
	
	indx = MAX(indx, 0);
	indx = MIN(indx, cnt);
	actualValue = indx;
        
	sz = (char *)[sl stringAt:indx];
	if (sz)
	    {
	    [super setStringValue:[NSString stringWithCString:sz]];
	    }
	else
	    {
	    [super setDoubleValue:actualValue];
	    }
	}
    else
        {
	actualValue = MIN(aDouble, limitMaxValue);
	actualValue = MAX(actualValue, limitMinValue);
    
	if (fExpandLow && (actualValue < limitMinBound))
            {
	    limitMinBound = actualValue;
	    }
    
	if (fExpandHigh && (actualValue > limitMaxBound))
            {
	    limitMaxBound = actualValue;
	    }
        
        [super setDoubleValue:actualValue];
        }
    
    fLoop = NO;
}

- (void)setStringValue:(NSString *)aString
    {
    double  value;
    
    [super setStringValue:aString];
    
    if (!fLoop)
        {
        sscanf([aString cString], "%lf", &value);
        [self setDoubleValue:value];
        }
}

-(int) intValue
    {
	[[self controlView] validateEditing];

    return (int)actualValue;
    }

-(float) floatValue
    {
	[[self controlView] validateEditing];

    return (float)actualValue;
    }

-(double) doubleValue
    {
	[[self controlView] validateEditing];

    return actualValue;
    }


- increment:sender
    {
    double  val;
    id      sl = stringList;
    
    if (!sl)
        {
	if ([[self controlView] class] == [MiscValueField class])
	    {
	    sl = [[self controlView] stringList];
	    }
        }
    
    if (sl)
        {
	[self setIntValue:[self intValue] + ((fAltKeyDown) ? sizeAltStep : 1)];
	}
    else
        {
	val = [self doubleValue] + ((fAltKeyDown) ? sizeAltStep : sizeStep);
    
	if (fExpandHigh && (val > limitMaxBound))
            {
	    val = limitMaxBound;
	    }
	[self setDoubleValue:val];
	}
    
    if ([self isContinuous])
        {
        [NSApp sendAction:[self action] to:[self target] from:[self controlView]];
        }
    
    return self;
    }

- decrement:sender
    {
    double  val;
    id      sl = stringList;
    
    if (!sl)
        {
	if ([[self controlView] class] == [MiscValueField class])
	    {
	    sl = [[self controlView] stringList];
	    }
        }
    
    if (sl)
        {
	[self setIntValue:[self intValue] - ((fAltKeyDown) ? sizeAltStep : 1)];
	}
    else
        {
	val = [self doubleValue] - ((fAltKeyDown) ? sizeAltStep : sizeStep);
    
	if (fExpandLow && (val < limitMinBound))
            {
	    val = limitMinBound;
	    }
	[self setDoubleValue:val];
	}
    
    if ([self isContinuous])
        {
        [NSApp sendAction:[self action] to:[self target] from:[self controlView]];
        }
    
    return self;
    }


- (void)setMinValue:(double)value
    {
    limitMinValue = value;
}

- (void)setMaxValue:(double)value
    {
    limitMaxValue = value;
}

- setMinBoundary:(double)value
    {
    limitMinBound = value;
    
    return self;
    }

- setMaxBoundary:(double)value
    {
    limitMaxBound = value;
    
    return self;
    }

-(double) minValue
    {
    return limitMinValue;
    }

-(double) maxValue
    {
    return limitMaxValue;
    }

-(double) minBoundary
    {
    return limitMinBound;
    }

-(double) maxBoundary
    {
    return limitMaxBound;
    }

- setStepSize:(double)size
    {
    sizeStep = size;
    
    return self;
    }

- setAltStepSize:(double)size
    {
    sizeAltStep = size;
    
    return self;
    }

-(double) stepSize
    {
    return sizeStep;
    }

-(double) altStepSize
    {
    return sizeAltStep;
    }

- setExpandMin:(BOOL)flag
    {
    fExpandLow = flag;
    
    [self setDoubleValue:actualValue];
    
    return self;
    }

- setExpandMax:(BOOL)flag
    {
    fExpandHigh = flag;
    
    [self setDoubleValue:actualValue];
    
    return self;
    }

-(BOOL) expandMin
    {
    return fExpandLow;
    }

-(BOOL) expandMax
    {
    return fExpandHigh;
    }


- setStringList:anObject
    {
    if (anObject && [anObject respondsToSelector:@selector(stringAt:)] &&
	[anObject respondsToSelector:@selector(count)])
        {
	stringList = anObject;
	[self setEditable:NO];
	[self setIntValue:(int)actualValue];
	}
    
    return self;
    }

- stringList
    {
    return stringList;
    }


- (void)encodeWithCoder:(NSCoder *)aCoder
    {
    [super encodeWithCoder:aCoder];
    
    [aCoder encodeValuesOfObjCTypes:"ddd", &actualValue, &sizeStep, &sizeAltStep];
    [aCoder encodeValuesOfObjCTypes:"dd", &limitMinValue, &limitMaxValue];
    [aCoder encodeValuesOfObjCTypes:"dd", &limitMinBound, &limitMaxBound];
    [aCoder encodeValueOfObjCType:@encode(BOOL) at:&fExpandLow];
    [aCoder encodeValueOfObjCType:@encode(BOOL) at:&fExpandHigh];
    [aCoder encodeObject:idUp];
    [aCoder encodeObject:idDown];
}

- (id)initWithCoder:(NSCoder *)aDecoder
    {
    [super initWithCoder:aDecoder];
    
    [aDecoder decodeValuesOfObjCTypes:"ddd", &actualValue, &sizeStep, &sizeAltStep];
    [aDecoder decodeValuesOfObjCTypes:"dd", &limitMinValue, &limitMaxValue];
    [aDecoder decodeValuesOfObjCTypes:"dd", &limitMinBound, &limitMaxBound];
    [aDecoder decodeValueOfObjCType:@encode(BOOL) at:&fExpandLow];
    [aDecoder decodeValueOfObjCType:@encode(BOOL) at:&fExpandHigh];
    idUp = [[aDecoder decodeObject] retain];
    idDown = [[aDecoder decodeObject] retain];
    
    return self;
    }

#error ArchiverConversion: put the contents of your 'awake' method at the end of your 'initWithCoder:' method instead
- awake
    {
    [self setDoubleValue:actualValue];
    
    return self;
    }

- (void)awakeFromNib
    {
#error ArchiverConversion: put the contents of your 'awake' method at the end of your 'initWithCoder:' method instead
    [self awake];
}

@end
