//
//	MiscValueField.m -- a TextField subclass that specializes in using
//                          the MiscValueCell.
//	   Written by David Fedchenko. Copyright 1994 by David Fedchenko.
//	       		 Version 2.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "MiscValueField.h"
#import "MiscValueCell.h"

@implementation MiscValueField

static id _idMiscValueCellClass;

+ (void)initialize
    {
    /* Class initialization code. */
    if (self == [MiscValueField class])
        {
        _idMiscValueCellClass = [MiscValueCell class];  // Default class
        }
    
    return;
    }

+ (void) setCellClass:(Class)classId
    {
    _idMiscValueCellClass = classId;
    
    return;
    }

- initWithFrame:(NSRect)frameRect
    {
    id oldCell;

    [super initWithFrame:frameRect];
    oldCell = [self cell];
    [self setCell:[[_idMiscValueCellClass alloc] initTextCell:@"0"]];
    [oldCell release];

    return self;
    }

// the methods to communicate with the cell
- (void)setMinValue:(double)value
    {
    [[self cell] setMinValue:value];
}

- (void)setMaxValue:(double)value
    {
    [[self cell] setMaxValue:value];
}

- setMinBoundary:(double)value
    {
    [[self cell] setMinBoundary:value];

    return self;
    }

- setMaxBoundary:(double)value
    {
    [[self cell] setMaxBoundary:value];

    return self;
    }

-(double) minValue
    {
    // there is casting here because the NeXT headers have a type conflict
    // for these two methods.
    return [(MiscValueCell *)[self cell] minValue];
    }

-(double) maxValue
    {
    return [(MiscValueCell *)[self cell] maxValue];
    }

-(double) minBoundary
    {
    return [[self cell] minBoundary];
    }

-(double) maxBoundary
    {
    return [[self cell] maxBoundary];
    }

- setStepSize:(double)size
    {
    [[self cell] setStepSize:size];
    
    return self;
    }

- setAltStepSize:(double)size
    {
    [[self cell] setAltStepSize:size];
    
    return self;
    }

-(double) stepSize
    {
    return [[self cell] stepSize];
    }

-(double) altStepSize
    {
    return [[self cell] altStepSize];
    }

- setExpandMin:(BOOL)flag
    {
    [[self cell] setExpandMin:flag];
    
    return self;
    }

- setExpandMax:(BOOL)flag
    {
    [[self cell] setExpandMax:flag];
    
    return self;
    }

-(BOOL) expandMin
    {
    return [[self cell] expandMin];
    }

-(BOOL) expandMax
    {
    return [[self cell] expandMax];
    }


- setStringList:anObject
    {
    if (anObject && [anObject respondsToSelector:@selector(stringAt:)] &&
        [anObject respondsToSelector:@selector(count)])
        {
	stringList = anObject;
	
	[[self cell] setStringList:anObject];
	}
    
    return self;
    }

- stringList
    {
    if ([[self cell] stringList])
        {
	return [[self cell] stringList];
	}
    else
        {
	return stringList;
	}
    }

@end
