//
//	MiscBundleLoader.m -- an object category to load .bundle files with
//			code for a specific ObjC class in them
//		Written by Mike Ferris Copyright (c) 1994 by Mike Ferris.
//		Unburied from original MOKit by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	


// Category to load bundle-ized objects into apps dynamically.  Just
// ask for the object by name...
// This was originally in Mike Ferris' MOKit code, where it was well hidden...

#import <misckit/MiscBundleLoader.h>
/*#import <AppKit/nextstd.h>*/
#import <Foundation/NSBundle.h>
#import <objc/objc-runtime.h>
#import <sys/param.h>		// for MAXPATHLEN

#define BUNDLE_TYPE							"bundle"

@implementation NSObject(MiscBundleLoader)

+ (Class)loadClassBundle:(const char *)className
// Finds the bundle of the same name as className, grabs it and loads the
// class from it and returns the named class.
{
	char pathBuff[MAXPATHLEN + 1];
	id classBundle = nil;
	Class class = nil;
	
	// Load the bundle
#warning NSNameConversion:  Class names have changed. Check that the class name passed in is real.
	if ((class = objc_lookUpClass(className)) == nil)  {
		// class is not already loaded... load it.
		
		// Look for the bundle in the main bundle first, 
		// else try in this class's bundle.
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called pathBuff will be changed into an (NSString *).  You must change the type of the variable called pathBuff by hand.
		if (((pathBuff = [[NSBundle mainBundle] pathForResource:[NSString stringWithCString:className] ofType:[NSString stringWithCString:BUNDLE_TYPE]]) == nil)) {
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called pathBuff will be changed into an (NSString *).  You must change the type of the variable called pathBuff by hand.
			if (((pathBuff = [[NSBundle bundleForClass:[self class]] pathForResource:[NSString stringWithCString:className] ofType:[NSString stringWithCString:BUNDLE_TYPE]]) == nil)) {
				NSLog([NSString stringWithCString:"[%s loadClassBundle] failed to "
						"find %s class bundle."], [[self name] cString], className);
				return nil;
			}
		}
		classBundle = [[NSBundle allocWithZone:[self zone]] initWithPath:pathBuff];
		if (!classBundle) {
			NSLog([NSString stringWithCString:"[%s loadClassBundle] failed to "
						"create bundle for class %s."], [[self name] cString], className);
			return nil;
		}
		if ((class = [classBundle classNamed:className]) == nil) {
			NSLog([NSString stringWithCString:"[%s loadClassBundle] failed to "
						"load %s class from bundle."], [[self name] cString], className);
			return nil;
		}
	}
	return class;
}

@end
