//
//	MiscClassVariable.m -- a class to implement true class variables
//		Written by Mike Ferris Copyright (c) 1994 by Mike Ferris.
//		Modified from original MOKit "MOClassVariable" class by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	


#import "misckit/MiscClassVariable.h"
#import <objc/objc-runtime.h>

#define CLASS_VERSION		0
#define CLASS_NAME			"MiscClassVariable"

@implementation MiscClassVariable

+ (void)initialize
// Set the version.
{
#warning NSNameConversion:  Class names have changed. Check that the class name passed in is real.
	if (self == objc_lookUpClass(CLASS_NAME))  {
		[self setVersion:CLASS_VERSION];
	}
	return;
}

- init
// Just call the DI.
{
	return [self initDoesFreeValues:NO];
}

- initDoesFreeValues:(BOOL)flag
// Designated Initializer.
// Allocate the hashtable used to store the values.
{
	[super init];
	ht = [[HashTable allocWithZone:[self zone]] 
				initKeyDesc:"%" valueDesc:"@"];
	doesFreeValues = flag;
	return self;
}

- (void)dealloc
// Free the hashtable (objects too, if we're supposed to).
{
	if (doesFreeValues)  [ht freeObjects];
	[ht release];
	{ [super dealloc]; return; };
}

- setObject:obj forClass:(Class)aClass
// The old value for the class (if any) is freed if doesFreeValues is YES, 
// otherwise it returns the previous value.  Returns nil if doesFreeValues is
// YES.
{
	NXAtom className = NXUniqueString([[aClass name] cString]);
	id prevVal = nil;
	
	if (obj)  {
		// insert the new value
		prevVal = (id)[ht insertKey:className value:obj];
	}  else  {
		// no new value, just remove the old
		prevVal = (id)[ht valueForKey:className];
		if (prevVal)  [ht removeKey:className];
	}
	
	if (doesFreeValues)  {
		if (prevVal)  [prevVal release];
		return nil;
	}  else  {
		return prevVal;
	}
}

- getObjectForClass:(Class)aClass
// Look up the value for the given class.  Returns nil if no value has 
// been set.
{
	NXAtom className = NXUniqueString([[aClass name] cString]);
	return (id)[ht valueForKey:className];
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	int version;

	[super initWithCoder:aDecoder];
	version = [aDecoder versionForClassName:[NSString stringWithCString:CLASS_NAME]];
	if ( version == CLASS_VERSION ) {
		[aDecoder decodeValuesOfObjCTypes:"@c",&ht,&doesFreeValues];
	} else {
#warning StringConversion: NSStringFromSelector return value is an NSString; use [ cString] if needed, otherwise change the target type to an NSString.
		NSLog(@"[%s %s] - unknown version of %s in typed stream", [[[self class] name] cString], NSStringFromSelector(_cmd), [[[self class] name] cString]);
	}
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];	
	[aCoder encodeValuesOfObjCTypes:"@c",&ht,&doesFreeValues];
}

@end
