//
//	MiscColor.m -- a wrapper class around the NXColor functions
//		Written by Don Yacktman Copyright (c) 1994 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/misckit.h>

@implementation MiscColor

+ (BOOL)canInitWithPasteboard:(NSPasteboard *)pasteboard
{ // ***** need to also check for the color object type
	const char *pBoardTypes[2] = { [NSColorPboardType cString], NULL };
	if ([[pasteboard availableTypeFromArray:[NSArray arrayWithObject:[NSString stringWithCString:*pBoardTypes]]] isEqualToString:NSColorPboardType]) return YES;
	return NO;
}

+ newFromPasteBoard:(NSPasteboard *)pasteboard
{
	return [[self alloc] initFromPasteBoard:pasteboard];
}

+ newColor:(NSColor *)color
{
	return [[self alloc] initColor:color];
}

- init
{
	return [self initColor:[NSColor whiteColor]];
}

- initColor:(NSColor *)color
{
	[self setColor:color];
	[self setDelegate:nil];
	return self;
}

- initFromPasteBoard:(NSPasteboard *)pasteboard
{
	if (![[self class] canInitWithPasteboard:pasteboard]) {
		[self release];
		return nil;
	}
	return [[self init] setFromPasteBoard:pasteboard];
}

- initColorFromPixel:(NSPoint *)location
{
	return [self initColor:NSReadPixel(*location)];
}

- readFromPasteBoard:(NSPasteboard *)pasteboard
{ // ***** need to also deal with color object type
	[self setColor:[NSColor colorFromPasteboard:pasteboard]];
	return self;
}

- setFromPasteBoard:(NSPasteboard *)pasteboard
{
	return [self readFromPasteBoard:pasteboard];
}

- writeToPasteBoard:(NSPasteboard *)pasteboard
{	// Need to add a pasteboard type for the color OBJECT, too... *****
	[[self actualColor] writeToPasteboard:pasteboard];
	return self;
}

- setRedComponent:(float)red
{
#error ColorConversion: [[self actualColor] colorWithRedComponent:red] (used to be NXChangeRedComponent([self actualColor],red)) is not in the API. Rewrite this code, or implement a category on NSColor to support this method.
	[self setColor:[[self actualColor] colorWithRedComponent:red]];
	return self;
}

- setGreenComponent:(float)green
{
#error ColorConversion: [[self actualColor] colorWithGreenComponent:green] (used to be NXChangeGreenComponent([self actualColor],green)) is not in the API. Rewrite this code, or implement a category on NSColor to support this method.
	[self setColor:[[self actualColor] colorWithGreenComponent:green]];
	return self;
}

- setBlueComponent:(float)blue
{
#error ColorConversion: [[self actualColor] colorWithBlueComponent:blue] (used to be NXChangeBlueComponent([self actualColor],blue)) is not in the API. Rewrite this code, or implement a category on NSColor to support this method.
	[self setColor:[[self actualColor] colorWithBlueComponent:blue]];
	return self;
}

- setCyanComponent:(float)cyan
{
#error ColorConversion: [[self actualColor] colorWithCyanComponent:cyan] (used to be NXChangeCyanComponent([self actualColor],cyan)) is not in the API. Rewrite this code, or implement a category on NSColor to support this method.
	[self setColor:[[self actualColor] colorWithCyanComponent:cyan]];
	return self;
}

- setMagentaComponent:(float)magenta
{
#error ColorConversion: [[self actualColor] colorWithMagentaComponent:magenta] (used to be NXChangeMagentaComponent([self actualColor],magenta)) is not in the API. Rewrite this code, or implement a category on NSColor to support this method.
	[self setColor:[[self actualColor] colorWithMagentaComponent:magenta]];
	return self;
}

- setYellowComponent:(float)yellow
{
#error ColorConversion: [[self actualColor] colorWithYellowComponent:yellow] (used to be NXChangeYellowComponent([self actualColor],yellow)) is not in the API. Rewrite this code, or implement a category on NSColor to support this method.
	[self setColor:[[self actualColor] colorWithYellowComponent:yellow]];
	return self;
}

- setBlackComponent:(float)black
{
#error ColorConversion: [[self actualColor] colorWithBlackComponent:black] (used to be NXChangeBlackComponent([self actualColor],black)) is not in the API. Rewrite this code, or implement a category on NSColor to support this method.
	[self setColor:[[self actualColor] colorWithBlackComponent:black]];
	return self;
}

- setHueComponent:(float)hue
{
#error ColorConversion: [[self actualColor] colorWithHueComponent:hue] (used to be NXChangeHueComponent([self actualColor],hue)) is not in the API. Rewrite this code, or implement a category on NSColor to support this method.
	[self setColor:[[self actualColor] colorWithHueComponent:hue]];
	return self;
}

- setSaturationComponent:(float)saturation
{
	[self setColor:[[self actualColor] colorWithSaturationComponent:saturation]];
	return self;
}

- setBrightnessComponent:(float)brightness
{
	[self setColor:[[self actualColor] colorWithBrightnessComponent:brightness]];
	return self;
}

- setGrayComponent:(float)gray
{
#error ColorConversion: [[self actualColor] colorWithWhiteComponent:gray] (used to be NXChangeGrayComponent([self actualColor],gray)) is not in the API. Rewrite this code, or implement a category on NSColor to support this method.
	[self setColor:[[self actualColor] colorWithWhiteComponent:gray]];
	return self;
}

- setAlphaComponent:(float)alpha
{
	[self setColor:[[self actualColor] colorWithAlphaComponent:alpha]];
	return self;
}

- (void)setColor:(NSColor *)color
{
	theColor = color;
}

- setColorFromObject:object
{
	if ([object respondsToSelector:@selector(actualColor)]) {
		[self setColor:[object actualColor]];
		return self;
	} else if ([object respondsToSelector:@selector(color)]) {
		[self setColor:[object color]];
		return self;
	}
	return nil;
}

- setColorFromPixel:(NSPoint *)location
{
	[self setColor:NSReadPixel(*location)];
	return self;
}

- (NSColor *)color { return [self filterColor:[self actualColor]]; }
- (NSColor *)actualColor { return theColor; }

- (void)display
{
	[self setColor];
}

- (NSColor *)filterColor:(NSColor *)input
{
	return input;
}

- (BOOL)setColor
{
	BOOL printOK = YES;
	id theDelegate = [self delegate];
	NSColor * realColor = [self actualColor];
	if ([theDelegate respondsToSelector:@selector(willSendColorToPS:shouldPrint:)])
		realColor = [theDelegate willSendColorToPS:[self actualColor]
				shouldPrint:&printOK];
	if (printOK) {
		NSColor * filtered = [self filterColor:realColor];
		[filtered set];
		if ([theDelegate respondsToSelector:@selector(didSendColorToPS:)])
			[theDelegate didSendColorToPS:filtered];
	}
	return printOK;
}

- getGray:(float *)gray
{
	[[[self actualColor] colorUsingColorSpaceName:NSCalibratedWhiteColorSpace] getWhite:gray alpha:NULL];
	return self;
}

- getGray:(float *)gray alpha:(float *)alpha
{
	[[[self actualColor] colorUsingColorSpaceName:NSCalibratedWhiteColorSpace] getWhite:gray alpha:alpha];
	return self;
}

- getRed:(float *)red green:(float *)green blue:(float *)blue
{
	[[[self actualColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:red green:green blue:blue alpha:NULL];
	return self;
}

- getRed:(float *)red green:(float *)green blue:(float *)blue
		alpha:(float *)alpha
{
	[[[self actualColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:red green:green blue:blue alpha:alpha];
	return self;
}

- getHue:(float *)hue saturation:(float *)saturation
		brightness:(float *)brightness
{
	[[[self actualColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getHue:hue saturation:saturation brightness:brightness alpha:NULL];
	return self;
}

- getHue:(float *)hue saturation:(float *)saturation
		brightness:(float *)brightness alpha:(float *)alpha
{
	[[[self actualColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getHue:hue saturation:saturation brightness:brightness alpha:alpha];
	return self;
}

- getCyan:(float *)cyan magenta:(float *)magenta yellow:(float *)yellow
		black:(float *)black
{
	[[[self actualColor] colorUsingColorSpaceName:NSDeviceCMYKColorSpace] getCyan:cyan magenta:magenta yellow:yellow black:black alpha:NULL];
	return self;
}

- getCyan:(float *)cyan magenta:(float *)magenta yellow:(float *)yellow
		black:(float *)black alpha:(float *)alpha
{
	[[[self actualColor] colorUsingColorSpaceName:NSDeviceCMYKColorSpace] getCyan:cyan magenta:magenta yellow:yellow black:black alpha:alpha];
	return self;
}

- setGray:(float)gray
{
	[self setColor:[NSColor colorWithCalibratedWhite:gray alpha:1.0]];
	return self;
}

- setGray:(float)gray alpha:(float)alpha
{
	[self setColor:[NSColor colorWithCalibratedWhite:gray alpha:alpha]];
	return self;
}

- setRed:(float)red green:(float)green blue:(float)blue
{
	[self setColor:[NSColor colorWithCalibratedRed:red green:green blue:blue alpha:1.0]];
	return self;
}

- setRed:(float)red green:(float)green blue:(float)blue alpha:(float)alpha
{
	[self setColor:[NSColor colorWithCalibratedRed:red green:green blue:blue alpha:alpha]];
	return self;
}

- setHue:(float)hue saturation:(float)saturation
		brightness:(float)brightness
{
	[self setColor:[NSColor colorWithCalibratedHue:hue saturation:saturation brightness:brightness alpha:1.0]];
	return self;
}

- setHue:(float)hue saturation:(float)saturation
		brightness:(float)brightness alpha:(float)alpha
{
	[self setColor:[NSColor colorWithCalibratedHue:hue saturation:saturation brightness:brightness alpha:alpha]];
	return self;
}

- setCyan:(float)cyan magenta:(float)magenta yellow:(float)yellow
		black:(float)black
{
	[self setColor:[NSColor colorWithDeviceCyan:cyan magenta:magenta yellow:yellow black:black alpha:1.0]];
	return self;
}

- setCyan:(float)cyan magenta:(float)magenta yellow:(float)yellow
		black:(float)black alpha:(float)alpha
{
	[self setColor:[NSColor colorWithDeviceCyan:cyan magenta:magenta yellow:yellow black:black alpha:alpha]];
	return self;
}

- (BOOL)isEqualColor:anObject
{
	if ([anObject respondsToSelector:@selector(actualColor)]) {
		return [[self actualColor] isEqual:[anObject actualColor]];
	} else if ([anObject respondsToSelector:@selector(color)]) {
		return [[self color] isEqual:[anObject color]];
	}
	return NO;
}

- (BOOL)isEqualToColor:(NSColor *)color
{
	return [[self color] isEqual:color];
}

- (BOOL)isEqual:anObject
{
	if ([anObject class] != [self class]) return NO;
	return [self isEqualColor:anObject];
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	[super initWithCoder:aDecoder];
#warning ColorConversion: [[aDecoder decodeNXColor] retain] used to be NXReadColor(aDecoder).  Use 'decodeNXColor' to read old style colors, but use 'decodeObject' to read OpenStep colors.
	[self setColor:[[aDecoder decodeNXColor] retain]];
	[self setDelegate:[[aDecoder decodeObject] retain]];
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];
	[aCoder encodeObject:[self actualColor]];
	[aCoder encodeObject:[self delegate]];
}

// NXTransport protocol:  (I hope these will work, since I don't know another
//		way to encode and pass a color through a portal!)
- (void)encodeWithCoder:(NSPortCoder *)aPortCoder
{
	NSColor * sendColor = [self actualColor];
	id sendDelegate = [self delegate];
	[aPortCoder encodeValueOfObjCType:@encode(NSColor *) at:&sendColor];
	[aPortCoder encodeValueOfObjCType:"@" at:&sendDelegate];
}

- (id)initWithCoder:(NSPortCoder *)aPortCoder
{
	NSColor * newColor;
	id newDelegate;
	[aPortCoder decodeValueOfObjCType:@encode(NSColor *) at:&newColor];
	[aPortCoder decodeValueOfObjCType:"@" at:&newDelegate];
	[self setColor:newColor];
	[self setDelegate:newDelegate];
	return self;
}

#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
- replacementObjectForPortCoder:(NSPortCoder *)aPortCoder
{
	if ([aPortCoder isBycopy]) {
		*flagp = NO; // object will not be freed (ie, copy,  not move)
		return self; //encode object (copy it)
	}
	*flagp = NO;
	// super will encode the proxy otherwise
#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
	return [super replacementObjectForPortCoder:[aPortCoder connection]];
}

// Interface Builder support
- (const char *)getInspectorClassName { return "MiscColorInspector"; }
- (NSImage *)getIBImage { return [NSImage imageNamed:@"MiscColorObj"]; }

- (float)redComponent { return [[[self actualColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] redComponent]; }
- (float)greenComponent { return [[[self actualColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] greenComponent]; }
- (float)blueComponent { return [[[self actualColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] blueComponent]; }
- (float)cyanComponent { return [[[self actualColor] colorUsingColorSpaceName:NSDeviceCMYKColorSpace] cyanComponent]; }
- (float)magentaComponent { return [[[self actualColor] colorUsingColorSpaceName:NSDeviceCMYKColorSpace] magentaComponent]; }
- (float)yellowComponent { return [[[self actualColor] colorUsingColorSpaceName:NSDeviceCMYKColorSpace] yellowComponent]; }
- (float)blackComponent { return [[[self actualColor] colorUsingColorSpaceName:NSDeviceCMYKColorSpace] blackComponent]; }
- (float)hueComponent { return [[[self actualColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] hueComponent]; }
- (float)saturationComponent { return [[[self actualColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] saturationComponent]; }
- (float)brightnessComponent { return [[[self actualColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] brightnessComponent]; }
- (float)grayComponent { return [[[self actualColor] colorUsingColorSpaceName:NSCalibratedWhiteColorSpace] whiteComponent]; }
- (float)alphaComponent { return [[self actualColor] alphaComponent]; }

+ findColorNamed:(const char *)aName inList:(const char *)aListName
{
	NSColor * newColor;
	if (!(&newColor = [[NSColorList colorListNamed:aListName] colorWithKey:aName])) return nil;
	return [[self alloc] initColor:newColor];
}

- (const char *)colorName { return [NSColorList([self actualColor]) cString]; }
- (const char *)colorListName { return [[[[self actualColor] colorUsingColorSpaceName:NSNamedColorSpace] catalogNameComponent] cString]; }

- delegate { return delegate; }
- (void)setDelegate:(id)anObject
{
	id ret = delegate;
	delegate = anObject;
	return ret;
}

@end


// Examples of a NULL delegate
@implementation MiscColor(delegate)

- (NSColor *)willSendColorToPS:(NSColor *)color shouldPrint:(BOOL *)printOK
{
	*printOK = YES;
	return color;
}

- didSendColorToPS:(NSColor *)color
{
	return self;
}

@end
