//
//	MiscDocManager.m -- class to manage multiple documents
//		Written by Mike Ferris Copyright (c) 1993, 1995 by Mike Ferris.
//		Modified for use in the MiscKit by Don Yacktman from MODocManager.
//				Version 1.0.  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/misckit.h>
#import <objc/objc-runtime.h>

#define CLASS_VERSION				0
#define CLASS_NAME					"MiscDocManager"

#define BUNDLE_TYPE					"bundle"

#define MISCKIT_BUNDLE				"MiscKitBundle"
#define QUITPANEL_NIBNAME			"MiscDocQuitPanel.nib"
#define QUITPANEL_REVIEW			0
#define QUITPANEL_SAVEALL			1
#define QUITPANEL_QUIT				2
#define QUITPANEL_CANCEL			3

#define MAX_FRAME_CYCLE				10
#define FRAME_ADJUST				25.0
#define FRAME_START_LOC_X			150.0
#define FRAME_START_LOC_Y			30.0  // top of screen to top of window

#define REVERTALERTPANEL_TITLE		[[MiscKitBundle localizedStringForKey:@"RevertTitle" value:@"Revert" table:@"MiscDocAlertPanels"] cString]
#define REVERTALERTPANEL_TEXT		[[MiscKitBundle localizedStringForKey:@"RevertText" value:@"Do you really want to revert to %s version?  This cannot be undone." table:@"MiscDocAlertPanels"] cString]
#define REVERTALERTPANEL_REVERT		[[MiscKitBundle localizedStringForKey:@"RevertRevertButton" value:@"Revert" table:@"MiscDocAlertPanels"] cString]
#define REVERTALERTPANEL_CANCEL		[[MiscKitBundle localizedStringForKey:@"RevertCancelButton" value:@"Cancel" table:@"MiscDocAlertPanels"] cString]
#define REVERTALERTPANEL_BLANK		[[MiscKitBundle localizedStringForKey:@"RevertSubText1" value:@"blank" table:@"MiscDocAlertPanels"] cString]
#define REVERTALERTPANEL_PREVSAVED	[[MiscKitBundle localizedStringForKey:@"RevertSubText2" value:@"previously saved" table:@"MiscDocAlertPanels"] cString]

#define NEW_MENU_TITLE			[[MiscKitBundle localizedStringForKey:@"NewMenuTitle" value:@"New" table:@"MiscDocMenu"] cString]
#define OPEN_MENU_TITLE			[[MiscKitBundle localizedStringForKey:@"OpenMenuTitle" value:@"Open" table:@"MiscDocMenu"] cString]


@interface MiscDocManager(Private)

- Misc_resetOpenMenus;

@end

static id MiscKitBundle = nil;

@implementation MiscDocManager

+ (void)initialize
// Set the version.  Load classes.  Load the MiscKit bundle.
{
	if (self == [MiscDocManager class])  {
		[self setVersion:CLASS_VERSION];
	}
	return;
}

- init
// Call the DI.
{
	return [self initDocumentClass:nil];
}

- initDocumentClass:(Class)docControllerClass;
// Deesignated initializer.
// Init the manager with defaults.  If docControllerClass is non-nil it is
// added as the first of potentially several document controller classes this
// manager will manage.
{
	NSScreen *screen *= *NULL;
	
	[super init];
	
	docList = [[List allocWithZone:[self zone]] init];
	currentDoc = nil;
	docClassList = [[List allocWithZone:[self zone]] init];
	if (docControllerClass)  [docClassList addObject:docControllerClass];
	
	untitledCount = 0;
	frameCycle = 0;
	
#error ScreenConversion: 'mainScreen' has been replaced by NSScreen 'mainScreen'
	screen = [NSScreen mainScreen];
	
	windowStartingLocation.x = FRAME_START_LOC_X;
	windowStartingLocation.y = [screen frame].size.height - 
				FRAME_START_LOC_Y;
	
	return self;
}

- (void)dealloc
// free stuff we allocated.
{
	[[docList freeObjects] release];
	[docClassList empty];
	[self Misc_resetOpenMenus];
	if (newSubmenuCell) [newSubmenuCell release];
	if (newMenu) [newMenu release];
	if (openSubmenuCell) [openSubmenuCell release];
	if (openMenu) [openMenu release];
	[docClassList release];
	
	{ [super dealloc]; return; };
}

- (void)awakeFromNib
// Set the updateActions for the menu items we'll manage.
{
	if (docMenu)  {
#error ApplicationConversion: 'setAutoupdate:' is obsolete ; autoupdate is always on at the application level
		[NSApp setAutoupdate:YES];
		[docMenu setDelegate:self];
#error MenuCell method setUpdateAction:forMenu: is obsolete, use validateCell: from NSMenuActionResponder protocol instead.
		[openCell setUpdateAction:@selector(menuUpdate:) forMenu:docMenu];
#error MenuCell method setUpdateAction:forMenu: is obsolete, use validateCell: from NSMenuActionResponder protocol instead.
		[newCell setUpdateAction:@selector(menuUpdate:) forMenu:docMenu];
#error MenuCell method setUpdateAction:forMenu: is obsolete, use validateCell: from NSMenuActionResponder protocol instead.
		[saveCell setUpdateAction:@selector(menuUpdate:) forMenu:docMenu];
#error MenuCell method setUpdateAction:forMenu: is obsolete, use validateCell: from NSMenuActionResponder protocol instead.
		[saveAsCell setUpdateAction:@selector(menuUpdate:) forMenu:docMenu];
#error MenuCell method setUpdateAction:forMenu: is obsolete, use validateCell: from NSMenuActionResponder protocol instead.
		[saveToCell setUpdateAction:@selector(menuUpdate:) forMenu:docMenu];
#error MenuCell method setUpdateAction:forMenu: is obsolete, use validateCell: from NSMenuActionResponder protocol instead.
		[saveAllCell setUpdateAction:@selector(menuUpdate:) forMenu:docMenu];
#error MenuCell method setUpdateAction:forMenu: is obsolete, use validateCell: from NSMenuActionResponder protocol instead.
		[revertCell setUpdateAction:@selector(menuUpdate:) forMenu:docMenu];
#error MenuCell method setUpdateAction:forMenu: is obsolete, use validateCell: from NSMenuActionResponder protocol instead.
		[closeCell setUpdateAction:@selector(menuUpdate:) forMenu:docMenu];
	}
	[self Misc_resetOpenMenus];
}


- new:sender
// create a new untitled doc.  This is only used when we have one 
// document class
{
	id newDoc;
	if ([docClassList count] != 1)  {
		NSBeep();
		return nil;
	}
	newDoc = [[[docClassList objectAt:0] allocWithZone:[self zone]] 
				initFromFile:NULL manager:self];
	if (!newDoc)  {
		return nil;
	}
	
	return self;
}

- newFromDocumentClass:sender
// Create a new untitled doc from the class that's at the same position 
// in our doc class list as the selectedRow of the sender.  This is only 
// used when there is more than one doc class.
{
	id newDoc = [[[docClassList objectAt:[sender selectedTag]] allocWithZone:[self zone]] initFromFile:NULL manager:self];
	if (!newDoc)  {
		return nil;
	}
	return self;
}

static BOOL isExtensionInList(const char *ext, List *list, int *pos)
// Return YES and the position of the type in the list if we find a match.
// If there is no match, try matching a NULL extension.  Returns YES only if
// the extension is actually in the list.  Returns NO if not.  pos is set to 
// the position of the entry if the match is made.  If no match is made, but 
// there is a null-extension type in the list, it's position is put in pos. 
// Otherwise pos is set to -1.
{
	int i, c = [list count];
	for (i=0; i<c; i++)  {
		const char *curExt = [[list objectAt:i] typeExtension];
		// They match if they are both empty or if they match
		if ((!ext || !*ext) && (!curExt || !*curExt))  {
			if (pos)  *pos = i;
			return YES;
		}
		if (ext && curExt && (strcmp(ext, curExt)) == 0)  {
			if (pos)  *pos = i;
			return YES;
		}
	}
	if (ext && *ext)  {
		if (!isExtensionInList(NULL, list, pos))  {
			if (pos)  *pos = -1;
		}
	}  else  {
		if (pos)  *pos = -1;
	}
	return NO;
}

- open:sender
// Allow the user to open a document from disk.  We must handle all 
// doc extensions supported by all doc controller classes.  If two 
// classes support reading the same extension, the first class which 
// supports that extension is used to open files with that extension.
{
#warning FactoryMethods: [OpenPanel openPanel] used to be [OpenPanel new].  Open panels are no longer shared.  'openPanel' returns a new, autoreleased open panel in the default configuration.  To maintain state, retain and reuse one open panel (or manually re-set the state each time.)
	id openPanel = [NSOpenPanel openPanel];

	int currentClass, docClassCount, currentType, typeCount;
	List *openTypes = nil;

	// We'll fill this list with each open type supported by each doc 
	// controller class.  No duplicates, though.
	List *allTypes = [[List allocWithZone:[self zone]] init];

	const char **types = NULL;
	const char *aType;
	BOOL useTypes = YES;
	
	// First, fill allTypes and classForTypes with info for all the extensions
	// we need to handle.
	docClassCount = [docClassList count];
	if (docClassCount == 0)  {
		[allTypes release];
		return nil;
	}
	for (currentClass = 0; currentClass < docClassCount; currentClass++)  {
		openTypes = [[docClassList objectAt:currentClass] 
					getOpenTypesList:nil];
		typeCount = [openTypes count];
		for (currentType = 0; currentType < typeCount; currentType++)  {
			MiscDocType *type = [openTypes objectAt:currentType];
			if (!isExtensionInList([type typeExtension], allTypes, 
						NULL))  {
				[allTypes addObject:type];
			}
		}
		[openTypes release];
	}
	
	
	// Now make a nasty array like the open panel wants.
	typeCount = [allTypes count];
	if (typeCount == 0)  {
		[allTypes release];
		return nil;
	}
	NX_MALLOC(types, char *, typeCount+1);
	for (currentType=0; currentType<typeCount; currentType++)  {
		aType = [[allTypes objectAt:currentType] typeExtension];
		useTypes = (useTypes && aType && *aType);
		types[currentType] = (aType?aType:"");
	}
	types[typeCount] = NULL;
	
	// Now run the damn thing.
	[openPanel setAllowsMultipleSelection:NO];
	[openPanel chooseDirectories:NO];
	[openPanel setAccessoryView:nil];
	[openPanel setDelegate:self];
	[openPanel setTreatsFilePackagesAsDirectories:NO];
#error StringConversion: Open panel types are now stored in an NSArray of NSStrings (used to use char**).  Change your variable declaration.
	if ([openPanel runModalForTypes:(useTypes?types:NULL)])  {
		MiscString *path = [[MiscString allocWithZone:[self zone]] 
					initString:[[openPanel filename] cString]];
		MiscString *extension = [path fileExtension];
		// Now we need to open the damn thing with the right class.
		int index = -1;
		
		isExtensionInList([[extension stringValue] cString], allTypes, &index);
		if (index < 0)  {
			// we shouldn't have been allowed to open the doc if we can't find
			// a type for it... why are we here
			[allTypes release];
			if (types)  free(types);
			[path release];
			[extension release];
			return nil;
		}

		[self openDocument:[[path stringValue] cString] 
					withClass:[[allTypes objectAt:index] controllerClass]];
		[extension release];
		[path release];
	}
	
	[allTypes release];
	if (types)  free(types);
	return self;
}

- openFromDocumentClass:sender
// This runs the open panel for the types which are supported by the 
// document class at the same position in our doc class list as the 
// selectedRow of the sender.  This is used when there is more than 
// one doc class.
{
#warning FactoryMethods: [OpenPanel openPanel] used to be [OpenPanel new].  Open panels are no longer shared.  'openPanel' returns a new, autoreleased open panel in the default configuration.  To maintain state, retain and reuse one open panel (or manually re-set the state each time.)
	id openPanel = [NSOpenPanel openPanel];
	id openTypes = [[docClassList objectAt:[sender selectedTag]] 
				getOpenTypesList:nil];
	int i, c;
	const char **types = NULL;
	const char * aType;
	BOOL useTypes = YES;
	
	c = [openTypes count];
	if (c == 0)  {
		[openTypes release];
		return nil;
	}
	NX_MALLOC(types, char *, c+1);
	for (i=0; i<c; i++)  {
		aType = [[openTypes objectAt:i] typeExtension];
		useTypes = (useTypes && aType && *aType);
		types[i] = (aType?aType:"");
	}
	types[i] = NULL;
	
	[openPanel setAllowsMultipleSelection:NO];
	[openPanel chooseDirectories:NO];
	[openPanel setAccessoryView:nil];
	[openPanel setDelegate:self];
	[openPanel setTreatsFilePackagesAsDirectories:NO];
#error StringConversion: Open panel types are now stored in an NSArray of NSStrings (used to use char**).  Change your variable declaration.
	if ([openPanel runModalForTypes:(useTypes?types:NULL)])  {
		[self openDocument:[[openPanel filename] cString] 
					withClass:[docClassList objectAt:[sender selectedTag]]];
	}
	
	free(types);
	[openTypes release];
	return self;
}

- openDocument:(const char *)path withClass:(Class)docClass;
// Make a new documentof the given class and initialize it from the given file.
{
	id newDoc;
	
	newDoc = [[docClass allocWithZone:[self zone]] 
				initFromFile:path manager:self];
	
	return self;
}

- save:sender
// Tell the current doc to save.
{
	if (currentDoc)  {
		[currentDoc save:self];
	}
	return self;
}

- saveAs:sender
// Tell the current doc to save as.
{
	if (currentDoc)  {
		[currentDoc saveAs:self];
	}
	return self;
}

#warning ColorConversion: 'writeToFile:' was 'saveTo:' ; conversion should only apply to NSColorList ; 'writeToFile:' returns BOOL rather than raising
- (BOOL)writeToFile:(id)sender
// Tell the current doc to save to.
{
	if (currentDoc)  {
#warning ColorConversion: 'writeToFile:' was 'saveTo:' ; conversion should only apply to NSColorList ; 'writeToFile:' returns BOOL rather than raising
		[currentDoc writeToFile:[NSString stringWithCString:self]];
	}
	return YES;
}

- saveAll:sender
// Save all the open docs.
{
	int i, c = [docList count];
	id temp;
	
	for (i=0; i<c; i++)  {
		temp = [docList objectAt:i];
		[temp saveIfNeeded];
	}
	return self;
}

- revert:sender
// Tell the current doc to revert.
{
	if (currentDoc)  {
		if ( NSRunAlertPanel([NSString stringWithCString:REVERTALERTPANEL_TITLE], [NSString stringWithCString:REVERTALERTPANEL_TEXT], [NSString stringWithCString:REVERTALERTPANEL_REVERT], [NSString stringWithCString:REVERTALERTPANEL_CANCEL], nil, ([currentDoc isUntitled] ?
				     REVERTALERTPANEL_BLANK:REVERTALERTPANEL_PREVSAVED))
				== NSAlertDefaultReturn )  {
			[currentDoc revert:self];
		}
	}
	return self;
}

- close:sender
// Tell the current doc to close.
{
	if (currentDoc)  {
		[currentDoc close:self];
	}
	return self;
}

#warning PrintingConversion:  printPSCode: has been renamed to print:.  Rename this method?
- (void)print:(id)sender
// Tell the current doc to print.
{
	if (currentDoc)  {
#warning PrintingConversion:  printPSCode: has been renamed to print:.  Rename this method?
		[currentDoc print:self];
	}
}

- getNextWindowLocation:(NSPoint *)pt
// Get the location for the next document window.
{
	pt->x = windowStartingLocation.x + (FRAME_ADJUST * frameCycle);
	pt->y = windowStartingLocation.y - (FRAME_ADJUST * frameCycle);
	frameCycle++;
	if (frameCycle>=MAX_FRAME_CYCLE)  frameCycle = 0;
	
	return self;
}

- (int)nextUntitledNum
// Get the number for the next untitled doc.
{
	untitledCount++;
	return untitledCount;
}

- setWindowStartingLocation:(const NSPoint *)pt
// Set the starting window location from which the staggering of doc 
// windows will begin.  This should be set before any windows are 
// created for best results.
{
	windowStartingLocation.x = pt->x;
	windowStartingLocation.y = pt->y;
	return self;
}

- addDocument:(MiscDocument *)aDocument
// Add a new document to our list.  DocController calls this automatically.
{
	[docList addObject:aDocument];
	
	return self;
}

- removeDocument:(MiscDocument *)aDocument
// Remove a document to our list.  (DocController calls this when the
// document is closing.)
{
	[docList removeObject:aDocument];
	if (currentDoc == aDocument)  {
		currentDoc = nil;
	}
	
	return self;
}

- (MiscDocument *)findDocumentForWindow:aWindow
// Find the controller that controls the given window (if any).
{
	int i, c = [docList count];
	id temp;
	
	for (i=0; i<c; i++)  {
		temp = [docList objectAt:i];
		if ([temp window] == aWindow)  {
			return temp;
		}
	}
	return nil;
}

- (List *)documentList
// Return the List object we use to store our docs.
{
	return docList;
}

- makeDocumentsPerform:(SEL)aMethod with:anArg
// Distribute a message to the documents.
{
	[docList makeObjectsPerform:aMethod with:anArg];
	
	return self;
}

- (BOOL)areDocumentsDirty
// Returns YES if at least one open document is dirty.
{
	int i, c = [docList count];
	
	for (i=0; i<c; i++)  {
		if ([[docList objectAt:i] isDirty])  {
			return YES;
		}
	}
	return NO;
}

- setCurrentDocument:(MiscDocument *)aDocument
// Sets the current doc.  Called from DocController's window delegate
// methods.
{
	currentDoc = aDocument;
	return self;
}

- currentDocument
// Returns the current doc's controller.
{
	return currentDoc;
}

- addDocumentClass:(Class)docControllerClass
// Add the class to the list of classes we manage.  Rebuilds the Document menu.
{
	[docClassList addObject:docControllerClass];
	[self Misc_resetOpenMenus];
	return self;
}

- removeDocumentClass:(Class)docControllerClass
// Remove the class from the list of classes we manage.  
// Rebuilds the Document menu.
{
	[docClassList removeObject:docControllerClass];
	[self Misc_resetOpenMenus];
	return self;
}

- (List *)documentClassList
// Return the list of classes we manage.
{
	return docClassList;
}

- (int)docClassCount
// Returns the number of classes that this manager is managing.
{
	return [docClassList count];
}

#warning NotificationConversion: applicationDidFinishLaunching:(NSNotification *)notification (used to be appDidInit:) is an NSApplication notification method (used to be a delegate method); delegates of NSApplication are automatically set to observe this notification; subclasses of NSApplication do not automatically receive this notification
- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
	NSApplication *theApplication = [notification object];
    if ([NSApp defaultBoolValue:"NXAutoLaunch"] &&
			[NSApp defaultBoolValue:"HideOnAutoLaunch"]) {
		// Note:  this will cause trouble if you bring up *any*
		// windows when launching!  The way around this is to
		// subclass this object and catch this method first before
		// passing execution to this code...and suppress ordering
		// up any windows until the app is unhid.  You'll need to
		// basically delay loading those windows until later--when
		// the app is unhidden.  The easiest way to to have a one
		// shot method that will bring up the windows with a static
		// var telling if it's done it's thing yet or not and call
		// it from where ever applicable--with it returning and
		// doing nothing if it has already done its work.  This also
		// means that any app windows should be loaded from another
		// .nib file than the main .nib, should not be visible upon
		// launch, and should probably not be deferred.
		[NSApp hide:theApplication];
	}
}

- (BOOL)applicationShouldTerminate:(id)sender
// If there are unsaved docs, put up a panel asking if the user wants to
// Review Unsaved, Save All, Quit Anyway, or Cancel.
{
	int ret;
	int i, c;
	
	if ([self areDocumentsDirty])  {
		if (!quitPanel)  {
			if (MiscKitBundle)  {
				char pathBuff[MAXPATHLEN+1];
#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called pathBuff will be changed into an (NSString *).  You must change the type of the variable called pathBuff by hand.
				if (((pathBuff = [MiscKitBundle pathForResource:[NSString stringWithCString:QUITPANEL_NIBNAME] ofType:@""]) == nil))  {
					NSLog([NSString stringWithCString:"%s: failed to find nib file %s in "
								"MiscKit bundle."], [[[self class] name] cString], QUITPANEL_NIBNAME);
					return NO;
				}
				if (![NSBundle loadNibFile:[NSString stringWithCString:pathBuff] externalNameTable:[NSDictionary dictionaryWithObjectsAndKeys:self, @"NSOwner", nil] withZone:[self zone]])  {
					NSLog(@"%s: failed to load nib file %s.", [[[self class] name] cString], pathBuff);
					return NO;
				}
			}  else  {
				if (![NSBundle loadNibNamed:[NSString stringWithCString:QUITPANEL_NIBNAME] owner:self])  {
					NSLog(@"%s: failed to load nib file %s.", [[[self class] name] cString], QUITPANEL_NIBNAME);
					return NO;
				}
			}
		}
		ret = [NSApp runModalForWindow:quitPanel];
		[quitPanel orderOut:self];
		
		if (ret == QUITPANEL_REVIEW)  {
			c = [docList count];
			for (i=c-1;i>=0;i--)  {
				[[[docList objectAt:i] window] makeKeyAndOrderFront:self];
				if ([[docList objectAt:i] windowShouldClose:[[docList 
							objectAt:i] window]] == nil)  {
					return NO;
				}
				[[[docList objectAt:i] window] orderOut:self];
			}
			return YES;
		}  else if (ret == QUITPANEL_SAVEALL)  {
			[self saveAll:self];
			return YES;
		}  else if (ret == QUITPANEL_QUIT)  {
			return YES;
		}  else if (ret == QUITPANEL_CANCEL)  {
			return NO;
		}
	}
	return YES;  // nil to abort termination
}

- quitPanelStopModalAction:sender
// This is the action for the buttons in the quit panel.
{
	[NSApp stopModalWithCode:[sender selectedTag]];
	return self;
}

#define QUITPANEL_OK_BUTTON_ROW		1
#define QUITPANEL_OK_BUTTON_COL		2

#warning NotificationConversion: windowDidBecomeKey:(NSNotification *)notification is an NSWindow notification method (used to be a delegate method); delegates of NSWindow are automatically set to observe this notification; subclasses of NSWindow do not automatically receive this notification
- (void)windowDidBecomeKey:(NSNotification *)notification
// For the quit panel... make sure that the return icon is only there
// if the window is key.
{
	NSWindow *theWindow = [notification object];
    if (theWindow == quitPanel)  {
		[[buttonMatrix cellAtRow:QUITPANEL_OK_BUTTON_ROW column:QUITPANEL_OK_BUTTON_COL] setImagePosition:NSImageRight];
		[buttonMatrix setImage:[NSImage imageNamed:@"NSReturnSign"] atRow:QUITPANEL_OK_BUTTON_ROW column:QUITPANEL_OK_BUTTON_COL];
	}
}

#warning NotificationConversion: windowDidResignKey:(NSNotification *)notification is an NSWindow notification method (used to be a delegate method); delegates of NSWindow are automatically set to observe this notification; subclasses of NSWindow do not automatically receive this notification
- (void)windowDidResignKey:(NSNotification *)notification
// For the quit panel... make sure that the return icon is only there
// if the window is key.
{
	NSWindow *theWindow = [notification object];
    if (theWindow == quitPanel)  {
		[[buttonMatrix cellAtRow:QUITPANEL_OK_BUTTON_ROW column:QUITPANEL_OK_BUTTON_COL] setImagePosition:NSNoImage];
		[buttonMatrix setImage:[NSImage imageNamed:@""] atRow:QUITPANEL_OK_BUTTON_ROW column:QUITPANEL_OK_BUTTON_COL];
	}
}

- (BOOL)menuUpdate:menuCell
// This is the updateAction for all the Document menu stuff.
{
	BOOL enabled = [menuCell isEnabled];
	BOOL docsDirty;
	if ((menuCell == openCell) || (menuCell == newCell))  {
		if (([docClassList count] == 0) && (enabled))  {
			[menuCell setEnabled:NO];
			return YES;
		}  else if (!enabled)  {
			[menuCell setEnabled:YES];
			return YES;
		}
	}  else if ((menuCell == saveCell) || (menuCell == saveAsCell) || 
				(menuCell == saveToCell) || (menuCell == closeCell))  {
		if ((currentDoc == nil) && (enabled))  {
			[menuCell setEnabled:NO];
			return YES;
		}  else if ((currentDoc != nil) && (!enabled))  {
			[menuCell setEnabled:YES];
			return YES;
		}
	}  else if (menuCell == saveAllCell)  {
		docsDirty = [self areDocumentsDirty];
		if ((docsDirty) && (!enabled))  {
			[menuCell setEnabled:YES];
			return YES;
		}  else if ((!docsDirty) && (enabled))  {
			[menuCell setEnabled:NO];
			return YES;
		}
	}  else if (menuCell == revertCell)  {
		if ((currentDoc == nil) && (enabled))  {
			[menuCell setEnabled:NO];
			return YES;
		}  else if (currentDoc != nil)  {
			if (([currentDoc isDirty]) && (!enabled))  {
				[menuCell setEnabled:YES];
				return YES;
			}  else if ((![currentDoc isDirty]) && (enabled))  {
				[menuCell setEnabled:NO];
				return YES;
			}
		}
	}
	return NO;
}

#error ArchiverConversion: put the contents of your 'awake' method at the end of your 'initWithCoder:' method instead
- awake
// Initialize stuff we don't archive.
{
	quitPanel = nil;
	buttonMatrix = nil;
	newMenu = nil;
	newSubmenuCell = nil;
	openMenu = nil;
	openSubmenuCell = nil;
	return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder
// This method is probably not useful.  But here it is.
{
	int classVersion;
	NSScreen *screen *= *NULL;

	[super initWithCoder:aDecoder];
	
	classVersion = [aDecoder versionForClassName:[NSString stringWithCString:CLASS_NAME]];
	
	switch (classVersion)  {
		case 0:		// First version.
			docList = [[aDecoder decodeObject] retain];
			currentDoc = [[aDecoder decodeObject] retain];
			docClassList = [[aDecoder decodeObject] retain];
			[aDecoder decodeValueOfObjCType:"i" at:&untitledCount];
			[aDecoder decodeValueOfObjCType:"i" at:&frameCycle];
			windowStartingLocation = [aDecoder decodePoint];
			
			docMenu = [[aDecoder decodeObject] retain];
			openCell = [[aDecoder decodeObject] retain];
			newCell = [[aDecoder decodeObject] retain];
			saveCell = [[aDecoder decodeObject] retain];
			saveAsCell = [[aDecoder decodeObject] retain];
			saveToCell = [[aDecoder decodeObject] retain];
			saveAllCell = [[aDecoder decodeObject] retain];
			revertCell = [[aDecoder decodeObject] retain];
			closeCell = [[aDecoder decodeObject] retain];
			break;
		default:
			NSLog([NSString stringWithCString:"[%s read:] class version %d cannot read "
						"instances archived with version %d"], CLASS_NAME, CLASS_VERSION, classVersion);
			docList = [[List allocWithZone:[self zone]] init];
			currentDoc = nil;
			docClassList = [[List allocWithZone:[self zone]] init];
			untitledCount = 0;
			frameCycle = 0;
#error ScreenConversion: 'mainScreen' has been replaced by NSScreen 'mainScreen'
			screen = [NSScreen mainScreen];
			windowStartingLocation.x = FRAME_START_LOC_X;
			windowStartingLocation.y = [screen frame].size.height - 
						FRAME_START_LOC_Y;
			docMenu = nil;
			openCell = nil;
			newCell = nil;
			saveCell = nil;
			saveAsCell = nil;
			saveToCell = nil;
			saveAllCell = nil;
			revertCell = nil;
			closeCell = nil;
			
			break;
	}
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
// This method is probably not useful.  But here it is.
{
	[super encodeWithCoder:aCoder];
	[aCoder encodeObject:docList];
	[aCoder encodeObject:currentDoc];
	[aCoder encodeObject:docClassList];
	[aCoder encodeValueOfObjCType:"i" at:&untitledCount];
	[aCoder encodeValueOfObjCType:"i" at:&frameCycle];
	[aCoder encodePoint:windowStartingLocation];
	
	[aCoder encodeConditionalObject:docMenu];
	[aCoder encodeConditionalObject:openCell];
	[aCoder encodeConditionalObject:newCell];
	[aCoder encodeConditionalObject:saveCell];
	[aCoder encodeConditionalObject:saveAsCell];
	[aCoder encodeConditionalObject:saveToCell];
	[aCoder encodeConditionalObject:saveAllCell];
	[aCoder encodeConditionalObject:revertCell];
	[aCoder encodeConditionalObject:closeCell];
}

- Misc_resetOpenMenus
// Keeps the document menu in step with changes in the manager's class list.
// This is the method that makes sure the Open and New stuff is correct for
// the document classes being managed.
{
	id itemList;
	int i, c;
	const char *name;
	id cell;
	int row, col, open;
	List *openList= NULL;
    
	// See if the nib is loaded and we do menus
	if (!docMenu)  {
		return nil;
	}
    
	// Count needed open submenu entries
	c = [docClassList count];
	open = 0;
	for (i=0; i<c; i++) {
		openList = [[docClassList objectAt:i] getOpenTypesList: openList];
		open += [openList count]>0;
		[openList empty];
	}

	// Handle openMenu
	if (open > 1)  {
		// We should have submenus for openMenu.
		if (!openMenu) {
			openMenu = [[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:[NSString stringWithCString:OPEN_MENU_TITLE]];
		}
#warning PopUpConversion: This message should be sent to an NSPopUpButton, but is probably being sent to an NSPopUpList
#warning PopUpConversion: Consider NSPopUpButton methods instead of using itemMatrix to access items in a pop-up list.
		itemList = [openMenu itemMatrix];
		[itemList renewRows:0 columns:1];
		c = [docClassList count];
		for (i=0; i<c; i++) {
			name = [[[[docClassList objectAt:i] controllerName] stringValue] cString];
			[itemList addRow];
			cell = [itemList cellAtRow:i column:0];
			[cell setTitle:[NSString stringWithCString:name]];
			[cell setTag:i];
			[cell setTarget:self];
			[cell setAction:@selector(openFromDocumentClass:)];
		}
		[openMenu sizeToFit];

		// Now make an openSubmenuItem
		if (!openSubmenuCell) {
			openSubmenuCell = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initTextCell:[NSString stringWithCString:OPEN_MENU_TITLE]];
		}

		// Put the submenu item in the doc menu below the open item
#warning PopUpConversion: This message should be sent to an NSPopUpButton, but is probably being sent to an NSPopUpList
#warning PopUpConversion: Consider NSPopUpButton methods instead of using itemMatrix to access items in a pop-up list.
		itemList = [docMenu itemMatrix];
		[itemList getRow:&row column:&col ofCell:openCell];
		if ([itemList cellAtRow:row+1 column:col] != openSubmenuCell)  {
			[itemList insertRow:row+1];
			[[itemList putCell:openSubmenuCell atRow:row+1 column:col] release];
			[docMenu setSubmenu:openMenu forItem:openSubmenuCell];
			[docMenu sizeToFit];
		}
	} else {
		// We should not have a submenu for openMenu.
#warning PopUpConversion: This message should be sent to an NSPopUpButton, but is probably being sent to an NSPopUpList
#warning PopUpConversion: Consider NSPopUpButton methods instead of using itemMatrix to access items in a pop-up list.
		itemList = [docMenu itemMatrix];
		// take out the open submenu item if its there
		if ([itemList getRow:&row column:&col ofCell:openSubmenuCell]) {
#warning GeneralNamingConversion: The old andFree:NO argument no longer exists; now the removed cell is released. You might want to assure that this is the correct behavior.
			[itemList removeRow:row];
		}
		[docMenu sizeToFit];
    }

	// Handle newMenu
	if ([docClassList count] > 1)  {
		// We should have submenus for newMenu.
		if (!newMenu)
			newMenu = [[NSMenu allocWithZone:[NSMenu menuZone]] initWithTitle:[NSString stringWithCString:NEW_MENU_TITLE]];
#warning PopUpConversion: This message should be sent to an NSPopUpButton, but is probably being sent to an NSPopUpList
#warning PopUpConversion: Consider NSPopUpButton methods instead of using itemMatrix to access items in a pop-up list.
		itemList = [newMenu itemMatrix];
		[itemList renewRows:0 columns:1];
		c = [docClassList count];
		for (i=0; i<c; i++)  {
			name = [[[[docClassList objectAt:i] controllerName] stringValue] cString];
			[itemList addRow];
			cell = [itemList cellAtRow:i column:0];
			[cell setTitle:[NSString stringWithCString:name]];
			[cell setTag:i];
			[cell setTarget:self];
			[cell setAction:@selector(newFromDocumentClass:)];
		}
		[newMenu sizeToFit];
	
		// Now make a newSubmenuItem
		if (!newSubmenuCell)
		newSubmenuCell = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initTextCell:[NSString stringWithCString:NEW_MENU_TITLE]];
	
		// Put the submenu item in the doc menu in place of the new item
#warning PopUpConversion: This message should be sent to an NSPopUpButton, but is probably being sent to an NSPopUpList
#warning PopUpConversion: Consider NSPopUpButton methods instead of using itemMatrix to access items in a pop-up list.
		itemList = [docMenu itemMatrix];
		if (![itemList getRow:&row column:&col ofCell:newSubmenuCell])  {
			[itemList getRow:&row column:&col ofCell:newCell];
			[itemList putCell:newSubmenuCell atRow:row column:col];
			[docMenu setSubmenu:newMenu forItem:newSubmenuCell];
			[docMenu sizeToFit];
		}
	} else {
		// We should not have a submenu for newMenu.
#warning PopUpConversion: This message should be sent to an NSPopUpButton, but is probably being sent to an NSPopUpList
#warning PopUpConversion: Consider NSPopUpButton methods instead of using itemMatrix to access items in a pop-up list.
		itemList = [docMenu itemMatrix];
		// take out the new submenu item if its there and put back the new item
		if ([itemList getRow:&row column:&col ofCell:newSubmenuCell])
			[itemList putCell:newCell atRow:row column:col];
		[docMenu sizeToFit];
	}
	[openList release];
	return(self);
}


@end
