/************************************************************************
  CATEGORY:			MiscFile (Modification)

  See the header for more information.

  This object is included in the MiscKit by permission from the author
  and its use is governed by the MiscKit license, found in the file
  "LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
  for a list of all applicable permissions and restrictions.

  Copyright (c) 1995 by author.  All rights reserved.

 *************************************************************************/

#import <misckit/MiscFile.h>


@interface MiscFile (ModificationPrivate)

- (int)_setBits: (int)perms for: (int) who;

@end


@implementation MiscFile (Modification)

- (int)setPermissions: (int)perms for: (int)who
{
	// Sets the permission of the receiver. If successful, MISCFILE_SUCCESS is 	// returned. Otherwise MISCFILE_ERROR is returned. The arg perms can any 	// ORed combo. of MISCFILE_READ, MISCFILE_WRITE, or MISCFILE_EXECUTE. 
	// The who message can be any ORed value of  MISCFILE_OWNER, 	
	// MISCFILE_GROUP, or MISCFILE_OTHER.
	// Example : [self setPermissions: MISCFILE_READ | MISCFILE_EXECUTE for:
	//	MISCFILE_GROUP]. This is the same as "chmod 050 filename" (I think :-)
	int  mode;
	BOOL  caching = [self enableCaching];
		
	// Stat caching is turned off just to get the most up to date permissions.
	// Otherwise we may be changing other permissions we shouldn't.
	[self setEnableCaching: NO];
	mode = (int)[self permissions];
	
	// Only enable caching if it was YES originally.
	if (caching == YES)
		[self setEnableCaching: YES];

	// Clear the 9 permission bits.	
	mode &= 0xfe00;
	mode |= [self _setBits: perms for: who];
	return [self chmod: mode];
}


- (int)addPermissions: (int)perms for: (int)who
{
	// Adds permissions given while leaving the rest alone.
	int  currMode;
	int  mode = [self _setBits: perms for: who];
	BOOL  caching = [self enableCaching];
	
	// Turn off stat caching to get the most up to date permissions.
	[self setEnableCaching: NO];
	currMode = (int)[self permissions];
	if (caching == YES)
		// Only enable caching if it was YES originally.
		[self setEnableCaching: YES];
	
	mode |= currMode;
	return [self chmod: mode];
}
	

- (int)removePermissions: (int)perms for: (int)who
{
	// Removes permissions given while leaving the rest alone.
	int  currMode; 
	int  mode = [self _setBits: perms for: who];
	BOOL  caching = [self enableCaching];
	
	[self setEnableCaching: NO];
	currMode = (int)[self permissions];
	if (caching == YES)
		// Only enable caching if it was YES originally.
		[self setEnableCaching: YES];
		
	mode ^= currMode;
	return [self chmod: mode];
}

@end


@implementation MiscFile (ModificationPrivate)

- (int)_setBits: (int)perms for: (int) who
{
	int  mode = 0;

	if (who & MISCFILE_OWNER)
		mode |= perms << 6;
	if (who & MISCFILE_GROUP)
		mode |= perms << 3;
	if (who & MISCFILE_OTHER)
		mode |= perms;

	if (perms & MISCFILE_STICKY)
		mode |= 0x0200;
	if (perms & MISCFILE_SETGID)
		mode |= 0x0400;
	if (perms & MISCFILE_SETUID)
		mode |= 0x0800;
	return mode;
}	

@end
