/************************************************************************
  CATEGORY:			MiscFile (Unix)

  See the header for more information.
 
  This object is included in the MiscKit by permission from the author
  and its use is governed by the MiscKit license, found in the file
  "LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
  for a list of all applicable permissions and restrictions.

  Copyright (c) 1995 by author.  All rights reserved.

 *************************************************************************/

#import <sys/types.h>
#import <errno.h>
#import <libc.h>				// for chmod(2) and chown(2)
#import <sys/stat.h>			// for stat(2)
#import <sys/file.h>			// for access(2)
#import <misckit/MiscFile.h>

// NeXT's headers don't seem to declare this anywhere...it is supposed to
// be in <bsd/sys/file.h> but it isn't.
extern int access (const char *path, int mode);


static BOOL  useLinkInfo = NO;


@implementation MiscFile (Unix)

+ (BOOL)useLinkInfo
{
	return useLinkInfo;
}


+ setUseLinkInfo: (BOOL)linkInfo
{
	useLinkInfo = linkInfo;
	return self;
}


- (int)errorCode
{
	return errorCode;
}


- (BOOL)enableCaching
{
	return mfFlags.enableCaching;
}


- setEnableCaching: (BOOL)cache
{
	mfFlags.enableCaching = cache;
	return self;
}


- (int)access: (int)mode
{
	int  succ = access ([self fullPath], mode);
	[self _setErrorCode: succ];
	return succ;
}


- (struct stat *)stat
{
	// stat and lstat break the return an integer with either success (0) or
	// errorCode rule. Any functions that use them though, should check the 	// value returned and if NULL (failed to stat file) return MISCFILE_ERROR
	// to be consistent with the other methods.
	int  succ;
	if (st == NULL || [self enableCaching] == NO || 
					mfFlags.lastStat == MISCFILE_LSTAT)
	{
		if (st == NULL)
			st = (struct stat *)malloc(sizeof(struct stat));
		succ = stat ([self fullPath], st);
		[self _setErrorCode: succ];
		mfFlags.lastStat = MISCFILE_STAT;
		if (succ == MISCFILE_ERROR)
		{
			free (st);
			st = NULL;
	 	 }
	 }
	return st;
}


- (struct stat *)lstat
{
	// stat and lstat break the return an integer with either success (0) or
	// errorCode rule. Any functions that use them though, should check the 	// value returned and if NULL (failed to stat file) return MISCFILE_ERROR
	// to be consistent with the other methods.
	int  succ;
	if (st == NULL || [self enableCaching] == NO || 
					mfFlags.lastStat == MISCFILE_STAT)
	{
		if (st == NULL)
			st = (struct stat *)malloc(sizeof(struct stat));
		succ = lstat ([self fullPath], st);
		[self _setErrorCode: succ];
		mfFlags.lastStat = MISCFILE_LSTAT;
		if (succ == MISCFILE_ERROR)
		{
			free (st);
			st = NULL;
	 	 }
	 }
	return st;
}


- (struct stat *)whichStat
{
	// If we are a symbolic link and we want the information about the
	// link then lstat the file. Normally if you stat a symbolic link
	// you will get information about the file the link references.
	if ([self isSymbolicLink] && [ [self class] useLinkInfo] == YES)
		return [self lstat];
	else
		return [self stat];
}

		
- (int)readlink: (char *)linkPath pathLen: (int)len
{
	// linkPath should be a buffer of MAXPATHLEN+1. Will return either 
	// MISCFILE_SUCCESS or MISCFILE_ERROR. Since the documentation for
	// readlink says it does not null terminate the string, we will
	// make sure it is.
	int  succ = readlink ([self fullPath], linkPath, len);
	if (succ > 0)
	{
		*(linkPath+succ)='\0'; // null terminate it
		succ = 0;
	 }
	[self _setErrorCode: succ];
	return succ;
}
	
	
- (int)chmod: (unsigned short)mode
{
	int  succ = chmod([self fullPath], mode);
	[self _setErrorCode: succ];
	return succ;
}


- (int)chown: (uid_t)owner group: (gid_t)group
{
	int  succ = chown ([self fullPath], owner, group);
	[self _setErrorCode: succ];
	return succ;
}	


- (int)link: (const char *)newpath
{
	int  succ = link ([self fullPath], newpath);
	[self _setErrorCode: succ];
	return succ;
}


- (int)symlink: (const char *)newpath
{
	int  succ = symlink ([self fullPath], newpath);
	[self _setErrorCode: succ];
	return succ;
}

@end


@implementation MiscFile (UnixPrivate)

- _setErrorCode: (int)retValue
{
	// Most of the file manipulation functions under UNIX return 0 for success
	// and -1 for error. You just have to pass the return value (from stat, 
	// etc.) and this method sets errorCode appropriately.
	if (retValue == MISCFILE_SUCCESS)
		errorCode = MISCFILE_SUCCESS; 
	else
		errorCode = errno;
	return self;
}	

@end

