/*
 *  Copyright (c) 1993 Christopher J. Kane.  All rights reserved.
 *
 *  This software is subject to the terms of the MiscKit license
 *  agreement.  Refer to the license document included with the
 *  MiscKit distribution for these terms.
 *
 *  Version: 1.1 (22 October 1993)
 */

#import <misckit/MiscFindPanelClass.h>

@implementation NSApplication (MiscFindPanelClass)

static BOOL _FPloaded=NO;
static BOOL _fpLoaded=NO;

- (Class)MiscFindPanelClass
{
  static Class _findPanelClass=Nil;
  if (!_FPloaded)
    {
      char path[MAXPATHLEN+16];
      _FPloaded = YES;
#warning FoundationConversion: The NSBundle bundlePath method returns the top-level directory of the bundle.  With the new bundle scheme, resources are installed in a subdirectory of the bundlePath (called Resources) instead of directly in the bundlePath.  If you are using the bundlePath to build paths to resources within your bundle, you should try to use pathForResource:ofType: instead.
      sprintf(path, "%s/MiscFindPanel.bundle", [[[NSBundle mainBundle] bundlePath] cString]);
      _findPanelClass = [[[NSBundle alloc] initWithPath:path] classNamed:"MiscFindPanel"];
      if ([[NSApp delegate] respondsToSelector:@selector(MiscFindPanelClassWasLoaded:)])
        _findPanelClass = [[NSApp delegate] MiscFindPanelClassWasLoaded:_findPanelClass];
      else if ([NSApp respondsToSelector:@selector(MiscFindPanelClassWasLoaded:)])
        _findPanelClass = [NSApp MiscFindPanelClassWasLoaded:_findPanelClass];
    }
  return _findPanelClass;
}

- (BOOL)isMiscFindPanelClassLoaded
{
  return _FPloaded;
}

- (MiscFindPanel *)findPanel
{
  static MiscFindPanel *_findPanel=nil;
  if (!_fpLoaded)
    {
      if ([[NSApp delegate] respondsToSelector:@selector(findPanelWillLoad:)])
        [[NSApp delegate] findPanelWillLoad:[self MiscFindPanelClass]];
      else if ([NSApp respondsToSelector:@selector(findPanelWillLoad:)])
        [NSApp findPanelWillLoad:[self MiscFindPanelClass]];
      _findPanel = [[self MiscFindPanelClass] sharedInstance];
      if ([[NSApp delegate] respondsToSelector:@selector(findPanelDidLoad:)])
        _findPanel = [[NSApp delegate] findPanelDidLoad:_findPanel];
      else if ([NSApp respondsToSelector:@selector(findPanelDidLoad:)])
        _findPanel = [NSApp findPanelDidLoad:_findPanel];
      _fpLoaded = YES;
    }
  return _findPanel;
}

- (BOOL)isFindPanelLoaded
{
  return _fpLoaded;
}

- enterSelection:sender
{
  return [[self findPanel] enterSelection:sender];
}

- findNext:sender
{
  return [[self findPanel] findForward:sender];
}

- findPrevious:sender
{
  return [[self findPanel] findBackward:sender];
}

- jumpToSelection:sender
{
  return [[self findPanel] jumpToSelection:sender];
}

- orderFrontFindPanel:sender
{
  [[self findPanel] makeKeyAndOrderFront:sender];
  return [self findPanel];
}

@end
