//
//	MiscHalftoneColor.m -- a class for color "tints"  (to specify a halftone)
//		Written by Don Yacktman Copyright (c) 1994 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/misckit.h>

@implementation MiscHalftoneColor

+ (BOOL)canInitWithPasteboard:(NSPasteboard *)pasteboard
{ // ***** need version to handle object type
	return [super canInitWithPasteboard:pasteboard];
}

- setFromPasteBoard:(NSPasteboard *)pasteboard
{ // ***** need version to handle object type
	return [super setFromPasteBoard:pasteboard];
}

- writeToPasteBoard:(NSPasteboard *)pasteboard
{ // ***** need version to handle object type
	return [super writeToPasteBoard:pasteboard];
}

- setHalftone:(float)amount
{
	percent = MAX(MIN(amount, 1.0), 0.0);
	return self;
}

- (float)halftone { return percent; }

- (NSColor *)filterColor:(NSColor *)input
{ // halftone the color to print before printing it:
	float c, m, y, k;
	id workingColor = [MiscColor newColor:input];
	[workingColor getCyan:&c magenta:&m yellow:&y black:&k];
	[workingColor setCyan:(c * percent) magenta:(m * percent)
			yellow:(y * percent) black:(k * percent)];
	return [workingColor color];
}

- setColorFromObject:object
{
	if ([super setColorFromObject:object]) {
		// preserve halftoning info if available, if not, assume 100% color
		if ([object respondsToSelector:@selector(halftone)])
			percent = [object halftone];
		else percent = 1.0;
		return self;
	}
	return nil;
}

- (BOOL)isEqualHalftoneAmount:anObject
{
	if ([anObject respondsToSelector:@selector(halftone)])
		if ([anObject halftone] == percent) return YES;
	return NO;
}

- (BOOL)isEqualColor:anObject
{
	if ([super isEqualColor:anObject]) {
		return [self isEqualHalftoneAmount:anObject];
	} else {
		if ([anObject respondsToSelector:@selector(isEqualToColor:)])
			return [anObject isEqualToColor:[self filterColor:theColor]];
		else if ([anObject respondsToSelector:@selector(color)]) {
			id tempColor = [MiscColor newColor:[self filterColor:theColor]];
			BOOL ret = [tempColor isEqualToColor:[anObject color]];
			[tempColor release];
			return ret;
		}
	}
	return NO;
}

- (BOOL)isEqual:anObject
{
	if ([super isEqual:anObject]) {
		return [self isEqualHalftoneAmount:anObject];
	}
	return NO;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	[super initWithCoder:aDecoder];
	[aDecoder decodeValuesOfObjCTypes:"f", &percent];
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];
	[aCoder encodeValuesOfObjCTypes:"f", &percent];
}

- (void)encodeWithCoder:(NSPortCoder *)aPortCoder
{
	[super encodeWithCoder:aPortCoder];
	[aPortCoder encodeValueOfObjCType:"f" at:&percent];
}

- (id)initWithCoder:(NSPortCoder *)aPortCoder
{
	[super initWithCoder:aPortCoder];
	[aPortCoder decodeValueOfObjCType:"f" at:&percent];
	return self;
}

#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
- replacementObjectForPortCoder:(NSPortCoder *)aPortCoder
{
#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
	return [super replacementObjectForPortCoder:[aPortCoder connection]];
}

- (const char *)getInspectorClassName
{
	return "MiscHalftoneColorInspector";
}

- (NSImage *)getIBImage
{
	return [NSImage imageNamed:@"MiscHalftoneColorObj"];
}

- setColorFromPixel:(NSPoint *)location
{
	[super setColorFromPixel:location];
	percent = 1.0;
	return self;
}

@end