
#import <misckit/misckit.h>
#import <misckit/MiscRecordParser.h>

@implementation MiscRecordParser

- init
{
	id ret = [super init];
	if (ret) {
		fieldDelimiter = ':';
		recordDelimiter = '\n';
		firstLineIsKeys = YES;
		useWhites = NO;
	}
	return ret;
}

- (void)dealloc
{
	[self freeLastParse];
	{ [super dealloc]; return; };
}

- freeLastParse
{
	if (parsedRecords) {
		[parsedRecords freeObjects];
		[parsedRecords release];
	}
	return self;
}


- (List *)records       { return parsedRecords;   }
- (List *)keyNames      { return keyNames;        }
- (BOOL)firstLineIsKeys { return firstLineIsKeys; }
- (char)recordDelimiter { return recordDelimiter; }
- (char)fieldDelimiter  { return fieldDelimiter;  }
- (BOOL)delimitByWhitespace  { return useWhites; }
- setKeyNames:(List *)keys       { keyNames        = keys;  return self; }
- setFirstLineIsKeys:(BOOL)aFlag { firstLineIsKeys = aFlag; return self; }
- setRecordDelimiter:(char)aChar { recordDelimiter = aChar; return self; }
- setFieldDelimiter:(char)aChar  { fieldDelimiter  = aChar; return self; }
- setDelimitByWhitespace:(char)aChar { useWhites   = aChar; return self; }

- loadFromFileNamed:(MiscString *)aFileName
{
	MiscString *fileString = [MiscString new];

	[fileString addRepresentations:[NSImageRep imageRepsWithContentsOfFile:[aFileName stringValue]]];
	[fileString recalcLength];
	[self loadFromString:fileString];
	[fileString release];
	return self;
}

- loadFromString:(MiscString *)aString
{
	List *recordList; int i;

	[self freeLastParse];
	parsedRecords = [[List alloc] init];
	if (keyNames && firstLineIsKeys) {
		[keyNames freeObjects];
		[keyNames release];
	}

	// break into records
	recordList = [self _breakIntoRecordsAndFields:aString];

	// assemble dictionaries for the records
	for (i=0; i<[recordList count]; i++) {
		[parsedRecords addObject:
				[self _buildDictionaryForFields:[recordList objectAt:i]]];
	}
	[recordList freeObjects];
	[recordList release];
	return self;
}

- (MiscDictionary *)_buildDictionaryForFields:(List *)fieldList
{
	int i;
	MiscDictionary *newDict = [[MiscDictionary alloc] init];

	for (i=0; i<MAX([keyNames count], [fieldList count]); i++) {
		id field = [fieldList objectAt:i];
		id key = [keyNames objectAt:i];
		if (!field) field = [MiscString new];
		if (!key) key = [[MiscString alloc] initFromFormat:"f%d", (i + 1)];
		[newDict insertKey:key value:field];
	}
	return newDict;
}

- (List *)_breakIntoRecordsAndFields:(MiscString *)aString
{
	List *recordList = [[List alloc] init];
	id record = nil; int i = 0;

	while (record = 
			[aString extractPart:i++ useAsDelimiter:recordDelimiter]) {
		int j = 0; id field, fieldList = [[List alloc] init];
		if (firstLineIsKeys && (i < 2)) keyNames = fieldList;
		else [recordList addObject:fieldList];
		// break record into fields
		if (useWhites) {
			while (field = [record wordNum:j++]) {
				[fieldList addObject:field];
			}
		} else {
			while (field =
					[record extractPart:j++ useAsDelimiter:fieldDelimiter]) {
				[fieldList addObject:field];
			}
		}
		[record release];
	}
	return recordList;
}

@end
