//
//	MiscRtMatrix.m -- a wrapper around the RtMatrix structure.
//		Written by Thomas Engel Copyright (c) 1994 by Thomas Engel.
//				Version 1.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

/* MiscRtMatrix.m				 
 *
 * PDO does not allow pasing RtMatrix by value. So this is a wrapper object.
 * Anyway. The object is the subject. More features than RtMatrix.
 *
 * For interface-info see the header file. The comments in this file mostly
 * cover only the real implementation details.
 *
 * Written by: 		Thomas Engel
 * Created:    		13.04.1994 (Copyleft)
 * Last modified: 	23.04.1994
 */

#import <misckit/MiscRtMatrix.h>

@implementation MiscRtMatrix

- setByTranslation:(float)x :(float)y :(float)z
	   andRotation:(float)a :(float)b :(float)c
{
	// This code will calculate a transformation matrix from the give angles
	// and translation values.
	// The calculation has been tuned for speed!
	
	float	degreeToPi;
	double 	sa, ca,
			sb, cb,
			sc, cc;
	double 	scca, ccca, sasc, sacc;
			
	degreeToPi = M_PI / 180;
	
	a *= degreeToPi;
	b *= degreeToPi;
	c *= degreeToPi;
	
	m[0][0] = 1;	m[1][0] = 0;	m[2][0] = 0;	m[3][0] = x;
	m[0][1] = 0;	m[1][1] = 1;	m[2][1] = 0;	m[3][1] = y;
	m[0][2] = 0;	m[1][2] = 0;	m[2][2] = 1;	m[3][2] = z;
	m[0][3] = 0;	m[1][3] = 0;	m[2][3] = 0;	m[3][3] = 1;

	if( a != 0 && b == 0.0 && c == 0.0 )
	{
		// Rot about the x axis.

		m[1][1] = cos(a);
		m[1][2] = sin(a);
		m[2][1] = -m[1][2];
		m[2][2] = m[1][1];
	}
	else if( b != 0.0 && c == 0.0 && a == 0.0 )
	{
		// y Axis only
	
		m[0][0] = cos(b);
		m[2][0] = sin(b);
		m[0][2] = -m[2][0];
		m[2][2] = m[0][0];
	}
	else if( c != 0.0 && a == 0.0 && b == 0.0 )
	{
		// z axis only
		
		m[0][0] = cos(c);
		m[0][1] = sin(c);
		m[1][0] = -m[0][1];
		m[1][1] = m[0][0];
	}

	else
	{
		// This is the code needed to use more than one rotation.
		// The rotation [3][3] matrix will be computed this way.
		// Ri = (c*b)*a
		//
		//      [ cc*cb    -(sc*ca)+sa*cc*sb      sa*sc+cc*ca*sb ]
  		// Ri = [ sc*cb	      cc*ca+sa*sc*sb   -(sa*cc)+sc*ca*sb ]
        //      [ -sb	         cb*sa	              cb*ca      ]
		//
		// This will lead to an a,b,c rotation wich meas first rotating around
		// the x axis then around the y axis and we'll close with the z axis.
		// This order is very important !! 
		

 		sa = sin(a); 	ca = cos(a);
 		sb = sin(b); 	cb = cos(b);
 		sc = sin(c);	cc = cos(c);

 		m[0][0] = cc*cb;
 		m[0][1] = sc*cb;
 		m[0][2] = -sb;

 		scca=sc*ca; sasc=sa*sc; ccca=cc*ca; sacc=sa*cc;

 		m[1][0] = sacc*sb - scca;
 		m[1][1] = ccca + sasc*sb;
 		m[1][2] = cb*sa;

 		m[2][0] = sasc + ccca*sb;
		m[2][1] = scca*sb - sacc;
 		m[2][2] = cb*ca;

		// Now we have the real transform matrix.
		// Always remember the rotation direction.
	}
	return self;
}
			 
- setTransformMatrix:(RtMatrix)newTransform
{
	// Well stupid code. But its fast and I can live with it. Sorry.
	
	m[0][0] = newTransform[0][0];
	m[0][1] = newTransform[0][1];
	m[0][2] = newTransform[0][2];
	m[0][3] = newTransform[0][3];
	m[1][0] = newTransform[1][0];
	m[1][1] = newTransform[1][1];
	m[1][2] = newTransform[1][2];
	m[1][3] = newTransform[1][3];
	m[2][0] = newTransform[2][0];
	m[2][1] = newTransform[2][1];
	m[2][2] = newTransform[2][2];
	m[2][3] = newTransform[2][3];
	m[3][0] = newTransform[3][0];
	m[3][1] = newTransform[3][1];
	m[3][2] = newTransform[3][2];
	m[3][3] = newTransform[3][3];

	return self;
}

- getTransformMatrix:(RtMatrix)theMatrix
{
	// Well stupid code. But its fast and I can live with it. Sorry.
	// We will return the whole matrix here.
	
	theMatrix[0][0] = m[0][0];
	theMatrix[0][1] = m[0][1];
	theMatrix[0][2] = m[0][2];
	theMatrix[0][3] = m[0][3];
	theMatrix[1][0] = m[1][0];
	theMatrix[1][1] = m[1][1];
	theMatrix[1][2] = m[1][2];
	theMatrix[1][3] = m[1][3];
	theMatrix[2][0] = m[2][0];
	theMatrix[2][1] = m[2][1];
	theMatrix[2][2] = m[2][2];
	theMatrix[2][3] = m[2][3];
	theMatrix[3][0] = m[3][0];
	theMatrix[3][1] = m[3][1];
	theMatrix[3][2] = m[3][2];
	theMatrix[3][3] = m[3][3];

	return self;
}

- getTranslateMatrix:(RtMatrix)theMatrix
{
	// Still stupid code.
	// Returns ONLY the TRANSLATIVE part of the matrix.
	
	theMatrix[0][0] = 1;
	theMatrix[0][1] = 0;
	theMatrix[0][2] = 0;
	theMatrix[0][3] = 0;
	theMatrix[1][0] = 0;
	theMatrix[1][1] = 1;
	theMatrix[1][2] = 0;
	theMatrix[1][3] = 0;
	theMatrix[2][0] = 0;
	theMatrix[2][1] = 0;
	theMatrix[2][2] = 1;
	theMatrix[2][3] = 0;
	theMatrix[3][0] = m[3][0];
	theMatrix[3][1] = m[3][1];
	theMatrix[3][2] = m[3][2];
	theMatrix[3][3] = 1;

	return self;
}

- getRotateMatrix:(RtMatrix)theMatrix
{
	// More stupid code.
	// We will return ONLY the ROTATION part of the matrix.
	
	theMatrix[0][0] = m[0][0];
	theMatrix[0][1] = m[0][1];
	theMatrix[0][2] = m[0][2];
	theMatrix[0][3] = 0;
	theMatrix[1][0] = m[1][0];
	theMatrix[1][1] = m[1][1];
	theMatrix[1][2] = m[1][2];
	theMatrix[1][3] = 0;
	theMatrix[2][0] = m[2][0];
	theMatrix[2][1] = m[2][1];
	theMatrix[2][2] = m[2][2];
	theMatrix[2][3] = 0;
	theMatrix[3][0] = 0;
	theMatrix[3][1] = 0;
	theMatrix[3][2] = 0;
	theMatrix[3][3] = 1;

	return self;
}

#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
- replacementObjectForPortCoder:(NSPortCoder *)aPortCoder
{
    if ([aPortCoder isBycopy]) return self;
#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
    return [super replacementObjectForPortCoder:[aPortCoder connection]];
}

- (void)encodeWithCoder:(NSPortCoder *)aPortCoder
{
	[aPortCoder encodeValueOfObjCType:"f" at:&m[0][0]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[0][1]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[0][2]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[0][3]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[1][0]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[1][1]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[1][2]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[1][3]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[2][0]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[2][1]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[2][2]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[2][3]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[3][0]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[3][1]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[3][2]];
	[aPortCoder encodeValueOfObjCType:"f" at:&m[3][3]];
}

- (id)initWithCoder:(NSPortCoder *)aPortCoder
{
	[aPortCoder decodeValueOfObjCType:"f" at:&m[0][0]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[0][1]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[0][2]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[0][3]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[1][0]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[1][1]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[1][2]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[1][3]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[2][0]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[2][1]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[2][2]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[2][3]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[3][0]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[3][1]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[3][2]];
	[aPortCoder decodeValueOfObjCType:"f" at:&m[3][3]];

	return self;
}

- (BOOL)doesRotate
{
	if( m[1][0] != 0 ||
		m[2][0] != 0 || 
		m[2][1] != 0 ) return YES;
	return NO;
}

- (BOOL)doesTranslate
{
	if( m[3][0] != 0 || 
		m[3][1] != 0 || 
		m[3][2] != 0 ) return YES;
	return NO;
}

- (BOOL)doesScale
{
	return NO;
}

@end

/*
 * History: 23.04.94 Added multiple rotation calculation.
 *
 *			16.04.04 Using a RtMatrix internally now.
 *
 *			13.04.94 First test. Should help to notify rotaion start/end.
 *
 *
 * Bugs: - doesRotate only tests relevant sections. It does not check them
 *		   for consitency or if it really is a useful rotation.
 */