//
//	MiscRuntimeExtensions.m -- an object category to extend the runtime
//		Written by Don Yacktman Copyright (c) 1994 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/MiscBundleLoader.h>
#import <misckit/MiscString.h>
/*#import <AppKit/nextstd.h>*/
#import <stdlib.h>
#import <objc/objc-runtime.h>

@implementation NSObject(MiscRuntimeExtensions)

// Return the amount of memory an instance of this class takes up.
// This _only_ takes into account the instance variables themselves,
// and NOT what they point to.  Neither method should be changed;
// if you want to take into account the space allocated to ivars,
// subclass -sizeOf below.
+ (int)instanceSize
{
	return ((struct objc_class *) self)->instance_size;
}

- (int)instanceSize
{
	return [[self class] instanceSize];
}

- (int)sizeOf
{
	return [[self class] instanceSize];
}

// This method does its thing without any checking.  It is very dangerous
// and should only be used as a last resort.
- _becomeClass:aClass
{
	self->isa = aClass;
	return self;
}

// Change your class to a new class.  WARNING:  This is dangerous and
// could seriously cause havoc in your code if you use it unwisely!
// For sanity's sake, the receiver must either be a subclass of the
// class you want to change it into or must be of a smaller size.
- becomeClass:aClass
{
	if (![[self class] isKindOfClass:aClass])
		if ([[self class] instanceSize] < [aClass instanceSize])
			return nil;
	return [self _becomeClass:aClass];
}

// Attempt to compare two objects.  Uses -stringValue to do it.  You
// may wish to use some other method in a subclass.
- (int)compare:anObject
{
	const char *string1, *string2;
	if (anObject == self) return (0);
	if ([self respondsToSelector:@selector(stringValue)] &&
			[anObject respondsToSelector:@selector(stringValue)]) {
		string1 = (const char *)[self perform:@selector(stringValue)];
		string2 = (const char *)[anObject perform:@selector(stringValue)];
		if (string1 && string2) return (strcmp(string1, string2));
	}
	return(1);
}

static BOOL _Misc_visuals = NO;

- doesNotRecognizeSelector:(SEL)aSelector
{
	id message = [MiscString new];
	
	[message setFromFormat:
		"unrecognized -%s message sent to instance (0x%x) of %s class.\n",
#warning StringConversion: NSStringFromSelector return value is an NSString; use [ cString] if needed, otherwise change the target type to an NSString.
		NSStringFromSelector(aSelector), (unsigned)self, [[[self class] name] cString]];
	if (_Misc_visuals)
		NSRunAlertPanel(@"", [NSString stringWithCString:[message stringValueAndFree]], @"OK", nil, nil);
	else
		fprintf(stderr, [message stringValueAndFree]);
	return self;
}

+ (BOOL)miscVisualAlerts
{
	return _Misc_visuals;
}

+ setMiscVisualAlerts:(BOOL)aFlag
{
	_Misc_visuals = aFlag;
	return self;
}

// I named these methods this way instead of -forward... because we aren't
// passing a message on (forwarding) we are initiating a message...
- dispatchToDelegate:aDelegate message:(SEL)message;
{
	if ( aDelegate && [aDelegate respondsToSelector:message] )
		return [aDelegate perform:message];
	// No delegate or doesn't respond to message.	
	return self;
}

- dispatchToDelegate:aDelegate message:(SEL)message with:anObject;
{	
	if ( aDelegate && [aDelegate respondsToSelector:message] )
		return [aDelegate perform:message withObject:anObject];
	// No delegate or doesn't respond to message.	
	return self;
}

@end
