//
//	MiscScreenColor.m -- a class for color "tints"  (to specify a halftone)
//		Written by Don Yacktman Copyright (c) 1994 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/misckit.h>

@implementation MiscScreenColor

+ (BOOL)canInitWithPasteboard:(NSPasteboard *)pasteboard
{ // ***** need version to handle object type
	return [super canInitWithPasteboard:pasteboard];
}

- setFromPasteBoard:(NSPasteboard *)pasteboard
{ // ***** need version to handle object type
	return [super setFromPasteBoard:pasteboard];
}

- writeToPasteBoard:(NSPasteboard *)pasteboard
{ // ***** need version to handle object type
	return [super writeToPasteBoard:pasteboard];
}

- initColor:(NSColor *)color
{
	return [self initColorFromPixel:NULL inView:nil];
}

- initColorFromPixel:(NSPoint *)location
{
	return [self initColorFromPixel:location inView:nil];
}

- initColorFromPixel:(NSPoint *)location inView:view
{
	[self setLocation:location];
	[self setView:view];
	[self setDelegate:nil];
	[self updateColor];
	return self;
}

- (NSColor *)actualColor
{ // keep us up to date
	[self updateColor];
	return [super actualColor];
}

- (NSPoint)location { return theLocation; }
- view
{
	return theView;
}

- setView:aView
{
	id ret = [self view];
	theView = aView;
	return ret;
}

- setLocation:(NSPoint *)location
{
	if (!location) {
		theLocation.x = 0.0;
		theLocation.y = 0.0;
		return nil;
	}
	theLocation.x = location->x;
	theLocation.y = location->y;
	return self;
}

- updateColor
{
	id view = [self view];
	NSPoint location = [self location];
#error ViewConversion: '-focusView' in NSApplication has been replaced by '+focusView' in NSView
	if (view && !([NSView focusView] == view)) [view lockFocus];
	[super setColorFromPixel:&location];
#error ViewConversion: '-focusView' in NSApplication has been replaced by '+focusView' in NSView
	if (view && !([NSView focusView] == view)) [view lockFocus];
	return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	NSPoint newPoint;
	id newView;
	[super initWithCoder:aDecoder];
	newPoint = [aDecoder decodePoint];
	newView = [[aDecoder decodeObject] retain];
	[self setLocation:&newPoint];
	[self setView:newView];
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	id view = [self view];
	NSPoint point = [self location];
	[super encodeWithCoder:aCoder];
	[aCoder encodePoint:point];
	[aCoder encodeObject:view];
}

- (void)encodeWithCoder:(NSPortCoder *)aPortCoder
{
	id view = [self view];
	NSPoint point = [self location];
	[super encodeWithCoder:aPortCoder];
	[aPortCoder encodeValueOfObjCType:"f" at:&(point.x)];
	[aPortCoder encodeValueOfObjCType:"f" at:&(point.y)];
	[aPortCoder encodeValueOfObjCType:"@" at:&view];
}

- (id)initWithCoder:(NSPortCoder *)aPortCoder
{
	NSPoint newPoint;
	id newView;
	[super initWithCoder:aPortCoder];
	[aPortCoder decodeValueOfObjCType:"f" at:&(newPoint.x)];
	[aPortCoder decodeValueOfObjCType:"f" at:&(newPoint.y)];
	[aPortCoder decodeValueOfObjCType:"@" at:&newView];
	[self setLocation:&newPoint];
	[self setView:newView];
	return self;
}

#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
- replacementObjectForPortCoder:(NSPortCoder *)aPortCoder
{
#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
	return [super replacementObjectForPortCoder:[aPortCoder connection]];
}

- (const char *)getInspectorClassName
{
	return "MiscScreenColorInspector";
}

- (NSImage *)getIBImage
{
	return [NSImage imageNamed:@"MiscScreenColorObj"];
}

@end