//
//	Hugh Ashton - hugh@furuike.twics.com - Feb 95
// Copyright under the terms and conditions of the MiscKit
//				Version 1.1.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//
// This object allows a search to take place in more than one Text object
// simultaneously.  It also allows searching without having to press the 
// Return key after each search string.
//
// It is inited with the initWithField:anObject andTarget:anotherObject
// method, where anObject is Text (not a TextField) and anotherObject is
// a List of Text objects.
//
// Added dynamic sizing to the original[Start|End] arrays, minor tweaks
//		-- Don Yacktman 4/13/95
//


#import <misckit/MiscSearcher.h>

@implementation MiscSearcher

- initWithField:anObject andList:anotherObject
{
	int i;
	[super init];
	
	theSearcher=anObject;
	[theSearcher setDelegate:self];
	theList=anotherObject;
	if (![theList isKindOfClassNamed:"List"] ||
			![theSearcher isKindOfClassNamed:"Text"]) {
		[self release];
		return nil;
	}
	areWeSearching=NO;
	if (originalStart) free(originalStart);
	if (originalEnd) free(originalEnd);
	originalStart = (int *)malloc(sizeof(int) * [theList count]);
	originalEnd = (int *)malloc(sizeof(int) * [theList count]);
	for (i=0;i<=[theList count];i++) {
		originalStart[i]=0;
		originalEnd[i]=0;
	}
	return self;
}

// delegate methods for theSearcher

#warning NotificationConversion: 'textDidChange:' used to be 'textDidGetKeys:'.  This conversion assumes this method is implemented or sent to a delegate of NSText.  If this method was implemented by a NSMatrix or NSTextField textDelegate, use the text notifications in NSControl.h.
- (void)textDidChange:(NSNotification *)notification
{
#warning NotificationConversion: if this notification was not posted by NSText (eg. was forwarded by NSMatrix or NSTextField from the field editor to their textDelegate), then the text object is found by [[notification userInfo] objectForKey:@"NSFieldEditor"] rather than [notification object]
	NSText *theText = [notification object];
    BOOL flag = [[theText text] isEqualToString:@""];
    int i;
	NSSelPt start,end;
	char theString[MAXPATHLEN-1]="";
	areWeSearching=YES;
	
	for(i=0;i<=[theList count];i++)
	{
		theTarget=[theList objectAt:i];
		if(flag)
		{
			[theTarget setSelectionStart:originalStart[i] end:originalEnd[i]];
			[theTarget setSelColor:[NSColor blackColor]];
#warning TextConversion: [[theSearcher text] cStringLength] was [theSearcher byteLength]. Use of 'cStringLength' to replace 'byteLength' may be wrong
#warning TextConversion: [[theSearcher text] cStringLength] was [theSearcher byteLength]. Use of 'cStringLength' to replace 'byteLength' may be wrong
			[theSearcher setSelectionStart:[[theSearcher text] cStringLength] end:[[theSearcher text] cStringLength]];
			areWeSearching=NO;
			return;
		}
#warning TextConversion: [[theSearcher text] cStringLength] was [theSearcher byteLength]. Use of 'cStringLength' to replace 'byteLength' may be wrong
#warning TextConversion: 'substringFromRange:' used to be 'getSubstring:start:length:'; make sure the range is valid.
		theString = [[theSearcher substringFromRange:NSMakeRange(0, [[theSearcher text] cStringLength])] cString];
		if([theTarget findText:[NSString stringWithCString:theString] ignoreCase:NO backwards:NO wrap:YES])
		{
			[theTarget getSelectionStart:&start end:&end];
			[theTarget setSelectionStart:originalStart[i] end:originalEnd[i]];
			[theTarget setSelColor:[NSColor blackColor]];
			[theTarget setSelectionStart:start.cp end:end.cp];
			[theTarget setSelColor:[NSColor darkGrayColor]];
			originalStart[i]=start.cp;
			originalEnd[i]=end.cp;
			[theTarget setSelectionStart:originalStart[i] end:originalEnd[i]];
		}
#warning TextConversion: [[theSearcher text] cStringLength] was [theSearcher byteLength]. Use of 'cStringLength' to replace 'byteLength' may be wrong
#warning TextConversion: [[theSearcher text] cStringLength] was [theSearcher byteLength]. Use of 'cStringLength' to replace 'byteLength' may be wrong
		[theSearcher setSelectionStart:[[theSearcher text] cStringLength] end:[[theSearcher text] cStringLength]];
	}
	areWeSearching=NO;
}


#error NSObjectConversion:  'forward:' has been replaced by forwardInvocation: which has different calling semantics.  Convert by hand.
- forward:sender
{
	int i;
	NSSelPt start,end;
	char theString[MAXPATHLEN-1]="";
	areWeSearching=YES;

#warning TextConversion: [[theSearcher text] cStringLength] was [theSearcher byteLength]. Use of 'cStringLength' to replace 'byteLength' may be wrong
#warning TextConversion: 'substringFromRange:' used to be 'getSubstring:start:length:'; make sure the range is valid.
	theString = [[theSearcher substringFromRange:NSMakeRange(0, [[theSearcher text] cStringLength])] cString];
	for(i=0;i<=[theList count];i++)
	{
		theTarget=[theList objectAt:i];
		if((strlen(theString))==0)
		{
			[theTarget setSelectionStart:originalStart[i] end:originalEnd[i]];
			[theTarget setSelColor:[NSColor blackColor]];
#warning TextConversion: [[theSearcher text] cStringLength] was [theSearcher byteLength]. Use of 'cStringLength' to replace 'byteLength' may be wrong
#warning TextConversion: [[theSearcher text] cStringLength] was [theSearcher byteLength]. Use of 'cStringLength' to replace 'byteLength' may be wrong
			[theSearcher setSelectionStart:[[theSearcher text] cStringLength] end:[[theSearcher text] cStringLength]];
			areWeSearching=NO;
			return self;
		}
		[theTarget setSelectionStart:originalEnd[i] end:originalEnd[i]];
		if([theTarget findText:[NSString stringWithCString:theString] ignoreCase:NO backwards:NO wrap:YES])
		{
			[theTarget getSelectionStart:&start end:&end];
			[theTarget setSelectionStart:originalStart[i] end:originalEnd[i]];
			[theTarget setSelColor:[NSColor blackColor]];
			[theTarget setSelectionStart:start.cp end:end.cp];
			[theTarget setSelColor:[NSColor darkGrayColor]];
			originalStart[i]=start.cp;
			originalEnd[i]=end.cp;
			[theTarget setSelectionStart:end.cp end:end.cp];
		}
#warning TextConversion: [[theSearcher text] cStringLength] was [theSearcher byteLength]. Use of 'cStringLength' to replace 'byteLength' may be wrong
#warning TextConversion: [[theSearcher text] cStringLength] was [theSearcher byteLength]. Use of 'cStringLength' to replace 'byteLength' may be wrong
		[theSearcher setSelectionStart:[[theSearcher text] cStringLength] end:[[theSearcher text] cStringLength]];
	}
	areWeSearching=NO;
	return self;
}

#warning NotificationConversion: 'textDidEndEditing:' used to be 'textDidEnd:'.  This conversion assumes this method is implemented or sent to a delegate of NSText.  If this method was implemented by a NSMatrix or NSTextField textDelegate, use the text notifications in NSControl.h.
- (void)textDidEndEditing:(NSNotification *)notification
{
#warning NotificationConversion: if this notification was not posted by NSText (eg. was forwarded by NSMatrix or NSTextField from the field editor to their textDelegate), then the text object is found by [[notification userInfo] objectForKey:@"NSFieldEditor"] rather than [notification object]
	NSText *theText = [notification object];
    int theChar = [[[notification userInfo] objectForKey:@"NSTextMovement"] intValue];
    int i;
	if(!areWeSearching)
	{
		[theSearcher setDelegate:nil];
// this stops us being interrupted while we clean up the target
		for(i=0;i<=[theList count];i++)
		{
			theTarget=[theList objectAt:i];
			[theTarget setSelectionStart:originalStart[i] end:originalEnd[i]];
			[theTarget setSelColor:[NSColor blackColor]];
			[theTarget setSelectionStart:originalStart[i] end:originalEnd[i]];
		}
//	go back to being a good delegate
		[theSearcher setDelegate:self];
//		NXRunAlertPanel("We've set the SEL",NULL,"OK",NULL,NULL);
		areWeSearching=YES;
	}
}

- setTheField:anObject
{
	theSearcher=anObject;
	if(![theSearcher isKindOfClassNamed:"Text"]) 
	{
		return nil;
	}
	[theSearcher setDelegate:self];
	areWeSearching=NO;
	return self;
}

- setTheList:anObject
{
	int i;
	
	if(![theList isKindOfClassNamed:"List"]) 
	{
		return nil;
	}
	theList=anObject;
	if (originalStart) free(originalStart);
	if (originalEnd) free(originalEnd);
	originalStart = (int *)malloc(sizeof(int) * [theList count]);
	originalEnd = (int *)malloc(sizeof(int) * [theList count]);
	for(i=0;i<=[theList count];i++)
	{
		originalStart[i]=0;
		originalEnd[i]=0;
	}
	areWeSearching=NO;
	return self;
}

// wrapper methods

- theList
{
	return theList;
}

- theField
{
	return theSearcher;
}

@end
