/*  Abstract class to act as a cover for sockets.
 *
 *  Copyright (c) 1994 Christopher J. Kane.
 *
 *  This software is subject to the terms of the MiscKit license
 *  agreement. Refer to the license document included with the
 *  MiscKit distribution for these terms.
 *
 *  Version 1.1 (18 September 1994)
 */

#import <misckit/MiscSocket.h>
#import <sys/errno.h>
extern int close(int);
extern int dup(int);
extern int errno;

@implementation MiscSocket

- init
{
	static int initCount = 0;
	if (0 <= sock && 0 < initCount) {
		int old_errno = errno;
		close(sock);
		errno = old_errno;
	}
	initCount++;
	sock = domain = type = -1;
	return self;
}

- initDomain:(int)aDomain type:(int)aType
{
	[self init];
	switch (aType) {
	case MiscSOCK_RAW:
		sock = socket(aDomain, SOCK_RAW, 0);
		break;
	case MiscSOCK_DGRAM:
		sock = socket(aDomain, SOCK_DGRAM, 0);
		break;
	case MiscSOCK_STREAM:
		sock = socket(aDomain, SOCK_STREAM, 0);
		break;
	default :
		sock = -1;
		errno = ESOCKTNOSUPPORT;
	}
	if (sock < 0)
		return [super release];
	domain = aDomain;
	type = aType;
	return self;
}

- (void)close
{
	return [self init];
}

- copyWithZone:(NSZone *)zone
{
	MiscSocket *copy = [super copyWithZone:zone];
	if (sock != -1) {
		copy->sock = dup(sock);
		if (copy->sock == -1)
			return [copy release];
	}
	return copy;
}

- (void)dealloc
{
	[self close];
	{ [super dealloc]; return; };
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	[super initWithCoder:aDecoder];
	[aDecoder decodeValuesOfObjCTypes:"iii", &sock, &domain, &type];
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];
	[aCoder encodeValuesOfObjCTypes:"iii", &sock, &domain, &type];
}

- (BOOL)isClosed
{
	return (sock == -1 && domain == -1 && type == -1);
}

- (NSString *)domain
{
	return [NSString stringWithCString:domain];
}

- (int)socket
{
	return sock;
}

- (int)type
{
	return type;
}

@end
