//
//	MiscStopwatch.m -- a generic class to time things (measure)
//		Written by Don Yacktman Copyright (c) 1993 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//		This is a free object!  Contact the author for the latest version.
//		Don Yacktman, 4279 N. Ivy Lane, Provo, UT, 84604
//		e-mail:  Don_Yacktman@byu.edu
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/MiscStopwatch.h>

@implementation MiscStopwatch

- init
{
	id ret = [super init];
	[self clearTiming:self];
	isRelative = YES;
	paused = YES;
	return ret;
}

- setRelative:(BOOL)t // overridden
{
	isRelative = YES;
	return self;
}


- clearTiming:sender
{
	struct timezone tzp;
	gettimeofday(&startTime, &tzp);
	startTime.tv_sec = 0;
	startTime.tv_usec = 0;
	[[[[[[[[self setMicrosecond:0] setSecond:0] setMinute:0] setHour:0]
			setDay:0] setMonth:0] setYear:0] calcDayOfWeek];
	return self;
}

- startTiming:sender
{
	[self clearTiming:sender];
	[self continueTiming:sender];
	return self;
}

- continueTiming:sender
{
	struct timezone tzp;
	if (!paused) return self;	// already running
	paused = NO;
	gettimeofday(&startTime, &tzp);
	return self;
}

- pauseTiming:sender
{
	[self calcElapsedTime:sender];
	paused = YES;
	return self;
}

- calcElapsedTime:sender
{	// adds elapsed time to the time's value and continues
	struct timeval endTime;
	struct timezone tzp;
	
	if (!paused) { // if we're paused, no time to add.
		gettimeofday(&endTime, &tzp);
		[self addMicroseconds:(endTime.tv_usec - startTime.tv_usec)];
		[self addSeconds:(endTime.tv_sec - startTime.tv_sec)];
		startTime.tv_sec = endTime.tv_sec;
		startTime.tv_usec = endTime.tv_usec;
	}
	return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	[super initWithCoder:aDecoder];
	[aDecoder decodeValuesOfObjCTypes:"cll",
			&paused, &startTime.tv_sec, &startTime.tv_usec];
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];
	[aCoder encodeValuesOfObjCTypes:"cll",
			&paused, &startTime.tv_sec, &startTime.tv_usec];
}

// NXTransport protocol implementation:
- (void)encodeWithCoder:(NSPortCoder *)aPortCoder
{
	[super encodeWithCoder:aPortCoder];
	[aPortCoder encodeValueOfObjCType:"c" at:&paused];
	[aPortCoder encodeValueOfObjCType:"i" at:&startTime.tv_sec];
	[aPortCoder encodeValueOfObjCType:"i" at:&startTime.tv_usec];
}

- (id)initWithCoder:(NSPortCoder *)aPortCoder
{
	[super initWithCoder:aPortCoder];
	[aPortCoder decodeValueOfObjCType:"c" at:&paused];
	[aPortCoder decodeValueOfObjCType:"i" at:&startTime.tv_sec];
	[aPortCoder decodeValueOfObjCType:"i" at:&startTime.tv_usec];
	return self;
}

- (const char *)format:(const char *)format
{
    struct tm tm;

    if (!myStringValue) myStringValue = (char *)malloc(256);
    tm.tm_sec = second;
    tm.tm_min = minute;
    tm.tm_hour = hour;
    tm.tm_mday  = day + 1;
    tm.tm_mon = month;
    tm.tm_year = year;

	// Note that day of week and day of year are essentially
	// meaningless for a relative time like this, so no call
	// to mktime is done.
	tm.tm_wday = 0;

    strftime( myStringValue, 256, format, &tm );
    
    return myStringValue;
}

@end
