/*--------------------------------------------------------------------------
 *
 * 	MiscTableSource
 *
 * 	Inherits From:	List
 *
 * 	Conforms To:	NXTransport,DBTableDataSources
 *
 * 	Declared In:	MiscTableSource.m
 *
 *
 * 	$Log$
 *
 * Copyright (C) 1995 Vince Demarco <vdemarco@bou.shl.com>
 * Use is governed by the MiscKit license
 *--------------------------------------------------------------------------*/
#import <Foundation/NSObject.h>
/*#import <AppKit/nextstd.h>*/
#import <Foundation/NSConnection.h>
#import <Foundation/NSDistantObject.h>
#import <Foundation/NSPort.h>
#import <machkit/reference.h>
#import <Foundation/NSPortCoder.h>
#import <stdlib.h>
#import <ctype.h>

#import <misckit/MiscTableSource.h>
#import <misckit/MiscListSorting.h>
#import <dbkit/DBValue.h>

@implementation MiscTableSource

- (unsigned int)rowCount
{
    return [self count];
}

- (unsigned int)columnCount
{
    return columnCount;
}

- setColumnCount:(unsigned int)aInt
{
    columnCount = aInt;
    return self;
}

- getValueFor:identifier at:(unsigned int) aPosition into:aValue
{
    Method methodStruct;
    SEL    aSelector;
    id     theObject;

    theObject    = [self objectAt:aPosition];
    aSelector	 = [self getSelectorForIdentifer:(const char *)identifier];
    methodStruct = class_getInstanceMethod([theObject class],(SEL)aSelector);

    if (aSelector && methodStruct && theObject){
	/* Here i am checking the return type of the the method (Well I hope that I am checking)
	 *                                                      (the Return type of the method)
	 */
	switch(methodStruct->method_types[0]){
	case _C_ID:
	    [aValue setObjectValue:(id)[theObject perform:aSelector]];
	    break;
	case _C_SHT:
	case _C_USHT:
	    {
		short int val = ((int(*)(id,SEL))[theObject methodForSelector:aSelector])
		    (theObject,aSelector);
		[aValue setIntValue:val];
		break;
	    }
	case _C_INT:
	case _C_UINT:
	    {
		int val = ((int(*)(id,SEL))[theObject methodForSelector:aSelector])
		    (theObject,aSelector);
		[aValue setIntValue:val];
		break;
	    }
	case _C_LNG:
	case _C_ULNG:
	    {
		long int val = ((int(*)(id,SEL))[theObject methodForSelector:aSelector])
		    (theObject,aSelector);
		[aValue setIntValue:val];
		break;
	}
	case _C_FLT:
	    {
		float val = ((float(*)(id,SEL))[theObject methodForSelector:aSelector])
		    (theObject,aSelector);
		[aValue setFloatValue:val];
		break;
	}
	case _C_DBL:
	    {
		double val = ((double(*)(id,SEL))[theObject methodForSelector:aSelector])
		    (theObject,aSelector);
		[aValue setDoubleValue:val];
		break;
	}
	case _C_CHARPTR:
	    {
		const char *val = ((const char*(*)(id,SEL))[theObject methodForSelector:aSelector])
		    (theObject,aSelector);
		[aValue setStringValue:[NSString stringWithCString:val]];
		break;
	    }
	default:
	    break;
	}
    }else{
	[aValue setNull];
	if(theObject){
#warning StringConversion: NSStringFromSelector return value is an NSString; use [ cString] if needed, otherwise change the target type to an NSString.
	    NSLog(@"[%s:%s]: %s has no getSelector for \"%s\" at %u", [[[self class] name] cString],NSStringFromSelector(_cmd),[[[theObject class] name] cString], identifier, aPosition);
	}else{
#warning StringConversion: NSStringFromSelector return value is an NSString; use [ cString] if needed, otherwise change the target type to an NSString.
	    NSLog(@"[%s:%s]: attempt to get value at position %u which is not in the list", [[[self class] name] cString],NSStringFromSelector(_cmd),aPosition);
	}
    }
    return self;
}

- setValueFor:identifier at:(unsigned int) aPosition from:aValue
{
    SEL         aSelector;
    Method      methodStruct;
    const char *type;
    int         offset;
    id          theObject;

    theObject    = [self objectAt:aPosition];
    aSelector	 = [self setSelectorForIdentifer:(const char *)identifier];
    methodStruct = class_getInstanceMethod([theObject class],(SEL)aSelector);

    if (aSelector && methodStruct && theObject && method_getNumberOfArguments(methodStruct) == 3){

	method_getArgumentInfo(methodStruct,2,&type,&offset);
	switch(type[0]){
	case _C_ID:
	    ((void(*)(id,SEL,id))[theObject methodForSelector:aSelector])
		(theObject,aSelector,[aValue objectValue]);
	    break;
	case _C_SHT:
	case _C_USHT:
	    ((void(*)(id,SEL,short int))[theObject methodForSelector:aSelector])
		(theObject,aSelector,[aValue intValue]);
	    break;
	case _C_INT:
	case _C_UINT:
	    ((void(*)(id,SEL,int))[theObject methodForSelector:aSelector])
		(theObject,aSelector,[aValue intValue]);
	    break;
	case _C_LNG:
	case _C_ULNG:
	    ((void(*)(id,SEL,long int))[theObject methodForSelector:aSelector])
		(theObject,aSelector,[aValue intValue]);
	    break;
	case _C_FLT:
	    ((void(*)(id,SEL,float))[theObject methodForSelector:aSelector])
		(theObject,aSelector,[aValue floatValue]);
	    break;
	case _C_DBL:
	    ((void(*)(id,SEL,double))[theObject methodForSelector:aSelector])
		(theObject,aSelector,[aValue doubleValue]);
	    break;
	case _C_CHARPTR:
	    ((void(*)(id,SEL,const char *))[theObject methodForSelector:aSelector])
		(theObject,aSelector,[[aValue stringValue] cString]);
	    break;
	default:
	    return nil;
	    break;
	}
    }else{
	return nil;
    }
    return self;
}

- (SEL)getSelectorForIdentifer:(const char *)identifier
{
    if (identifier){
#warning StringConversion: check argument type for NSSelectorFromString().
	if (NSSelectorFromString([NSString stringWithCString:identifier])){            /* Try just identifer */
#warning StringConversion: check argument type for NSSelectorFromString().
	    return NSSelectorFromString([NSString stringWithCString:identifier]);
	}else{
	    char *selectorName;

	    selectorName = alloca(strlen(identifier)+8); /* Try getIdentifer */
	    sprintf(selectorName,"get%s",identifier);
	    selectorName[3] = toupper(selectorName[3]);
#warning StringConversion: check argument type for NSSelectorFromString().
	    return NSSelectorFromString([NSString stringWithCString:selectorName]);
	}
    }
    return (SEL)0;
}

- (SEL)setSelectorForIdentifer:(const char *)identifier
{
    if (identifier){
	char *selectorName;

	selectorName = alloca(strlen(identifier)+8); /* Try getIdentifer */
	sprintf(selectorName,"set%s:",identifier);
	selectorName[3] = toupper(selectorName[3]);

#warning StringConversion: check argument type for NSSelectorFromString().
	if (NSSelectorFromString([NSString stringWithCString:selectorName])){        /* try setIdentifer: */
#warning StringConversion: check argument type for NSSelectorFromString().
	    return NSSelectorFromString([NSString stringWithCString:selectorName]);
	}else{                                /* try setidentifer: */
	    sprintf(selectorName,"set%s:",identifier);
#warning StringConversion: check argument type for NSSelectorFromString().
	    return NSSelectorFromString([NSString stringWithCString:selectorName]);
	}
    }
    return (SEL)0;
}

- sortUsingIdentifer:(const char *)identifer ascending:(BOOL)aBool
{
    return [self sortUsing:[self getSelectorForIdentifer:identifer] ascending:aBool];
}

/* Archiving Methods */
- (id)initWithCoder:(NSCoder *)aDecoder
{
    [super initWithCoder:aDecoder];
    [aDecoder decodeValueOfObjCType:"i" at:&columnCount];
    return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
    [super encodeWithCoder:aCoder];
    [aCoder encodeValueOfObjCType:"i" at:&columnCount];
}

/* NXTransport Protocol*/
#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
- replacementObjectForPortCoder:(NSPortCoder *)aPortCoder
{
    if ([aPortCoder isBycopy])
        return self;
#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
    return [super replacementObjectForPortCoder:[aPortCoder connection]];
}

- (void)encodeWithCoder:(NSPortCoder *)aPortCoder
{
    int     n = [self count];
    int counter;

    [aPortCoder encodeValueOfObjCType:"i" at:&columnCount];
    [aPortCoder encodeValueOfObjCType:"i" at:&n];
    for (counter = 0; counter < n; ++counter) {
        id      anObject = [self objectAt:counter];
        [aPortCoder encodeObject:anObject isBycopy:YES];
    }
}

- (id)initWithCoder:(NSPortCoder *)aPortCoder
{
    int counter, n;

    [aPortCoder decodeValueOfObjCType:"i" at:&columnCount];
    [aPortCoder decodeValueOfObjCType:"i" at:&n];
    [self initCount:n];
    for (counter = 0; counter < n; ++counter) {
        id anObject = [aPortCoder decodeObject];
        [self addObject:anObject];
    }
    return self;
}

@end
