//
//	MiscTaggedImage.m -- NXImage subclass that has an integer tag
//		Written by Don Yacktman Copyright (c) 1994 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	


#import <misckit/MiscTaggedImage.h>
#import <AppKit/AppKit.h>
#import <objc/objc-runtime.h>

#define CLASS_VERSION		1
#define CLASS_NAME			"MiscTaggedImage"

@interface MiscTaggedImage(private)

+ _tagHash;

@end


@implementation MiscTaggedImage

+ (void)initialize
// Set the version.
{
#warning NSNameConversion:  Class names have changed. Check that the class name passed in is real.
	if (self == objc_lookUpClass(CLASS_NAME))  {
		[self setVersion:CLASS_VERSION];
	}
	return;
}

+ _tagHash
{
	static id tagHash = nil;
	if (!tagHash)
		tagHash = [[HashTable allocWithZone:[self zone]] 
				initKeyDesc:"i" valueDesc:"@"];
	return tagHash;
}

+ findImageWithTag:(int)aTag
{
	return (id)[[self _tagHash] valueForKey:&aTag];
}

- (void)setTag:(int)newTag
{
	int oldTag = tag;
	id _tagHash = [[self class] _tagHash];

	tag = newTag;
	if (self == [[self class] findImageWithTag:oldTag]) {
		[_tagHash removeKey:&oldTag];
	}
	[_tagHash insertKey:&tag value:self];
}

- (int)tag
{
	return tag;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
	int version;

	[super initWithCoder:aDecoder];
	version = [aDecoder versionForClassName:[NSString stringWithCString:CLASS_NAME]];
	if ( version == CLASS_VERSION ) {
		[aDecoder decodeValuesOfObjCTypes:"i",&tag];
	} else {
#warning StringConversion: NSStringFromSelector return value is an NSString; use [ cString] if needed, otherwise change the target type to an NSString.
		NSLog(@"[%s %s] - unknown version of %s in typed stream", [[[self class] name] cString], NSStringFromSelector(_cmd), [[[self class] name] cString]);
	}
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];	
	[aCoder encodeValuesOfObjCTypes:"i",&tag];
}

@end
