//
//	MiscTime.m -- a generic class to simplify manipulation of times
//		Written by Don Yacktman Copyright (c) 1993 by Don Yacktman.
//				Version 1.2.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//		This is a free object!  Contact the author for the latest version.
//		Don Yacktman, 4279 N. Ivy Lane, Provo, UT, 84604
//		e-mail:  Don_Yacktman@byu.edu
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

	// *****  denotes an unfinished/unimplemented method.

#import <misckit/MiscTime.h>
#import <sys/time.h>

// a few functions to handle date wierdnesses

static int isLeapYear(int year)
{
	if ( year % 400 == 0 ) return 1;	// 2000 is a leap year
	if ( year % 100 == 0 ) return 0;	// 1900 is not a leap year
	if ( year %   4 == 0)  return 1;	// 1996 is a leap year
	return 0;				// 1994 is not a leap year
}

static int Misc_daysUpTo(int month, int year)
{
	int array[12] = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };
	if (month < 2) return array[month];
	return (array[month] + (isLeapYear(year) ? 1 : 0));
}

static int Misc_daysIn(int month, int year)
{
	int array[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
	if (month != 1) return array[month];
	return (array[month] + (isLeapYear(year) ? 1 : 0));
}

@implementation MiscTime

+ (void)initialize
{
	if ( self == [MiscTime class] ) [MiscTime setVersion:1];
	return;
}

+ (int)daysInMonth:(int)aMonth ofYear:(int)aYear
{
	return Misc_daysIn(aMonth, aYear);
}

+ (int)daysUpToMonth:(int)aMonth ofYear:(int)aYear
{
	return Misc_daysUpTo(aMonth, aYear);
}

+ (int)isLeapYear:(int)aYear
{
	return isLeapYear(aYear);
}

- init
{
	id ret;
	if (!_initted) ret = [super init];
	else ret = self;
	_initted = YES;
	dow = MISC_TIME_UNKNOWN_DOW;
	if (!myStringValue) myStringValue = (char *)malloc(256);
	year = 0; day = 0; month = 0; hour = 0;
	minute = 0; second = 0; microsecond = 0;
	isRelative = NO;
	return ret;
}

- initWithCurrentTime
{
	struct timeval tp;
	struct timezone tzp;
	struct tm *tv;
	
	id ret = [self init];

	gettimeofday(&tp, &tzp);
	tv = localtime(&tp.tv_sec);
	
	microsecond = tp.tv_usec;
	second = tv->tm_sec;
	minute = tv->tm_min;
	hour   = tv->tm_hour;
	day    = tv->tm_mday - 1;
	dow    = tv->tm_wday;
	month  = tv->tm_mon;
	year   = tv->tm_year + 1900;
	return ret;
}

- initWithTime_t: (time_t)aTime
{
	id  ret = [self init];
	struct tm  *tv;
	
	tv = localtime ((const time_t *)&aTime);
	microsecond = 0;
	second = tv->tm_sec;
	minute = tv->tm_min;
	hour   = tv->tm_hour;
	day    = tv->tm_mday - 1;
	dow    = tv->tm_wday;
	month  = tv->tm_mon;
	year   = tv->tm_year + 1900;
	return ret;
}

/* This is the original

- calcDayOfWeek
{
	// Fill in a struct tm and let mktime figure out the day of the week.
	// (0 = Sunday)
	struct tm tm;
	time_t t;
	
	tm.tm_sec = second;
	tm.tm_min = minute;
	tm.tm_hour = hour;
	tm.tm_mday  = day + 1;
	tm.tm_mon = month;
	tm.tm_year = year - 1900;
	
	t = mktime(&tm);
	
	// Upon succesful completion of mktime(), the tm_mday and tm_wday fields
	// are set correctly. Record the day of the week.
	
	dow = tm.tm_wday;
	
	return self;
}
*/
/*	This is an alternate version by Kurt Werle.  Which is better?
	Does it even matter?

- calcDayOfWeek
{
	// I need the next two just to get timezone info
	struct timeval tp;
	struct timezone tzp;

	struct tm tv;
	time_t realtime;
	
	tv.tm_sec = second;
	tv.tm_min = minute;
	tv.tm_hour = hour;
	tv.tm_mday = day + 1;
	tv.tm_mon = month;
	tv.tm_year = year - 1900;

	gettimeofday(&tp, &tzp);
	tv.tm_gmtoff = localtime(&tp.tv_sec)->tm_gmtoff;

	realtime = mktime(&tv);
	dow = localtime(&realtime)->tm_wday;
	
	return self;
}
*/
/*	This is an alternate version by Carl Lindberg.  This is probably
	faster, but has the disadvantage of working only after the Gregorian
	calendar.  Seems to be more reliable than mktime() for dates _after_
	that, however.
	
	We're using it since it seems to have the fewest problems, though it
	is somewhat imperfect.  At least it works outside of the range of
	years from 1970 to 2037, which seems to be mktime's limit.  This one
	works fom 1752 onward.  (Note that the calendar changed in 1752...) 
*/
- calcDayOfWeek
{
	long tmonth = month+1;
	long tyear = year;
	long tday = day+1;
	long century;
	
	if ((tmonth == 1) || (tmonth == 2)) {
	  tmonth += 12;
	  tyear--;
	}
	tmonth -= 2;
	century = tyear / 100;
	tyear %= 100;
	dow = ((260 * tmonth - 19) / 100) + tday + tyear + (tyear / 4) 
	      + (century / 4) - (2 * century);
	dow %= 7;
	
	return self;
}


- setYear:(int)t { year = t; dow = MISC_TIME_UNKNOWN_DOW; return self; }
- setMonth:(int)t { month = t; dow = MISC_TIME_UNKNOWN_DOW; return self; }
- setDayOfWeek:(int)t { dow = t; return self; }
- setDay:(int)t { day = t; dow = MISC_TIME_UNKNOWN_DOW; return self; }
- setHour:(int)t { hour = t; return self; }
- setMinute:(int)t { minute = t; return self; }
- setSecond:(int)t { second = t; return self; }
- setMicrosecond:(int)t { microsecond = t; return self; }
- setRelative:(BOOL)t { isRelative = t; return self; }

- (int)_nintValue // ignores usecs.
{
	return ((((((year - 1970) * MISC_TIME_MAX_DAYS
				+ Misc_daysUpTo(month, year) + day)
				* MISC_TIME_MAX_HOURS) + hour)
				* MISC_TIME_MAX_MINUTES + minute)
				* MISC_TIME_MAX_SECONDS + second);
}

- (int)intValue // # seconds since Jan 1, 1970
{ // rounds usec. and add in...
	return ([self _nintValue] +
				((microsecond < MISC_TIME_MAX_MICROSECONDS / 2) ? 0 : 1));
}

- (double)doubleValue
{
	// *****
	return 0.0;
}

- (float)floatValue
{
	// *****
	return 0.0;
}


/*
 * Return a formatted string in the manner of strftime(), where
          %%   identical to %
          %a   abbreviated weekday name
          %A   full weekday name
          %b   abbreviated month name
          %B   full month name
          %c   time and date using the time and date representa-
               tion for the locale (%X %x)
          %d   day of the month as a decimal number (01-31)
          %H   hour based on a 24-hour clock as a decimal number
               (00-23)
          %I   hour based on a 12-hour clock as a decimal number
               (01-12)
          %j   day of the year as a decimal number (001-366)
          %m   month as a decimal number (01-12)
          %M   minute as a decimal number (00-59)
          %p   AM/PM designation associated with a 12-hour clock
          %S   second as a decimal number (00-61)
          %w   weekday as a decimal number (0-6), where Sunday is 0
          %x   date using the date representation for the locale
          %X   time using the time representation for the locale
          %y   year without century (00-99)
          %Y   year with century (e.g. 1990)
          %Z   time zone name or abbreviation, or no characters
               if no time zone is determinable.
*/

/*
 *  don't free the result!.  (need to rethink this to use MiscString or something
 *  instead of blindly allocating 256 chars.)
 */	       
- (const char *)format:(const char *)format
{ /* Warning:  This is buggy due to mktime() and strftime(), which may
	 crash your app!  Worse, mktime() ssems to only work for the range
	 of years between 1970 and 2037.  Be forewarned!  Anyone have a
	 better strftime() we can use instead?  That would help! */

    struct tm tm;
    
     
    if (!myStringValue) myStringValue = (char *)malloc(256);
    tm.tm_sec = second;
    tm.tm_min = minute;
    tm.tm_hour = hour;
    tm.tm_mday  = day + 1;
    tm.tm_mon = month;
    tm.tm_year = year - 1900;
    
    /*
     * this will calculate and fill in the day of week and day of year fields,
     * which we might need.
     */
    mktime(&tm);
    
    strftime( myStringValue, 256, format, &tm );
    
    return myStringValue;
}

- (NSString *)stringValue
{ 	
	// For historical reasons we return things in this format, although
	// I think [self format:"%c"] would be better, which would produce
	// something appropriate for the locale.
	
	return [NSString stringWithCString:[self format:"%H : %M : %S"]];
	// old way:
	// Don't free this!  that's why it's a const!
	// if (!myStringValue) myStringValue = (char *)malloc(256);

	// sprintf(myStringValue, "%ld : %ld : %ld", hour, minute, second);
	// return myStringValue;
}

- (void)dealloc
{
	free(myStringValue);
	{ [super dealloc]; return; };
}

- (int)dayOfWeek
{
	if (dow < 0) [self calcDayOfWeek];
	return dow;
}

- (int)year { return year; }
- (int)month { return month; }
- (int)day { return day; }
- (int)hour { return hour; }
- (int)minute { return minute; }
- (int)second { return second; }
- (int)microsecond { return microsecond; }
- (BOOL)isRelative { return isRelative; }


- addTime:aTime
{
	if (![aTime isKindOfClass:[MiscTime class]]) return nil;
	[self addMicroseconds:[aTime microsecond]];
	[self addSeconds:[aTime second]];
	[self addMinutes:[aTime minute]];
	[self addHours:[aTime hour]];
	[self addDays:[aTime day]];
	[self addMonths:[aTime month]];
	[self addYears:[aTime year]];
	return self;
}

- subtractTime:aTime
{
	if (![aTime isKindOfClass:[MiscTime class]]) return nil;
	[self subtractMicroseconds:[aTime microsecond]];
	[self subtractSeconds:[aTime second]];
	[self subtractMinutes:[aTime minute]];
	[self subtractHours:[aTime hour]];
	[self subtractDays:[aTime day]];
	[self subtractMonths:[aTime month]];
	[self subtractYears:[aTime year]];
	return self;
}

- addMicroseconds:(long)t
{
	long x = [self microsecond] + t;
	if (!t) return self;
	[self setMicrosecond:(x % MISC_TIME_MAX_MICROSECONDS)];
	[self addSeconds:(x / MISC_TIME_MAX_MICROSECONDS)];
	return self;
}

- addSeconds:(long)t
{
	long x = [self second] + t;
	if (!t) return self;
	[self setSecond:(x % MISC_TIME_MAX_SECONDS)];
	[self addMinutes:(x / MISC_TIME_MAX_SECONDS)];
	return self;
}

- addMinutes:(long)t
{
	long x = [self minute] + t;
	if (!t) return self;
	[self setMinute:(x % MISC_TIME_MAX_MINUTES)];
	[self addHours:(x / MISC_TIME_MAX_MINUTES)];
	return self;
}

- addHours:(long)t
{
	long x = [self hour] + t;
	if (!t) return self;
	[self setHour:(x % MISC_TIME_MAX_HOURS)];
	[self addDays:(x / MISC_TIME_MAX_HOURS)];
	return self;
}

- addDays:(long)t
{
	long x = [self day] + t; int k;
	if (!t) return self;
	
	k = Misc_daysIn(month, year);
	while (x >= k) { // This is slow, but it works.
		[self addMonths:1];
		x -= k; k = Misc_daysIn(month, year);
	}
	[self setDay:(x % MISC_TIME_MAX_DAYS)];
	return self;
}

- addWeeks:(long)t
{
	return [self addDays:(t * 7)];
}

- addMonths:(long)t
{
	long x = [self month] + t;
	if (!t) return self;
	[self setMonth:(x % MISC_TIME_MAX_MONTHS)];
	[self addYears:(x / MISC_TIME_MAX_MONTHS)];
	return self;
}

- addYears:(long)t
{
	if (t) [self setYear:([self year] + t)];
	return self;
}

- (BOOL)isAfter:aTime
{ // returns YES if we are "older" than time passed in
	int si = [self _nintValue];
	int oi = [aTime _nintValue];
	if (si > oi) return YES;
	if ((si == oi) && ([self microsecond] > [aTime microsecond])) return YES;
	return NO;
}

- (BOOL)isEqual:aTime
{ // returns YES if we are "same" as time passed in
	if (([self _nintValue] == [aTime _nintValue]) &&
			([self microsecond] == [aTime microsecond])) return YES;
	return NO;
}

- subtractMicroseconds:(long)t
{
	long us = t % MISC_TIME_MAX_MICROSECONDS;
	long s = t / MISC_TIME_MAX_MICROSECONDS;
	long sus = [self microsecond];
	if (us > sus) { s++; sus += MISC_TIME_MAX_MICROSECONDS; }
	[self subtractSeconds:s];
	[self setMicrosecond:(sus - us)];
	return self;
}

- subtractSeconds:(long)t
{
	long s = t % MISC_TIME_MAX_SECONDS;
	long m = t / MISC_TIME_MAX_SECONDS;
	long ss = [self second];
	if (s > ss) { m++; ss += MISC_TIME_MAX_SECONDS; }
	[self subtractMinutes:m];
	[self setSecond:(ss - s)];
	return self;
}

- subtractMinutes:(long)t
{
	long m = t % MISC_TIME_MAX_MINUTES;
	long h = t / MISC_TIME_MAX_MINUTES;
	long sm = [self minute];
	if (m > sm) { h++; sm += MISC_TIME_MAX_MINUTES; }
	[self subtractHours:h];
	[self setMinute:(sm - m)];
	return self;
}

- subtractHours:(long)t
{
	long h = t % MISC_TIME_MAX_HOURS;
	long d = t / MISC_TIME_MAX_HOURS;
	long sh = [self hour];
	if (h > sh) { d++; sh += MISC_TIME_MAX_HOURS; }
	[self subtractDays:d];
	[self setHour:(sh - h)];
	return self;
}

- subtractDays:(long)t
{
	long sd = [self day];
	while (t > sd) { // this is slow, but it works.
		[self subtractMonths:1];
		sd += Misc_daysIn(month, year);
	}
	[self setDay:(sd - t)];
	return self;
}

- subtractWeeks:(long)t
{
	return [self subtractDays:(t * 7)];
}

- subtractMonths:(long)t
{
	long m = t % MISC_TIME_MAX_MONTHS;
	long y = t / MISC_TIME_MAX_MONTHS;
	long sm = [self month];
	if (m > sm) { y++; sm += MISC_TIME_MAX_MONTHS; }
	[self subtractYears:y];
	[self setMonth:(sm - m)];
	return self;
}

- subtractYears:(long)t
{
	if (t) [self setYear:([self year] - t)];
	return self;
}

- copy
{
	id myCopy = [[[self class] alloc] init];
	[myCopy setYear:year];
	[myCopy setMonth:month];
	[myCopy setDay:day];
	[myCopy setDayOfWeek:dow];
	[myCopy setHour:hour];
	[myCopy setMinute:minute];
	[myCopy setSecond:second];
	[myCopy setMicrosecond:microsecond];
	[myCopy setRelative:isRelative];
	return myCopy;
}

- copyTimeFrom:aTime
{
	[self setYear:[aTime year]];
	[self setMonth:[aTime month]];
	[self setDay:[aTime day]];
	[self setDayOfWeek:[aTime dayOfWeek]];
	[self setHour:[aTime hour]];
	[self setMinute:[aTime minute]];
	[self setSecond:[aTime second]];
	[self setMicrosecond:[aTime microsecond]];
	[self setRelative:[aTime isRelative]];
	return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    int version;
	[super initWithCoder:aDecoder];
    version = [aDecoder versionForClassName:[MiscTime name]];
	[aDecoder decodeValuesOfObjCTypes:"cllllllll", &isRelative, &microsecond, &second,
			&minute, &hour, &dow, &day, &month, &year];
	if (!myStringValue) myStringValue = (char *)malloc(256);
	if (version < 1) year += 1900;
	return self;
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
	[super encodeWithCoder:aCoder];
	[aCoder encodeValuesOfObjCTypes:"cllllllll", &isRelative, &microsecond, &second,
			&minute, &hour, &dow, &day, &month, &year];
}

// NXTransport protocol implementation:
- (void)encodeWithCoder:(NSPortCoder *)aPortCoder
{
	[aPortCoder encodeValueOfObjCType:"c" at:&isRelative];
	[aPortCoder encodeValueOfObjCType:"i" at:&microsecond];
	[aPortCoder encodeValueOfObjCType:"i" at:&second];
	[aPortCoder encodeValueOfObjCType:"i" at:&minute];
	[aPortCoder encodeValueOfObjCType:"i" at:&hour];
	[aPortCoder encodeValueOfObjCType:"i" at:&dow];
	[aPortCoder encodeValueOfObjCType:"i" at:&day];
	[aPortCoder encodeValueOfObjCType:"i" at:&month];
	[aPortCoder encodeValueOfObjCType:"i" at:&year];
}

- (id)initWithCoder:(NSPortCoder *)aPortCoder
{
	[aPortCoder decodeValueOfObjCType:"c" at:&isRelative];
	[aPortCoder decodeValueOfObjCType:"i" at:&microsecond];
	[aPortCoder decodeValueOfObjCType:"i" at:&second];
	[aPortCoder decodeValueOfObjCType:"i" at:&minute];
	[aPortCoder decodeValueOfObjCType:"i" at:&hour];
	[aPortCoder decodeValueOfObjCType:"i" at:&dow];
	[aPortCoder decodeValueOfObjCType:"i" at:&day];
	[aPortCoder decodeValueOfObjCType:"i" at:&month];
	[aPortCoder decodeValueOfObjCType:"i" at:&year];
	return self;
}

#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
- replacementObjectForPortCoder:(NSPortCoder *)aPortCoder
{
	if ([aPortCoder isBycopy]) {
		*flagp = NO; // object will copy.
		return self; //encode object (copy it)
	}
	*flagp = NO; // object will copy.
	// super will encode the proxy otherwise
#warning DO Conversion: if freeAfterEncoding was true use autorelease instead.
	return [super replacementObjectForPortCoder:[aPortCoder connection]];
}

@end
