//
// MiscTree.Search.m -- Category of MiscTree that allows a MiscTree to be
// 		searched
//		Written by Stefan Bohringer Copyright (c) 1996 by Stefan Bohringer.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This program is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/MiscTree.Search.h>

@implementation MiscTree(Search)

//static char pathSeparator;	// i deliberately need the seperator to be configurable!!!
//	(e.g take the one from NXBrowser's)

// private fnc that collects MiscTree objects whose value matches sstr
//	This algorithm is very slow. a polished one should run no-recursively
//	and accept a starting-point-path
void doSearchTree(MiscTree *sTree,id sstr,id eList,BOOL useLabel, MiscTreeCmpMode_t cmpMode)
{	int	i,cnt;
	List *bl=[sTree branches];
	for(i=0,cnt=[bl count];i< cnt; i++)
	{	MiscTree *o=[bl objectAt:i];
		switch(cmpMode)
		{	case MiscTreeCmp_regex:
				if([useLabel? o->label:o->value spotOfRegex:[[sstr stringValue] cString]] >=0)
					[eList addObject:o];
			break;
			case MiscTreeCmp_strcmp:
				if(![useLabel? o->label:o->value cmp:[[sstr stringValue] cString]])
					[eList addObject:o];
			break;
			case MiscTreeCmp_casestrcmp:
				if(![useLabel? o->label:o->value casecmp:[[sstr stringValue] cString]])
					[eList addObject:o];
			break;
			case MiscTreeCmp_idcmp:
				if(o->value == sstr) [eList addObject:o];
			break;
		} if([o branches]) doSearchTree(o,sstr,eList,useLabel,cmpMode);
	}
}

// these functions would make fine in MiscString (in a more generalized form)
// e.g. -removeEntryAt:n usingDelimiter:del;

void removeLastEntry(MiscString *str)
{	int i=[str rspotOfChars:"/"];
	if(i>= 0) [str fixStringLengthAt:i];
}
void delLeadingSeparators(MiscString *str)
{	const char *p=[[str stringValue] cString],*po=p;
	while(*p== '/') p++;
	if(p != po) [str setStringValue:[NSString stringWithCString:p]];

}

// private fnc that collects pathes in MiscString which point to node
//	sTree should be root
//	regrettably this algorithm is very slow (traversing the whole tree). perhaps
//	MiscTree should optionally include a link
//	from child to parent (special 'one parent mode') which would make the pathbuilding
//	process straightforward and fast.
void doPathTree(MiscTree *sTree,MiscTree *node,MiscString *pathstr,List *eList)
{	int	i,cnt;
	List *bl=[sTree branches];
	MiscString *added;
	[pathstr cat:"/"],[pathstr cat:[sTree label]];
	if([bl indexOf:node] != NSNotFound)
	{	[eList addObject:added=[pathstr copy]];
		[added cat:"/"],[added cat:[node label]];
		if([node branches]) [added cat:"/"];
		delLeadingSeparators(added);
	}
	for(i=0,cnt=[bl count];i< cnt; i++)
	{	id o=[bl objectAt:i];
		if([o branches]) doPathTree(o,node,pathstr,eList);
	} removeLastEntry(pathstr);
}

// gives a list of nodes matching the regex sstr.
// cave: the list does not contain copys so dont free the cont. of the returned list

-(MiscList*) listOfOccurence:sstr useLabel:(BOOL) useIt cmp:(MiscTreeCmpMode_t) cmpMode
{	MiscList *eList=[[MiscList alloc] init];
	doSearchTree(self,sstr,eList,useIt,cmpMode);
	if(![eList count]) eList=[eList release];	//nil if empty
	return eList;
}
-(MiscList*) listOfOccurence: sstr useLabel:(BOOL) useIt regex:(BOOL) regex
{	return [self listOfOccurence:sstr useLabel:useIt cmp:regex?MiscTreeCmp_regex:MiscTreeCmp_strcmp];
}
-(MiscList*) listOfOccurence: sstr
{	return [self listOfOccurence:sstr useLabel:NO cmp:MiscTreeCmp_strcmp];
}

// private method (its is actually the reverse of pathToChild:)
// cave: the list _does_ contain copys so you have to free before freein the list
-(MiscList*) pathToNode:(MiscTree*)theNode	// Get list of pathes pointing to theNode
{	MiscList *eList=[[MiscList alloc] init];
	MiscString	*pathstr=[[MiscString alloc] init];
	doPathTree(self,theNode,pathstr,eList);
	if(![eList count]) eList=[eList release];	//nil if empty
	return eList;
}

// Should actually be included in the Browser category:
//	Selects theNode's representation(label) in theBrowser
//	call this method in the rootObjc of the tree
- selectNode:(MiscTree*)theNode inBrowser:theBrowser
{	List *serg=[self pathToNode:theNode];
	if(serg)	// is theNode in theTree (self) ?
	{	[theBrowser loadColumnZero];	//seems to be necessary for setPath: to work
		[theBrowser setPath:[[serg objectAt:0] stringValue]];
		[theBrowser sendAction];		// readjust the GUI
		[[serg freeObjects] release];		// Free all the found pathes
		return self;
	}	return nil;		// nothing found->nothing selected
}

// convenience methods for small browsers
// send them to the root

- selectValueId:theId inBrowser:theBrowser
{	List	*searchList;
	id		 erg=nil;
	searchList=[self listOfOccurence:theId useLabel:NO cmp:MiscTreeCmp_idcmp];
	if(searchList)
	{	erg=[self selectNode:[searchList objectAt:0] inBrowser:theBrowser];
		searchList=[searchList release];
	} else [theBrowser loadColumnZero];	// deselect
	return erg;
}
- selectString:(const char*)theString inBrowser:theBrowser
{	id	erg=nil;
	List *searchList;
	MiscString *ts=[[MiscString alloc] initString:theString];
	if(![ts emptyString])
	{	if(searchList=[self listOfOccurence:ts useLabel:YES regex:NO])
		{	erg=[self selectNode:[searchList objectAt:0] inBrowser:theBrowser];
			searchList=[searchList release];
		} else [theBrowser loadColumnZero];	// deselect
	}	ts=[ts release];
	return erg;
}

@end

