
//  Copyright (c) 1995 by author.  All rights reserved.
// Usage is governed by the MiscKit license.

#import <sys/types.h>
#import <pwd.h>
#import <grp.h>
/*#import <AppKit/nextstd.h>*/
#import <objc/List.h>
#import <misckit/MiscListExtensions.h>
#import <misckit/MiscUserGroup.h>
#import <misckit/MiscUser.h>

static BOOL  filterSystemUsers = YES;


@interface MiscUser (Private)

- (BOOL)_isInGroup:(struct group *)gr;
	
@end


@implementation MiscUser 

+ (BOOL)filterSystemUsers
{
	// If YES is returned, the class method allUsers will filter out all
	// users that are part of the system (like uucp, nobody, news, etc).
	// NOT IMPLEMENTED YET
	return filterSystemUsers;
}


+ setFilterSystemUsers: (BOOL)filter
{
	// NOT IMPLEMENTED YET
	filterSystemUsers = filter;
	return self;
}


+ (List *)allUsers
{
	// Returns a List of MiscUsers, representing all the users on the system.
	// It is your responsibility to free the list and it's contents. The
	// Unix functions below will traverse all the domains, so we have to
	// filter out duplicates we find. The list returned is a combination of
	// all the domains (from local to the root).
	List  *userList = nil;
	MiscUser  *aUser;
	struct passwd  *pw;
	
	setpwent();
	do
	{
		pw = getpwent();
		if (pw == NULL)
			break;

		if (userList == nil)
			userList = [ [List alloc] init];

		// Check to see if the username is already in the list,
		// since there can be duplicates from multiple domains.
		if ([userList matchString: pw->pw_name 
						usingSel: @selector(username)] == NO)
		{
			aUser = [ [self alloc] initWithUsername: pw->pw_name];
			[userList addObject: aUser];
		 }
		
	 }
	while (pw != NULL);
	endpwent();
	
	return userList;
}
	
	
+ (MiscUser *)whoAmI
{
	// Returns the current user. It is your responsibility to free the
	// instance returned.
	// Thanks to Stephan Wacker for the method name.
	return [ [self alloc] initWithUserId: getuid()];
}


- initWithUserId: (int)userId
{
	// Inits if the userId is a valid uid, else returns nil.
	struct passwd  *pw;
	
	pw = getpwuid (userId);
	if (pw == NULL)
	{
		// Not a valid userId.
		[super release];
		return nil;
	 }
	
	return [self initWithUsername: pw->pw_name];
}


- initWithUsername: (const char *)uname
{
	// This method has been promoted to the designated initializer since there
	// are duplicate uids (for a couple of the system users) but no duplicate
	// login ids (usernames).
	struct passwd  *pw;
	
	if (uname == NULL)
		return nil;
	
	[super init];	
	pw = getpwnam (uname);
	if (pw == NULL)
	{
		[super release];
		return nil;
	 }
	uid = pw->pw_uid;
	gid = pw->pw_gid;
	
	// Probably a waste of space to save all this... I will
	// fix it to return autoreleased NSStrings for the OpenStep release.
	username = malloc((strlen (pw->pw_name)+1) * sizeof(char));
	strcpy (username, pw->pw_name);
	realname = malloc((strlen (pw->pw_gecos)+1) * sizeof(char));
	strcpy (realname, pw->pw_gecos);
	shell = malloc((strlen (pw->pw_shell)+1) * sizeof(char));
	strcpy (shell, pw->pw_shell);
	homeDirectory = malloc((strlen(pw->pw_dir)+1) * sizeof(char));
	strcpy (homeDirectory, pw->pw_dir);

	return self;
}	
	

- (void)dealloc
{
	if (username != NULL)
		free (username);
	if (realname != NULL)
		free (realname);
	if (shell != NULL)
		free (shell);
	if (homeDirectory != NULL)
		free (homeDirectory);
	{ [super dealloc]; return; };
}


- (const char *)username
{
	// Don't free the string returned.
	return username;
}


- (const char *)realname
{
	// Don't free the string returned.
	return realname;
}


- (int)userId
{
	return uid;
}


- (MiscUserGroup *)primaryGroup
{
	// It is your responsibility to free the returned instance.
	return [ [MiscUserGroup alloc] initWithGroupId: gid];
}


- (List *)groups
{
	// Returns a list of MiscUserGroups representing the groups the user
	// belongs to. You are responsible for freeing the list and it's contents. 
	List  *groupList = nil;
	MiscUserGroup  *aGroup;
	struct group  *gr;
	
	setgrent();
	do
	{
		gr = getgrent();
		if (gr != NULL && [self _isInGroup: gr])
		{
			if (groupList == nil)
				groupList = [ [List alloc] init];
				
			// Filter out duplicate names, since we could possibly cross
			// multiple domains and be in duplicate groups.
			if ([groupList matchString: gr->gr_name 
							usingSel: @selector(groupName)] == NO)
			{
				aGroup = [ [MiscUserGroup alloc] initWithGroupId: gr->gr_gid];
				[groupList addObject: aGroup];
			 }
		 }
	 }
	while (gr != NULL);
	
	endgrent();
	return  groupList;
}


- (const char *)shell
{
	// Don't free the string returned.
	return shell;
}


- (const char *)homeDirectory
{
	// Don't free the string returned.
	return homeDirectory;
}

@end


@implementation MiscUser (Private)

- (BOOL)_isInGroup:(struct group *)gr	
{
	// Search for us in a given group. Returns YES if we are a member,
	// NO otherwise.
	int x;
	for ( x = 0 ; gr->gr_mem[x] ; ++x )
		if (strcmp(gr->gr_mem[x], username) == 0) 
			return YES;
	return NO;
}

@end


/**************************************************************************
  CHANGES
  January 18, 1995 Version 0.2
  	1. Changed the designated initializer to initWithUsername: since there 
	were potential problems with duplicate uids (system users) and the old
	designated initializer initWithUid:.
  January 23, 1995 Version 0.3
    1. Changed some of the methods to remove duplicates while scanning 
	through all the users (which will cross multiple domains if they exist).
	
 ***************************************************************************/
 
