//
//	MiscWindowButtons.m -- extension to Window class
//		Written by John Randolph
//		Copyright (c) 1994 by John Randolph.
//				Version 1.0.  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/misckit.h>

#import "MiscWBWraps.h"

@interface NSWindow (MiscWindowButtonsPrivate)

- (int) globalWindowNum;
- toggleClose:sender;
- toggleMiniaturize:sender;
- updateBorder;

@end

@interface NSView (MiscWindowButtons)
- _setMask:(unsigned int) mask;
@end

@implementation NSWindow (MiscWindowButtonsPrivate)

- (int) globalWindowNum
	{
	unsigned int
		global;
	NSConvertWindowNumberToGlobal([self windowNumber],&global);
	return global;
	}
	
- toggleClose:sender
	{
	wFlags.buttonMask ^= NSClosableWindowMask;
	return [self updateBorder];
	}

- toggleMiniaturize:sender
	{
	wFlags.buttonMask ^= NSMiniaturizableWindowMask;
	return [self updateBorder];
	}
	
- updateBorder
	{
	[_borderView _setMask:wFlags.buttonMask];
	[_borderView tile];
	[_borderView display];
	[self display];
	return self;
	}
	
@end

@implementation NSWindow (MiscWindowButtons)
- showCloseBox:sender
	{
	if (!(wFlags.buttonMask & NSClosableWindowMask))
		{
		PSWaddCloseButtonRect([self windowNumber]);
		return [self toggleClose:sender];
		}
	return self;
	}
	
- showMiniaturizeButton:sender
	{
	if (!(wFlags.buttonMask & NSMiniaturizableWindowMask))
		{
		PSWaddMiniaturizeButtonRect([self windowNumber]);
		return [self toggleMiniaturize:sender];
		}
	return self;
	}
	
- hideCloseBox:sender
	{
	if (wFlags.buttonMask & NSClosableWindowMask)
		{
		PSWremoveCloseButtonRect([self windowNumber]);
		return [self toggleClose:sender];
		}
	return self;
	}
	
- hideMiniaturizeButton:sender
	{
	if (wFlags.buttonMask & NSMiniaturizableWindowMask)
		{
		PSWremoveMiniaturizeButtonRect([self windowNumber]);
		return [self toggleMiniaturize:sender];
		}
	return self;
	}
	

	
@end

