/**************************************************************************
 * CLASS:		_MiscDirectoryStream
 *
 *	Please see the header file for more information on this small class.
 **************************************************************************/

// Copyright 1995 Todd Thomas.  All Rights Reserved.
// Use permitted under terms of the MiscKit license.

#import <stdlib.h>
#import <sys/stat.h>
#import <sys/param.h>	
#import <stdio.h>	
/*#import <AppKit/nextstd.h>*/	
#import <misckit/_MiscDirectoryStream.h>


@implementation _MiscDirectoryStream

- init
{
	return [self initWithDirectory: NULL];
}



// The destinated initializer.

- initWithDirectory: (const char *)directory
{
	[super init];
	
	dirStream = NULL;
	dp = NULL;
	currentDirName = NULL;
	
	[self openDir: directory];
	return self;
}



// Calling closedir will free the pointer to struct direct and dirStream,
// so the only thing left is the currentDirName if set to anything.

- (void)dealloc
{
	if (dirStream != NULL)
		[self closeDir];
		
	if (currentDirName)
		free (currentDirName);
	
	{ [super dealloc]; return; };
}



// Opens a directory stream. If a stream is already open, then it is 
// closed and the new one is opened. Directory cannot be NULL.
	
- openDir: (const char *)directory
{  
	if (directory == NULL)
		return nil;
		
	if (dirStream)
		[self closeDir];
	 
	dirStream = opendir (directory);
	
	// If NULL then something went wrong, so return nil
	
	if (dirStream == NULL)
		return nil;
	
	if (currentDirName)
	{
		free (currentDirName);
		currentDirName = NULL;
	 }
	
	currentDirName = malloc((strlen(directory)+1) * sizeof(char));
	strcpy (currentDirName, directory);
	
	return self;
}

	 

// Returns the directory name that was passed to either initWithDirectory:  or
// openDir (in other words, the directory that the stream represents). If no
// directory is open, NULL is returned.

- (NSString *)directory
{
	return [NSString stringWithCString:(const char *)currentDirName];
}



// Just like the unix version except that nil is returned when the end of
// the stream is reached or no dir stream has been opened.

- readDir
{
	if (dirStream == NULL)
		return nil;
		
	dp = readdir (dirStream);
	if (dp)
		return self;
	else
		return nil;
}



// Returns nil if there is no stream to rewind.
		
- rewindDir
{
	if (dirStream == NULL)
		return nil;
		
	rewinddir (dirStream);
	return self;
}


	
- closeDir
{
	if (dirStream)
	{	
		closedir (dirStream);
		dirStream = NULL;
	 }
	 
	return self;
}



// Returns the filename that is currently being pointed to in the stream.

- (NSString *)filename
{
	if (dp)
		return [NSString stringWithCString:dp->d_name];
	else
		return nil;
}



// Returns the length of the filename being pointed to in the stream.

- (int)filenameLength
{
	if (dp)
		return dp->d_namlen;
	else
		return 0;
}



// If the current filename in the directory stream is a directory
// YES is returned.

- (BOOL)isADirectory
{
  struct stat  s;
  char  tmp[MAXPATHLEN+1];

  	if (dp == NULL)
		return NO;

	sprintf (tmp, "%s/%s", [[self directory] cString], dp->d_name);	
	
	if (lstat (tmp, &s) == -1)
		return NO;
		
	if ((s.st_mode & S_IFMT) == S_IFDIR)
//	if (S_ISDIR(s.st_mode)) 
		return YES;
		
	return NO;
}



// If the current filename in the directory stream is a symbolic link,
// then YES is returned.

- (BOOL)isASymbolicLink
{
  struct stat s;
  char  tmp[MAXPATHLEN+1];
  
  	if (dp == NULL)
		return NO;

	sprintf (tmp, "%s/%s", [[self directory] cString], dp->d_name);		
	if (lstat (tmp, &s) == -1)
		return NO;
		
	if ((s.st_mode & S_IFMT) == S_IFLNK)
		return YES;
		
	return NO;
}

@end
