/*
	* 	Net Image Filter -- a netpbm based image filtering service
	* 					by Marco Scheurer, Sen:te
	*
	* 	Version 1.0
	* 
	* 	Copyright 1995, Sente Sarl. All rights reserved.
	*
	*		This notice may not be removed from this source code.
	*
	*	This object is included in the MiscKit by permission from the author
	*	and its use is governed by the MiscKit license, found in the file
	*	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
	*	for a list of all applicable permissions and restrictions.
	*
*/

#import "NetImageConverter.h"
#import <AppKit/AppKit.h>

#import <mach/mach.h>

#define DEFAULTS_DB	"NetImageFilter"

#error DefaultsConversion: the NXDefaultsVector type is obsolete. Construct a dictionary of default registrations and use the NSUserDefaults 'registerDefaults:' method
#error DefaultsConversion: the NXDefaultsVector type is obsolete. Construct a dictionary of default registrations and use the NSUserDefaults 'registerDefaults:' method
static NXDefaultsVector FilterDefaults = {
	{"jpeg", "/usr/local/bin"},
	{"netpbm", "/usr/local/netpbm"},
	{"Debug", "NO"},
	{NULL}
};

static const char *BinPaths[] = {
	"/LocalDeveloper/bin",
	"/LocalDeveloper/netpbm",
	"/LocalDeveloper/netpbm/bin",
	"/LocalApps",
	"/LocalApps/netpbm",
	"/LocalApps/netpbm/bin",
	"/usr/local/bin",
	"/usr/local/netpbm",
	"/usr/local/netpbm/bin",
	"~/Unix/bin",
	"~/Unix/netpbm",
	"~/Unix/netpbm/bin",
	NULL	
};

@implementation NetImageConverter
+ sharedInstance
{
	static id sharedInstance = nil;

	if (!sharedInstance) {
		sharedInstance = [self alloc];
		[sharedInstance init];
	}

	return sharedInstance;
}	

- init
{
	const char **paths;
    /* register defaults vector
     */
#error DefaultsConversion: NXRegisterDefaults() is obsolete. Construct a dictionary of default registrations and use the NSUserDefaults 'registerDefaults:' method
#error DefaultsConversion: NXRegisterDefaults() is obsolete. Construct a dictionary of default registrations and use the NSUserDefaults 'registerDefaults:' method
    NXRegisterDefaults(DEFAULTS_DB, FilterDefaults);

	fprintf(stderr, "NetImageFilter v1.1\n");

    /* Create binaryHash -- keys and values are strings [that is,
       we manage all the memory for the keys/values stored in the
       table].
     */
	binaryHash = [[HashTable allocWithZone:[self zone]]
					  initKeyDesc: "*" valueDesc: "*"];


	binPaths = [[List allocWithZone:[self zone]] init];
	paths = BinPaths;

	while (*paths) {
		id bundle;
		id vBundle;

		bundle = [NSBundle allocWithZone:[self zone]];
		vBundle = [bundle initWithPath:*paths];

		if(vBundle == bundle)
			[binPaths addObject:bundle];

		paths++;
	}

	dataBuf = (char *) malloc(vm_page_size);

	return self;
}

- (const char *) filterPathFromData: (const char *) userData
{
	const char *filterPath;
	char *separator;
	char *userDataMunge;
	char *converter;
	char *family;

	
/* BTW:  MAXPATHLEN is safe to use without the traditional "+1" --
   a friend tested out the library routines and found that they all
   assume that MAXPATHLEN chars INCLUDES the terminating NULL!

   b.bum
 */


	if (!userData) return NULL; // validate argument.

    /* Check to see if the path to the filter program described by
       userData is already hashed in binaryHash.  If it is, just
       return that path.  Note if the empty string (a string of
       length 0) is returned by valueForKey, it indicates that a
       search has already been performed, but the binary in question
       was not found.
     */
	filterPath = (const char *) [binaryHash valueForKey: userData];

	if (filterPath)
		return filterPath[0] ? filterPath : NULL;

    /* Make a mungeable copy of userData
     */
	userDataMunge = NXCopyStringBufferFromZone(userData, (NXZone *)(NSZone *)[self zone]);

    /* Search for the binary.
     */

    /* Determine whether or not a family name was specified.  If it
       was, then split the family name from the converter.
     */
	separator = index (userDataMunge, '/');
	if (separator && *separator) {
        /* kill separator
         */
		separator[0] = '\0';
		converter	 = separator + 1;
		family		 = userDataMunge;
	} else {
		converter	 = userDataMunge;
		family		 = NULL;
	}

    /* 1. Look for path in defaults database using "converter" as the
       key
     */
	filterPath = [self validateConverterPath:
					   [[[NSUserDefaults standardUserDefaults] objectForKey:[NSString stringWithCString:converter]] cString]];

    /* 2. [if not already found] If the family has been specified,
       look for the binary rooted at the value of the default value
       associated with the family name.
     */
	if (!filterPath && family)
		filterPath = [self validateConverter:converter
						   inDirectory:[[[NSUserDefaults standardUserDefaults] objectForKey:[NSString stringWithCString:family]] cString]];

    /* 3. [if not already found] run popen() and use return value
     */
	if (!filterPath) {
		char cmdBuf[1024];
		FILE *process;

		sprintf(cmdBuf, "which %s\n", converter);

		process = popen(cmdBuf, "r");

		if (process) {
			int fd = fileno(process);
			int count;
			char buf[MAXPATHLEN];

			count = read(fd, buf, MAXPATHLEN);

			if (count && (buf[0] == '/')) {
				char *index = index(buf, '\n');
				
				if(index)
					*index = '\0';

				filterPath = [self validateConverterPath: buf];
			}
			
			pclose(process);
		}
	}

    /* 4. check list of standard paths.
     */
	if (!filterPath) {
		id *ida, *max;

		ida = NX_ADDRESS(binPaths);
		max = ida + [binPaths count];

		for(;ida<max && !filterPath;ida++)
			filterPath = [self validateConverter:converter inBundle:*ida];
	}

	if (!filterPath) {
		id bundle = [NSBundle bundleForClass:[self class]];
		
		filterPath = [self validateConverter:converter inBundle:bundle];

		if (!filterPath) {
			char pathBuf[MAXPATHLEN];

			sprintf(pathBuf, "%s/netpbm", [[bundle directory] cString]);
			filterPath = [self validateConverter:converter
							   inDirectory:pathBuf];

			if(!filterPath) {
				strcat(pathBuf, "bin");
				filterPath = [self validateConverter:converter
								   inDirectory:pathBuf];
			}
		}
	}

	if(filterPath)
		[binaryHash insertKey:userData value:(void *) filterPath];

	return filterPath;
}

- (const char *) validateConverter:(const char *) aConverter
                          inBundle:aBundle
{
	char pathBuf[MAXPATHLEN];

	if (!aConverter || !aBundle)
		return NULL;

#error StringConversion: This call to -[NXBundle getPath:forResource:ofType:] has been converted to the similar NSBundle method.  The conversion has been made assuming that the variable called pathBuf will be changed into an (NSString *).  You must change the type of the variable called pathBuf by hand.
	if(pathBuf = [aBundle pathForResource:[NSString stringWithCString:aConverter] ofType:@""])
		return [self validateConverterPath:pathBuf];

	return NULL;
}

- (const char *) validateConverter:(const char *) aConverter
                       inDirectory:(const char *) aDirPath
{
	char pathBuf[MAXPATHLEN];

	if(!aConverter || !aDirPath)
		return NULL;

    /* Technically, this could fail if aConverter+aDirPath is longer
       than MAXPATHLEN.  That isn't very likely.  I'm choosing to
       ignore that possibility in the hopes that many other things
       will break first.
     */
	sprintf(pathBuf, "%s/%s", aDirPath, aConverter);

	return [self validateConverterPath: pathBuf];
}

	
- (const char *) validateConverterPath:(const char *) aPotentialPath
{
	struct stat statBuf;

	fprintf(stderr, "Validating {%s}\n", aPotentialPath);
	
	if (!aPotentialPath)
		return NULL; // no path specified

	if(stat(aPotentialPath, &statBuf) == -1)
		return NULL; // could not stat

	if(statBuf.st_mode & S_IFDIR)
		return NULL; // is a directory

    /* check file read and exec attributes
     */
	if( (statBuf.st_mode & S_IREAD) && (statBuf.st_mode & S_IEXEC) )
        /* If the file is both readable and executable, then make
           a copy of the path and return the newly created string.
         */
		return NXCopyStringBufferFromZone(aPotentialPath, (NXZone *)(NSZone *)[self zone]);

	return NULL; // by default, fail.
}

- (BOOL) isDebugOn
{
	const char *defaultValue = [[[NSUserDefaults standardUserDefaults] objectForKey:@"Debug"] cString];
	return (defaultValue && !strcasecmp(defaultValue, "YES"));
}


- anyToTIFF:pasteboard
   userData:(const char *)userData
	  error:(char **)errorMessage
{
#error StringCoversion: return type of types is now an NSArray of NSStrings (used to be NXAtom *).  Change your variable declaration.
#error StringCoversion: return type of types is now an NSArray of NSStrings (used to be NXAtom *).  Change your variable declaration.
    NXAtom  foundType = [pasteboard types][0];

	fprintf(stderr,
			"NetImageFilter v1.1: Received Request {%s}\n",
			userData);	

	if (foundType) {
		char    *data = NULL;
		int     length = 0;
        
#error StreamConversion: 'dataForType:' (used to be 'readType:data:length:') returns an NSData instance
		&data = [pasteboard dataForType:[NSString stringWithCString:foundType]];

		fprintf(stderr,
				"NetImageFilter v1.1: Read Pasteboard {%d bytes}\n",
				length);

		if (data && length && userData && *userData) {
			char commandFormat[] = "%s %s | %s\n";
			char command [(3 * MAXPATHLEN) + strlen (commandFormat)];
			char sourcePath [MAXPATHLEN];

			const char *toPNMConverter;
			const char *toTIFFConverter;

			FILE *process;

			strncpy (sourcePath, data, length);
			sourcePath[length] = 0;

			toPNMConverter = [self filterPathFromData:userData];
			toTIFFConverter = [self filterPathFromData:"netpbm/pnmtotiff"];

			fprintf(stderr, "%s -> %s\n", toPNMConverter, toTIFFConverter);

			if(!toPNMConverter || !toTIFFConverter)
				return self;

			sprintf (command, commandFormat,
					 toPNMConverter, sourcePath, toTIFFConverter);

			process = popen(command, "r");

			if (process) {
				NXStream *tiffStream = NXOpenMemory(NULL, 0, NX_READWRITE);
				size_t bytes_read;

				while (bytes_read = fread(dataBuf, 1, vm_page_size, process)) {
					fprintf(stderr, "Writing {%d} bytes...\n", bytes_read);
					NXWrite(tiffStream, dataBuf, bytes_read);
				}

				fprintf(stderr, "Writing Tiff\n");

				[pasteboard declareTypes:[NSArray arrayWithObject:NSTIFFPboardType] owner:self];
				NXSeek(tiffStream, 0, NX_FROMSTART);
#error StreamConversion: 'setData:forType:' (used to be 'writeType:fromStream:') takes an NSData instance as its first argument (used to take NXStream).
				[pasteboard setData:tiffStream forType:NSTIFFPboardType];
				NXCloseMemory (tiffStream, NX_FREEBUFFER);
				
				pclose(process);
			}
		}
	}

	return self;
}
@end
