//
//	MiscMatrix.h -- a class to implement variable-sized matrices
//		Written by Mike Ferris Copyright (c) 1994 by Mike Ferris.
//		Modified from original MOKit "MOMatrix" class by Don Yacktman.
//				Version 1.0.  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

// MiscMatrix is a subclass of Matrix that allows independantly sizable
// rows and columns.  Each row can have a different height and each column
// can have a different width.

#import <AppKit/AppKit.h>

typedef struct _MiscColumnSize_ {
	float		x;
	float		width;
}  MiscColumnSize;
#define MISC_COLUMNSIZE_DESC		"{ff}"

typedef struct _MiscRowSize_ {
	float		y;
	float		height;
}  MiscRowSize;
#define MISC_ROWSIZE_DESC		"{ff}"


@interface MiscMatrix : NSMatrix
{
	Storage *columnSizes, *rowSizes;
	
}

+ (void)initialize;

- initWithFrame:(NSRect)frm mode:(int)aMode prototype:cellId numberOfRows:(int)rowsHigh numberOfColumns:(int)colsWide;
- initWithFrame:(NSRect)frm mode:(int)aMode cellClass:factoryId numberOfRows:(int)rowsHigh numberOfColumns:(int)colsWide;

- copyWithZone:(NSZone *)zone;

- (void)dealloc;

- setWidth:(float)newWidth ofCol:(int)col;
- setHeight:(float)newHeight ofRow:(int)row;

- (void)sizeToCells;
- (void)sizeToFit;
- sizeRowsToFitCells;
- (void)setFrameSize:(NSSize)_newSize;
  
- (void)renewRows:(int)newRows columns:(int)newCols;

- (void)addColumn;
- (void)addRow;
- (void)insertColumn:(int)col;
- (void)insertRow:(int)row;
- removeColAt:(int)col andFree:(BOOL)flag;
- removeRowAt:(int)row andFree:(BOOL)flag;

#warning RectConversion: drawRect:(NSRect)rects (used to be drawSelf:(const NXRect *)rects :(int)rectCount) no longer takes an array of rects
- (void)drawRect:(NSRect)rects;

- (NSRect)cellFrameAtRow:(int)row column:(int)col;
- getRow:(int *)row column:(int *)col forPoint:(NSPoint)aPoint;

- (void)setIntercellSpacing:(NSSize)aSize;

- (void)encodeWithCoder:(NSCoder *)aCoder;
- (id)initWithCoder:(NSCoder *)aDecoder;

@end

@interface Storage(MiscLastElementCategory)
	
- (void *)lastElement;

@end

