/*	TDNodeWell.m

	Copyright 1996 Uwe Hoffmann.

	This notice may not be removed from this source code.
	The use and distribution of this software is governed by the
	terms of the MiscKit license agreement.  Refer to the license
	document included with the MiscKit distribution for the terms.

	Version 2 (August 1996)
*/

#import "TDNodeWell.h"
#import <MiscAppKit/MiscDiagramTree.h>
#import <MiscAppKit/MiscTreeDiagram.h>
#import <MiscAppKit/MiscTreeDiagramView.h>
#import <MiscAppKit/MiscHitPath.h>
#import <MiscAppKit/MiscTDViewUtils.h>
#import <MiscAppKit/MiscNodeStyle.h>


@interface TDNodeWell(PrivateMethods)

- (BOOL)_internalWriteToPasteboard:(NSPasteboard *)pb;
- (void)_internalUpdateImage;
- (void)_internalResizeEvent:(NSEvent *)event control:(int)aInt;

@end

@implementation TDNodeWell

- initWithFrame:(NSRect)theFrame
{
	NSRect bds;

    	[super initWithFrame:theFrame];
        bds = [self bounds];
    	[self registerForDraggedTypes:[NSArray arrayWithObjects:MiscTreeDiagramStylePboardType, NSColorPboardType, nil]];
	dragOffset.x = dragOffset.y = 10.0;
	dragRect.origin.x = bds.origin.x + dragOffset.x;
	dragRect.origin.y = bds.origin.y + dragOffset.y;
	dragRect.size.width = bds.size.width - 2 * dragOffset.x;
	dragRect.size.height = bds.size.height - 2 * dragOffset.y;
	dragImage = [[NSImage alloc] initWithSize:dragRect.size];
	[dragImage setFlipped:YES];
	nodeSize.width = MIN(dragRect.size.width - 4,50);
	nodeSize.height = MIN(dragRect.size.height - 4,30);
	dragNode = [[MiscDiagramTree alloc] initWithSize:nodeSize shapeType:MiscRectangleShapeType];
	[dragNode layout];
	nodePos.x = (bds.size.width - [dragNode size].width) / 2;
	nodePos.y = (bds.size.height - [dragNode size].height) / 2;
	[dragNode setPos:nodePos];
	hitPath = [[MiscHitPath alloc] initWithWidth:4.0];
	numberControl = nil;
    	return self;
}

- (void)dealloc
{
	[dragImage release];
	[dragNode release];
	return [super dealloc];
}

- (BOOL)isFlipped
{
    return YES;
}

- (void)mouseDown:(NSEvent *)event 
{
    	NSPasteboard *dragPboard;
	BOOL error,onOutline;
	NSSize zero = {0.0,0.0};
	NSPoint p;
	int controlPointNumber;
	NSPoint imageLocation;
	
    	p = [self convertPoint:[event locationInWindow] fromView:nil];
	[hitPath moveTo:p];
	if([dragNode hit:hitPath controlPoint:&controlPointNumber outline:&onOutline]){
		if(controlPointNumber > -1)
			[self _internalResizeEvent:event control:controlPointNumber];
		else {
    			[NSApp preventWindowOrdering];
    			dragPboard = [NSPasteboard pasteboardWithName:NSDragPboard];
			error = [self _internalWriteToPasteboard:dragPboard];
			if(error)
				return;
			[self _internalUpdateImage];
			imageLocation.x = dragOffset.x;
			imageLocation.y = dragOffset.y + dragRect.size.height;
    			[self dragImage:dragImage 
				at:imageLocation 
				offset:zero event:event pasteboard:dragPboard source:self slideBack:YES];			
		}
	}
}

- (void)draggedImage:(NSImage *)image beganAt:(NSPoint)screenPoint
{
	
}

- (unsigned int)draggingSourceOperationMaskForLocal:(BOOL)flag
{
    return (NSDragOperationCopy | NSDragOperationGeneric);
}

- (void)draggedImage:(NSImage *)image endedAt:(NSPoint)screenPoint deposited:(BOOL)flag
{
    
}

- (unsigned int)draggingEntered:sender
{
	NSPasteboard *pboard;
    	unsigned int sourceMask;
	NSPoint p;
	BOOL onOutline;
	int controlPointNumber;
	
    	sourceMask = [sender draggingSourceOperationMask];
    	pboard = [sender draggingPasteboard];
	p = [self convertPoint:[sender draggingLocation] fromView:nil];
	[hitPath moveTo:p];
	if([dragNode hit:hitPath controlPoint:&controlPointNumber outline:&onOutline])
			return NSDragOperationGeneric;
	return NSDragOperationNone;
}

- (unsigned int)draggingUpdated:sender
{
	NSPasteboard *pboard;
    	unsigned int sourceMask;
	NSPoint p;
	BOOL onOutline;
	int controlPointNumber;
	
    	sourceMask = [sender draggingSourceOperationMask];
    	pboard = [sender draggingPasteboard];
	p = [self convertPoint:[sender draggingLocation] fromView:nil];
	[hitPath moveTo:p];
	if([dragNode hit:hitPath controlPoint:&controlPointNumber outline:&onOutline])
			return NSDragOperationGeneric;
	return NSDragOperationNone;
}

- (BOOL)performDragOperation:sender
{
	NSPasteboard *pboard;
    	unsigned int sourceMask;
	NSPoint p;
	BOOL success = NO;
	NSData *data;
	BOOL onOutline;
	id pasteObject;
	int controlPointNumber;
	
	sourceMask = [sender draggingSourceOperationMask];
    	pboard = [sender draggingPasteboard];
	p = [self convertPoint:[sender draggingLocation] fromView:nil];
	[hitPath moveTo:p];
	if([dragNode hit:hitPath controlPoint:&controlPointNumber outline:&onOutline]){
		if(miscTD_includesType([pboard types], MiscTreeDiagramStylePboardType)){
			data = [pboard dataForType:MiscTreeDiagramStylePboardType];
			if(data){
				pasteObject = [NSUnarchiver unarchiveObjectWithData:data];
				[dragNode setNodeStyle:[pasteObject objectForKey:@"NodeStyle"]];
				[[dragNode nodeStyle] setAutomaticResize:NO];
				[dragNode setShapeType:[[pasteObject objectForKey:@"ShapeType"] intValue]];
				[self display];
				success = YES;
			}
		} else if(miscTD_includesType([pboard types], NSColorPboardType )){
                	NSColor *color = [NSColor colorFromPasteboard:pboard];
                	if(onOutline)
                        	[[dragNode nodeStyle] setOutlineColor:color];
                	else
                        	[[dragNode nodeStyle] setFillColor:color];
                        [self display];
                	success = YES;
        	}
	}
    	return success;
}

- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
{
    return YES;
}

- (BOOL)shouldDelayWindowOrderingForEvent:(NSEvent *)theEvent 
{
    return YES;
}

- (void)setFrameSize:(NSSize)aSize
{
    	[super setFrameSize:aSize];
	
	dragRect.origin.x = [self bounds].origin.x + dragOffset.x;
	dragRect.origin.y = [self bounds].origin.y + dragOffset.y;
	dragRect.size.width = [self bounds].size.width - 2 * dragOffset.x;
	dragRect.size.height = [self bounds].size.height - 2 * dragOffset.y;
	
	nodeSize.width = MIN(dragRect.size.width - 4,50);
	nodeSize.height = MIN(dragRect.size.height - 4,30);
	
    	[dragImage setSize:dragRect.size];
	[dragNode setSize:nodeSize];
	
	nodePos.x = ([self bounds].size.width - [dragNode size].width) / 2;
	nodePos.y = ([self bounds].size.height - [dragNode size].height) / 2;
	
	[dragNode setPos:nodePos];
}

- (void)drawRect:(NSRect)r
{	
    	NSRect kr,b;
        
	PSsetgray(NSLightGray); 
	NSRectFill([self bounds]);
	NSDrawGrayBezel([self bounds] , [self bounds]);
	[dragNode drawResizing];
        b = [dragNode nodeBounds];
        PSsetgray(NSBlack);
        kr = NSMakeRect(b.origin.x - 2, b.origin.y - 2, 4, 4);
        NSRectFill(kr);
        kr = NSMakeRect(b.origin.x + b.size.width - 2, b.origin.y - 2, 4, 4);
        NSRectFill(kr);
        kr = NSMakeRect(b.origin.x + b.size.width - 2, b.origin.y + b.size.height - 2, 4, 4);
        NSRectFill(kr);
        kr = NSMakeRect(b.origin.x - 2, b.origin.y + b.size.height - 2, 4, 4);
        NSRectFill(kr);
}

@end

@implementation TDNodeWell(PrivateMethods)

- (BOOL)_internalWriteToPasteboard:(NSPasteboard *)pb
{
	NSData *data = nil;
	BOOL error = YES;
	int nrNodes;
	NSDictionary *dragNodes;
	NSString *keys[2];
	NSObject *values[2];

	if(!numberControl)
		nrNodes = 1;
	else
		nrNodes = [numberControl intValue];
	if(nrNodes == 1) 
		data = [NSArchiver archivedDataWithRootObject:dragNode];
	else if(nrNodes > 1){
		keys[0] = @"Template";
		keys[1] = @"NumberOfCopies";
		values[0] = [[dragNode copy] autorelease];
		values[1] = [NSNumber numberWithInt:nrNodes];
		dragNodes = [NSDictionary dictionaryWithObjects:(id *)values 
					forKeys:keys count:2]; 
		data = [NSArchiver archivedDataWithRootObject:dragNodes];
	}
	if(data){
		[pb declareTypes:[NSArray arrayWithObject:MiscTreeDiagramPboardType] owner:[self class]];
		[pb setData:data forType:MiscTreeDiagramPboardType];
		error = NO;
	}
	return error;
}

- (void)_internalUpdateImage
{
	[dragImage lockFocus];
		PStranslate(-dragOffset.x,-dragOffset.y);
		[[NSColor colorWithCalibratedRed:1 green:1 blue:1 alpha:0] set];
		NSRectFill(dragRect);
		[[NSColor colorWithCalibratedRed:0 green:0 blue:0 alpha:1] set];	  
		[dragNode drawResizing];
	[dragImage unlockFocus];
}

- (void)_internalResizeEvent:(NSEvent *)event control:(int)aInt
{
	BOOL oldWindowAcceptsMouseMoved;
	NSEvent *ev;
	BOOL finished;
	NSPoint pt,ptLast,delta;
	int controlPointNumber;
	NSPoint constrainMin, constrainMax;
	NSWindow *window = [self window];

	controlPointNumber = aInt;	
	ptLast = [dragNode controlPoint:aInt];
    	[self  lockFocus];
	oldWindowAcceptsMouseMoved = [window acceptsMouseMovedEvents];
	[window setAcceptsMouseMovedEvents:YES];
	ev = [window nextEventMatchingMask:NSLeftMouseUpMask|NSLeftMouseDraggedMask];
	finished = ([ev type] == NSLeftMouseUp);
	if(controlPointNumber == 0){
		constrainMin.x = dragRect.origin.x + 2;
		constrainMin.y = dragRect.origin.y + 2;
		constrainMax.x = [dragNode pos].x + [dragNode size].width - 2;
		constrainMax.y = [dragNode pos].y + [dragNode size].height - 2;	
	} else if(controlPointNumber == 1){
		constrainMin.x = [dragNode pos].x + 2;
		constrainMin.y = dragRect.origin.y + 2;
		constrainMax.x = dragRect.origin.x + dragRect.size.width - 2;
		constrainMax.y = [dragNode pos].y + [dragNode size].height - 2;	
	} else if(controlPointNumber == 2){
		constrainMin.x = [dragNode pos].x + 2;
		constrainMin.y = [dragNode pos].y + 2;
		constrainMax.x = dragRect.origin.x + dragRect.size.width - 2;
		constrainMax.y = dragRect.origin.y + dragRect.size.height - 2;	
	} else if(controlPointNumber == 3){
		constrainMin.x = dragRect.origin.x + 2;
		constrainMin.y = [dragNode pos].y + 2;
		constrainMax.x = [dragNode pos].x + [dragNode size].width - 2;
		constrainMax.y = dragRect.origin.y + dragRect.size.height - 2;	
	}
	while(!finished){
		pt = [self convertPoint:[ev locationInWindow] fromView:nil];
		if(pt.x < constrainMin.x)
			pt.x = constrainMin.x;
		if(pt.y < constrainMin.y)
			pt.y = constrainMin.y;
		if(pt.x > constrainMax.x)
			pt.x = constrainMax.x;	
		if(pt.y > constrainMax.y)
			pt.y = constrainMax.y;
		delta.x = pt.x - ptLast.x;
	   	delta.y = pt.y - ptLast.y;
		if(delta.x || delta.y){
			PSsetgray(NSLightGray); 
			NSRectFill([self bounds]);
			NSDrawGrayBezel([self bounds] , [self bounds]);
			[dragNode resizeToControlPoint:pt :&controlPointNumber];
			[dragNode drawResizing];
			[window flushWindow];
			PSWait();
			ptLast = pt;
		} 
		ev = [window nextEventMatchingMask:NSLeftMouseUpMask|NSLeftMouseDraggedMask];
		finished = ([ev type] == NSLeftMouseUp);
	}
	nodePos.x = ([self bounds].size.width - [dragNode size].width) / 2;
	nodePos.y = ([self bounds].size.height - [dragNode size].height) / 2;
	[dragNode setPos:nodePos];
	[window disableFlushWindow];
	[self display];
	[window setAcceptsMouseMovedEvents:oldWindowAcceptsMouseMoved];
	[self unlockFocus];
	[window enableFlushWindow];
	[window flushWindow];
	PSWait();
}

@end


